#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $certValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";
require "lib/CPKI/cert_ext.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($h);

our $CPKI_TITLE = getTitle("TestCase::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
        $h->{init_policy_mapping_inhibit_flag}  ='off';
        $h->{init_explicit_policy_flag}  = 'off';
        $h->{init_any_policy_inhibit_flag} = 'off';
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "
SELECT
  testcase_code, category_id, level, certpath_id, serverset_id,
  ta_cert_id, ee_cert_id, exp_value, 
  init_policy_set, init_policy_mapping_inhibit_flag,
  init_explicit_policy_flag, init_any_policy_inhibit_flag,
  name, keyword, note
  FROM testcase 
  WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});
    $db->querySession([$sql]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{testcase_code}	= $row->[0];
    $h->{category_id}	= $row->[1];
    $h->{level}		= $row->[2];
    $h->{certpath_id}	= $row->[3];
    $h->{serverset_id}	= $row->[4];
    $h->{ta_cert_id}	= $row->[5];
    $h->{ee_cert_id}	= $row->[6];
    $h->{exp_value}	= $row->[7];
    $h->{init_policy_set}	= $row->[8];
    $h->{init_policy_mapping_inhibit_flag} = ($row->[9] eq '1')? 'on' : 'off';
    $h->{init_explicit_policy_flag} = ($row->[10] eq '1')? 'on' : 'off';
    $h->{init_any_policy_inhibit_flag}  = ($row->[11] eq '1')? 'on' : 'off';
    $h->{name}  = $row->[12];
    $h->{keyword}  = $row->[13];
    $h->{note}  = $row->[14];      
}

sub _setInput {
    my ($h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"testcase_code",	$h," size=\"64\" wrap=off");
    &set_input_txt($input,"category_id",	$h," size=\"10\"");
    &set_input_txt($input,"level",		$h," size=\"10\"");
    &set_input_txt($input,"certpath_id",	$h," size=\"10\"");
    &set_link_openform($input,"certpath_id","certpath");
    &set_input_txt($input,"serverset_id",	$h," size=\"10\"");
    &set_input_txt($input,"ta_cert_id",		$h," size=\"10\"");
    &set_input_txt($input,"ee_cert_id",		$h," size=\"10\"");
    &set_input_txt($input,"exp_value",		$h," size=\"10\"");
    &set_input_txa($input,"init_policy_set",	$h," cols=\"64\" rows=\"5\" ");
    &set_input_rad($input,"init_policy_mapping_inhibit_flag__on",	$h);
    &set_input_rad($input,"init_policy_mapping_inhibit_flag__off",	$h);
    &set_input_rad($input,"init_explicit_policy_flag__on",		$h);
    &set_input_rad($input,"init_explicit_policy_flag__off",		$h);
    &set_input_rad($input,"init_any_policy_inhibit_flag__on",		$h);
    &set_input_rad($input,"init_any_policy_inhibit_flag__off",		$h);  
    &set_input_txa($input,"name",		$h," cols=\"64\" rows=\"2\" ");
    &set_input_txa($input,"keyword",	$h," cols=\"64\" rows=\"5\" ");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" ");

}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_case_editconfirm.cgi" method="POST">
$input->{diff_mode}
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID**</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1" nowrap >Test Case Code</td><td>
$input->{testcase_code}
</td></tr>
<tr><td class="1" nowrap>Category ID</td><td>$input->{category_id}
</td></tr>
<tr><td class="1">Level</td><td>$input->{level}
</td></tr>
<tr><td class="1" colspan="2">Input Values</td></tr>
<tr><td class="1" nowrap>Server Set ID</td><td>$input->{serverset_id}</td></tr>
<tr><td class="1" nowrap>Certification Path ID</td><td>
  $input->{certpath_id} $input->{certpath_id_link}</td></tr>
<tr><td class="1">Trust Anchor Cert ID</td><td>$input->{ta_cert_id}</td></tr>
<tr><td class="1">Subscriber Cert ID</td><td>$input->{ee_cert_id}</td></tr>
<tr><td class="1">Expected Value</td><td>$input->{exp_value}
</td></tr>
<tr><td class="1">Initial Policy Set</td><td>$input->{init_policy_set}
</td></tr>
<tr><td class="1" nowrap>Initial Policy Mapping Inhibit Flag</td>
<td>$input->{init_policy_mapping_inhibit_flag__on} true / $input->{init_policy_mapping_inhibit_flag__off} false
</td></tr>
<tr><td class="1" nowrap>Initial Explicit Policy Flag</td>
<td>$input->{init_explicit_policy_flag__on} true / $input->{init_explicit_policy_flag__off} false
</td></tr>
<tr><td class="1" nowrap>Initial Any Policy Inhibit Flag</td>
<td>$input->{init_any_policy_inhibit_flag__on} true / $input->{init_any_policy_inhibit_flag__off} false
</td></tr>
<tr><td class="1" colspan="2">Notes</td></tr>
<tr><td class="1">Name</td><td>$input->{name}
</td></tr>
<tr><td class="1">Key Words</td><td>$input->{keyword}
</td></tr>
<tr><td class="1">Note</td><td>$input->{note}
</td></tr>
</tbody>
</table>
<div style="margin-left: 20px">
(**) Item must be filled to insert DB. 
</div>

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_case_list.cgi">Back to testcase list</a>
</p>
</form>
___EOF___
}
