#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;

#require "gpkitdm_util.pl";
require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $formHash = &decodeHttpRequest;
if ($formHash->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$formHash->{db}/;
}

our $sql = "SELECT id, testcase_code,category_id, level, exp_value, name, ta_cert_id, ee_cert_id FROM testcase ORDER BY id;";
our $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

$db->querySession([$sql]);

our $CPKI_TITLE = getTitle("Test Case List");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print "<p><a href=\"cpki_case_edit.cgi\">CREATE NEW TEST CASE</a></p>";

&_printTestCaseList($db->{resultSetArray}->[0]);

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($formHash);

&printLog($db);
&printPoweredBy;
&printFooter;

sub _printTestCaseList {
    my ($tbl) = @_;
    my ($i);

    if ($#{$tbl} < 0) { 
	print "<strong class=\"warn\">No item in list.</strong><br>";
	return;
    }

    print "<table style=\"margin-left:0px\" class=\"rowover\" STRIPED=\"true\">\n";
    print "<thead><tr><th>Action</th><th>ID</th><th>Code</th><th>Category</th><th>Lv</th><th>TA Cert</th><th>EE Cert</th><th>Expected<br>Value</th><th>Name</th></tr></thead>\n";
    print "<tbody>\n";
    for ($i = 0; $i <= $#{$tbl}; $i++) {
	my ($row) = $tbl->[$i];
	my ($id, $s);
	my ($ta_contents, $ee_contents);
	$id = $row->[0];
	my $ta_id = $row->[6];
	my $ee_id = $row->[7];
	if ($ta_id =~ /^\d+$/) {
	    $ta_contents = $ta_id . 
		"<a href=\"cpki_cert_getfile.cgi?id=" . $ta_id . "\">[DL]</a>";
	}
	if ($ee_id =~ /^\d+$/) {
	    $ee_contents = $ee_id . 
		"<a href=\"cpki_cert_getfile.cgi?id=" . $ee_id . "\">[DL]</a>";
	}

	$s .= "<tr>";
	$s .= "<td>";
	$s .= "<a href=\"cpki_common_delconfirm.cgi?type=case&id=$id\">Del</a> / ";
	$s .= "<a href=\"cpki_case_edit.cgi?id=$id\">Edit</a>";
	$s .= "</td>";
      	$s .= tdesch($row->[0]);
       	$s .= tdesch($row->[1]);
      	$s .= tdesch($row->[2]);
       	$s .= tdesch($row->[3]);
      	$s .= "<td>" . $ta_contents . "</td>";
      	$s .= "<td>" . $ee_contents . "</td>";
      	$s .= tdesch($row->[4]);
      	$s .= tdesch($row->[5]);
	$s .= "</tr>\n";
	print $s;
    }
    print "</tbody>\n";
    print "</table>\n";
}
