#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $crlValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($input, $h);

our $CPKI_TITLE = getTitle("Cross Certificate Pair::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "SELECT * FROM ccp WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{issued_to}	= $row->[1];
    $h->{issued_by}	= $row->[2];
    $h->{issued_to_raw}	= $row->[3];
    $h->{issued_by_raw}	= $row->[4];
    $h->{name}		= $row->[5];
    $h->{keyword}	= $row->[6];
    $h->{note}		= $row->[7];
}

sub _setInput {
    my ($input, $h) = @_;

    &set_input_txt($input,"id",			$h," size=\"10\"");
    &set_input_hdv($input,"org_id",		$h);
    &set_input_txt($input,"issued_to",		$h," size=\"10\"");
    &set_link_openform($input,"issued_to","cert");
    &set_input_txt($input,"issued_by",		$h," size=\"10\"");
    &set_link_openform($input,"issued_by","cert");
    &set_input_txt($input,"issued_to_raw",	$h," size=\"10\"");
    &set_input_txt($input,"issued_by_raw",	$h," size=\"10\"");

    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");
}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_ccp_editconfirm.cgi" method="POST">
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">issuedToThisCA</td><td>
certificate ID 
$input->{issued_to}
$input->{issued_to_link}
 -OR- 
raw certificate ID $input->{issued_to_raw}
</td></tr>
<tr><td class="1">issuedByThisCA</td><td>
certificate ID 
$input->{issued_by}
$input->{issued_by_link}
 -OR-
raw certificate ID $input->{issued_by_raw}
</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keyword</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>

<p>

<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 

<a href="cpki_ccp_list.cgi">Back CCP list</a>
</p>
</form>
___EOF___
}
