#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONST;
our $CPKI_CONFIG;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $formHash = &decodeHttpRequest;
if ($formHash->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$formHash->{db}/;
}

our $sql = "SELECT id, issued_to, issued_by, name
  FROM ccp ORDER by id;";
our $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

$db->querySession([$sql]);

our $CPKI_TITLE = getTitle("Cross Certificate Pair List");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print "<p><a href=\"cpki_ccp_edit.cgi\">CREATE NEW CROSS CERTIFICATE PAIR</a></p>\n";

&_printList($db->{resultSetArray}->[0]);

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($formHash);

print <<"___EOF___";
<p>
<b>Tool</b>
<ul>
<li><a href="cpki_cert_diffcfg.cgi" target="_blank">Display diff among two certificate configuration file.</a></li>
</ul>
</p>
___EOF___

&printLog($db);
&printPoweredBy;
&printFooter;

sub _printList {
    my ($tbl) = @_;
    my ($i);

    if ($#{$tbl} < 0) { 
	print "<strong class=\"warn\">No item in list.</strong><br>";
	return;
    }

    print "<table class=\"rowover\" STRIPED=\"true\">\n";
    print "<thead><tr><th>Action</th><th>ID</th>";
    print "<th>Issued To</th><th>Issued By</th><th>Note</th></tr></thead>\n";
    print "<tbody>\n";
    for ($i = 0; $i <= $#{$tbl}; $i++) {
	my ($row) = $tbl->[$i];
	my ($id, $s);
	$id = $row->[0];
	my $crypto_method = 
	    &selectorValue($CPKI_CONST->{selector}->{rsadsa}, $row->[1]);
	$s .= "<tr>";
	$s .= "<td>";
	$s .= "<a href=\"cpki_common_delconfirm.cgi?type=ccp&id=$id\">Del</a> / ";
	$s .= "<a href=\"cpki_ccp_edit.cgi?id=$id\">Edit</a>";
	$s .= "</td>";
	$s .= tdesch($row->[0]);
	$s .= tdesch($row->[1]);
	$s .= tdesch($row->[2]);
	$s .= tdesch($row->[3]);
	$s .= "</tr>\n";
	print $s;
    }
    print "</tbody>\n";
    print "</table>\n";
}

