#!/usr/bin/perl

use strict;
use lib "./lib";
use JNSA::DB::UTIL;
use JNSA::HTML::UTIL;

our $CPKI_CONFIG;
our $CPKI_CONST;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $h = &decodeHttpRequest;
$h->{db} = getDbnameByDsn($CPKI_CONFIG->{db}->{dsn});
$h->{usr} = $CPKI_CONFIG->{db}->{usr};
$h->{pw} = $CPKI_CONFIG->{db}->{pw};

$h->{head} = "/tmp/certdiffcfg.$$";

our $SCRIPT_OUTPUT = "";
if ($h->{id1} ne '' && $h->{id2} ne '') {
    &_setDiff($h->{id1}, $h->{id2});
}

our $CPKI_TITLE = getTitle("Certificate::Diff Certgen Config");

# === PRINT HTML ========================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print "<form name=\"f1\" action=\"cpki_cert_diffcfg.cgi\" method=\"POST\">\n";
print "Cert ID1: ";
print "<input type=\"text\" name=\"id1\" size=20 maxsize=20>\n";
print "&nbsp;Cert ID2: ";
print "<input type=\"text\" name=\"id2\" size=20 maxsize=20>\n";
print "<input type=\"submit\" value=\"Show Diff\">\n";
print "</form>\n";

print "<p>Display diff of the 'certgen' configuration file between '$h->{id1}' and '$h->{id2}'.</p>\n";

print "<p><strong class=\"warn\">Please specify ID1 and ID2.</strong></p>\n"
    if ($h->{id1} eq '' || $h->{id2} eq '');

print "<pre>\n";
print esch($SCRIPT_OUTPUT);
print "</pre>\n";

&printPoweredBy;
&printFooter;

sub _setDiff {
    my ($id1, $id2) = @_;
    $h->{file1} = $h->{head} . ".id1.cfg";
    $h->{file2} = $h->{head} . ".id2.cfg";
    
    _saveCfgFile($h->{file1}, $id1);
    _saveCfgFile($h->{file2}, $id2);

    if (! -f $h->{file1} || ! -f $h->{file2}) {

	if (! -f $h->{file1}) {
	    $SCRIPT_OUTPUT .= " cert_id='$id1' not found.";
	}
	if (! -f $h->{file2}) {
	    $SCRIPT_OUTPUT .= " cert_id='$id2' not found.";
	}
	return;
    }

    open(FOUT, "diff $h->{file1} $h->{file2}|") or return;
    while (<FOUT>) {
	$SCRIPT_OUTPUT .= $_;
    }
    close FOUT;

    #unlink $h->{file1};
    #unlink $h->{file2};
}

sub _saveCfgFile {
    my ($file, $id) = @_;

    my $argUsr = "--usr=$h->{usr}" if $h->{usr} ne '';
    my $argPw = "--pw=$h->{pw}" if $h->{pw} ne '';
    system "/usr/local/cpki/bin/dbcertgencfg --id=$id --out=$file --db=$h->{db} --nokey $argUsr $argPw"
}
