#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $certValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";
require "lib/CPKI/cert_ext.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($h);

our $CPKI_TITLE = getTitle("Certificate::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql1 = "SELECT * FROM cert WHERE id = $h->{id};";
    my  $sql2 = "SELECT * FROM cert_value WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql1,$sql2]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{base_id}  = $row->[1];
    $h->{issuer_keypair} = $row->[2];
    $h->{subject_keypair}  = $row->[3];
    $h->{hash_alg} = $row->[4];
    $h->{name}     = $row->[5];
    $h->{keyword}  = $row->[6];
    $h->{note}  = $row->[7];

    &set_hvalue_from_table($h, $certValueProc, $db->{resultSetArray}->[1]);
}

sub _setInput {
    my ($h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"base_id",	$h," size=\"10\"");
    &set_input_txt($input,"issuer_keypair",	$h," size=\"10\"");
    &set_input_txt($input,"subject_keypair",	$h," size=\"10\"");
    &set_input_sel($input,"hash_alg",	$h,"sha1md5");
    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");

    &set_input_sel($input,"sn_type",		$h,"dechex");
    &set_input_txt($input,"sn_value",		$h," size=\"72\"");

    &set_input_sel($input,"issuer_type",	$h,"dn");
    &set_input_txt($input,"issuer_value",	$h," size=\"72\"");

    &set_input_sel($input,"validity_before_ttype",	$h,"time");
    &set_input_txt($input,"validity_before_value",	$h," size=\"30\"");
    &set_input_sel($input,"validity_after_ttype",	$h,"time");
    &set_input_txt($input,"validity_after_value",	$h," size=\"30\"");

    &set_input_sel($input,"subject_type",	$h,"dn");
    &set_input_txt($input,"subject_value",	$h," size=\"72\"");

    set_input_cbx($input,"authkeyident_exists",		$h);
    set_input_cbx($input,"authkeyident_critical",	$h);
    set_input_txt($input,"authkeyident_keyident_value",	$h," size=\"84\"");
    set_input_sel($input,"authkeyident_certiss_dtype",	$h,"dn");
    set_input_txt($input,"authkeyident_certiss_value",	$h," size=\"84\"");
    set_input_sel($input,"authkeyident_certsn_stype",	$h,"dechex");
    set_input_txt($input,"authkeyident_certsn_value",	$h," size=\"84\"");

    set_input_cbx($input,"subjkeyident_exists",		$h);
    set_input_cbx($input,"subjkeyident_critical",	$h);
    set_input_txt($input,"subjkeyident_keyident_value",	$h," size=\"84\"");

    set_input_cbx($input,"keyusage_exists",		$h);
    set_input_cbx($input,"keyusage_critical",		$h);
    set_input_cbx($input,"keyusage_digitsign",		$h);
    set_input_cbx($input,"keyusage_nonrepu",		$h);
    set_input_cbx($input,"keyusage_keyenciph",		$h);
    set_input_cbx($input,"keyusage_dataenciph",		$h);
    set_input_cbx($input,"keyusage_keyagree",		$h);
    set_input_cbx($input,"keyusage_keycertsgn",		$h);
    set_input_cbx($input,"keyusage_crlsign",		$h);
    set_input_cbx($input,"keyusage_enciphonly",		$h);
    set_input_cbx($input,"keyusage_deciphonly",		$h);

    set_input_cbx($input,"certpolicy_exists",		$h);
    set_input_cbx($input,"certpolicy_critical",		$h);
    set_input_cbx($input,"certpolicy_anypolicy",	$h);
    set_input_txa($input,"certpolicy_oids_value",	$h," cols=\"70\" rows=\"3\" wrap=off");

    set_input_cbx($input,"policymap_exists",		$h);
    set_input_cbx($input,"policymap_critical",		$h);
    set_input_txa($input,"policymap_value",		$h," cols=\"80\" rows=\"3\" wrap=off");

    set_input_cbx($input,"subjaltname_exists",		$h);
    set_input_cbx($input,"subjaltname_critical",	$h);
    set_input_sel($input,"subjaltname_rfc_ord",		$h,"order4");
    set_input_txt($input,"subjaltname_rfc_value",	$h," size=\"84\"");
    set_input_sel($input,"subjaltname_dns_ord",		$h,"order4");
    set_input_txt($input,"subjaltname_dns_value",	$h," size=\"84\"");
    set_input_sel($input,"subjaltname_dn_ord",		$h,"order4");
    set_input_sel($input,"subjaltname_dn_type",		$h,"dn");
    set_input_txt($input,"subjaltname_dn_value",	$h," size=\"84\"");
    set_input_sel($input,"subjaltname_ip_ord",		$h,"order4");
    set_input_txt($input,"subjaltname_ip_value",	$h," size=\"84\"");

    set_input_cbx($input,"issaltname_exists",		$h);
    set_input_cbx($input,"issaltname_critical",		$h);
    set_input_sel($input,"issaltname_rfc_ord",		$h,"order4");
    set_input_txt($input,"issaltname_rfc_value",	$h," size=\"84\"");
    set_input_sel($input,"issaltname_dns_ord",		$h,"order4");
    set_input_txt($input,"issaltname_dns_value",	$h," size=\"84\"");
    set_input_sel($input,"issaltname_dn_ord",		$h,"order4");
    set_input_sel($input,"issaltname_dn_type",		$h,"dn");
    set_input_txt($input,"issaltname_dn_value",		$h," size=\"84\"");
    set_input_sel($input,"issaltname_ip_ord",		$h,"order4");
    set_input_txt($input,"issaltname_ip_value",		$h," size=\"84\"");

    set_input_cbx($input,"basicconst_exists",		$h);
    set_input_cbx($input,"basicconst_critical",		$h);
    set_input_cbx($input,"basicconst_ca_value",		$h);
    set_input_txt($input,"basicconst_pathlen_value",	$h," size=\"84\"");

    set_input_cbx($input,"nameconst_exists",		$h);
    set_input_cbx($input,"nameconst_critical",		$h);
    set_input_sel($input,"nameconst_permit_dtype1",	$h,"dn");
    set_input_txt($input,"nameconst_permit_value1",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_permit_dtype2",	$h,"dn");
    set_input_txt($input,"nameconst_permit_value2",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_permit_dtype3",	$h,"dn");
    set_input_txt($input,"nameconst_permit_value3",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_permit_dtype4",	$h,"dn");
    set_input_txt($input,"nameconst_permit_value4",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_permit_dtype5",	$h,"dn");
    set_input_txt($input,"nameconst_permit_value5",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_exclude_dtype1",	$h,"dn");
    set_input_txt($input,"nameconst_exclude_value1",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_exclude_dtype2",	$h,"dn");
    set_input_txt($input,"nameconst_exclude_value2",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_exclude_dtype3",	$h,"dn");
    set_input_txt($input,"nameconst_exclude_value3",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_exclude_dtype4",	$h,"dn");
    set_input_txt($input,"nameconst_exclude_value4",	$h," size=\"84\"");
    set_input_sel($input,"nameconst_exclude_dtype5",	$h,"dn");
    set_input_txt($input,"nameconst_exclude_value5",	$h," size=\"84\"");

    set_input_cbx($input,"policyconst_exists",		$h);
    set_input_cbx($input,"policyconst_critical",	$h);
    set_input_txt($input,"policyconst_req_value",	$h," size=\"84\"");
    set_input_txt($input,"policyconst_inhibit_value",	$h," size=\"84\"");

    set_input_cbx($input,"extkeyusage_exists",		$h);
    set_input_cbx($input,"extkeyusage_critical",	$h);
    set_input_txa($input,"extkeyusage_oids_value",	$h," cols=\"40\" rows=\"3\" wrap=off");

    set_input_cbx($input,"crldp_exists",		$h);
    set_input_cbx($input,"crldp_critical",		$h);
    set_input_sel($input,"crldp_dn_ord",		$h,"order2");
    set_input_sel($input,"crldp_dn_type",		$h,"dn");
    set_input_txt($input,"crldp_dn_value",		$h," size=\"84\"");
    set_input_sel($input,"crldp_uri_ord",		$h,"order2");
    set_input_txa($input,"crldp_uri_value",		$h," cols=\"70\" rows=\"3\" wrap=off");

    set_input_cbx($input,"authinfoacc_exists",		$h);
    set_input_cbx($input,"authinfoacc_critical",	$h);
    set_input_txt($input,"authinfoacc_uri_value",	$h," size=\"84\"");

    set_input_cbx($input,"ocspnochk_exists",		$h);
    set_input_cbx($input,"ocspnochk_critical",		$h);

    set_input_cbx($input,"privateext_exists",		$h);
    set_input_txa($input,"privateext_value",		$h," cols=\"80\" rows=\"3\" wrap=off");
}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_cert_editconfirm.cgi" method="POST">
$input->{diff_mode}
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID**</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">Base ID</td><td>
$input->{base_id}
You may enter ID when you copy certificate.
</td></tr>
<tr><td class="1">Issuer Keypair*</td><td>$input->{issuer_keypair}
<a href="javascript:openForm('key',document.f1.issuer_keypair)">[view]</a>
</td></tr>
<tr><td class="1" nowrap>Subject Keypair*</td><td>$input->{subject_keypair}
<a href="javascript:openForm('key',document.f1.subject_keypair)">[view]</a>
</td></tr>
<tr><td class="1" nowrap>Hash Algorithm</td><td>$input->{hash_alg}</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keywords</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>
<div style="margin-left: 20px">
(**) Item must be filled to insert DB. (*) Item must be filled to generate certificate file. (i.e. to issue certificate)
</div>

<h4>Basic Certificate</h4>

<table width="90%">
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr><td class="1">
  serialNumber*
  <a href="cpki_faq.cgi#positiveint" target="_blank">[?]</a>
  </td><td>
  $input->{sn_type} $input->{sn_value}</td></tr>
<tr><td class="1">issuer*</td><td>
  $input->{issuer_type} $input->{issuer_value}</td></tr>
<tr><td class="1">Validity*</td>
  <td>
    <table class="none">
    <tr><td><b>notBefore</b></td>
    <td>$input->{validity_before_ttype}</td>
    <td>$input->{validity_before_value} (YY)YYMMDDHHMMSS</td></tr>
    <tr><td><b>notAfter</b></td>
    <td>$input->{validity_after_ttype}</td>
    <td>$input->{validity_after_value} (YY)YYMMDDHHMMSS</td></tr>
    </table>
  </td></tr>
<tr><td class="1">subject*</td><td>
  $input->{subject_type} $input->{subject_value}</td></tr>
</tbody>
</table>

<!-- Authority Key Identifier ========================================== -->
<h4>$input->{authkeyident_exists}Authority Key Identifier - 
Critical:$input->{authkeyident_critical}
$input->{authkeyident_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>keyIdentifier</b></td>
<td>$input->{authkeyident_keyident_value}</td>
</tr>
<tr>
<td><b>authorityCertIssuer</b></td>
<td>$input->{authkeyident_certiss_dtype} $input->{authkeyident_certiss_value}</td>
</tr>
<tr>
<td><b>authorityCertSerialNumber</b></td>
<td>$input->{authkeyident_certsn_stype} $input->{authkeyident_certsn_value}</td>
</tr>
</tbody>
</table>

<!-- Subject Key Identifier ========================================== -->
<h4>$input->{subjkeyident_exists}Subject Key Identifier - 
Critical:$input->{subjkeyident_critical}
$input->{subjkeyident_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>keyIdentifier</b></td>
<td>$input->{subjkeyident_keyident_value}</td>
</tr>
</tbody>
</table>

<!-- Key Usage ========================================== -->
<h4>$input->{keyusage_exists}Key Usage - 
Critical:$input->{keyusage_critical}
$input->{keyusage_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr>
<td><b>keyUsage</b></td>
<td>
$input->{keyusage_digitsign}:digitalSignature
$input->{keyusage_nonrepu}:nonRepudiation
$input->{keyusage_keyenciph}:keyEncipherment
$input->{keyusage_dataenciph}:dataEncipherment
$input->{keyusage_keyagree}:keyAgreement
$input->{keyusage_keycertsgn}:keyCertSign
$input->{keyusage_crlsign}:cRLSign
$input->{keyusage_enciphonly}:encipherOnly
$input->{keyusage_deciphonly}:decipherOnly
</td>
</tr>
</tbody>
</table>

<!-- Certificate Policies ========================================== -->
<h4>$input->{certpolicy_exists}Certificate Policies - 
Critical:$input->{certpolicy_critical}
$input->{certpolicy_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr><td><b>anyPolicy</b></td><td>$input->{certpolicy_anypolicy}</td></tr>
<tr><td><b>OID1<br>OID2<br>&nbsp;&nbsp;:</b></td><td>$input->{certpolicy_oids_value}</td></tr>
</tbody>
</table>

<!-- Policy Mappings ========================================== -->
<h4>$input->{policymap_exists}Policy Mappings - 
Critical:$input->{policymap_critical}
$input->{policymap_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr><td><b>PolicyMappings</b></td><td>$input->{policymap_value}</td></tr>
</tbody>
</table>

<!-- Subject Alternative Name ========================================== -->
<h4>$input->{subjaltname_exists}Subject Alternative Name - 
Critical:$input->{subjaltname_critical}
$input->{subjaltname_invalid}
</h4>

<table>
<thead><tr><th>Order</th><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td>$input->{subjaltname_rfc_ord}</td>
<td><b>rfc822Name</b></td>
<td>$input->{subjaltname_rfc_value}</td>
</tr>
<tr>
<td>$input->{subjaltname_dns_ord}</td>
<td><b>dNSName</b></td>
<td>$input->{subjaltname_dns_value}</td>
</tr>
<tr>
<td>$input->{subjaltname_dn_ord}</td>
<td><b>directoryName</b></td>
<td>$input->{subjaltname_dn_type} $input->{subjaltname_dn_value}</td>
</tr>
<tr>
<td>$input->{subjaltname_ip_ord}</td>
<td><b>iPAddress</b></td>
<td>$input->{subjaltname_ip_value}</td>
</tr>
</tbody>
</table>

<!-- Issuer Alternative Name ========================================== -->
<h4>$input->{issaltname_exists}Issuer Alternative Name - 
Critical:$input->{issaltname_critical}
$input->{issaltname_invalid}
</h4>

<table>
<thead><tr><th>Order</th><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td>$input->{issaltname_rfc_ord}</td>
<td><b>rfc822Name</b></td>
<td>$input->{issaltname_rfc_value}</td>
</tr>
<tr>
<td>$input->{issaltname_dns_ord}</td>
<td><b>dNSName</b></td>
<td>$input->{issaltname_dns_value}</td>
</tr>
<tr>
<td>$input->{issaltname_dn_ord}</td>
<td><b>directoryName</b></td>
<td>$input->{issaltname_dn_type} $input->{issaltname_dn_value}</td>
</tr>
<tr>
<td>$input->{issaltname_ip_ord}</td>
<td><b>iPAddress</b></td>
<td>$input->{issaltname_ip_value}</td>
</tr>
</tbody>
</table>

<!-- Basic Constraints ========================================== -->
<h4>$input->{basicconst_exists}Basic Constraints - 
Critical:$input->{basicconst_critical}
$input->{basicconst_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr><td><b>cA</b></td><td>$input->{basicconst_ca_value}</td></tr>
<tr><td><b>pathLenConstraint</b></td><td>$input->{basicconst_pathlen_value}</td></tr>
</tbody>
</table>

<!-- Name Constraints ========================================== -->
<h4>$input->{nameconst_exists}Name Constraints - 
Critical:$input->{nameconst_critical}
$input->{nameconst_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>permittedSubtrees</b></td>
<td>
<table class="none">
<tr><td>$input->{nameconst_permit_dtype1}</td>
<td>$input->{nameconst_permit_value1}</td></tr>
<tr><td>$input->{nameconst_permit_dtype2}</td>
<td>$input->{nameconst_permit_value2}</td></tr>
<tr><td>$input->{nameconst_permit_dtype3}</td>
<td>$input->{nameconst_permit_value3}</td></tr>
<tr><td>$input->{nameconst_permit_dtype4}</td>
<td>$input->{nameconst_permit_value4}</td></tr>
<tr><td>$input->{nameconst_permit_dtype5}</td>
<td>$input->{nameconst_permit_value5}</td></tr>
</table>
</td>
</tr>
<tr>
<td><b>excludedSubtrees</b></td>
<td>
<table class="none">
<tr><td>$input->{nameconst_exclude_dtype1}</td>
<td> $input->{nameconst_exclude_value1}</td></tr>
<tr><td>$input->{nameconst_exclude_dtype2}</td>
<td> $input->{nameconst_exclude_value2}</td></tr>
<tr><td>$input->{nameconst_exclude_dtype3}</td>
<td> $input->{nameconst_exclude_value3}</td></tr>
<tr><td>$input->{nameconst_exclude_dtype4}</td>
<td> $input->{nameconst_exclude_value4}</td></tr>
<tr><td>$input->{nameconst_exclude_dtype5}</td>
<td> $input->{nameconst_exclude_value5}</td></tr>
</table>
</td>
</tr>
</tbody>
</table>

<!-- Policy Constraints ========================================== -->
<h4>$input->{policyconst_exists}Policy Constraints - 
Critical:$input->{policyconst_critical}
$input->{policyconst_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>requireExplicitPolicy</b></td>
<td>$input->{policyconst_req_value}</td>
</tr>
<tr>
<td><b>inhibitPolicyMapping</b></td>
<td>$input->{policyconst_inhibit_value}</td>
</tr>
</tbody>
</table>

<!-- Extended Key Usage ========================================== -->
<h4>$input->{extkeyusage_exists}Extended Key Usage - 
Critical:$input->{extkeyusage_critical}
$input->{extkeyusage_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>OID1<br>OID2<br>&nbsp;&nbsp;:</b></td>
<td>$input->{extkeyusage_oids_value}</td>
</tr>
</tbody>
</table>

<!-- CRL Distribution Points ========================================== -->
<h4>$input->{crldp_exists}CRL Distribution Points - 
Critical:$input->{crldp_critical}
$input->{crldp_invalid}
</h4>

<table>
<thead><tr><th>Order</th><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td>$input->{crldp_dn_ord}</td>
<td><b>directoryName</b></td>
<td>$input->{crldp_dn_type} $input->{crldp_dn_value}</td>
</tr>
<tr>
<td>$input->{crldp_uri_ord}</td>
<td><b>URI
<a href="cpki_tool_urihelper.cgi" target="_blank">[?]</a>
</b></td>
<td>$input->{crldp_uri_value}</td>
</tr>
</tbody>
</table>

<!-- Authority Information Access ======================================== -->
<h4>$input->{authinfoacc_exists}Authority Information Access - 
Critical:$input->{authinfoacc_critical}
$input->{authinfoacc_invalid}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>ocsp=URI<br>caIssuers=URI</b></td>
<td>$input->{authinfoacc_uri_value}</td>
</tr>
</tbody>
</table>

<!-- OCSP No Check ======================================== -->
<h4>$input->{ocspnochk_exists}OCSP No Check - 
Critical:$input->{ocspnochk_critical}
$input->{ocspnochk_invalid}
</h4>

<!-- Private Extension ==================================== -->
<h4>$input->{privateext_exists}Private Extensions - 
<input type="hidden" name="privateext_critical" value="on">
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td nowrap><b>
OID1=[cn]:DER1<br>
OID2=[cn]:DER2<br>
&nbsp;&nbsp;:<br>
c - critical<br>
n - non-critical
</b></td>
<td>
$input->{privateext_value}<br>
2.16.840.1.113730.1.1=n:03020780 -- 
for SSL client certification of netscape-cert-type extension
</td>
</tr>
</tbody>
</table>

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_cert_list.cgi">Back to certificate list</a>
</p>
</form>
___EOF___
}
