#!/usr/bin/perl

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
use lib "./lib";
use Data::Dumper;

our $f = &decodeHttpRequest;

&_setFormValue($f);

our $CPKI_TITLE = getTitle("Certificate::ConfirmUpdate");

$aPostname =
    [qw(id org_id base_id
	issuer_keypair subject_keypair hash_alg
	name keyword note
	sn issuer validity
	subject
	authkeyident_exists authkeyident_critical
	 authkeyident_keyident authkeyident_certiss authkeyident_certsn
	subjkeyident_exists subjkeyident_critical
	 subjkeyident_keyident
	keyusage_exists keyusage_critical
	 keyusage
	certpolicy_exists certpolicy_critical
	 certpolicy_anypolicy certpolicy_oids
	policymap_exists policymap_critical
	 policymap
	subjaltname_exists subjaltname_critical
	 subjaltname_rfc_ord subjaltname_rfc
	 subjaltname_dns_ord subjaltname_dns
	 subjaltname_dn_ord subjaltname_dn
	 subjaltname_ip_ord subjaltname_ip
	issaltname_exists issaltname_critical
	 issaltname_rfc_ord issaltname_rfc
	 issaltname_dns_ord issaltname_dns
	 issaltname_dn_ord issaltname_dn
	 issaltname_ip_ord issaltname_ip
	basicconst_exists basicconst_critical
	 basicconst_ca basicconst_pathlen
	nameconst_exists nameconst_critical
	 nameconst_permit nameconst_exclude
	policyconst_exists policyconst_critical
	 policyconst_req policyconst_inhibit
	extkeyusage_exists extkeyusage_critical
	 extkeyusage_oids
	crldp_exists crldp_critical
	 crldp_dn_ord crldp_dn
	 crldp_uri_ord crldp_uri
	authinfoacc_exists authinfoacc_critical
	 authinfoacc_uri
	ocspnochk_exists ocspnochk_critical
	privateext_exists privateext_critical
         privateext
	)];
$postdata = &string_html_trth_hash($f,$aPostname);

# Print HTML ================================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print <<"__EOFEOFEOF__";
<p><b>Do you update following certificate(No.$f->{id})?</b></p>

<table>
<thead><th>Item</th><th>Value</th></thead>
<tbody>
$postdata
</tbody>
</table>
<form action="cpki_cert_editresult.cgi" method="POST">
__EOFEOFEOF__

printHtmlInputHiddenHashByArray($f,$aPostname);

#printDumper($f);

print <<"__EOFEOFEOF__";
<input type="submit" value="Register"> | 
<a href="cpki_cert_list.cgi">Back to certificate list</a>
</form>
__EOFEOFEOF__

&printPoweredBy;
&printFooter;

sub _setFormValue {
    my($h) = @_;

    # serialNumber
    if ($h->{sn_value} ne '') {
	$h->{sn} = $h->{sn_type} . ":" . $h->{sn_value};
    }

    # issuer
    if ($h->{issuer_value} ne '') {
	$h->{issuer} = $h->{issuer_type} . ":" . $h->{issuer_value};
    }

    # validity
    my ($v);
    $v .= "notBefore=" . 
	$h->{validity_before_ttype} . ":" . $h->{validity_before_value};
    $v .= "\tnotAfter=" . 
	$h->{validity_after_ttype} . ":" . $h->{validity_after_value};
    $h->{validity} = $v;

    # subject
    if ($h->{subject_value} ne '') {
	$h->{subject} = $h->{subject_type} . ":" . $h->{subject_value};
    }

    # == AuthorityKeyIdentifier =========================================
    $h->{authkeyident_critical} =
	($h->{authkeyident_critical} eq 'on') ? "true" : "false";

    if ($h->{authkeyident_keyident_value} ne '') {
	$h->{authkeyident_keyident} =
	    "keyIdentifier=" . $h->{authkeyident_keyident_value}
    }

    if ($h->{authkeyident_certiss_value} ne '') {
	$h->{authkeyident_certiss} =
	    "authorityCertIssuer=" . $h->{authkeyident_certiss_dtype} . ":" 
		. $h->{authkeyident_certiss_value};
    }

    if ($h->{authkeyident_certsn_value} ne '') {
	$h->{authkeyident_certsn} = "authorityCertSerialNumber=" . 
	    $h->{authkeyident_certsn_stype} . ":" .
		$h->{authkeyident_certsn_value};
    }

    # == SubjectKeyIdentifier ==========================================
    $h->{subjkeyident_critical} =
	($h->{subjkeyident_critical} eq 'on') ? "true" : "false";

    if ($h->{subjkeyident_keyident_value} ne '') {
	$h->{subjkeyident_keyident} = $h->{subjkeyident_keyident_value};
    }

    # == KeyUsage ======================================================
    $h->{keyusage_critical} =
	($h->{keyusage_critical} eq 'on') ? "true" : "false";

    my @a = ();
    push @a, 'digitalSignature'	if ($h->{keyusage_digitsign}	eq 'on');
    push @a, 'nonRepudiation'	if ($h->{keyusage_nonrepu}	eq 'on');
    push @a, 'keyEncipherment'	if ($h->{keyusage_keyenciph}	eq 'on');
    push @a, 'dataEncipherment'	if ($h->{keyusage_dataenciph}	eq 'on');
    push @a, 'keyAgreement'	if ($h->{keyusage_keyagree}	eq 'on');
    push @a, 'keyCertSign'	if ($h->{keyusage_keycertsgn}	eq 'on');
    push @a, 'cRLSign'		if ($h->{keyusage_crlsign}	eq 'on');
    push @a, 'encipherOnly'	if ($h->{keyusage_enciphonly}	eq 'on');
    push @a, 'decipherOnly'	if ($h->{keyusage_deciphonly}	eq 'on');
    $h->{keyusage} = join("\t", @a);
    undef @a;

    # == Certificate Policies ===========================================
    $h->{certpolicy_critical} =
	($h->{certpolicy_critical} eq 'on') ? "true" : "false";
    $h->{certpolicy_oids} = canonical_tab($h->{certpolicy_oids_value});

    # == Policy Mappings ================================================
    $h->{policymap_critical} =
	($h->{policymap_critical} eq 'on') ? "true" : "false";
    $h->{policymap} = canonical_tab($h->{policymap_value});

    # == Subject Alt Name ===============================================
    $h->{subjaltname_critical} =
	($h->{subjaltname_critical} eq 'on') ? "true" : "false";

    &ord_minusone($h,
		  [qw(subjaltname_rfc_ord subjaltname_dns_ord
		      subjaltname_dn_ord subjaltname_ip_ord)],
		  [qw(subjaltname_rfc_value subjaltname_dns_value
		      subjaltname_dn_value subjaltname_ip_value)]);
    &renumber_seq($h,[qw(subjaltname_rfc_ord subjaltname_dns_ord
			 subjaltname_dn_ord subjaltname_ip_ord)]);

    if ($h->{subjaltname_rfc_value} ne '') {
	$h->{subjaltname_rfc} = "rfc822Name=" . $h->{subjaltname_rfc_value};
    }
    if ($h->{subjaltname_dns_value} ne '') {
	$h->{subjaltname_dns} = "dNSName=" . $h->{subjaltname_dns_value};
    }
    if ($h->{subjaltname_dn_value} ne '') {
	$h->{subjaltname_dn} =
	    "directoryName=" . $h->{subjaltname_dn_type} . ":"
		. $h->{subjaltname_dn_value};
    }
    if ($h->{subjaltname_ip_value} ne '') {
	$h->{subjaltname_ip} = "iPAddress=" . $h->{subjaltname_ip_value};
    }

    # == IssuerAltName ==================================================
    $h->{issaltname_critical} =
	($h->{issaltname_critical} eq 'on') ? "true" : "false";

    &ord_minusone($h,
		  [qw(issaltname_rfc_ord issaltname_dns_ord
		      issaltname_dn_ord issaltname_ip_ord)],
		  [qw(issaltname_rfc_value issaltname_dns_value
		      issaltname_dn_value issaltname_ip_value)]);
    &renumber_seq($h,[qw(issaltname_rfc_ord issaltname_dns_ord
			 issaltname_dn_ord issaltname_ip_ord)]);

    if ($h->{issaltname_rfc_value} ne '') {
	$h->{issaltname_rfc} = "rfc822Name=" . $h->{issaltname_rfc_value};
    }
    if ($h->{issaltname_dns_value} ne '') {
	$h->{issaltname_dns} = "dNSName=" . $h->{issaltname_dns_value};
    }
    if ($h->{issaltname_dn_value} ne '') {
	$h->{issaltname_dn} =
	    "directoryName=" . $h->{issaltname_dn_type} .
		":" . $h->{issaltname_dn_value};
    }
    if ($h->{issaltname_ip_value} ne '') {
	$h->{issaltname_ip} = "iPAddress=" . $h->{issaltname_ip_value};
    }

    # == Basic Constraint ===============================================
    $h->{basicconst_critical} = 
	($h->{basicconst_critical} eq 'on') ? "true" : "false";
    $h->{basicconst_ca} =
	($h->{basicconst_ca_value} eq 'on') ? "cA=TRUE" : "";

    if ($h->{basicconst_pathlen_value} ne '') {
	$h->{basicconst_pathlen} =
	    "pathLenConstraint=" . $h->{basicconst_pathlen_value};
    }

    # == Name Constraint ================================================
    $h->{nameconst_critical} =
	($h->{nameconst_critical} eq 'on') ? "true" : "false";

    $h->{nameconst_permit} = _getNameConst($h,"permit");
    $h->{nameconst_exclude} = _getNameConst($h,"exclude");

    # == Policy Constraint ===============================================
    $h->{policyconst_critical} =
	($h->{policyconst_critical} eq 'on') ? "true" : "false";

    if ($h->{policyconst_req_value} ne '') {
	$h->{policyconst_req} =
	    "requireExplicitPolicy=" . $h->{policyconst_req_value};
    }

    if ($h->{policyconst_inhibit_value} ne '') {
	$h->{policyconst_inhibit} =
	    "inhibitPolicyMapping=" . $h->{policyconst_inhibit_value};
    }

    # == Extended Key Usage ==============================================
    $h->{extkeyusage_critical} =
	($h->{extkeyusage_critical} eq 'on') ? "true" : "false";

    if ($h->{extkeyusage_oids_value} ne '') {
	$h->{extkeyusage_oids} = canonical_tab($h->{extkeyusage_oids_value});
    }

    # == CRL Distribution Point ==========================================
    $h->{crldp_critical} =
	($h->{crldp_critical} eq 'on') ? "true" : "false";

    &ord_minusone($h,
		  [qw(crldp_dn_ord crldp_uri_ord)],
		  [qw(crldp_dn_value crldp_uri_value)]);
    &renumber_seq($h,[qw(crldp_dn_ord crldp_uri_ord)]);

    if ($h->{crldp_dn_value} ne '') {
	$h->{crldp_dn} =
	    "directoryName=" .
		$h->{crldp_dn_type} . ":" . $h->{crldp_dn_value};
    }

    if ($h->{crldp_uri_value} ne '') {
	$h->{crldp_uri} =
	    "URI=" . canonical_tab($h->{crldp_uri_value});
    }

    # == Authority Information Access =====================================
    $h->{authinfoacc_critical} =
	($h->{authinfoacc_critical} eq 'on') ? "true" : "false";
    if ($h->{authinfoacc_uri_value} ne '') {
	$h->{authinfoacc_uri} = $h->{authinfoacc_uri_value};
    }

    # == OCSP No Check ====================================================
    $h->{ocspnochk_critical} =
	($h->{ocspnochk_critical} eq 'on') ? "true" : "false";
    if ($h->{crlnum_value} ne '') {
	$h->{crlnum} = $h->{crlnum_stype} . ":" . $h->{crlnum_value};
    }

    # == Private Extension ==============================================
    $h->{privateext_critical} =
	($h->{privateext_critical} eq 'on') ? "true" : "false";

    if ($h->{privateext_value} ne '') {
	$h->{privateext} = canonical_tab($h->{privateext_value});
    }
}

sub _getNameConst {
    my ($h,$type) = @_;
    my @list, $i;

    for $i (1..5) {
	my $key1 = "nameconst_" . $type . "_value" . $i;
	my $key2 = "nameconst_" . $type . "_dtype" . $i;
	my $v = $h->{$key1};
	next if $v eq "";
	push @list, $h->{$key2} . ":" . $v;
    }

    return undef if ($#list == -1);

    my $lhs_eq = ($type eq "permit") ? 
	"permittedSubtrees=" : "excludedSubtrees=";

    return $lhs_eq . join("\t", @list);
}
