#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $crlValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($input, $h);

our $CPKI_TITLE = getTitle("Certification Path::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm("cpki_certpath_editconfirm.cgi");

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "SELECT * FROM certpath WHERE id = $h->{id} ORDER BY id;";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql]);

    for my $i (0..$#{$db->{resultSetArray}->[0]}) {
	my $row = $db->{resultSetArray}->[0]->[$i];
	my $seq = $row->[1];
	my $idx = $i + 1;
	$h->{"cert_id_$idx"} = $row->[2];
	$h->{"crl_id_$idx"}  = $row->[3];
	$h->{"ocsp_id_$idx"} = $row->[4];
    }
    $h->{num_entities_input} = $#{$db->{resultSetArray}->[0]} + 1 + 5;
}

sub _setInput {
    my ($input, $h) = @_;

    &set_input_txt($input,"id",			$h," size=\"10\"");
    &set_input_hdv($input,"org_id",		$h);

    for my $idx (1..$h->{num_entities_input}) {
	&set_input_txt($input,"cert_id_$idx",	$h," size=\"10\"");
	&set_input_txt($input,"crl_id_$idx",	$h," size=\"10\"");
	&set_input_txt($input,"ocsp_id_$idx",	$h," size=\"10\"");
	&set_link_openform($input,"cert_id_$idx","cert");
	&set_link_openform($input,"crl_id_$idx","crl");
	&set_link_openform($input,"ocsp_id_$idx","ocsp");
    }
}

sub _printForm {
    my ($next_url, $extedit_link) = @_;
    my (%t, $list_url, $backto);
    print <<"___EOF___";
<form name="f1" action="$next_url" method="POST">
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
$input->{num_entities_input}
<input type="hidden" name="num_entities_input" value=\"$h->{num_entities_input}\">
</td></tr>
<tr><th colspan="2">Intermediate Certificates/CRLs/OCSP Simulators</th></tr>
___EOF___

    for my $idx (1..$h->{num_entities_input}) {
	print "<tr><td class=\"1\" nowrap>Entity $idx</td><td>";
	print "&nbsp;Cert: ", $input->{"cert_id_$idx"};
	print $input->{"cert_id_${idx}_link"};
	print "&nbsp;CRL: ", $input->{"crl_id_$idx"};
	print $input->{"crl_id_${idx}_link"};
        print "&nbsp;OCSP: ", $input->{"ocsp_id_$idx"};
	print $input->{"ocsp_id_${idx}_link"};
	print "</td></tr>\n";
    }

    print <<"___EOF___";
<tr><td colspan=2>NOTE: If you want more entities input field
save this record and edit again. You'll get five more field.
</td></tr>
</tbody>
</table>



<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="$list_url">$backto</a>
</p>
</form>
___EOF___
}
