#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $crlValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";
require "lib/CPKI/crl_ext.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($h);

our $CPKI_TITLE = getTitle("CRL::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql1 = "SELECT * FROM crl WHERE id = $h->{id};";
    my  $sql2 = "SELECT * FROM crl_value WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql1,$sql2]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{base_id}  = $row->[1];
    $h->{issuer_keypair} = $row->[2];
    $h->{hash_alg} = $row->[3];
    $h->{name}     = $row->[4];
    $h->{keyword}  = $row->[5];
    $h->{note}  = $row->[6];

    &set_hvalue_from_table($h, $crlValueProc, $db->{resultSetArray}->[1]);
}

sub _setInput {
    my ($h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"base_id",	$h," size=\"10\"");
    &set_input_txt($input,"issuer_keypair",	$h," size=\"10\"");
    &set_link_openform($input,"issuer_keypair","key");
    &set_input_sel($input,"hash_alg",	$h,"sha1md5");
    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");

    &set_input_sel($input,"issuer_type",	$h,"dn");
    &set_input_txt($input,"issuer_value",	$h," size=\"72\"");

    &set_input_sel($input,"validity_this_ttype",	$h,"time");
    &set_input_txt($input,"validity_this_value",	$h," size=\"30\"");
    &set_input_sel($input,"validity_next_ttype",	$h,"time");
    &set_input_txt($input,"validity_next_value",	$h," size=\"30\"");

    &set_input_sel($input,"revokecert_stype1",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial1",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype1",	$h,"time");
    &set_input_txt($input,"revokecert_time1",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason1",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype2",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial2",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype2",	$h,"time");
    &set_input_txt($input,"revokecert_time2",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason2",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype3",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial3",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype3",	$h,"time");
    &set_input_txt($input,"revokecert_time3",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason3",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype4",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial4",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype4",	$h,"time");
    &set_input_txt($input,"revokecert_time4",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason4",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype5",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial5",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype5",	$h,"time");
    &set_input_txt($input,"revokecert_time5",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason5",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype6",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial6",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype6",	$h,"time");
    &set_input_txt($input,"revokecert_time6",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason6",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype7",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial7",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype7",	$h,"time");
    &set_input_txt($input,"revokecert_time7",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason7",	$h," size=\"3\"");

    &set_input_sel($input,"revokecert_stype8",	$h,"dechex");
    &set_input_txt($input,"revokecert_serial8",	$h," size=\"15\"");
    &set_input_sel($input,"revokecert_ttype8",	$h,"time");
    &set_input_txt($input,"revokecert_time8",	$h," size=\"15\"");
    &set_input_txt($input,"revokecert_reason8",	$h," size=\"3\"");

    set_input_cbx($input,"authkeyident_exists",		$h);
    set_input_cbx($input,"authkeyident_critical",	$h);
    set_input_txt($input,"authkeyident_keyident",	$h," size=\"84\"");
    set_input_sel($input,"authkeyident_certiss_dtype",	$h,"dn");
    set_input_txt($input,"authkeyident_certiss_value",	$h," size=\"84\"");
    set_input_sel($input,"authkeyident_certsn_stype",	$h,"dechex");
    set_input_txt($input,"authkeyident_certsn_value",	$h," size=\"84\"");

    set_input_cbx($input,"issaltname_exists",		$h);
    set_input_cbx($input,"issaltname_critical",		$h);
    set_input_sel($input,"issaltname_rfc_ord",		$h,"order4");
    set_input_txt($input,"issaltname_rfc_value",	$h," size=\"84\"");
    set_input_sel($input,"issaltname_dns_ord",		$h,"order4");
    set_input_txt($input,"issaltname_dns_value",	$h," size=\"84\"");
    set_input_sel($input,"issaltname_dn_ord",		$h,"order4");
    set_input_sel($input,"issaltname_dn_type",		$h,"dn");
    set_input_txt($input,"issaltname_dn_value",		$h," size=\"84\"");
    set_input_sel($input,"issaltname_ip_ord",		$h,"order4");
    set_input_txt($input,"issaltname_ip_value",		$h," size=\"84\"");

    &set_input_cbx($input,"crlnum_exists",		$h);
    &set_input_cbx($input,"crlnum_critical",		$h);
    &set_input_sel($input,"crlnum_stype",		$h,"dechex");
    &set_input_txt($input,"crlnum_value",		$h," size=\"84\"");

    &set_input_cbx($input,"idp_exists",		$h);
    &set_input_cbx($input,"idp_critical",		$h);
    &set_input_sel($input,"idp_dn_ord",		$h,"order2");
    &set_input_sel($input,"idp_dn_type",		$h,"dn");
    &set_input_txt($input,"idp_dn_value",		$h," size=\"84\"");
    &set_input_sel($input,"idp_uri_ord",		$h,"order2");
    &set_input_txa($input,"idp_uri_value",	$h," cols=\"70\" rows=\"3\" wrap=off");
    &set_input_cbx($input,"idp_onlyusr",		$h);
    &set_input_cbx($input,"idp_onlyca",		$h);

    set_input_cbx($input,"privateext_exists",		$h);
    set_input_txa($input,"privateext_value",		$h," cols=\"80\" rows=\"3\" wrap=off");
}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_crl_editconfirm.cgi" method="POST">
$input->{diff_mode}
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID**</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">Base ID</td><td>
$input->{base_id}
You may enter ID when you copy CRL.
</td></tr>
<tr><td class="1">Issuer Key*</td><td>$input->{issuer_keypair} $input->{issuer_keypair_link}</td></tr>
<tr><td class="1" nowrap>Hash Algorithm</td><td>$input->{hash_alg}</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keywords</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>
<div style="margin-left: 20px">
(**) Item must be filled to insert DB. (*) Item must be filled to generate CRL file. (i.e. to issue CRL)
</div>


<h4>Basic CRL</h4>

<table width="90%">
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr><td class="1">issuer*</td><td>$input->{issuer_type} $input->{issuer_value}</td></tr>
<tr><td class="1">Validity*</td>
  <td>
    <table class="none">
    <tr><td><b>thisUpdate</b></td>
    <td>$input->{validity_this_ttype}</td>
    <td>$input->{validity_this_value} (YY)YYMMDDHHMMSS</td></tr>
    <tr><td><b>nextUpdate</b></td>
    <td>$input->{validity_next_ttype}</td>
    <td>$input->{validity_next_value} (YY)YYMMDDHHMMSS</td></tr>
    </table>
  </td></tr>
<tr><td class="1">revokedCert</td>
  <td>
  <table class="none">
  <thead>
  <tr>
  <td class="1" colspan="2">serialNumber*</td>
  <td class="1" colspan="2">time*</td>
  <td class="1">reason</td>
  </tr>
  </thead>
  <tbody>
  <tr>
  <td>$input->{revokecert_stype1}</td>
  <td>$input->{revokecert_serial1}</td>
  <td>$input->{revokecert_ttype1}</td>
  <td>$input->{revokecert_time1}</td>
  <td>$input->{revokecert_reason1}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype2}</td>
  <td>$input->{revokecert_serial2}</td>
  <td>$input->{revokecert_ttype2}</td>
  <td>$input->{revokecert_time2}</td>
  <td>$input->{revokecert_reason2}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype3}</td>
  <td>$input->{revokecert_serial3}</td>
  <td>$input->{revokecert_ttype3}</td>
  <td>$input->{revokecert_time3}</td>
  <td>$input->{revokecert_reason3}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype4}</td>
  <td>$input->{revokecert_serial4}</td>
  <td>$input->{revokecert_ttype4}</td>
  <td>$input->{revokecert_time4}</td>
  <td>$input->{revokecert_reason4}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype5}</td>
  <td>$input->{revokecert_serial5}</td>
  <td>$input->{revokecert_ttype5}</td>
  <td>$input->{revokecert_time5}</td>
  <td>$input->{revokecert_reason5}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype6}</td>
  <td>$input->{revokecert_serial6}</td>
  <td>$input->{revokecert_ttype6}</td>
  <td>$input->{revokecert_time6}</td>
  <td>$input->{revokecert_reason6}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype7}</td>
  <td>$input->{revokecert_serial7}</td>
  <td>$input->{revokecert_ttype7}</td>
  <td>$input->{revokecert_time7}</td>
  <td>$input->{revokecert_reason7}</td>
  </tr>
  <tr>
  <td>$input->{revokecert_stype8}</td>
  <td>$input->{revokecert_serial8}</td>
  <td>$input->{revokecert_ttype8}</td>
  <td>$input->{revokecert_time8}</td>
  <td>$input->{revokecert_reason8}</td>
  </tr>
  </tbody>
  </table>
  reason: empty or any integer (unspecified:0 keyCompromise:1 
  cACompromise:2 affiliationChanged:3 superseded:4
  cessationOfOperation:5 certificateHold:6 removeFromCRL:8
  privilegeWithdrawn:9 aACompromise:10)
  </td></tr>
</tbody>
</table>

<h4>$input->{authkeyident_exists}Authority Key Identifier - 
Critical:$input->{authkeyident_critical}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td><b>keyIdentifier</b></td>
<td>$input->{authkeyident_keyident}</td>
</tr>
<tr>
<td><b>authorityCertIssuer</b></td>
<td>$input->{authkeyident_certiss_dtype} $input->{authkeyident_certiss_value}</td>
</tr>
<tr>
<td><b>authorityCertSerialNumber</b></td>
<td>$input->{authkeyident_certsn_stype} $input->{authkeyident_certsn_value}</td>
</tr>
</tbody>
</table>

<h4>$input->{issaltname_exists}Issuer Alternative Name - 
Critical:$input->{issaltname_critical}
</h4>

<table>
<thead><tr><th>Order</th><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td>$input->{issaltname_rfc_ord}</td>
<td><b>rfc822Name</b></td>
<td>$input->{issaltname_rfc_value}</td>
</tr>
<tr>
<td>$input->{issaltname_dns_ord}</td>
<td><b>dNSName</b></td>
<td>$input->{issaltname_dns_value}</td>
</tr>
<tr>
<td>$input->{issaltname_dn_ord}</td>
<td><b>directoryName</b></td>
<td>$input->{issaltname_dn_type} $input->{issaltname_dn_value}</td>
</tr>
<tr>
<td>$input->{issaltname_ip_ord}</td>
<td><b>iPAddress</b></td>
<td>$input->{issaltname_ip_value}</td>
</tr>
</tbody>
</table>

<h4>$input->{crlnum_exists}CRL Number - 
Critical:$input->{crlnum_critical}
</h4>

<table>
<tr><td>
$input->{crlnum_stype} $input->{crlnum_value}
</td></tr>
</table>

<h4>$input->{idp_exists}Issuing Distribution Point - 
Critical:$input->{idp_critical}
</h4>

<table>
<thead><tr><th>Order</th><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td>$input->{idp_dn_ord}</td>
<td><b>directoryName</b></td>
<td>$input->{idp_dn_type} $input->{idp_dn_value}</td>
</tr>
<tr>
<td>$input->{idp_uri_ord}</td>
<td><b>URI
<a href="cpki_tool_urihelper.cgi" target="_blank">[?]</a>
</b></td>
<td>$input->{idp_uri_value}</td>
</tr>
<tr>
<td colspan="3">
$input->{idp_onlyusr}<b>:onlyContainsUserCerts<b><br>
$input->{idp_onlyca}<b>:onlyContainsCACerts</b><br>
</td>
</tr>
</tbody>
</table>

<!-- Private Extension ==================================== -->
<h4>$input->{privateext_exists}Private Extensions</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td nowrap><b>OID1=[cn]:DER1<br>OID2=[cn]:DER2<br>&nbsp;&nbsp;:</b></td>
<td>$input->{privateext_value}</td>
</tr>
</tbody>
</table>

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_crl_list.cgi">Back to CRL list</a>
</p>
</form>
___EOF___
}
