#!/usr/bin/perl

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
use lib "./lib";
use Data::Dumper;

our $f = &decodeHttpRequest;

&_setFormValue($f);

our $CPKI_TITLE = getTitle("CRL::ConfirmUpdate");

$aPostname =
    [qw(id org_id base_id
	issuer_keypair hash_alg
	name keyword note
	issuer validity
	revokecert
	authkeyident_exists authkeyident_critical
	  authkeyident_keyident authkeyident_certiss authkeyident_certsn
	issaltname_exists issaltname_critical
	  issaltname_rfc_ord issaltname_rfc issaltname_dns_ord issaltname_dns
	  issaltname_dn_ord issaltname_dn issaltname_ip_ord issaltname_ip
	crlnum_exists crlnum_critical
	  crlnum
	idp_exists idp_critical
	  idp_dn_ord idp_dn
	  idp_uri_ord idp_uri
	  idp_onlyusr_ord idp_onlyusr
	  idp_onlyca_ord idp_onlyca
	privateext_exists privateext_critical
	  privateext)];
$postdata = &string_html_trth_hash($f,$aPostname);

# Print HTML ================================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print <<"__EOFEOFEOF__";
<p><b>Do you update following CRL(No.$f->{id})?</b></p>

<table>
<thead><th>Item</th><th>Value</th></thead>
<tbody>
$postdata
</tbody>
</table>
<form action="cpki_crl_editresult.cgi" method="POST">
__EOFEOFEOF__

printHtmlInputHiddenHashByArray($f,$aPostname);

#printDumper($f);

print <<"__EOFEOFEOF__";
<input type="submit" value="Register"> | 
<a href="cpki_crl_list.cgi">Back to CRL list</a>
</form>
__EOFEOFEOF__

&printPoweredBy;
&printFooter;

sub _setFormValue {
    my($h) = @_;

    # issuer
    if ($h->{issuer_value} ne '') {
	$h->{issuer} = $h->{issuer_type} . ":" . $h->{issuer_value};
    }

    # validity
    my ($v);
    $v .= "thisUpdate=" . 
	$h->{validity_this_ttype} . ":" . $h->{validity_this_value};
    $v .= "\tnextUpdate=" . 
	$h->{validity_next_ttype} . ":" . $h->{validity_next_value};
    $h->{validity} = $v;

    # revokecert
    for $i (1..8) {
	my $v;
	next if ($h->{"revokecert_serial$i"} eq '' || $h->{"revokecert_time$i"} eq '');

	$v = $h->{"revokecert_stype$i"} . ":" . $h->{"revokecert_serial$i"} . "/";
	$v .= $h->{"revokecert_ttype$i"} . ":" . $h->{"revokecert_time$i"};
	if ($h->{"revokecert_reason$i"} =~ /^\d+$/) {
	    $v .= "/" . "reasonCode:" . $h->{"revokecert_reason$i"};
	}
	push @a, $v;
    }
    $h->{revokecert} = join("\t",@a);

    # == AuthorityKeyIdentifier =============================================
    $h->{authkeyident_critical} = 
	($h->{authkeyident_critical} eq 'on') ? "true" : "false";

    if ($h->{authkeyident_keyident} ne '') {
	$h->{authkeyident_keyident} =
	    "keyIdentifier=" . $h->{authkeyident_keyident}
    }

    if ($h->{authkeyident_certiss_value} ne '') {
	$h->{authkeyident_certiss} = "authorityCertIssuer=" . 
	    $h->{authkeyident_certiss_dtype} . ":" . 
		$h->{authkeyident_certiss_value};
    }

    if ($h->{authkeyident_certsn_value} ne '') {
	$h->{authkeyident_certsn} = "authorityCertSerialNumber=" . 
	    $h->{authkeyident_certsn_stype} . ":" .
		$h->{authkeyident_certsn_value};
    }

    # == IssuerAltName =======================================================
    $h->{issaltname_critical} = ($h->{issaltname_critical} eq 'on') ? "true" : "false";

    &ord_minusone($h,
		  [qw(issaltname_rfc_ord issaltname_dns_ord
		      issaltname_dn_ord issaltname_ip_ord)],
		  [qw(issaltname_rfc_value issaltname_dns_value
		      issaltname_dn_value issaltname_ip_value)]);
    &renumber_seq($h,[qw(issaltname_rfc_ord issaltname_dns_ord
			 issaltname_dn_ord issaltname_ip_ord)]);

    if ($h->{issaltname_rfc_value} ne '') {
	$h->{issaltname_rfc} = "rfc822Name=" . $h->{issaltname_rfc_value};
    }
    if ($h->{issaltname_dns_value} ne '') {
	$h->{issaltname_dns} = "dNSName=" . $h->{issaltname_dns_value};
    }
    if ($h->{issaltname_dn_value} ne '') {
	$h->{issaltname_dn} = "directoryName=" . $h->{issaltname_dn_type} . 
	    ":" . $h->{issaltname_dn_value};
    }
    if ($h->{issaltname_ip_value} ne '') {
	$h->{issaltname_ip} = "iPAddress=" . $h->{issaltname_ip_value};
    }

    # == CRL Number =========================================================
    $h->{crlnum_critical} = ($h->{crlnum_critical} eq 'on') ? "true" : "false";
    if ($h->{crlnum_value} ne '') {
	$h->{crlnum} = $h->{crlnum_stype} . ":" . $h->{crlnum_value};
    }

    # == Issuing Distribution Point =========================================
    $h->{idp_critical} = ($h->{idp_critical} eq 'on') ? "true" : "false";
    $h->{idp_onlyusr_ord} = 3;
    $h->{idp_onlyca_ord} = 4;

    &ord_minusone($h,
		  [qw(idp_dn_ord idp_uri_ord
		      idp_onlyusr_ord idp_onlyca_ord)],
		  [qw(idp_dn_value idp_uri_value
		      idp_onlyusr idp_onlyca)]);
    &renumber_seq($h,[qw(idp_dn_ord idp_uri_ord
			 idp_onlyusr_ord idp_onlyca_ord)]);

    if ($h->{idp_dn_value} ne '') {
	$h->{idp_dn} = "directoryName=" . $h->{idp_dn_type} . 
	    ":" . $h->{idp_dn_value};
    }

    if ($h->{idp_uri_value} ne '') {
	$h->{idp_uri} = 
	    "URI=" . canonical_tab($h->{idp_uri_value});
    }

    if ($h->{idp_onlyusr} eq 'on') {
	$h->{idp_onlyusr} = "onlyContainsUserCerts=TRUE";
    }
    if ($h->{idp_onlyca} eq 'on') {
	$h->{idp_onlyca} = "onlyContainsCACerts=TRUE";
    }

    # == Private Extension ==============================================
    $h->{privateext_critical} =
	($h->{privateext_critical} eq 'on') ? "true" : "false";

    if ($h->{privateext_value} ne '') {
	$h->{privateext} = canonical_tab($h->{privateext_value});
    }
}

