#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $formHash = &decodeHttpRequest;
if ($formHash->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$formHash->{db}/;
}

our $sql = "
SELECT CI.id, CI.name, CV.value FROM crl CI, crl_value CV
  WHERE
    CI.id = CV.id AND
    CV.name = 'issuer'
  ORDER BY id;";
our $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

$db->querySession([$sql]);

our $CPKI_TITLE = getTitle("CRL List");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print "<p><a href=\"cpki_crl_edit.cgi\">CREATE NEW CRL</a></p>\n";

&_printCRLList($db->{resultSetArray}->[0]);

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($formHash);

&printLog($db);

print <<"___EOF___";
<p>
<b>Tool</b>
<ul>
<li><a href="cpki_crl_diffcfg.cgi" target="_blank">Display diff among two 'crlgen' configuration files for CRLs.</a></li>
</ul>
</p>
___EOF___


&printPoweredBy;
&printFooter;

sub _printCRLList {
    my ($tbl) = @_;
    my ($i);

    if ($#{$tbl} < 0) { 
	print "<strong class=\"warn\">No item in list.</strong><br>";
	return;
    }

    print "<table class=\"rowover\" STRIPED=\"true\">\n";
    print "<thead><tr><th>Action</th><th>ID</th><th>Note</th><th>Util</th></tr></thead>\n";
    print "<tbody>\n";
    for ($i = 0; $i <= $#{$tbl}; $i++) {
	my ($row) = $tbl->[$i];
	my ($id, $s);
	$id = $row->[0];

	$s .= "<tr>";
	$s .= "<td>";
	$s .= "<a href=\"cpki_common_delconfirm.cgi?type=crl&id=$id\">Del</a> / ";
	$s .= "<a href=\"cpki_crl_edit.cgi?id=$id\">Edit</a>";
	$s .= "</td>";
	$s .= tdesch($row->[0]);
	$s .= tdesch(($row->[1] ne "") ? $row->[1] : $row->[2]);
	$s .= "<td>";
	$s .= "<a href=\"cpki_crl_getfile.cgi?id=$id\">[file]</a>";
	$s .= "<a href=\"cpki_crl_viewcfg.cgi?id=$id\">[view cfg]</a>";
	$s .= "<a href=\"cpki_crl_viewasn1.cgi?id=$id\">[view asn1]</a>";
	$s .= "</td>";
	$s .= "</tr>\n";
	print $s;
    }
    print "</tbody>\n";
    print "</table>\n";
}

