#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $crlValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($input, $h);

our $CPKI_TITLE = getTitle("LDAP Entry::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "SELECT * FROM ldap_entry WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{dn}		= $row->[1];
    $h->{entry}		= $row->[2];
    $h->{name}		= $row->[3];
    $h->{keyword}	= $row->[4];
    $h->{note}		= $row->[5];
}

sub _setInput {
    my ($input, $h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"dn",		$h," size=\"84\"");
    &set_input_txa($input,"entry",	$h," cols=\"64\" rows=\"9\" wrap=off id=\"entry_textarea\"");

    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");
}

sub _printForm {
    print <<"___EOF___";
<SCRIPT type="text/javascript">
function setEntryOU() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = "";
    s += 'ou: NAME\\n';
    s += 'objectclass: top\\n';
    s += 'objectclass: organizationalUnit';
    o.value = s;
  }
}

function setEntryC() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = "";
    s += 'c: CNAME\\n';
    s += 'objectclass: top\\n';
    s += 'objectclass: country';
    o.value = s;
  }
}

function setEntryO() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = "";
    s += 'o: NAME\\n';
    s += 'objectclass: top\\n';
    s += 'objectclass: organization';
    o.value = s;
  }
}

function setEntryCN() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = "";
    s += 'cn: NAME\\n';
    s += 'sn: NAME\\n';
    s += 'objectclass: top\\n';
    s += 'objectclass: person';
    o.value = s;
  }
}

function setEntryRef() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = "";
    s += 'o: NAME or \\n';
    s += 'ou: NAME or ANY ATTRIBUTE\\n';
    s += 'objectclass: top\\n';
    s += 'objectclass: referral\\n';
    s += 'objectclass: extensibleObject\\n';
    s += 'ref: ldap://foo.com/o=PPTG,c=JP';
    o.value = s;
  }
}

function setEntryAddPkiCA() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = o.value;
    s += '\\nobjectclass: pkiCA\\n';
    s += 'cacertificate;binary:< db_cert_77\\n';
    s += 'authorityrevocationlist;binary:< db_crl_77\\n';
    s += 'certificaterevocationlist;binary:< db_crl_77\\n';
    s += 'crosscertificatepair;binary:< db_ccp_77\\n';
    s += 'crosscertificatepair;binary:< db_ccp_777\\n';
    o.value = s;
  }
}

function setEntryAddPkiUser() {
  if (document.all) {
    var o = document.all('entry_textarea');
    var s = o.value;
    s += '\\nobjectclass: pkiUser\\n';
    s += 'usercertificate;binary:< db_cert_77\\n';
    o.value = s;
  }
}

</SCRIPT>
<form action="cpki_direntry_editconfirm.cgi" method="POST">
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">DN</td><td>$input->{dn}</td></tr>
<tr><td class="1">Entry</td><td>
$input->{entry}
<br>
If you want to use a template, CLICK ANY OF HERE &gt;&gt;&nbsp; 
<a href="javascript:eval('setEntryC()')">C</a> | 
<a href="javascript:eval('setEntryO()')">O</a> | 
<a href="javascript:eval('setEntryOU()')">OU</a> | 
<a href="javascript:eval('setEntryCN()')">CN</a> | 
<a href="javascript:eval('setEntryRef()')">Ref</a> | 
<a href="javascript:eval('setEntryAddPkiCA()')">+pkiCA</a> | 
<a href="javascript:eval('setEntryAddPkiUser()')">+pkiUser</a>
(IE only) 
</td>
</tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keyword</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_direntry_list.cgi">Back to LDAP Entry list</a>
</p>
</form>
___EOF___
}
