#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $crlValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($input, $h);

our $CPKI_TITLE = getTitle("Key Pair::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm("cpki_key_editconfirm.cgi");

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "SELECT * FROM keypair WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{crypto_method}	= $row->[1];
    $h->{crypto_bit}	= $row->[2];
    $h->{private_key}	= $row->[3];
    $h->{public_key}	= $row->[4];
    $h->{keyid}		= $row->[5];
    $h->{keyid_sha1_64}	= $row->[6];
    $h->{name}		= $row->[7];
    $h->{keyword}	= $row->[8];
    $h->{note}		= $row->[9];
}

sub _setInput {
    my ($input, $h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_sel($input,"crypto_method",	$h,"rsadsa");
    &set_input_txt($input,"crypto_bit",	$h," size=\"6\"");

    &set_input_txa($input,"private_key",	$h," cols=\"65\" rows=\"9\"");
    &set_input_txa($input,"public_key",		$h," cols=\"65\" rows=\"9\"");
    &set_input_txt($input,"keyid",		$h," size=\"84\"");
    &set_input_txt($input,"keyid_sha1_64",	$h," size=\"84\"");

    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");
}

sub _printForm {
    my ($next_url, $extedit_link) = @_;
    my (%t, $list_url, $backto);
    print <<"___EOF___";
<form action="$next_url" method="POST">
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">Crypto Method</td><td>$input->{crypto_method}</td></tr>
<tr><td class="1">Size</td><td>$input->{crypto_bit}
To (re)generate key,
fill 'ID', 'Method' and 'Size' then check 
<a href="#genkey_check_box">bottom</a>.
For RSA 4096 bit key, see <a href="cpki_tips.cgi#rsa4096">here</a>
</td></tr>
<tr><td class="1">160bit Key ID</td><td>$input->{keyid}</td></tr>
<tr><td class="1">64bit Key ID</td><td>$input->{keyid_sha1_64}</td></tr>
<tr><td class="1">Public Key</td><td>$input->{public_key}</td></tr>
<tr><td class="1">Private Key</td><td>$input->{private_key}</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keyword</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>

<p>

<a name="genkey_check_box"><input type="checkbox" name="genkey"></a>
<strong class="warn">
Check HERE and Push 'Next' if you want to GENERATE A KEYPAIR 
according to cryptographic method and size of key.
</strong>
<br>

<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 

<a href="$list_url">$backto</a>
</p>
</form>
___EOF___
}
