#!/usr/bin/perl

use strict;
use lib "./lib";
use JNSA::DB::UTIL;
use JNSA::HTML::UTIL;

our $CPKI_CONFIG;
our $CPKI_CONST;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $h = &decodeHttpRequest;
$h->{db} = getDbnameByDsn($CPKI_CONFIG->{db}->{dsn});
$h->{usr} = $CPKI_CONFIG->{db}->{usr};
$h->{pw} = $CPKI_CONFIG->{db}->{pw};

$h->{head} = "/tmp/diffcfg.ocsp.$$";

our $SCRIPT_OUTPUT = "";
if ($h->{id1} ne '' && $h->{id2} ne '') {
    &_setDiff($h->{id1}, $h->{id2});
}

our $CPKI_TITLE = getTitle("OCSP Response::Diff ocspresgen Config");

# === PRINT HTML ========================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print "<form name=\"f1\" action=\"cpki_ocsp_diffcfg.cgi\" method=\"POST\">\n";
print "OCSP Response ID1: ";
print "<input type=\"text\" name=\"id1\" size=20 maxsize=20>\n";
print "&nbsp;OCSP Response ID2: ";
print "<input type=\"text\" name=\"id2\" size=20 maxsize=20>\n";
print "<input type=\"submit\" value=\"Show Diff\">\n";
print "</form>\n";

print "<p>Display diff of the 'ocspresgen' configuration file between '$h->{id1}' and '$h->{id2}'.</p>\n";

print "<pre>\n";
print esch($SCRIPT_OUTPUT);
print "</pre>\n";

print <<"___EOF___";
<p>
<b>Tool</b>
<ul>
<li><a href="cpki_cert_diffcfg.cgi" target="_blank">Display diff among two 'certgen' configuration files for certficates.</a></li>
</ul>
</p>
___EOF___

&printPoweredBy;
&printFooter;

sub _setDiff {
    my ($id1, $id2) = @_;
    $h->{file1} = $h->{head} . ".id1.cfg";
    $h->{file2} = $h->{head} . ".id2.cfg";
    
    _saveCfgFile($h->{file1}, $id1);
    _saveCfgFile($h->{file2}, $id2);

    open(FOUT, "diff $h->{file1} $h->{file2}|") or return;
    while (<FOUT>) {
	$SCRIPT_OUTPUT .= $_;
    }
    close FOUT;

    unlink $h->{file2};
    unlink $h->{file2};
}

sub _saveCfgFile {
    my ($file, $id) = @_;

    my $argUsr = "--usr=$h->{usr}" if $h->{usr} ne '';
    my $argPw = "--usr=$h->{pw}" if $h->{pw} ne '';
    system "/usr/local/cpki/bin/dbocspresgencfg --id=$id --out=$file --db=$h->{db} --nokey $argUsr $argPw"
}
