#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $ocspValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";
require "lib/CPKI/ocsp_ext.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($h);

our $CPKI_TITLE = getTitle("OCSP Response::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql1 = "SELECT * FROM ocsp WHERE id = $h->{id};";
    my  $sql2 = "SELECT * FROM ocsp_value WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql1,$sql2]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{base_id}  = $row->[1];
    $h->{issuer_keypair} = $row->[2];
    $h->{ocsp_cert} = $row->[3];
    $h->{name}     = $row->[4];
    $h->{keyword}  = $row->[5];
    $h->{note}  = $row->[6];

    &set_hvalue_from_table($h, $ocspValueProc, $db->{resultSetArray}->[1]);
}

sub _setInput {
    my ($h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"base_id",	$h," size=\"10\"");
    &set_input_txt($input,"issuer_keypair",	$h," size=\"10\"");
    &set_link_openform($input,"issuer_keypair","key");
    &set_input_txt($input,"ocsp_cert",	$h," size=\"10\"");
    &set_link_openform($input,"ocsp_cert","cert");
    &set_input_txt($input,"name",	$h," size=\"84\"");
    &set_input_txt($input,"keyword",	$h," size=\"84\"");
    &set_input_txa($input,"note",	$h," cols=\"64\" rows=\"5\" wrap=off");

    # responseStatus
    &set_input_sel($input,"responseStatus",		$h,"ocspresstat");
    # responderID
    &set_input_sel($input,"responderID_by",		$h,"ocspresid");
    &set_input_sel($input,"responderID_dn_type",	$h,"dn");
    &set_input_txt($input,"responderID_dn_value",	$h," size=\"84\"");
    # producedAt
    &set_input_txt($input,"producedAt",			$h," size=\"20\"");
    # certID
    &set_input_sel($input,"certID_issuerNameHash_type",	$h,"dn");
    &set_input_txt($input,"certID_issuerNameHash_value",$h," size=\"84\"");
    &set_input_txt($input,"certID_issuerKeyHash_value",	$h," size=\"84\"");
    &set_input_sel($input,"certID_serialNumber_type",	$h,"dechex");
    &set_input_txt($input,"certID_serialNumber_value",	$h," size=\"84\"");
    # certStatus
    &set_input_sel($input,"certStatus_CertStatus",	$h,"ocspcertstat");
    &set_input_txt($input,"certStatus_revocationTime",	$h," size=\"20\"");
    &set_input_sel($input,"certStatus_revocationReason",$h,"ocsprevores");
    # thisUpdate
    &set_input_txt($input,"thisUpdate",			$h," size=\"20\"");

    # oCSPStatusCode for MOJ
    &set_input_cbx($input,"oCSPStatusCode_exists",	$h);
    &set_input_cbx($input,"oCSPStatusCode_critical",	$h);
    &set_input_sel($input,"oCSPStatusCode_value", 	$h,"ocspmojcode");

    # certPathStatus for CVS
    &set_input_cbx($input,"certPathStatus_exists",	$h);
    &set_input_cbx($input,"certPathStatus_critical",	$h);
    &set_input_sel($input,"certPathStatus_value", 	$h,"ocspcvsstat");
}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_ocsp_editconfirm.cgi" method="POST">
$input->{diff_mode}
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID**</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">Base ID</td><td>
$input->{base_id}
You may enter ID when you copy CRL.
</td></tr>
<tr><td class="1" nowrap>Issuer Key*</td>
  <td>$input->{issuer_keypair} $input->{issuer_keypair_link}</td></tr>
<tr><td class="1" nowrap>OCSP Cert</td>
  <td>$input->{ocsp_cert} $input->{ocsp_cert_link}</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keywords</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>
<div style="margin-left: 20px">
(**) Item must be filled to insert DB. (*) Item must be filled to generate CRL file. (i.e. to issue CRL)
</div>


<h4>OCSP Simulator Basic Response</h4>

<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">responseStatus</td><td>$input->{responseStatus}</td></tr>
<tr><td class="1">responderID</td>
  <td>
  $input->{responderID_by}
  $input->{responderID_dn_type}
  $input->{responderID_dn_value}
  </td></tr>
<tr><td class="1">producedAt</td><td>$input->{producedAt}</td></tr>
<tr><td class="1">certID</td><td>
  <table class="none">
  <tr><td>issuerNameHash</td>
    <td>$input->{certID_issuerNameHash_type}
        $input->{certID_issuerNameHash_value}</td></tr>
  <tr><td>issuerKeyHash</td>
    <td>$input->{certID_issuerKeyHash_value}</td></tr>
  <tr><td>serialNumber</td>
    <td>$input->{certID_serialNumber_type}
        $input->{certID_serialNumber_value}</td></tr>
  </table>
  </td></tr>
<tr><td class="1">CertStatus</td><td>
  <table class="none">
  <tr><td>CertStatus</td><td>$input->{certStatus_CertStatus}</td></tr>
  <tr><td>revocationTime</td><td>$input->{certStatus_revocationTime}</td></tr>
  <tr><td>revocationReason</td><td>$input->{certStatus_revocationReason}</td></tr>
  </table>
  </td></tr>
<tr><td class="1">thisUpdate</td><td>$input->{thisUpdate}</td></tr>
</tbody>
</table>

<h4>$input->{oCSPStatusCode_exists}oCSPStatusCode for MOJ - 
Critical:$input->{oCSPStatusCode_critical}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td nowrap><b>oCSPStatusCode</b></td>
<td>$input->{oCSPStatusCode_value}</td>
</tr>
</tbody>
</table>

This option will be used for business registration by
<a href="http://www.moj.go.jp/ENGLISH/preface.html">the Ministory of Justice</a>
of the Japanese Government. See
<a href="http://www.moj.go.jp/ONLINE/CERTIFICATION/SYSTEM/system.html">this document(in Japanese)</a>
for available values.

<h4>$input->{certPathStatus_exists}certPathStatus for CVS - 
Critical:$input->{certPathStatus_critical}
</h4>

<table>
<thead><tr><th>Item</th><th>Value</th></tr></thead>
<tbody>
<tr>
<td nowrap><b>certPathStatus</b></td>
<td>$input->{certPathStatus_value}</td>
</tr>
</tbody>
</table>

This option will be used for 
<a href="http://www.hitachi.co.jp/Prod/comp/app/kenshou/">
the Certificate Validation Server(CVS)</a> which is a kind of OCSP responder
however response values are not conformed to RFC.
CVS have been used by Ministries of the Japanese Government.

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_ocsp_list.cgi">Back to OCSP Responder Simulator list</a>
</p>
</form>
___EOF___
}
