#!/usr/bin/perl

use strict;

our ($_0f) = $0; $_0f =~ s/^.*\/([^\/]+)$/$1/;
our ($_0d) = $0; $_0d =~ s/[\/\\][^\/\\]+$//; push(@INC, $_0d);

my $ocsptestroot= $_0d . "/data";

my $ocspresgen 	= "/usr/local/cpki/bin/ocspresgen";
my $ocspreq 	= "/usr/local/cpki/bin/ocspreq";

my $req  	= $ocsptestroot . "/ocspReq.$$.der";
my $reqSNNONCE 	= $ocsptestroot . "/ocspReq.$$.txt";
my $priv 	= $ocsptestroot . '/z.prv.pem';
my $cfg  	= $ocsptestroot . '/z.res.cfg';
my $cert 	= $ocsptestroot . '/z.crt.crt';
my $res  	= $ocsptestroot . "/ocspRes.$$.der";
my $req_stub 	= $ocsptestroot . '/req.der';
my ($cmd, $stdout, $error, $nonce, $serial);
my ($size, $buffer);

# check environment
if (! $ENV{'CONTENT_LENGTH'}) {
    print "Content-type: text/html\n\n";
    print "<HTML><BODY>\n";
    print "<H1>Welcome to OCSP Responder Simulator</H1>\n";
    print "</HTML></BODY>\n";
    exit 1;
}

if (! -d $ocsptestroot) {
    umask 0;
    mkdir( $ocsptestroot, 0777 ) or 
	&exit_http("$0: cannot create directory $ocsptestroot");;
}

&_saveOcspRequestToFile;
&_setSnAndNonceFromRequest;
&_ocspResgenExecute;

# print HTTP header and OCSP response data
$size = (-s $res);
if (! $size) {
    &exit_http("$0: $res file is empty");
}
open( INFILE, "< $res" ) or &exit_http("$0: cannot open $res for input");
binmode( INFILE );
print "Content-Type: application/ocsp-response\n";
print "Content-Transfer-Encodeing: Binary\n";
print "Content-Length: $size\n\n";

binmode( STDOUT );
while ( read( INFILE, $buffer, 1024 ) ) {
    print STDOUT $buffer;
}
close( INFILE );
print STDERR "$0: success (nonce=$nonce)\n";
exit;

#== END OF MAIN ============================================================

#
# Print HTTP error message and die
#
sub exit_http {
    my ($msg) = @_;
    print STDERR "$msg\n";
    print "Content-type: text/html\n\n";
    print "<HTML><BODY><PRE>\n";
    print "<H1>OCSP Responder Simulator Error</H1>\n\n";
    print "$msg\n\n</PRE></BODY></HTML>\n";
    exit (1);
}

sub _saveOcspRequestToFile {
    read (STDIN, my $post, $ENV{'CONTENT_LENGTH'});
    open ( OUT, "> $req" ) or &exit_http("$0: cannot open $req for output");
    print OUT $post;
    close (OUT);
}

sub _setSnAndNonceFromRequest {
    my $result = "";

    if ( ! -e "$ocspreq" ) {
	&exit_http("$0: file $ocspreq not exist");
    }

    $cmd = "$ocspreq $req";
    $result = `$cmd 2>&1`;

    $error = ($? >> 8);
    if ($error != 0) {
	&exit_http("$0: ocspreq failed ( command=$cmd, exit_status=$error ), dumping debug info...\n$nonce");
    }

    chomp($result);

    open(FOUT, "> $reqSNNONCE");
    print FOUT $result;
    close FOUT;

    if ($result =~ /^([0-9A-Fa-f]+):([0-9A-Fa-f]+)$/) {
	$serial = $1;
	$nonce = $2;
    }
}

# 
# _ocspResgenExecute()
#
#	execute ocspresgen to generate response data
#
sub _ocspResgenExecute {
    my $argSn = ($serial ne '') ? "-s $serial" : "";

    if (! -e "$ocspresgen") {
	&exit_http("$0: file $ocspresgen not exist");
    }

    

    $cmd = "$ocspresgen -K $priv -c $cert -v $cfg -n $nonce $argSn -o $res";
    $stdout = `$cmd 2>&1`;
    $error = ($? >> 8);
    if ($error) {
	&exit_http("$0: ocspresgen failed ( command=$cmd, exit_status=$error ), dumping debug info...\n<STDOUT>$stdout\n<CFG>$cfg");
    }
}
