#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;
our $certValueProc;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our $h = &decodeHttpRequest;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setValue($h);
&_setInput($h);

our $CPKI_TITLE = getTitle("Raw Certificate::Edit");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm;

#printDumper($db);
#printDumper($CPKI_CONFIG->{db});
#printDumper($h);
#printDumper($input);

&printLog($db,$h);
&printPoweredBy;
&printFooter;

sub _setValue {
    if ((! defined $h->{id}) || $h->{id} !~ /^\d+/) {
	return;
    }

    $h->{org_id} = $h->{id};

    my $sql = "SELECT * FROM rawcert WHERE id = $h->{id};";
    $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

    $db->querySession([$sql]);

    my $row = $db->{resultSetArray}->[0]->[0];
    $h->{base_id}  = $row->[1];
    $h->{issuer_keypair} = $row->[2];
    $h->{subject_keypair}  = $row->[3];
    $h->{pem} = $row->[4];
    $h->{name}     = $row->[5];
    $h->{keyword}  = $row->[6];
    $h->{note}  = $row->[7];

    &set_hvalue_from_table($h, $certValueProc, $db->{resultSetArray}->[1]);
}

sub _setInput {
    my ($h) = @_;

    &set_input_txt($input,"id",		$h," size=\"10\"");
    &set_input_hdv($input,"org_id",	$h);
    &set_input_txt($input,"base_id",	$h," size=\"10\"");
    &set_input_txt($input,"issuer_keypair",	$h," size=\"10\"");
    &set_input_txt($input,"subject_keypair",	$h," size=\"10\"");
    &set_input_txa($input,"pem",	$h," cols=\"66\" rows=\"20\" wrap=off");
    &set_input_txt($input,"name",	$h," size=\"86\"");
    &set_input_txt($input,"keyword",	$h," size=\"86\"");
    &set_input_txa($input,"note",	$h," cols=\"66\" rows=\"5\" wrap=off");
}

sub _printForm {
    print <<"___EOF___";
<form name="f1" action="cpki_rawcert_editconfirm.cgi" method="POST">
$input->{diff_mode}
<table width="90%">
<thead><tr><th>Item</th><th width="100%">Value</th></tr></thead>
<tbody>
<tr><td class="1">ID**</td><td>
$input->{id}
(original ID:$input->{org_id}) Copy will be made if you specify different number.
</td></tr>
<tr><td class="1">Base ID</td><td>
$input->{base_id}
You may enter ID when you copy certificate.
</td></tr>
<tr><td class="1">Issuer Keypair</td><td>$input->{issuer_keypair}
<a href="javascript:openForm('key',document.f1.issuer_keypair)">[view]</a>
</td></tr>
<tr><td class="1" nowrap>Subject Keypair</td><td>$input->{subject_keypair}
<a href="javascript:openForm('key',document.f1.subject_keypair)">[view]</a>
</td></tr>
<tr><td class="1" nowrap>PEM Cert*</td><td>
$input->{pem}
<br>
input PEM formatted certificate.
</td></tr>
<tr><td class="1">Name</td><td>$input->{name}</td></tr>
<tr><td class="1">Keywords</td><td>$input->{keyword}</td></tr>
<tr><td class="1">Note</td><td>$input->{note}</td></tr>
</tbody>
</table>
<div style="margin-left: 20px">
(**) Item must be filled to insert DB. (*) Item must be filled to generate certificate file.
</div>

<p>
<input type="submit" value="Next"> 
<input type="reset" value="Reset"> 
<a href="cpki_rawcert_list.cgi">Back to certificate list</a>
</p>
</form>
___EOF___
}
