#!/usr/bin/perl

use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONST;
our $CPKI_CONFIG;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $formHash = &decodeHttpRequest;
if ($formHash->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$formHash->{db}/;
}

our $sql1 = "SELECT * FROM test_result ORDER by id;";
our $sql2 = "SELECT id, name FROM test_category ORDER BY id;";
our $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});

$db->querySession([$sql1,$sql2]);

our $CPKI_TITLE = getTitle("Test Result List");

# Print HTML ===============================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printListResult($db->{resultSetArray}->[0]);
&_printListCategory($db->{resultSetArray}->[1]);

&printPoweredBy;
&printFooter;

# Sub Routine ===============================================================

sub _printListResult {
    my ($tbl) = @_;
    my ($i);

    if ($#{$tbl} < 0) { 
	print "<strong class=\"warn\">No item in list.</strong><br>";
	return;
    }

    print "<table class=\"rowover\" STRIPED=\"true\"><thead>\n";
    print "<thead><tr><th>Action</th><th>No.</th><th>Category</th><th>Client</th><th>Start</th><th>End</th><th>Time</th><th>Case</th><th>True</th><th>False</th><th>Note</th></tr></thead><tbody>\n";

    for $i (0..$#{$tbl}) {
	my $s = "";
	my $row = $tbl->[$i];
	my $urlhead = "cpki_testresult";
	$s .= "<tr>";
	$s .= "<td nowrap><a href=\"cpki_common_delconfirm.cgi?type=testresult&id=$row->[0]\">Del</a>/";
	$s .= "<a href=\"cpki_testresultdetail_list.cgi?id=$row->[0]\">Detail</a></td>";
	$s .= tdesch($row->[0]);
	$s .= tdesch($row->[1]);
	$s .= tdesch($row->[2]);
	$s .= "<td nowrap>" . esch(strLocalTime($row->[3])) . "</td>";
	$s .= tdesch(substr(strLocalTime($row->[4]),10));
	$s .= tdesch(strTimeRange($row->[5]));
	$s .= tdesch($row->[6]);
	$s .= tdesch($row->[7]);
	$s .= tdesch($row->[8]);
	$s .= tdesch($row->[9]);
	$s .= "</tr>";
	print $s, "\n";
    }

    print "</tbody></table>\n";
}

sub _printListCategory {
    my ($tbl) = @_;
    my ($i);

    return if ($#{$tbl} == -1);

    print "<H4>True/False Sheet For Each Category</H4>\n";

    print "<table class=\"rowover\" STRIPED=\"true\"><thead>\n";
    print "<thead><tr><th>ID</th><th>Name</th><th>Sheet</th></tr></thead><tbody>\n";

    for $i (0..$#{$tbl}) {
	my $item = $tbl->[$i];
	print "<tr>";
	print tdesch($item->[0]);
	print tdesch($item->[1]);
	print "<td><a href=\"cpki_testresult_status_list.cgi?id=$item->[0]\" target=\"_blank\">Open</a></td>";
	print "</tr>";
    }

    print "</tbody></table>\n";
}

