#!/usr/bin/perl

use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONST;
our $CPKI_CONFIG;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $formHash = &decodeHttpRequest;
if ($formHash->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$formHash->{db}/;
}
$h = $formHash;


if ($h->{id} =~ /^\d+/) {
    $sql1 = "
SELECT T1.testcase_id, T1.id, result_status
FROM test_result_detail T1, testcase T2
WHERE
  T1.testcase_id = T2.id AND
  T2.category_id = $h->{id};";
    
    $sql2 = "
SELECT id, name FROM test_result
WHERE category_id = $h->{id};";

    $db = new JNSA::DB({config => $TDMCFG->{db}});
    $db->querySession([$sql1,$sql2]);

    our $db = new JNSA::DB({config => $CPKI_CONFIG->{db}});
    $db->querySession([$sql1,$sql2]);

}

our $CPKI_TITLE = getTitle("True/False Sheet::Category=" . $h->{id} . "::List");

# Print HTML ===============================================================

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

#printDumper($db);

#&print_html_testresult_determi_list($db->{resultSetArray});
#if ($f->{test_category_id} =~ /^\d+/) {
#    &printLog($db,$f) if $db->isError;
#}

&_printList($db->{resultSetArray});
if ($f->{test_category_id} =~ /^\d+/) {
    &printLog($db,$f) if $db->isError;
}

&printPoweredBy;
&printFooter;

# Sub Routine ===============================================================

sub _printList {
    my ($rs) = @_;
    my ($i,$j);

    my $table = $rs->[0];
    my $table2 = $rs->[1];

    if ($#{$table} == -1) {
       &print_html_if_warn("$t{'No Entry'}");
    return;
    }

    my $hComment = {};
    for $i (0..$#{$table2}) {
	my ($result,$comment) = @{$table2->[$i]};
	$hComment->{$result} = $comment;
    }	

    my $hCase = {};
    my $hResult = {};
    my $hDetermi = {};
    my $info = {};
    _processResultDetailArray($table,$hCase,$hResult,$hDetermi,$info);
    my $maxLen = _maxLen($info->{maxCase});

    my @aCase = sort {int($a) <=> int($b)} keys(%{$hCase});
    my @aResult = sort {int($a) <=> int($b)} keys(%{$hResult});

    #
    # PRINT TABLE HEADER
    # 
    print "<table><thead><tr>\n";
    print "<th nowrap>Case<br>Result</th>";
    for $i (@aCase) {
	my $caseStr = _strVertInteger($i,$maxLen);
	print "<th>$caseStr</th>";
    }
    print "<th>$t{Comment}</th>";
    print "</tr></thead><tbody>\n";

    #
    # PRINT TABLE BODY
    # 
    for $j (@aResult) {
	print "<tr><td align=\"right\">$j</td>";
	for $i (@aCase) {
	    my $value = "?";
	    my $key = $i . ":" . $j;
	    $value = "-" if $hDetermi->{$key} == 2;
	    $value = "o" if $hDetermi->{$key} == 1;
	    $value = "x" if $hDetermi->{$key} == 0;
	    $value = "-" if (! defined $hDetermi->{$key});
	    print "<td>$value</td>";
	}
	#print tdesch($hComment->{$j});
	print "<td nowrap>" . esch($hComment->{$j}) . "</td>";
	print "</tr>\n";
    }

    print "</tbody></table>\n";

    print "<pre>LEGEND: o:TRUE x:FALSE -:NOT TESTED</pre>";
}

sub _strVertInteger {
    my ($i,$len) = @_;
    my $s = sprintf("%${len}s",$i);
    return join("<br>",split(//,$s));
}


sub _maxLen {
    my ($i) = @_;
    return 0 if $i eq '';
    my $s = " " . $i;
    return (length($s)-1);
}

sub _processResultDetailArray {
    my ($t,$hCase,$hResult,$hDetermi,$info) = @_;
    my $i;
    my $maxCase = 0;
    
    for $i (0..$#{$t}) {
	my ($case,$result,$determi) = @{$t->[$i]};
	$case = int($case);
	$maxCase = ($case > $maxCase) ? $case : $maxCase;
	$hCase->{$case} = 1;
	$hResult->{$result} = 1;
	my $key = $case . ":" . $result;
	$hDetermi->{$key} = $determi;
    }
    $info->{maxCase} = $maxCase;
}
