#!/usr/bin/perl

use strict;
use lib "./lib";

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";

our $CPKI_TITLE = getTitle("Tips");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

print <<"___EOF___";
<h2>Tips Index</h2>
<ul>
<li><a href="#customcgi">How to customize 'Test Data Editor' CGI</a></li>
<li><a href="#customdb">How to customize database</a></li>
<li><a href="#rsa4096">How to generate RSA key pair larger than 2048 bits.</a></li>
<li>
See also <a href="cpki_help.cgi">online help</a> or
<a href="cpki_faq.cgi">FAQ</a>.
</li>
</ul>

<h4><a name="customcgi">How to customize 'Test Data Editor' CGI</a></h4>
<p>
You may customize following features of 'Test Data Editor' CGI.
<ul>
<li><b>Title and links in page header.</b><br>
You can modify the HTML in 'printHeader' function which defines the
header, links and styles file of pages
localed in '/var/www/cgi-bin/lib/CPKI/site_config.pl' file.
<p><b>NOTE:</b><br>Charactor encoding of all Perl files including
'site_config.pl' must be <b>'UTF8N'</b>
encoding(UTF8 without BOM - Byte order mark).
Also charset value 'UTF-8' in 'printHeader' should not be modified.
</p>
</li>
<li><b>Title and links in page footer.</b><br>
You can modify HTML in 'printFooter' function like above.
</li>
<li><b>Top page contents.</b><br>
You can modify HTML in 'printTopContents' function.
</li>
<li><b>Page styles (fonts, colors)</b><br>
Cascading style sheet (CSS) file which defines the fonts, colors and layout
defined in 'printHeader' function like this.
<pre>
&lt;LINK rel="stylesheet" href="/cpki/index.css" type="text/css"&gt;
</pre>
The CSS file is located in '/var/www/html/cpki/index.css" as default.
</li>
<li><b>Database</b><br>
See <a href="#customdb">this section in detail</a>.
</li>
</ul>
</p>

<h4><a name="customdb">How to customize database</a></h4>
<p>
The database used by 'Test Data Editor' is specified as
'CPKI_CONFIG' global variable in 
'/var/www/cgi-bin/lib/CPKI/site_config.pl' file. 
<pre>
&#36;CPKI_CONFIG = {
    db => {
	dsn => "dbi:Pg:dbname=DBNAME",
	usr => "DB_USER_NAME",
	pw => "DB_USER_PASSWORD"
    }
};
</pre>
You can specify
<a href="http://www.postgresql.org/" target="_blank">PostgreSQL</a>
database name, user name and password.
'Test Data Editor' uses 'DBI' and 'DBD::Pg' Perl package to
access database.
If you want to use other database system like MySQL, Interbase or Oracle
'dsn'(data source name) attribute of 'CPKI_CONFIG' can be modified.
</p>
<p>
The <b>most important</b> thing about database is 
that database charactor encoding must be <b>'UNICODE(UTF8)'</b>.
When you create database in PostgreSQL,  '-E UNICODE' option 
should be specified like follows.
<pre>
% createdb -E UNICODE DB_NAME
</pre>
</p>

<h4><a name="rsa4096">How to generate RSA key pair larger than 2048 bits</a></h4>
<p>
Key pair generation with CGI will fail in most cases because 
CGI causes time out. But you can generate keypair manually 
by 'genkeyfp' command which is bundled with this test suite as following.
<pre>
% genkeyp -type rsa -size 4096
</pre>
This will create a public key, private key, 160 bit keyid and
64 bit keyid in '/tmp' directory, then you can fill key pair editor form
by pasting them. 
In the file 'keyid.txt', the first line is 160 bit keyid and
the second line is 64 bit keyid. You can also generate a RSA key by 
openssl as following.
<pre>
generate RSA 4096 bit privte key.
% openssl genrsa -out private.pem 4096
extract its public key.
% openssl rsa -in private.pub -pubout -out public.pem
#TBD HOW TO KNOW KID
</pre>
</p>
___EOF___

&printPoweredBy;
&printFooter;

