#!/usr/bin/perl

use strict;
use Data::Dumper;
use lib "./lib";
use JNSA::DB;

our $CPKI_CONFIG;
our $CPKI_CONST;

require "lib/CPKI/site_config.pl";
require "lib/CPKI/util.pl";
require "lib/CPKI/form.pl";

our ($h, $hRaw) = &decodeHttpRequestWithRawValue;

if ($h->{db} ne "") {
    $CPKI_CONFIG->{db}->{dsn} =~ s/dbname=[0-9A-Za-z]+/dbname=$h->{db}/;
}
our ($db);
our $input = {};
&_setInput($input, $h);

our $CPKI_TITLE = getTitle("LDAP URI Helper");

print "Content-type: text/html; charset=UTF-8\n\n";

&printHeader;

&_printForm("cpki_tool_urihelper.cgi");
&_printResult;

#printDumper($h);
#printDumper($hRaw);
#printDumper($input);

print <<"___EOF___";
<h2>ENCODING EXAMPLE FOR YOUR REFERENCE</h2>
For example, when the following DN is written in a certificate
<pre>
Country = JP
Organization = ABC Co.<font color=red>,</font> Ltd.
</pre>

The software first should escape the comma character
to distinguish the delimiter function by doing:

<pre>
o=ABC Co.<font color="red">\\2c</font> Ltd., c=JP
o=ABC Co.<font color="red">\\,</font> Ltd., c=JP
o=<font color=red>"</font>ABC Co., Ltd.<font color=red>"</font>, c=JP (LDAP v2)
</pre>

Then the software should escape the special characters by doing:

<pre>
1'. ldap://example.tld/o=ABC%20Co.<font color=red>%5c2c</font>%20Ltd., c=JP
2'. ldap://example.tld/o=ABC%20Co.<font color=red>%5c,</font>%20Ltd., c=JP
3'. ldap://example.tld/o=<font color=red>%22</font>ABC%20Co.,%20Ltd.<font color=red>%22</font>, c=JP
</pre>

(From JKST-IWG CA-CA Guideline)
</pre>
___EOF___

&printPoweredBy;
&printFooter;

# -- END OF MAIN ---------------------------------------------------------

sub _setInput {
    my ($input, $h) = @_;

    &set_input_txt($input,"rawtext",	$h," size=\"60\"");
}

sub _printForm {
    my ($next_url, $extedit_link) = @_;
    my (%t, $list_url, $backto);
    print <<"___EOF___";
<form action="$next_url" method="POST">
<p>
LDAP URI can be used in cRLDistributionPoints of 
X.509 certificate extension and issuingDistributionPoint
of X.509 CRL extension. Available LDAP URI format is below.
<pre>
ldap://hostname(:portnumber)/reverse-ordered-dn?attribute(;binary)
for example.
ldap://foo.com/cn=RDN2,o=RDN1,c=JP?cACertificate;binary
</pre>
</p>
<p>
Try to input CJK characters like '&#x6C34;' which means 'water' in Japanese
or something.(Can you see this?)<br>
<b>INPUT RDN VALUE: &nbsp;</b>$input->{rawtext}<br>
<br>

<input type="submit" value="ENCODE TEXT"> 
<input type="reset" value="Reset"> 

<a href="$list_url">$backto</a>
</p>
</form>
___EOF___
}

sub _printResult {
    return if ($h->{rawtext} eq "");

    my $value = esch($h->{rawtext});
    my $valueRaw1 = $hRaw->{rawtext};
    my $valueRaw2 = $hRaw->{rawtext};
    $valueRaw2 =~ s/\%/\%5C/g;

    print <<"___EOF___";
<h2>ESCAPE ENCODE RESULT</h2>
<p>
<b>TEXT YOU ENTERED:</b><br>
$value<br>
<b>FOR LDAP FORM1 - no back slash(%5C) escaping</b><br>
$valueRaw1<br>
<b>FOR LDAP FORM2 - with back slash(%5C) escaping</b><br>
$valueRaw2<br>
</p>
___EOF___
}
