#!/usr/bin/perl
use lib "/var/www/cgi-bin/lib";
use strict;
use JNSA::DB;
use IO::File;
use POSIX qw(tmpnam);

our $CPKI_CONFIG;
require "lib/CPKI/site_config.pl";

main();

sub main{
    my($id, $tsqFileName, $params, $tsrFileName);
    $id = getId();
    $params = getDBParams($id);
    $tsqFileName = createTSQ();
    $tsrFileName = createTSR($params, $tsqFileName);
    printData($params, $tsrFileName);
    exit 0;
}

sub getDBParams($){
    my($id) = @_;
    my $sql = "select tsr_id, http_header from tsp_protocol_testcase where id = $id";
    my $db = new JNSA::DB({config=>$CPKI_CONFIG->{db}});
    $db->querySession([$sql]);
    my $result = {};
    $result->{tsr_id} = $db->{resultSetArray}->[0]->[0]->[0];
    $result->{http_header} = $db->{resultSetArray}->[0]->[0]->[1];
    return $result;
}

sub getId(){
    my $buffer = $ENV{'QUERY_STRING'};
    my @pairs = split(/&/,$buffer);
    my $pair;
    foreach $pair (@pairs) {
	my ($name, $value) = split(/=/, $pair);
	if($name =~ /^id$/){
	    $value =~ tr/+/ /;
	    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	    $value =~s/ \^/-\^/g;
	    return $value;
	}
    }
}

sub createTSR($$){
    my($params, $tsqFileName) = @_;
    my $db = $CPKI_CONFIG->{db};
    my($tsrFileName, $dbname, $tmpdir);

    $tmpdir = "/tmp";
    $tsrFileName = $tmpdir . "/tsa.cgi.$$.tsr";

    $dbname = $db->{dsn};
    $dbname =~ s/.*=//;

    my $cmd = 'PATH=/usr/local/cpki/bin:$PATH';;
    $cmd .= " dbtsrgen --tsrid=$params->{tsr_id} --tsq=$tsqFileName -db=$dbname --out=$tsrFileName";
    if($db->{usr}){
	$cmd .= " --usr=$db->{usr}";
    }
    if($db->{pw}){
	$cmd .= " --pw=$db->{pw}";
    }
    $cmd .= " 2>&1";
#    print STDERR $cmd;
    system($cmd);
    return($tsrFileName);
}

sub createTSQ(){
    my $buf;
    read(STDIN, $buf, $ENV{'CONTENT_LENGTH'});
    my($fname, $fh);
    $fh = makeTempFile(\$fname);
    print $fh $buf;
    close($fh);
    return $fname;
}

sub printData($$){
    my($params, $fname) = @_;
    my($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size
       ,$atime,$mtime,$ctime,$blksize,$blocks) = stat($fname);
    print "$params->{http_header}\n";
    print("Content-length: $size\n");
    print("\n");

    open(TSR, "<$fname");
    while(<TSR>){
	print;
    }
    close(TSR);
}

sub makeTempFile($;){
    my($f) = @_;
    my($fname, $fh);
    do {$fname = tmpnam() } until $fh = IO::File->new($fname, O_RDWR|O_CREAT|O_EXCL);
    if($f){
	$$f = $fname;
    }
    return $fh;
}
