#============================================================================
# certValueProc
#============================================================================
$certValueProc = {};

#=========================================================================
# Basic Certificate Field
#=========================================================================

$certValueProc->{serialNumber} = sub {
    my($h,$row) = @_;

    my $value = $row->[3];

    if ($value =~ /^([^:]+):(.*)$/) {
	$h->{sn_type} = $1;
	$h->{sn_value} = $2;
    }
};

$certValueProc->{issuer} = sub {
    my($h,$row) = @_;

    my $value = $row->[3];

    if ($value =~ /^([^:]+):(.*)$/) {
	$h->{issuer_type} = $1;
	$h->{issuer_value} = $2;
    }
};

$certValueProc->{validity} = sub {
    my($h,$row) = @_;
    my($beforetype,$beforevalue,$aftertype,$aftervalue);

    my $value = $row->[3];

    if ($value =~ /^(.*):days:(.*)/) {
	$h->{validity_short_flag} = "on";
	$h->{validity_short_ttype} = $1;
	$h->{validity_short_days} = $2;
	return;
    } else {
	$h->{validity_short_flag} = "off";
	if (($beforetype,$beforevalue,$aftertype,$aftervalue) = 
	    ($value =~ /notBefore=([^:]+):([^\t]+)\tnotAfter=([^:]+):(.*)$/)) {
	    $h->{validity_before_value} = $beforevalue;
	    $h->{validity_after_value} = $aftervalue;
	    $h->{validity_before_ttype} = $beforetype;
	    $h->{validity_after_ttype} = $aftertype;
	}
    }
};

$certValueProc->{subject} = sub {
    my($h,$row) = @_;

    my $value = $row->[3];

    if ($value =~ /^([^:]+):(.*)$/) {
	$h->{subject_type} = $1;
	$h->{subject_value} = $2;
    }
};

#=========================================================================
# X.509 Extensions
#=========================================================================

$certValueProc->{authorityKeyIdentifier} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];

    $h->{authkeyident_exists} = 1;
    $h->{authkeyident_critical} = $row->[4];

    if ($value =~ /keyIdentifier=(.*)$/) {
	$h->{authkeyident_keyident_value} = $1;
	return;
    }
    if ($value =~ /authorityCertIssuer=([^:]+):(.*)$/) {
	$h->{authkeyident_certiss_dtype} = $1;
	$h->{authkeyident_certiss_value} = $2;
	return;
    }
    if ($value =~ /authorityCertSerialNumber=([^:]+):(.*)$/) {
	$h->{authkeyident_certsn_stype} = $1;
	$h->{authkeyident_certsn_value} = $2;
	return;
    }
};


$certValueProc->{subjectKeyIdentifier} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];

    $h->{subjkeyident_exists} = 1;
    $h->{subjkeyident_critical} = $row->[4];
    $h->{subjkeyident_keyident_value} = $value;
};

$certValueProc->{keyUsage} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{keyusage_exists} = 1;
    $h->{keyusage_critical} = $row->[4];

    my $info = $CPKI_CONST->{const}->{keyusage};

    for $key (split(/\t/,$value)) {
	$h->{$info->{$key}} = 'on' if (defined $info->{$key});
    }
};
    
$certValueProc->{certificatePolicies} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{certpolicy_exists} = 1;
    $h->{certpolicy_critical} = $row->[4];

    if ($value eq 'anyPolicy') {
	$h->{certpolicy_anypolicy} = 'on';
    } else {
	$h->{certpolicy_oids_value} = canonical_nl($value);
    }
};

$certValueProc->{policyMappings} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{policymap_exists} = 1;
    $h->{policymap_critical} = $row->[4];
    $h->{policymap_value} = canonical_nl($value);
};

$certValueProc->{subjectAltName} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{subjaltname_exists} = 1;
    $h->{subjaltname_critical} = $row->[4];

    if ($value =~ /rfc822Name=(.*)$/) {
	$h->{subjaltname_rfc_ord} = $seq;
	$h->{subjaltname_rfc_value} = $1;
	return;
    }
    if ($value =~ /dNSName=(.*)$/) {
	$h->{subjaltname_dns_ord} = $seq;
	$h->{subjaltname_dns_value} = $1;
	return;
    }
    if ($value =~ /directoryName=([^:]+):(.*)$/) {
	$h->{subjaltname_dn_ord} = $seq;
	$h->{subjaltname_dn_type} = $1;
	$h->{subjaltname_dn_value} = $2;
	return;
    }
    if ($value =~ /iPAddress=(.*)$/) {
	$h->{subjaltname_ip_ord} = $seq;
	$h->{subjaltname_ip_value} = $1;
	return;
    }
};

$certValueProc->{issuerAltName} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{issaltname_exists} = 1;
    $h->{issaltname_critical} = $row->[4];

    if ($value =~ /rfc822Name=(.*)$/) {
	$h->{issaltname_rfc_ord} = $seq;
	$h->{issaltname_rfc_value} = $1;
	return;
    }
    if ($value =~ /dNSName=(.*)$/) {
	$h->{issaltname_dns_ord} = $seq;
	$h->{issaltname_dns_value} = $1;
	return;
    }
    if ($value =~ /directoryName=([^:]+):(.*)$/) {
	$h->{issaltname_dn_ord} = $seq;
	$h->{issaltname_dn_type} = $1;
	$h->{issaltname_dn_value} = $2;
	return;
    }
    if ($value =~ /iPAddress=(.*)$/) {
	$h->{issaltname_ip_ord} = $seq;
	$h->{issaltname_ip_value} = $1;
	return;
    }
};

$certValueProc->{basicConstraints} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{basicconst_exists} = 1;
    $h->{basicconst_critical} = $row->[4];

    if ($value eq 'cA=TRUE') {
	$h->{basicconst_ca_value} = "on";
    }
    if ($value =~ /pathLenConstraint=(.*)$/) {
	$h->{basicconst_pathlen_value} = $1;
    }
};

$certValueProc->{nameConstraints} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{nameconst_exists} = 1;
    $h->{nameconst_critical} = $row->[4];

    if ($value =~ /permittedSubtrees=(.*)$/) {
	my $pair = "";
	my $i = 1;
	for $pair (split(/\t/,$1)) {
	    my ($t,$v) = ($pair =~ /^([^:]+):(.*)$/);
	    $h->{"nameconst_permit_dtype$i"} = $t;
	    $h->{"nameconst_permit_value$i"} = $v;
	    $i++;
	}
	return;
    }

    if ($value =~ /excludedSubtrees=(.*)$/) {
	my $pair = "";
	my $i = 1;
	for $pair (split(/\t/,$1)) {
	    my ($t,$v) = ($pair =~ /^([^:]+):(.*)$/);
	    $h->{"nameconst_exclude_dtype$i"} = $t;
	    $h->{"nameconst_exclude_value$i"} = $v;
	    $i++;
	}
	return;
    }

};

$certValueProc->{policyConstraints} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{policyconst_exists} = 1;
    $h->{policyconst_critical} = $row->[4];

    if ($value =~ /requireExplicitPolicy=(.*)$/) {
	$h->{policyconst_req_value} = $1;
	return;
    }

    if ($value =~ /inhibitPolicyMapping=(.*)$/) {
	$h->{policyconst_inhibit_value} = $1;
	return;
    }
};

$certValueProc->{extendedKeyUsage} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{extkeyusage_exists} = 1;
    $h->{extkeyusage_critical} = $row->[4];
    $h->{extkeyusage_oids_value} = canonical_nl($value);
};

$certValueProc->{cRLDistributionPoint} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{crldp_exists} = 1;
    $h->{crldp_critical} = $row->[4];

    if ($value =~ /directoryName=([^:]+):(.*)$/) {
	$h->{crldp_dn_ord} = $seq;
	$h->{crldp_dn_type} = $1;
	$h->{crldp_dn_value} = $2;
	return;
    }
    if ($value =~ /URI=(.*)$/) {
	$h->{crldp_uri_ord} = $seq;
	$h->{crldp_uri_value} = canonical_nl($1);
	return;
    }
};

$certValueProc->{authorityInformationAccess} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{authinfoacc_exists} = 1;
    $h->{authinfoacc_critical} = $row->[4];
    $h->{authinfoacc_uri_value} = $value;
};

$certValueProc->{oCSPNoCheck} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{ocspnochk_exists} = 1;
    $h->{ocspnochk_critical} = $row->[4];
};

$certValueProc->{privateExtension} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{privateext_exists} = 1;
    $h->{privateext_critical} = 0; # set to non critical
    $h->{privateext_value} = canonical_nl($value);
};


1;

