#============================================================================
# crlValueProc
#============================================================================
$crlValueProc = {};

$crlValueProc->{issuer} = sub {
    my($h,$row) = @_;

    my $value = $row->[3];
    ($h->{issuer_type},$h->{issuer_value}) = ($value =~ /^([^:]+):(.+)$/);
};

$crlValueProc->{validity} = sub {
    my($h,$row) = @_;
    my($thistype,$thisvalue,$nexttype,$nextvalue);

    my $value = $row->[3];

    if ($value =~ /^(.*):days:(.*)/) {
	$h->{validity_short_flag} = "on";
	$h->{validity_short_ttype} = $1;
	$h->{validity_short_days} = $2;
	return;
    } else {
	$h->{validity_short_flag} = "off";
	if (($thistype,$thisvalue,$nexttype,$nextvalue) = 
	    ($value =~ /thisUpdate=([^:]+):([^\t]+)\tnextUpdate=([^:]+):(.*)$/)) {
	    $h->{validity_this_value} = $thisvalue;
	    $h->{validity_next_value} = $nextvalue;
	    $h->{validity_this_ttype} = $thistype;
	    $h->{validity_next_ttype} = $nexttype;
	}
    }
};

$crlValueProc->{revokedCertificates} = sub {
    my($h,$row) = @_;
    my(@a,$item,$i);

    my $value = $row->[3];

    $i = 1;
    for $item (split(/\t/,$value)) {
	my ($serial, $time, $reason) = split(/\//, $item);

	($h->{"revokecert_stype$i"},$h->{"revokecert_serial$i"}) =
	    ($serial =~ /^([^:]+):(.+)$/);

	($h->{"revokecert_ttype$i"},$h->{"revokecert_time$i"}) = 
	    ($time =~ /^([^:]+):(.+)$/);

	($h->{"revokecert_reason$i"}) =
	    ($reason =~ /^reasonCode:(\d+)$/);

	$i++;
    }
};

$crlValueProc->{authorityKeyIdentifier} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{authkeyident_exists} = 1;
    $h->{authkeyident_critical} = $row->[4];

    if ($value =~ /keyIdentifier=(.*)$/) {
	$h->{authkeyident_keyident} = $1;
	return;
    }
    if ($value =~ /authorityCertIssuer=([^:]+):(.*)$/) {
	$h->{authkeyident_certiss_dtype} = $1;
	$h->{authkeyident_certiss_value} = $2;
	return;
    }
    if ($value =~ /authorityCertSerialNumber=([^:]+):(.*)$/) {
	$h->{authkeyident_certsn_stype} = $1;
	$h->{authkeyident_certsn_value} = $2;
	return;
    }
};

$crlValueProc->{issuerAltName} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{issaltname_exists} = 1;
    $h->{issaltname_critical} = $row->[4];

    if ($value =~ /rfc822Name=(.*)$/) {
	$h->{issaltname_rfc_ord} = $seq;
	$h->{issaltname_rfc_value} = $1;
	return;
    }
    if ($value =~ /dNSName=(.*)$/) {
	$h->{issaltname_dns_ord} = $seq;
	$h->{issaltname_dns_value} = $1;
	return;
    }
    if ($value =~ /directoryName=([^:]+):(.*)$/) {
	$h->{issaltname_dn_ord} = $seq;
	$h->{issaltname_dn_type} = $1;
	$h->{issaltname_dn_value} = $2;
	return;
    }
    if ($value =~ /iPAddress=(.*)$/) {
	$h->{issaltname_ip_ord} = $seq;
	$h->{issaltname_ip_value} = $1;
	return;
    }
};

$crlValueProc->{cRLNumber} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{crlnum_exists} = 1;
    $h->{crlnum_critical} = $row->[4];

    if ($value =~ /(\w+):(.*)$/) {
	$h->{crlnum_stype} = $1;
	$h->{crlnum_value} = $2;
	return;
    }
};

$crlValueProc->{issuingDistributionPoint} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{idp_exists} = 1;
    $h->{idp_critical} = $row->[4];

    if ($value eq "onlyContainsUserCerts=TRUE") {$h->{idp_onlyusr}=1;}
    if ($value eq "onlyContainsCACerts=TRUE")   {$h->{idp_onlyca}=1;}
    if ($value =~ /directoryName=([^:]+):(.*)$/) {
	$h->{idp_dn_ord} = $seq;
	$h->{idp_dn_type} = $1;
	$h->{idp_dn_value} = $2;
    }
    if ($value =~ /URI=(.*)$/) {
	$h->{idp_uri_ord} = $seq;
	$h->{idp_uri_value} = canonical_nl($1);
    }
};

$crlValueProc->{privateExtension} = sub {
    my($h,$row) = @_;

    my $seq = $row->[2];
    my $value = $row->[3];
    my $key;

    $h->{privateext_exists} = 1;
    $h->{privateext_critical} = 0; # set to non critical
    $h->{privateext_value} = canonical_nl($value);
};

1;

