use JNSA::DB;
use JNSA::DB::UTIL;
use JNSA::HTML::UTIL;
use Data::Dumper;

sub printDumper {
    print "<pre>", esch(Dumper($_[0])), "</pre>\n";
}

sub set_hvalue_from_table {
    my($h, $procHash, $t) = @_;
    my($i);

    for $i (0..$#{$t}) {
        my $row = $t->[$i];
        my $iname = $row->[1];
        $h->{"now_processing_$iname"} = 1;
        &call_setter($procHash, $iname, $h, $row);
    }
}

sub call_setter {
    my($setterref,$name,$arg1,$arg2) = @_;
    if (! defined $setterref->{$name}) {
	$arg1->{"setter_notfound_$name"} = "NG?";
    } else {
	my $funcref = $setterref->{$name};
	&$funcref($arg1,$arg2);
    }
}

#
# printResult(db,name,id,action)
#
sub printResult {
    my($db,$name,$id,$action) = @_;

    if (! $db->isError) {
	$ok_msg = "A $name (No.$id) was $action.";
	print_html_if_warn($ok_msg);
    } else {
	$ng_msg = "It couldn\'t be $action because of database errors.";
	print_html_if_warn($ng_msg);
	$db->printErrorAssumption($db->getErrorMessageString);
    }
}

#
# printLog(db,form)
#
# Args:
#	db:   database object(JNSA::DB module instanse)
#	form: hash of input form values
#
#	Output the result of DB session and the form inputs as HTML to STDOUT
#
sub printLog {
    my ($db,$hForm) = @_;
    my ($log, $dbErr, $cgiErr);
    my $log = "";

    $dbErr = $db->getErrorMessageString() if $db ne "";
    $log .= "'******** DB Error Message *******'\n" . esch($dbErr)
	if ($dbErr ne "");

    $cgiErr = strInfo($hForm);
    $log .= "'******** CGI Form Input *********'\n" . esch($cgiErr)
	if ($cgiErr ne "");

    if ($log ne "") {
	print "<h4>Message Log</h4>\n";
	print "<textarea cols=80 rows=10>$log</textarea>\n";
    }
}

sub print_html_error {
    my($msg,$note) = @_;
    print "<P>\n";
    print "<STRONG class=\"error\">$note</STRONG><br>\n";
    print "<TEXTAREA cols=\"80\" rows=\"5\">";
    print $msg;
    print "</TEXTAREA>\n";
    print "</P>\n";
}

sub print_html_if_warn {
    my($msg) = @_;
    if ($msg ne '') {
	print "<P><STRONG class=\"warn\">$msg</STRONG></P>\n";
    }
}

sub selectorValue {
    my($info, $value) = @_;
    my $i;

    for $i (0..$#{$info->{value}}) {
	return $info->{disp}->[$i] if ($info->{value}->[$i] eq $value);
    }
}

# v$CPKI_CONST->{version}
sub printPoweredBy {
    print <<"___EOF___";
<p style="text-align: right;">
POWERED BY <a href="http://www.jnsa.org/mpki/">
CPKI-TS(Challenge PKI Test Suite) v$CPKI_CONST->{version}
</a></p>
___EOF___
}

sub decodeHttpRequest {
    my ($buffer, @pairs, $resultHash);
    
    if ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    } else {
	$buffer = $ENV{'QUERY_STRING'};
    }

    @pairs = split(/&/,$buffer);
    foreach $pair (@pairs) {
	my ($name, $value) = split(/=/, $pair);
	$value =~ tr/+/ /;
	$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	$value =~s/ \^/-\^/g;
	$resultHash->{$name} = $value;
    }
    return $resultHash;
}

sub decodeHttpRequestWithRawValue {
    my ($buffer, @pairs, $resultHash, $resultRawHash);
    
    if ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    } else {
	$buffer = $ENV{'QUERY_STRING'};
    }

    @pairs = split(/&/,$buffer);
    foreach $pair (@pairs) {
	my ($name, $value) = split(/=/, $pair);

	$resultRawHash->{$name} = $value;

	$value =~ tr/+/ /;
	$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	$value =~s/ \^/-\^/g;
	$resultHash->{$name} = $value;
    }
    return ($resultHash, $resultRawHash);
}

#
# cbox2bool 
#
sub cbox2bool {return ($_[0] eq 'on') ? 'true' : 'false';}

#
# canonical_tab
#
sub canonical_tab {
    my ($v) = @_;

    $v =~ s/^\s+//;
    $v =~ s/\s+$//;
    return join("\t",split(/\r\n/,$v));
}

#
# canonical_nl
#
sub canonical_nl {return join("\n",split(/\t/,$_[0]));}

#
# ord_minusone
#
#	Set value to the sequence number -1 if the value is NULL.
#
sub ord_minusone {
    my($h, $aOrd, $aValue) = @_;
    my($i);

    for $i (0..$#{$aValue}) {
	$h->{$aOrd->[$i]} = -1 if ($h->{$aValue->[$i]} eq '');
    }
}

#
# renumber_seq(h, keylist)
#
# Args:
#	h:		reference of hash
#	keylist:	ordered list(array) of key strings
#
#	renumber values of hash.
#	If value is -1, value is unchanged.
#	If key is not exists in the keylist, value is unchanged
#	Otherwise values are set to keylist order.
#	The start value is 1.
#
# $aaa->{kome} = '3';
# $aaa->{hie} = '10';
# $aaa->{gunjo} = '-1';
# $aaa->{mugitoro_ok} = 'aa';
# 	
# renumber_seq($aaa,[qw(kome hie gunjo)]) ... result to ...
# $aaa->{kome} = 1;
# $aaa->{hie} = 2;
# $aaa->{gunjo} = '-1';
# $aaa->{mugitoro_ok} = 'aa';
#
sub renumber_seq {
    my($h,$keylist) = @_;
    my($k);

    my @sorted_h = sort {$h->{$a} <=> $h->{$b}} (@{$keylist});

    my $i = 1;
    for $k (@sorted_h) {
	next if ($h->{$k} == -1);
	$h->{$k} = $i++;
    }
}

#
# gettitle
#
sub getTitle {
    my ($title) = @_;
    return $title . "[" . 
	$CPKI_CONFIG->{db}->{usr} . '@' .
	    getDbnameByDsn($CPKI_CONFIG->{db}->{dsn}) . "]";
}

#
# string_html_trth_hash
#
sub string_html_trth_hash {
    my($h, $aKey, $hPreName) = @_;
    my $i;

    my $s = "";
    for $i (0..$#{$aKey}) {
	my $k = esch($aKey->[$i]);
	my $v = esch($h->{$k});
	$s .= "<tr><td class=\"1\">$k</td><td>";
	$s .= "<pre>" if ($hPreName->{$k} == 1);
	$s .= $v;
	$s .= "</pre>" if ($hPreName->{$k} == 1);
	$s .= "</td></tr>\n";
    }
    return $s;
}

$CPKI_CONST = {
    version => "2.0.1 (DB:1.9.2.2a)",
    selector => {
	dn => {
	    value => [qw(Printable UTF8)],
	    disp => [qw(Printable UTF8)]
	},
	time => {
	    value => [qw(UTC Generalized)],
	    disp => [qw(UTCTime GeneralizedTime)]
	},
	dechex => {
	    value => [qw(dec hex)],
	    disp => [qw(dec hex)]
	},
	sha1md5 => {
	    value => [qw(0 1)],
	    disp => [qw(0:SHA1 1:MD5)]
	},
	rsadsa => {
	    value => [qw(0 1)],
	    disp => [qw(0:RSA 1:DSA)]
	},
	keypairusage => {
	    value => [qw(0 1 2)],
	    disp => [qw(0:CA 1:EE 2:OCSP)]
	},
	keypairidtype => {
	    value => [qw(0 1)],
	    disp => [qw(0:160bit_SHA1 1:0100+60bit_SHA1)]
	},
	keypairidtype_short => {
	    value => [qw(0 1)],
	    disp => [qw(0:160 1:0100+601)]
	},
	ocspresstat => {
	    value => [qw(0 1)],
	    disp => [qw(0:successful 1:malformedRequest)]
	},
	ocspresid => {
	    value => [qw(byName byKey)],
	    disp => [qw(byName byKey)]
	},
	ocspcertstat => {
	    value => [qw(0 1 2)],
	    disp => [qw(0:good 1:revoked 2:unknown)]
	},
	ocsprevores => {
	    value => [qw(2 3 5 6)],
	    disp => [qw(2:cACompromise 3:affiliationChanged 5:cessationOfOperation 6:certificateHold)]
	},
	ocspmojcode => {
	    value => [qw(0 1 2 3 4)],
	    disp => [qw(0 1 2 3 4)]
	},
	ocspcvsstat => {
	    value => [qw(0 101 200 201 202 203 204 205 206 901 902)],
	    disp => [qw(0:good 101:cantBuildPath 200: 201: 202:wrongSignature 203:revokedCert 204:anyPolicyExists 205:constraintViolation 206:ocspCertStatusUnknown 901:rejectYourRequest 902:requestTimeOut)]
	},
	order2 => {
	    value => [qw(-1 1 2)],
	    disp => [qw(None 1 2)]
	},
	order4 => {
	    value => [qw(-1 1 2 3 4)],
	    disp => [qw(None 1 2 3 4)]
	},
	certbase => {
	    value => [qw(1 2 3 4 5 21 22 23 24 25)],
	    disp => [qw(01:SelfSigned 02:CA 03:EE 04:OCSP-Responder 05:Link 21:GPKI-SelfSigned 22:GPKI-CA 23:GPKI-EE 24:GPKI-OCSP-Responder 25:GPKI-Link)]
	},
	certdiff => {
	    value => [qw(11 12 13 14 15 16 31 32 33 34 35 36 37)],
	    disp => [qw(11:SelfSigned 12:Cross 13:Sub-CA 14:EE 15:OCSP-Responder 16:Link 31:GPKI-SelfSigned 32:GPKI-Cross 33:GPKI-Sub-CA 34:GPKI-METI-Sub-CA 35:GPKI-EE 36:GPKI-OCSP-Responder 37:GPKI-Link)]
	},
	crlbase => {
	    value => [qw(1 2 3)],
	    disp => [qw(01:full 02:EE 03:ARL)]
	},
	crldiff => {
	    value => [qw(11 12 13)],
	    disp => [qw(11:full 12:EE 13:ARL)]
	},
    },
    const => {
        keyusage => {
	    digitalSignature	=> 'keyusage_digitsign',
	    nonRepudiation	=> 'keyusage_nonrepu',
	    keyEncipherment	=> 'keyusage_keyenciph',
	    dataEncipherment	=> 'keyusage_dataenciph',
	    keyAgreement	=> 'keyusage_keyagree',
	    keyCertSign		=> 'keyusage_keycertsgn',
	    cRLSign		=> 'keyusage_crlsign',
	    encipherOnly	=> 'keyusage_enciphonly',
	    decipherOnly	=> 'keyusage_deciphonly',
        },
	javascript => "
<SCRIPT type=\"text/javascript\">
function openForm(cgi,f) {
    if (f.value != \"\") {
	var v = parseInt(f.value);
        if (isNaN(v)) {
	    alert('specified value is not id: ' + v);
        } else {
	    open(\"cpki_\"+cgi+\"_edit.cgi?id=\"+v);
        }
    } else {
	alert('id not specified');
    }
}
function openID(cgi,pk,id) {
    if (isNaN(id)) {
        alert('specified value is not id: ' + id);
    } else {
        open(\"cpki_\"+cgi+\".cgi?\"+pk+\"=\"+id);
    }
}
function openEd(cgi,id) {
    if (isNaN(id)) {
        alert('specified value is not id: ' + id);
    } else {
        open(\"cpki_\"+cgi+\"_edit.cgi?id=\"+id);
    }
}
function formCheckboxSetAll(form, value) {
  var f = form;
  var len = f.elements.length;
  for (var i = 0; i < len; i++) {
    var e = f.elements[i];
    if (e.type == \"checkbox\") e.checked = value;
  }
}
</SCRIPT>
",
    },
};

1;
