package JNSA::DB::CERT;		# -*-Perl-*-

# $Id: CERT.pm,v 1.2 2003/10/24 05:04:21 urushima Exp $

use JNSA::DB::UTIL;
use vars qw($VERSION);

$VERSION = '1.911';

require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(sqlInsertCert
	   sqlInsertX509Extension
	   sqlInsertCertValueBasic
	   sqlInsertCertValueExt
	   sqlUpdateCert
	   sqlUpdateCertNote
	   sqlDeleteCertValue
	   sqlDeleteCertValueBasic
	   sqlDeleteCertValueExt);

#
# sqlInsertCert(certInfo)
#
#	
sub sqlInsertCert {
    my($h) = @_;

    qwsH($h, qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair subject_keypair));

    $sql = "
INSERT INTO cert VALUES (
  $h->{id},$h->{base_id},$h->{issuer_keypair},$h->{subject_keypair},
  $h->{hash_alg},$h->{name},$h->{keyword},$h->{note});";

    return $sql;
}
# -- end ------------------------------------------------------------

#
# sqlInsertX509Extension(hashInfo, tableName)
#
sub sqlInsertX509Extension {
    my($h, $tableName) = @_;
    my $sql;

    qwsH($h,qw(name value));

    $sql = "
INSERT INTO $tableName VALUES (
  $h->{id},$h->{name},$h->{seq_no},
  $h->{value},$h->{is_critical},$h->{is_ext});";
    return $sql;
}

#
# sqlInsertCertValueBasic(certInfo)
#
sub sqlInsertCertValueBasic {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(sn issuer validity subject));

    my $CRITICAL_FLAG = 'false';	# default

    if ($h->{sn} ne '') {
	$sql .= "
INSERT INTO cert_value VALUES (
  $h->{id},'serialNumber',0,$h->{sn},false,false);";
    }

    if ($h->{issuer} ne '') {
	$sql .= "
INSERT INTO cert_value VALUES (
  $h->{id},'issuer',0,$h->{issuer},false,false);";
    }

    if ($h->{validity} ne '') {
	$sql .= "
INSERT INTO cert_value VALUES (
  $h->{id},'validity',0,$h->{validity},false,false);"; 
    }

    if ($h->{subject} ne '') {
	$sql .= "
INSERT INTO cert_value VALUES (
  $h->{id},'subject',0,$h->{subject},false,false);";
    }
    return $sql;
}

# -- end ------------------------------------------------------------

#
# sqlInsertCertValueExt(certInfo)
#
sub sqlInsertCertValueExt {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(ext_note
	       authkeyident_keyident
	       authkeyident_certiss
	       authkeyident_certsn
	       subjkeyident_keyident
	       certpolicy_oids
	       policymap
	       subjaltname_rfc
	       subjaltname_dns
	       subjaltname_dn
	       subjaltname_ip
	       issaltname_rfc
	       issaltname_dns
	       issaltname_dn
	       issaltname_ip
	       basicconst_ca
	       basicconst_pathlen
	       nameconst_permit
	       nameconst_exclude
	       policyconst_req
	       policyconst_inhibit
	       extkeyusage_oids
	       crldp_dn
	       crldp_uri
	       authinfoacc_uri
	       privateext
	       ));

    _addEauthorityKeyIdentifier(\$sql,$h);
    _addEsubjectKeyIdentifier(\$sql,$h);
    _addEkeyUsage(\$sql,$h);
    _addEcertificatePolicies(\$sql,$h);
    _addEpolicyMappings(\$sql,$h);
    _addEsubjectAltName(\$sql,$h);
    _addEissuerAltName(\$sql,$h);
    _addEbasicConstraints(\$sql,$h);
    _addEnameConstraints(\$sql,$h);
    _addEpolicyConstraints(\$sql,$h);
    _addEextendedKeyUsage(\$sql,$h);
    _addEcRLDistributionPoint(\$sql,$h);
    _addEauthorityInformationAccess(\$sql,$h);
    _addEoCSPNoCheck(\$sql,$h);
    _addEprivateExtension(\$sql,$h);

    return $sql;
}

sub _addEauthorityKeyIdentifier {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{authkeyident_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'authorityKeyIdentifier';
    my $critical = $h->{authkeyident_critical};
    
    for $key (qw(authkeyident_keyident
		 authkeyident_certiss
		 authkeyident_certsn)) {
	if ($h->{$key} ne "''") {
	    $$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
	    $i++;
	}
    }
}

sub _addEsubjectKeyIdentifier {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{subjkeyident_exists} ne 'on') { return; }

    my $i = 0;
    my $iname = 'subjectKeyIdentifier';
    my $critical = $h->{subjkeyident_critical};

    my $key = 'subjkeyident_keyident';
    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
    }
}

sub _addEkeyUsage {
    my ($sqlref, $h) = @_;
    my $key;
    
    if ($h->{keyusage_exists} ne 'on') { return; }

    my $i = 0;
    my $iname = 'keyUsage';
    my $critical = $h->{keyusage_critical};
    my $key = 'keyusage';
    
    if ($h->{$key} ne '') {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,'$h->{$key}',$critical,true);";
    }
}

sub _addEcertificatePolicies {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{certpolicy_exists} ne 'on') { return; }

    my $i = 0;
    my $iname = 'certificatePolicies';
    my $critical = $h->{certpolicy_critical};

    if ($h->{certpolicy_anypolicy} eq 'on') {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,'anyPolicy',$critical,true);";
    } else {
	if ($h->{certpolicy_oids} ne '') {
	    $$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{certpolicy_oids},$critical,true);";
	}
    }
}

sub _addEpolicyMappings {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{policymap_exists} ne 'on') { return; }

    my $i = 0;
    my $iname = 'policyMappings';
    my $critical = $h->{policymap_critical};

    if ($h->{policymap} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{policymap},$critical,true);";
    }
}

sub _addEsubjectAltName {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{subjaltname_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'subjectAltName';
    my $critical = $h->{subjaltname_critical};

    for $key (qw(subjaltname_rfc
		 subjaltname_dns
		 subjaltname_dn
		 subjaltname_ip)) {
	my $ord = $key . "_ord";
	next if ($h->{$ord} == -1);
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$h->{$ord},$h->{$key},$critical,true);";
    }
}

sub _addEissuerAltName {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{issaltname_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'issuerAltName';
    my $critical = $h->{issaltname_critical};

    for $key (qw(issaltname_rfc issaltname_dns issaltname_dn issaltname_ip)) {
	my $ord = $key . "_ord";
	next if ($h->{$ord} == -1);
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$h->{$ord},$h->{$key},$critical,true);";
    }
}

sub _addEbasicConstraints {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{basicconst_exists} ne 'on') { return; }

    my $iname = 'basicConstraints';
    my $critical = $h->{basicconst_critical};
    my $key;
    
    $ord = 1;

    $key = "basicconst_ca";
    if ($h->{basicconst_ca} eq "'cA=TRUE'") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$ord,$h->{$key},$critical,true);";
	$ord++;
    }

    $key = "basicconst_pathlen";
    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$ord,$h->{$key},$critical,true);";
    }
}

sub _addEnameConstraints {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{nameconst_exists} ne 'on') { return; }

    my $iname = 'nameConstraints';
    my $critical = $h->{nameconst_critical};
    my ($key, $i);
    
    $i = 1;
    for $key (qw(nameconst_permit nameconst_exclude)) {
	next if ($h->{$key} eq "''");
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
	$i++;
    }
}

sub _addEpolicyConstraints {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{policyconst_exists} ne 'on') { return; }

    my $iname = 'policyConstraints';
    my $critical = $h->{policyconst_critical};
    my ($key, $i);
    
    $i = 1;
    for $key (qw(policyconst_req policyconst_inhibit)) {
	next if ($h->{$key} eq "''");
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
	$i++;
    }
}

sub _addEextendedKeyUsage {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{extkeyusage_exists} ne 'on') { return; }

    my $iname = 'extendedKeyUsage';
    my $critical = $h->{extkeyusage_critical};
    my ($key, $i);

    $key = "extkeyusage_oids";
    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',0,$h->{$key},$critical,true);";
	$i++;
    }
}

sub _addEcRLDistributionPoint {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{crldp_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'cRLDistributionPoint';
    my $critical = $h->{crldp_critical};

    for $key (qw(crldp_dn crldp_uri)) {
	my $ord = $key . "_ord";
	next if ($h->{$ord} == -1);
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$h->{$ord},$h->{$key},$critical,true);";
    }
}

sub _addEauthorityInformationAccess {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{authinfoacc_exists} ne 'on') { return; }

    my $i = 0;
    my $iname = 'authorityInformationAccess';
    my $critical = $h->{authinfoacc_critical};
    my $key = "authinfoacc_uri";

    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
    }
}

sub _addEoCSPNoCheck {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{ocspnochk_exists} ne 'on') { return; }

    my $iname = 'oCSPNoCheck';
    my $critical = $h->{ocspnochk_critical};

    $$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',0,'TRUE',$critical,true);";
}

sub _addEprivateExtension {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{privateext_exists} ne 'on') { return; }

    my $iname = 'privateExtension';
    my $critical = $h->{privateext_critical};
    my ($key, $i);

    $key = "privateext";
    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO cert_value VALUES (
  $h->{id},'$iname',0,$h->{$key},$critical,true);";
	$i++;
    }
}

# -- end ------------------------------------------------------------

#
# sqlUpdateCert(certInfo)
#
#	
sub sqlUpdateCert {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair subject_keypair));

    my $sql = "
UPDATE cert SET
  base_id = $h->{base_id},
  issuer_keypair = $h->{issuer_keypair},
  subject_keypair = $h->{subject_keypair},
  hash_alg = $h->{hash_alg},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}
# -- end ------------------------------------------------------------

#
# sqlUpdateCertNote(certInfo)
#
#	
sub sqlUpdateCertNote {
    my($h) = @_;

    my $note = qws($h->{note});

    my $sql = "
UPDATE cert SET note = $note WHERE id = $h->{id};";

    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteCertValue {
    my($id) = @_;

    my $sql = "
DELETE FROM cert_value WHERE id = $id;";
    return $sql;
}

# -- end ------------------------------------------------------------
# DEPRECATED
sub sqlDeleteCertValueBasic {
    my($cert_no) = @_;

    my $sql = "
DELETE FROM cert_value
  WHERE cert_no = $cert_no AND cert_item_name LIKE 'B%';";

    return $sql;
}
# -- end ------------------------------------------------------------
# DEPRECATED
sub sqlDeleteCertValueExt {
    my($cert_no) = @_;

    my $sql = "
DELETE FROM cert_value
  WHERE cert_no = $cert_no AND cert_item_name LIKE 'E%';";

    return $sql;
}
# -- end ------------------------------------------------------------
1;
