package JNSA::DB::CRL;		# -*-Perl-*-

# $Id: CRL.pm,v 1.2 2003/10/24 05:04:21 urushima Exp $

use JNSA::DB::UTIL;
use vars qw($VERSION);

$VERSION = '1.911';

require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(sqlInsertCrl
	   sqlInsertCrlValueBasic
	   sqlInsertCrlValueExt
	   sqlUpdateCrl
	   sqlUpdateCrlNote
	   sqlDeleteCrlValue
	   sqlDeleteCrlValueBasic
	   sqlDeleteCrlValueExt
	   );

#
# sqlInsertCrl(crlInfo)
#
#	
sub sqlInsertCrl {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair));

    my $sql = "
INSERT INTO crl VALUES (
  $h->{id},$h->{base_id},$h->{issuer_keypair},$h->{hash_alg},
  $h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

# -- end ------------------------------------------------------------

#
# sqlInsertCrlValueBasic(crlInfo)
#
sub sqlInsertCrlValueBasic {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(issuer validity revokecert));

    $sql .= "
INSERT INTO crl_value VALUES (
  $h->{id},'issuer',0,$h->{issuer},false,false);";

    if ($h->{validity} ne "''") {
	$sql .= "
INSERT INTO crl_value VALUES (
  $h->{id},'validity',0,$h->{validity},false,false);";
    }

    if ($h->{revokecert} ne "''") {
	$sql .= "
INSERT INTO crl_value VALUES (
  $h->{id},'revokedCertificates',0,$h->{revokecert},false,false);";
    }
    return $sql;
}

# -- end ------------------------------------------------------------

#
# sqlInsertCrlValueExt(crlInfo)
#
sub sqlInsertCrlValueExt {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(authkeyident_keyident
	       authkeyident_certiss
	       authkeyident_certsn
	       issaltname_rfc
	       issaltname_dns
	       issaltname_dn
	       issaltname_ip
	       crlnum
	       idp_dn
	       idp_uri
	       idp_onlyusr
	       idp_onlyca
	       privateext
	       ));

    _addEauthorityKeyIdentifier(\$sql,$h);
    _addEissuerAltName(\$sql,$h);
    _addEcRLNumber(\$sql,$h);
    _addEissuingDistributionPoint(\$sql,$h);
    _addEprivateExtension(\$sql,$h);
    return $sql;
}

sub _addEauthorityKeyIdentifier {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{authkeyident_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'authorityKeyIdentifier';
    my $critical = $h->{authkeyident_critical};
    
    for $key (qw(authkeyident_keyident
		 authkeyident_certiss
		 authkeyident_certsn)) {
	if ($h->{$key} ne "''") {
	    $$sqlref .= "
INSERT INTO crl_value VALUES (
  $h->{id},'$iname',$i,$h->{$key},$critical,true);";
	    $i++;
	}
    }
}

sub _addEissuerAltName {
    my ($sqlref, $h) = @_;
    my $key;

    if ($h->{issaltname_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'issuerAltName';
    my $critical = $h->{issaltname_critical};

    for $key (qw(issaltname_rfc
		 issaltname_dns
		 issaltname_dn
		 issaltname_ip)) {
	my $ord = $key . "_ord";
	next if ($h->{$ord} == -1);
	$$sqlref .= "
INSERT INTO crl_value VALUES (
  $h->{id},'$iname',$h->{$ord},$h->{$key},$critical,true);";
    }
}

sub _addEcRLNumber {
    my ($sqlref, $h) = @_;
    my $key, $i;
    
    if ($h->{crlnum_exists} ne 'on') { return; }

    my $iname = 'cRLNumber';
    my $critical = $h->{crlnum_critical};

    if ($h->{crlnum} ne "''") {
	$$sqlref .= "
INSERT INTO crl_value VALUES (
  $h->{id},'$iname',0,$h->{crlnum},$critical,true);";
    }
}    

sub _addEissuingDistributionPoint {
    my ($sqlref, $h) = @_;
    my $key, $i;
    
    if ($h->{idp_exists} ne 'on') { return; }

    my $i = 1;
    my $iname = 'issuingDistributionPoint';
    my $critical = $h->{idp_critical};

    for $key (qw(idp_dn
		 idp_uri
		 idp_onlyusr
		 idp_onlyca)) {
	my $ord = $key . "_ord";
	next if ($h->{$ord} == -1);
	$$sqlref .= "
INSERT INTO crl_value VALUES (
  $h->{id},'$iname',$h->{$ord},$h->{$key},$critical,true);";
    }
}

sub _addEprivateExtension {
    my ($sqlref, $h) = @_;
    my $key, $i;

    if ($h->{privateext_exists} ne 'on') { return; }

    my $iname = 'privateExtension';
    my $critical = $h->{privateext_critical};
    my ($key, $i);

    $key = "privateext";
    if ($h->{$key} ne "''") {
	$$sqlref .= "
INSERT INTO crl_value VALUES (
  $h->{id},'$iname',0,$h->{$key},$critical,true);";
	$i++;
    }
}
# -- end ------------------------------------------------------------

#
# sqlUpdateCrl(crlInfo)
#
#	
sub sqlUpdateCrl {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair));

    my $sql = "
UPDATE crl SET
  base_id = $h->{base_id},
  issuer_keypair = $h->{issuer_keypair},
  hash_alg = $h->{hash_alg},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}
# -- end ------------------------------------------------------------

#
# sqlUpdateCrlNote(crlInfo)
#
#	
sub sqlUpdateCrlNote {
    my($h) = @_;

    my $note = qws($h->{note});

    my $sql = "
UPDATE crl SET note = $note WHERE id = $h->{id};";
    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteCrlValue {
    my($id) = @_;

    my $sql = "
DELETE FROM crl_value WHERE id = $id;";
    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlDeleteCrlValueBasic {
    my($crl_no) = @_;

    my $sql = "
DELETE FROM crl_value
  WHERE crl_no = $crl_no AND crl_item_name LIKE 'B%';";

    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteCrlValueExt {
    my($crl_no) = @_;

    my $sql = "
DELETE FROM crl_value
  WHERE crl_no = $crl_no AND crl_item_name LIKE 'E%';";

    return $sql;
}
# -- end ------------------------------------------------------------
1;
