package JNSA::DB::OCSP;		# -*-Perl-*-

use JNSA::DB::UTIL;
use vars qw($VERSION);

$VERSION = '1.911';

require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(sqlInsertOcsp
	   sqlInsertOcspValueBasic
	   sqlInsertOcspValueExt
	   sqlUpdateOcsp
	   sqlUpdateOcspNote
	   sqlDeleteOcspValue
	   sqlDeleteOcspValueBasic
	   sqlDeleteOcspValueExt);

sub sqlInsertOcsp {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair ocsp_cert));

    my $sql = "
INSERT INTO ocsp VALUES (
  $h->{id},$h->{base_id},$h->{issuer_keypair},$h->{ocsp_cert},
  $h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

sub sqlInsertOcspValueBasic {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(responseStatus
	       responderID
	       producedAt
	       certID_issuerNameHash
	       certID_issuerKeyHash
	       certID_serialNumber
	       certStatus_CertStatus
	       certStatus_revocationTime
	       certStatus_revocationReason
	       thisUpdate
	       ));

    $sql .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'responseStatus',0,$h->{responseStatus},false,false);";

    if ($h->{resid} ne "''") {
	$sql .= "                            
INSERT INTO ocsp_value VALUES (
  $h->{id},'responderID',0,$h->{responderID},false,false);";
    }

    if ($h->{producedAt} ne "''") {
	$sql .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'producedAt',0,$h->{producedAt},false,false);";
    }

    _addBcertID(\$sql,$h);
    _addBcertStatus(\$sql,$h);

    if ($h->{thisUpdate} ne '') {
	$sql .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'thisUpdate',0,$h->{thisUpdate},false,false);";
    }

    return $sql;
}

sub _addBcertID {
    my ($sqlref, $h) = @_;

    my $this_seq_no = 1;

    if ($h->{certID_issuerNameHash} ne "''") {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certID',$this_seq_no,$h->{certID_issuerNameHash},false,false);";
	$this_seq_no++;  
    }

    if ($h->{certID_issuerKeyHash} ne "''") {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certID',$this_seq_no,$h->{certID_issuerKeyHash},false,false);";
	$this_seq_no++;  
    }

    if ($h->{certID_serialNumber} ne "''") {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certID',$this_seq_no,$h->{certID_serialNumber},false,false);";
	$this_seq_no++;  
    }
}

sub _addBcertStatus {
    my ($sqlref, $h) = @_;

    my $this_seq_no = 1;

    if ($h->{certStatus_CertStatus} ne "''") {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certStatus',$this_seq_no,$h->{certStatus_CertStatus},false,false);";
	$this_seq_no++;  
    }

    if ($h->{certStatus_revocatoinTime} ne "''") {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certStatus',$this_seq_no,$h->{certStatus_revocationTime},false,false);";
	$this_seq_no++;  
    }
    if ($h->{certStatus_revocationReason} ne '') {
	$$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'certStatus',$this_seq_no,$h->{certStatus_revocationReason},false,false);";
	$this_seq_no++;  
    }
}
# -- end ------------------------------------------------------------


#
# sqlInsertOcspValueExt(ocspInfo)
#
sub sqlInsertOcspValueExt {
    my($h) = @_;
    my $sql = "";

    _addEoCSPStatusCode(\$sql,$h);
    _addEcertPathStatus(\$sql,$h);

    return $sql;
}

sub _addEoCSPStatusCode {
    my ($sqlref, $h) = @_;

    if ($h->{oCSPStatusCode_exists} ne 'on') { return; }

    qwsH($h,qw(oCSPStatusCode_value));

    my $iname = 'oCSPStatusCode';
    my $critical = $h->{oCSPStatusCode_critical};
    
    $$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'$iname',0,$h->{oCSPStatusCode_value},$critical,true);";
}

sub _addEcertPathStatus {
    my ($sqlref, $h) = @_;

    if ($h->{certPathStatus_exists} ne 'on') { return; }

    qwsH($h,qw(certPathStatus_value));

    my $iname = 'certPathStatus';
    my $critical = $h->{certPathStatus_critical};
    
    $$sqlref .= "
INSERT INTO ocsp_value VALUES (
  $h->{id},'$iname',0,$h->{certPathStatus_value},$critical,true);";
}

# -- end ------------------------------------------------------------

#
# sqlUpdateOcsp(ocspInfo)
#
#	
sub sqlUpdateOcsp {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(base_id issuer_keypair ocsp_cert));

    my $sql = "
UPDATE ocsp SET
  base_id = $h->{base_id},
  issuer_keypair = $h->{issuer_keypair},
  ocsp_cert = $h->{ocsp_cert},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}
# -- end ------------------------------------------------------------

#
# sqlUpdateCertNote(certInfo)
#
#	
sub sqlUpdateOcspNote {
    my($h) = @_;

    my $note = qws($h->{note});

    my $sql = "
UPDATE ocsp SET note = $note WHERE id = $h->{id};";

    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteOcspValue {
    my($id) = @_;

    my $sql = "DELETE FROM ocsp_value WHERE id = $id;";

    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteOcspValueBasic {
    my($ocsp_no) = @_;

    my $sql = "
DELETE FROM ocsp_value
  WHERE ocsp_no = $ocsp_no AND ocsp_item_name LIKE 'B%';";

    return $sql;
}
# -- end ------------------------------------------------------------

sub sqlDeleteOcspValueExt {
    my($ocsp_no) = @_;

    my $sql = "
DELETE FROM ocsp_value
  WHERE ocsp_no = $ocsp_no AND ocsp_item_name LIKE 'E%';";

    return $sql;
}
# -- end ------------------------------------------------------------

1;
