package JNSA::DB::SQL;	# -*-Perl-*-

# $Id: SQL.pm,v 1.6 2004/07/05 01:18:48 urushima Exp $

use JNSA::DB::UTIL;
use vars qw($VERSION);

$VERSION = '1.922';

require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(sqlInsertTestCase
	   sqlUpdateTestCase
	   sqlInsertKeyPair
	   sqlUpdateKeyPair
	   sqlInsertCcp
	   sqlUpdateCcp
	   sqlInsertLdapEntry
	   sqlUpdateLdapEntry
	   sqlInsertLdapRepositoryItem
	   sqlInsertLdapRepository
	   sqlUpdateLdapRepository
	   sqlInsertRepositorySet
	   sqlUpdateRepositorySet
	   sqlInsertTestCategory
	   sqlUpdateTestCategory
	   sqlInsertTestClient
	   sqlUpdateTestClient);

sub sqlInsertTestCase {
    my($h) = @_;

    qwsH($h,qw(test_case_name accep_policy
	       test_explanation reference));

    my $sql = "
INSERT INTO test_case VALUES (
  $h->{test_case_no},$h->{test_case_name},$h->{test_category_id},
  $h->{test_level},$h->{cert_path_no},
  $h->{test_exp_value},$h->{accep_policy},
  $h->{init_policy},$h->{init_explicit},$h->{init_any},
  $h->{test_explanation},$h->{reference});";

    return $sql;
}


sub sqlUpdateTestCase {
    my($h) = @_;

    qwsH($h,qw(test_case_name accep_policy
	       test_explanation reference));

    my $sql = "
UPDATE test_case SET
  test_case_name = $h->{test_case_name},
  test_category_id = $h->{test_category_id},
  test_level = $h->{test_level},
  cert_path_no = $h->{cert_path_no},
  test_exp_value = $h->{test_exp_value},
  accep_policy = $h->{accep_policy},
  init_policy_mapping_inhibit = $h->{init_policy},
  init_explicit_policy = $h->{init_explicit},
  init_any_policy_inhibit = $h->{init_any},
  test_explanation = $h->{test_explanation},
  reference = $h->{reference}
  WHERE test_case_no = $h->{test_case_no};";

    return $sql;
}

sub sqlInsertKeyPair {
    my($h) = @_;

    qwsH($h,qw(public_key private_key keyid keyid_sha1_64 
	       name keyword note));

    my $sql = "
INSERT INTO keypair VALUES (
  $h->{id},$h->{crypto_method},$h->{crypto_bit},
  $h->{private_key},$h->{public_key},
  $h->{keyid},$h->{keyid_sha1_64},
  $h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateKeyPair {
    my($h) = @_;

    qwsH($h,qw(public_key private_key keyid keyid_sha1_64
	       name keyword note));

    my $sql = "
UPDATE keypair SET
  crypto_method = $h->{crypto_method},
  crypto_bit = $h->{crypto_bit},
  public_key = $h->{public_key},
  private_key = $h->{private_key},
  keyid = $h->{keyid},
  keyid_sha1_64 = $h->{keyid_sha1_64},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertCcp {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(issued_to issued_by issued_to_raw issued_by_raw));

    my $sql = "
INSERT INTO ccp VALUES (
  $h->{id},$h->{issued_to},$h->{issued_by},
  $h->{issued_to_raw},$h->{issued_by_raw},
  $h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateCcp {
    my($h) = @_;

    qwsH($h,qw(name keyword note));
    numNullH($h, qw(issued_to issued_by issued_to_raw issued_by_raw));

    my $sql = "
UPDATE ccp SET
  issued_to = $h->{issued_to},
  issued_by = $h->{issued_by},
  issued_to_raw = $h->{issued_to_raw},
  issued_by_raw = $h->{issued_by_raw},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertLdapEntry {
    my($h) = @_;

    qwsH($h,qw(dn entry name keyword note));

    my $sql = "
INSERT INTO ldap_entry VALUES (
  $h->{id},$h->{dn},$h->{entry},
  $h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateLdapEntry {
    my($h) = @_;

    qwsH($h,qw(dn entry name keyword note));

    my $sql = "
UPDATE ldap_entry SET
  dn = $h->{dn},
  entry = $h->{entry},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertLdapRepositoryItem {
    my($h) = @_;

    my $sql = "
INSERT INTO ldap_repository_item VALUES ($h->{id},$h->{entry_id});";
    return $sql;
}

sub sqlInsertLdapRepository {
    my($h) = @_;

    qwsH($h,qw(host name keyword note));
    numNullH($h, qw(port));

    my $sql = "
INSERT INTO ldap_repository VALUES (
  $h->{id},$h->{host},$h->{port},
  $h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateLdapRepository {
    my($h) = @_;

    qwsH($h,qw(host name keyword note));
    numNullH($h, qw(port));

    my $sql = "
UPDATE ldap_repository SET
  host = $h->{host},
  port = $h->{port},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertRepositorySet {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
INSERT INTO repository_set VALUES (
  $h->{id},$h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateRepositorySet {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
UPDATE repository_set SET
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertTestCategory {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
INSERT INTO test_category VALUES (
  $h->{id},$h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

sub sqlUpdateTestCategory {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
UPDATE test_category SET
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

sub sqlInsertTestClient {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
INSERT INTO test_client VALUES (
  $h->{id},$h->{name},$h->{keyword},$h->{note});";
    return $sql;
}

sub sqlUpdateTestClient {
    my($h) = @_;

    qwsH($h,qw(name keyword note));

    my $sql = "
UPDATE test_client SET
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

1;
