package JNSA::DB::TSP;		# -*-Perl-*-

use JNSA::DB::UTIL;
use vars qw($VERSION);

$VERSION = '1.911';

require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(sqlInsertTsq
	   sqlUpdateTsq
	   sqlInsertTsqValue
	   sqlInsertTsr
	   sqlUpdateTsr
	   sqlInsertTsrValue
	   sqlInsertTspProtocolTestcase
	   sqlUpdateTspProtocolTestcase
	   sqlInsertTspAccuracyTestcase
	   sqlUpdateTspAccuracyTestcase
	   );

sub sqlInsertTsq {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(name keyword note));
    $h->{base_id} = numNull($h->{base_id});

    my $sql = "
INSERT INTO tsq VALUES (
  $h->{id},$h->{base_id},
  $h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

# -- end ------------------------------------------------------------

#
# sqlInsertTsqValue(info)
#
sub sqlInsertTsqValue {
    my($h) = @_;
    my $sql = "";
    my @attrNames = qw(version hashAlgorithm hashedMessage
		       reqPolicy nonce certReq);
    

    #qwsH($h,@attrNames);

    for my $attrName (@attrNames) {
	my $name = qws($attrName);
	my $value = qws($h->{$attrName});
	next if ($name eq "'reqPolicy'" && $value eq "''");
	next if ($name eq "'certReq'" && $value eq "''");
	$sql .= "
INSERT INTO tsq_value VALUES (
  $h->{id},$name,0,$value,false,false);";
    }
    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlUpdateTsq {
    my($h) = @_;

    qwsH($h, qw(name keyword note));
    numNullH($h, qw(base_id));

    my $sql = "
UPDATE tsq SET
  base_id = $h->{base_id},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}


# -- end ------------------------------------------------------------

sub sqlInsertTsr {
    my($h) = @_;
    my $sql = "";

    qwsH($h,qw(name keyword note));
    $h->{base_id} = numNull($h->{base_id});

    my $sql = "
INSERT INTO tsr VALUES (
  $h->{id},$h->{base_id},
  $h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlInsertTsrValue {
    my($h) = @_;
    my $sql = "";
    my @attrNames = qw(TimeStampResp_status
		       PKIStatusInfo_statusString
		       PKIStatusInfo_failInfo
		       TimeStampResp_containsTST
		       TSTInfo_version
		       TSTInfo_policy
		       TSTInfo_messageImprint_hashAlgorithm
		       TSTInfo_messageImprint_AUTO
		       TSTInfo_messageImprint_hashedMessage
		       TSTInfo_serialNumber
		       TSTInfo_genTime
		       TSTInfo_accuracy
		       TSTInfo_ordering
		       TSTInfo_nonce
		       TSTInfo_nonce_AUTO
		       TSTInfo_tsa
		       TSTInfo_extensions
		       TSTInfo_binary
		       SignerInfo_count
		       Signature_tampered
		       SignedData_version
		       ContentInfo_contentType
		       EncapsulatedContentInfo_eContentType
		       Exclude_ESSCertID
		       ESSCertID_certHash
		       ESSCertID_issuerSerial_issuer
		       ESSCertID_issuerSerial_serialNumber
		       );
    
    #qwsH($h,@attrNames);

    for my $attrName (@attrNames) {
	my $name = qws($attrName);
	my $value = qws($h->{$attrName});
	$sql .= "
INSERT INTO tsr_value VALUES (
  $h->{id},$name,0,$value,false,false);";
    }

    my $cert_seq = 1;
    for my $i (1..5) {
	if ($h->{"SigningCertificate_certID$i"} ne '') {
	    my $value = $h->{"SigningCertificate_certID$i"};
	    $sql .= "
INSERT INTO tsr_value VALUES (
  $h->{id},'SigningCertificate_certID',$cert_seq,$value,false,false);";
	    $cert_seq++;
	}
    }

    my $keypair_seq = 1;
    for my $i (1..5) {
	if ($h->{"SigningCertificate_keypairID$i"} ne '') {
	    my $value = $h->{"SigningCertificate_keypairID$i"};
	    $sql .= "
INSERT INTO tsr_value VALUES (
  $h->{id},'SigningCertificate_keypairID',$keypair_seq,$value,false,false);";
	    $keypair_seq++;
	}
    }

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlUpdateTsr {
    my($h) = @_;

    qwsH($h, qw(name keyword note));
    numNullH($h, qw(base_id));

    my $sql = "
UPDATE tsr SET
  base_id = $h->{base_id},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlInsertTspProtocolTestcase {
    my($h) = @_;

    qwsH($h,qw(testcase_code http_header
	       name keyword note));
    numNullH($h, qw(category_id tsq_id tsr_id 
		    level exp_value));

    my $sql = "
INSERT INTO tsp_protocol_testcase VALUES (
  $h->{id},$h->{testcase_code},$h->{category_id},
  $h->{level},$h->{tsq_id},$h->{tsr_id},$h->{http_header},$h->{exp_value},
  $h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlUpdateTspProtocolTestcase {
    my($h) = @_;

    qwsH($h,qw(testcase_code http_header
	       name keyword note));
    numNullH($h, qw(category_id tsq_id tsr_id 
		    level exp_value));

    my $sql = "
UPDATE tsp_protocol_testcase SET
  testcase_code = $h->{testcase_code},
  category_id = $h->{category_id},
  level = $h->{level},
  tsq_id = $h->{tsq_id},
  tsr_id = $h->{tsr_id},
  http_header = $h->{http_header},
  exp_value = $h->{exp_value},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlInsertTspAccuracyTestcase {
    my($h) = @_;

    qwsH($h,qw(testcase_code
	       name keyword note));
    numNullH($h, qw(category_id tsr_id1 tsr_id2 
		    level exp_value));

    my $sql = "
INSERT INTO tsp_accuracy_testcase VALUES (
  $h->{id},$h->{testcase_code},$h->{category_id},
  $h->{level},$h->{tsr_id1},$h->{tsr_id2},$h->{exp_value},
  $h->{name},$h->{keyword},$h->{note});";

    return $sql;
}

# -- end ------------------------------------------------------------

sub sqlUpdateTspAccuracyTestcase {
    my($h) = @_;

    qwsH($h,qw(testcase_code
	       name keyword note));
    numNullH($h, qw(category_id tsr_id1 tsr_id2
		    level exp_value));

    my $sql = "
UPDATE tsp_accuracy_testcase SET
  testcase_code = $h->{testcase_code},
  category_id = $h->{category_id},
  level = $h->{level},
  tsr_id1 = $h->{tsq_id1},
  tsr_id2 = $h->{tsr_id2},
  exp_value = $h->{exp_value},
  name = $h->{name},
  keyword = $h->{keyword},
  note = $h->{note}
  WHERE id = $h->{id};";

    return $sql;
}

