--
-- Multi-domain PKI Interoperability Test Suite Database Schema v1.9.1.4c
--
--							Oct 22, 2003
--
--			NPO JNSA http://www.jnsa.org/
--			SECOM Co., Ltd. http://www.secom.co.jp/
--
CREATE TABLE "ts_info" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" text,
	"value" text
);

CREATE TABLE "keypair" (
	"id" numeric(9,0) PRIMARY KEY,
	"crypto_method" numeric(1,0),
	"crypto_bit" numeric(4,0),
	"private_key" text,
	"public_key" text,
	"keyid" varchar(256),
	"keyid_sha1_64" varchar(256),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "test_category" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "test_target" (
	"id" numeric(3,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "cert" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"issuer_keypair" numeric(9,0),
	"subject_keypair" numeric(9,0),
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "cert_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);

CREATE TABLE "crl" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"issuer_keypair" numeric(9,0),
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "crl_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);


CREATE TABLE "ocsp" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"issuer_keypair" numeric(9,0),
	"ocsp_cert" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "ocsp_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);


CREATE TABLE "ccp" (
	"id" numeric(9,0) PRIMARY KEY,
	"issued_to" numeric(9,0),
	"issued_by" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "certpath" (
	"id" numeric(9,0) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"cert_id" numeric(9,0),
	"crl_id" numeric(9,0),
	"ocsp_id" numeric(9,0)
);


CREATE TABLE "testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0),
	"level" numeric(2,0),
	"certpath_id" numeric(9,0),
	"exp_value" numeric(2,0),
	"init_policy_set" text,
	"init_policy_mapping_inhibit_flag" boolean,
	"init_explicit_policy_flag" boolean,
	"init_any_policy_inhibit_flag" boolean,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "test_result" (
	"id" numeric(9,0) PRIMARY KEY,
	"category_id" numeric(9,0),
	"target_id" numeric(3,0),
	"start_time" integer,
	"end_time" integer,
	"process_time" integer,
	"num_testcase" numeric(4,0),
	"num_true" numeric(4,0),
	"num_false" numeric(4,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "test_result_detail" (
	"test_result_id" numeric(9,0) NOT NULL,
	"testcase_id" numeric(9,0) NOT NULL,
	"exit_status" numeric(8,0),
	"standard_output" text,
	"result_status" numeric(1,0)
);

CREATE TABLE "ldap_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"dn" varchar(1024),
	"entry" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository_item" (
	"id" numeric(9,0) NOT NULL,
	"entry_id" numeric(9,0) NOT NULL
);

CREATE TABLE "http_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"repository_id" numeric(9,0),
	"type" numeric(3,0),
	"path" text,
	"value" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "http_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "repository_set_item" (
	"id" numeric(9,0) PRIMARY KEY,
	"repository_type" numeric(2,0) NOT NULL,
	"repository_id" numeric(9,0) NOT NULL
);

CREATE TABLE "repository_set" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

-- =======================================================================
--
-- SCHEMA FOR TIME STAMP
--
-- =======================================================================

CREATE TABLE "tsr_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0),
	"level" numeric(2,0),
	"tsq_id" numeric(9,0),
	"http_header_id" numeric(9,0),
	"tsr_id" numeric(9,0),
	"tst_id" numeric(9,0),
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tst_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0),
	"level" numeric(2,0),
	"tst1" numeric(9,0),
	"tst2" numeric(9,0),
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsq" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsq_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);

CREATE TABLE "tsr" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsr_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);

CREATE TABLE "http_header" (
	"id" numeric(9,0) PRIMARY KEY,
	"header" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tst" (
	"id" numeric(9,0),
	"tstinfo_id" numeric(9,0),
	"certpath_id" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tstinfo" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tstinfo_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean
);

