--
-- $Id: schema.1.9.1.6a.sql,v 1.2 2003/10/24 03:12:02 urushima Exp $
--
-- Multi-domain PKI Interoperability Test Suite Database Schema v1.9.1.6a
--
--							Oct 22, 2003
--
--			NPO JNSA http://www.jnsa.org/
--			SECOM Co., Ltd. http://www.secom.co.jp/
--
CREATE TABLE "ts_info" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" text,
	"value" text
);

CREATE TABLE "keypair" (
	"id" numeric(9,0) PRIMARY KEY,
	"crypto_method" numeric(1,0),
	"crypto_bit" numeric(4,0),
	"private_key" text,
	"public_key" text,
	"keyid" varchar(256),
	"keyid_sha1_64" varchar(256),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "test_category" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "test_target" (
	"id" numeric(3,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "cert" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL,
	"issuer_keypair" numeric(9,0) default NULL,
	"subject_keypair" numeric(9,0) default NULL,
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (base_id) references cert(id) on delete set default,
	foreign key (issuer_keypair) references keypair(id) on delete set default,
	foreign key (subject_keypair) references keypair(id) on delete set default
);


CREATE TABLE "cert_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id, name,seq_no),
	foreign key (id) references cert(id) on delete cascade
);

CREATE TABLE "crl" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL,
	"issuer_keypair" numeric(9,0) default NULL,
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (base_id) references crl(id) on delete set default,
	foreign key (issuer_keypair) references keypair(id) on delete set default
);


CREATE TABLE "crl_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no),
	foreign key (id) references crl(id) on delete cascade
);


CREATE TABLE "ocsp" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL,
	"issuer_keypair" numeric(9,0) default NULL,
	"ocsp_cert" numeric(9,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (base_id) references ocsp(id) on delete set default,
	foreign key (issuer_keypair) references keypair(id) on delete set default
);


CREATE TABLE "ocsp_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no),
	foreign key (id) references ocsp(id) on delete cascade
);


CREATE TABLE "ccp" (
	"id" numeric(9,0) PRIMARY KEY,
	"issued_to" numeric(9,0) default NULL,
	"issued_by" numeric(9,0) default NULL,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (issued_to) references cert(id) on delete set default
);


CREATE TABLE "certpath" (
	"id" numeric(9,0) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"cert_id" numeric(9,0) default NULL,
	"crl_id" numeric(9,0) default NULL ,
	"ocsp_id" numeric(9,0) default NULL,
	primary key (id,seq_no),
	foreign key (cert_id) references cert(id) on delete set default,
	foreign key (crl_id) references crl(id) on delete set default, 
	foreign key (ocsp_id) references ocsp(id) on delete set default 
);


CREATE TABLE "testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL,
	"level" numeric(2,0),
	"certpath_id" numeric(9,0) default NULL,
	"exp_value" numeric(2,0),
	"init_policy_set" text,
	"init_policy_mapping_inhibit_flag" boolean,
	"init_explicit_policy_flag" boolean,
	"init_any_policy_inhibit_flag" boolean,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (category_id) references test_category(id) on delete set default
);

CREATE TABLE "test_result" (
	"id" numeric(9,0) PRIMARY KEY,
	"category_id" numeric(9,0) default NULL,
	"target_id" numeric(3,0) default NULL,
	"start_time" integer,
	"end_time" integer,
	"process_time" integer,
	"num_testcase" numeric(4,0),
	"num_true" numeric(4,0),
	"num_false" numeric(4,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (category_id) references test_category(id) on delete set default,
	foreign key (target_id) references test_target(id) on delete set default
);


CREATE TABLE "test_result_detail" (
	"test_result_id" numeric(9,0) NOT NULL,
	"testcase_id" numeric(9,0) NOT NULL,
	"exit_status" numeric(8,0),
	"standard_output" text,
	"result_status" numeric(1,0),
	foreign key (test_result_id ) references test_result(id) on delete cascade,
	foreign key (testcase_id) references testcase(id) on delete cascade,
	primary key (test_result_id,testcase_id)
);

CREATE TABLE "ldap_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"dn" varchar(1024),
	"entry" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository_item" (
	"id" numeric(9,0) NOT NULL,
	"entry_id" numeric(9,0) NOT NULL,
	primary key (id,entry_id),
	foreign key (id) references ldap_repository(id) on delete cascade,
	foreign key (entry_id) references ldap_entry(id) on delete cascade
);

CREATE TABLE "http_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "http_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"repository_id" numeric(9,0),
	"type" numeric(3,0),
	"path" text,
	"value" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (repository_id) references http_repository(id) on delete cascade
);

CREATE TABLE "repository_set" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

-- =======================================================================
--
-- SCHEMA FOR TIME STAMP
--
-- =======================================================================

CREATE TABLE "tsq" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (base_id) references tsq(id) on delete set default
);

CREATE TABLE "tsq_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key(id,name,seq_no),
	foreign key (id) references tsq(id) on delete cascade
);

CREATE TABLE "tsr" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (id) references tsr(id) on delete set default
);

CREATE TABLE "tsr_value" (
	"id" numeric(9,0) NOT NULL,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no),
	foreign key (id) references tsr(id) on delete cascade
);

CREATE TABLE "tsr_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL,
	"level" numeric(2,0),
	"tsq_id" numeric(9,0) default NULL,
	"http_header" text,
	"tsr_id" numeric(9,0) default NULL,
	"tst_id" numeric(9,0) default NULL ,
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (category_id) references test_category(id) on delete set default,
	foreign key (tsq_id) references tsq(id) on delete set default,
	foreign key (tsr_id) references tsr(id) on delete set default
);

CREATE TABLE "tst_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL,
	"level" numeric(2,0),
	"tst1" numeric(9,0) default NULL,
	"tst2" numeric(9,0) default NULL,
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text,
	foreign key (tst1) references tsr(id) on delete set default,
	foreign key (tst2) references tsr(id) on delete set default	
);

INSERT INTO ts_info (id,name,value) VALUES (1,'schema_version','1.916');
