--
-- $Id: schema.1.9.2.2a.sql,v 1.2 2004/05/11 10:56:09 urushima Exp $
--
-- Challenge PKI Test Suite Database Schema v1.9.2.2a
--
--							May 9, 2004
--
--			Copyright 2003, 2004
--			IPA http://www.ipa.go.jp/
--			NPO JNSA http://www.jnsa.org/
--			SECOM Co., Ltd. http://www.secom.co.jp/
--
CREATE TABLE "ts_info" (
	"id" serial PRIMARY KEY,
	"name" text,
	"value" text
);

CREATE TABLE "keypair" (
	"id" numeric(9,0) PRIMARY KEY,
	"crypto_method" numeric(1,0),
	"crypto_bit" numeric(4,0),
	"private_key" text,
	"public_key" text,
	"keyid" varchar(256),
	"keyid_sha1_64" varchar(256),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "test_category" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "test_client" (
	"id" numeric(3,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "cert" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL 
           CONSTRAINT cert_base_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issuer_keypair" numeric(9,0) default NULL
           CONSTRAINT cert_issuer_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"subject_keypair" numeric(9,0) default NULL
           CONSTRAINT cert_subject_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "cert_value" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT cert_value_ref REFERENCES cert(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id, name,seq_no)
);

CREATE TABLE "crl" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL
           CONSTRAINT crl_base_ref REFERENCES crl(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issuer_keypair" numeric(9,0) default NULL
           CONSTRAINT crl_issuer_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"hash_alg" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "crl_value" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT crl_value_ref REFERENCES crl(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no)
);


CREATE TABLE "ocsp" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL
           CONSTRAINT ocsp_base_ref REFERENCES ocsp(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issuer_keypair" numeric(9,0) default NULL
           CONSTRAINT ocsp_issuer_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"ocsp_cert" numeric(9,0) default NULL
           CONSTRAINT ocsp_ocsp_cert_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "ocsp_value" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT ocsp_value_ref REFERENCES ocsp(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no)
);


CREATE TABLE "rawcert" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL
           CONSTRAINT rawcert_base_ref REFERENCES rawcert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issuer_keypair" numeric(9,0) default NULL
           CONSTRAINT rawcert_issuer_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"subject_keypair" numeric(9,0) default NULL
           CONSTRAINT rawcert_subject_keypair_ref REFERENCES keypair(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"pem" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "ccp" (
	"id" numeric(9,0) PRIMARY KEY,
	"issued_to" numeric(9,0) default NULL
           CONSTRAINT issuer_to_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issued_by" numeric(9,0) default NULL
           CONSTRAINT issuer_by_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issued_to_raw" numeric(9,0) default NULL
           CONSTRAINT issuer_to_raw_ref REFERENCES rawcert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"issued_by_raw" numeric(9,0) default NULL
           CONSTRAINT issuer_by_raw_ref REFERENCES rawcert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "certpath" (
	"id" numeric(9,0) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"cert_id" numeric(9,0) default NULL
           CONSTRAINT certpath_cert_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"crl_id" numeric(9,0) default NULL 
           CONSTRAINT certpath_crl_ref REFERENCES crl(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"ocsp_id" numeric(9,0) default NULL
           CONSTRAINT certpath_ocsp_ref REFERENCES ocsp(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	primary key (id,seq_no)
);


CREATE TABLE "testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL
           CONSTRAINT testcase_category_ref REFERENCES test_category(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"level" numeric(2,0),
	"certpath_id" numeric(9,0) default NULL,
	"serverset_id" numeric(9,0) default NULL,
	"ta_cert_id" numeric(9,0) default NULL
           CONSTRAINT testcase_ta_cert_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"ee_cert_id" numeric(9,0) default NULL
           CONSTRAINT testcase_ee_cert_ref REFERENCES cert(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"exp_value" numeric(2,0),
	"init_policy_set" text,
	"init_policy_mapping_inhibit_flag" boolean,
	"init_explicit_policy_flag" boolean,
	"init_any_policy_inhibit_flag" boolean,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "test_result" (
	"id" numeric(9,0) PRIMARY KEY,
	"category_id" numeric(9,0) default NULL,
	"client_id" numeric(3,0) default NULL, 
	"start_time" integer,
	"end_time" integer,
	"process_time" integer,
	"num_testcase" numeric(4,0),
	"num_true" numeric(4,0),
	"num_false" numeric(4,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);


CREATE TABLE "test_result_detail" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT result_detail_ref REFERENCES test_result(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"testcase_id" numeric(9,0),
	"exit_status" numeric(8,0),
	"standard_output" text,
	"result_status" numeric(1,0),
	primary key (id,testcase_id)
);

CREATE TABLE "repository_set" (
	"id" numeric(9,0) PRIMARY KEY,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"dn" varchar(1024),
	"entry" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "ldap_repository_item" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT ldap_repository_item_ref REFERENCES ldap_repository(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"entry_id" numeric(9,0) NOT NULL
           CONSTRAINT ldap_repository_entry_ref REFERENCES ldap_entry(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	primary key (id,entry_id)
);

CREATE TABLE "http_repository" (
	"id" numeric(9,0) PRIMARY KEY,
	"host" varchar(512),
	"port" numeric(5,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "http_entry" (
	"id" numeric(9,0) PRIMARY KEY,
	"repository_id" numeric(9,0)
           CONSTRAINT http_entry_repositry_ref REFERENCES http_repository(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"type" numeric(3,0),
	"path" text,
	"value" text,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "repository_set_ldap_item" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT repository_set_ldap_item_repository_set_ref REFERENCES repository_set(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"repository_id" numeric(9,0) NOT NULL
           CONSTRAINT repositroy_set_ldap_item_ldap_repositry_ref REFERENCES ldap_repository(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	primary key (id, repository_id)
);

CREATE TABLE "repository_set_http_item" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT repositroy_set_http_item_repositry_set_ref REFERENCES repository_set(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"repository_id" numeric(9,0) NOT NULL
           CONSTRAINT repositroy_set_http_item_http_repositry_ref REFERENCES http_repository(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	primary key (id, repository_id)
);

-- ==================================================================
--
-- SCHEMA FOR TIME STAMP
--
-- ==================================================================

CREATE TABLE "tsq" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL
           CONSTRAINT tsq_base_ref REFERENCES tsq(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsq_value" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT tsq_value_tsq_ref REFERENCES tsq(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key(id,name,seq_no)
);

CREATE TABLE "tsr" (
	"id" numeric(9,0) PRIMARY KEY,
	"base_id" numeric(9,0) default NULL
           CONSTRAINT tsr_base_ref REFERENCES tsr(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsr_value" (
	"id" numeric(9,0) NOT NULL
           CONSTRAINT tsr_value_tsr_ref REFERENCES tsr(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
	"name" varchar(128) NOT NULL,
	"seq_no" numeric(2,0) NOT NULL,
	"value" text,
	"is_critical" boolean,
	"is_ext" boolean,
	primary key (id,name,seq_no)
);

CREATE TABLE "tsp_protocol_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL
           CONSTRAINT tsp_protocol_testcase_category_ref REFERENCES test_category(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"level" numeric(2,0),
	"tsq_id" numeric(9,0) default NULL
           CONSTRAINT tsp_protocol_testcase_tsq_ref REFERENCES tsq(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"tsr_id" numeric(9,0) default NULL
           CONSTRAINT tsp_protocol_testcase_tsr_ref REFERENCES tsr(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"http_header" text,
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

CREATE TABLE "tsp_accuracy_testcase" (
	"id" numeric(9,0) PRIMARY KEY,
	"testcase_code" varchar(256),
	"category_id" numeric(9,0) default NULL,
	"level" numeric(2,0),
	"tsr_id1" numeric(9,0) default NULL
           CONSTRAINT tsp_accuracy_testcase_tsr1_ref REFERENCES tsr(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"tsr_id2" numeric(9,0) default NULL
           CONSTRAINT tsp_accuracy_testcase_tsr2_ref REFERENCES tsr(id) ON DELETE SET DEFAULT DEFERRABLE INITIALLY DEFERRED,
	"exp_value" numeric(2,0),
	"name" varchar(128),
	"keyword" varchar(256),
	"note" text
);

