# $Id: cpki_ldif.pl,v 1.1 2004/06/10 05:16:21 urushima Exp $
#===================================================================
#
# Challenge PKI Test Suite 2.0
#
#							Mar 6, 2004
#
#		Copyright (c) 2003, 2004
#		Challenge PKI Project
#			IPA http://www.ipa.go.jp/
#			NPO JNSA http://www.jnsa.org/
#			SECOM Co.,Ltd. http://www.secom.co.jp/
#			All Rights Reserved.
#
#===================================================================

#
# Naming Convensions:
#   u:userCertificate
#   c:caCertificate
#   a:AuthorityRevocationList
#   r:CertificateRevocationList
#   p:CrossCertificatePair
#

#
# DnEncode
#
sub DnEncode {
    my ($dn) = @_;

    $dn =~ s/([^0-9A-Za-z.-_@])/sprintf("%%%02X",ord($1))/eg;
    $dn =~ s/\\/%5c/g;
    return $dn;
}

#
# DnReverse
#
sub DnReverse {
    my ($dn) = @_;
    my (@rdns,@rev_rdns);

    @rev_rdns = ();
    while ($dn =~ s/,\s*(\w+=[^=]+)$//) {
	push @rev_rdns, $1;
    }
    push @rev_rdns, $dn;
    return join(",",@rev_rdns);
}

sub DnReverseBug___ {
    my ($dn) = @_;
    my (@rdns,@rev_rdns);

    $dn =~ s/^\s+//;
    $dn =~ s/\s+$//;

    @rdns = split(/\s*,\s*/,$dn);
    @rev_rdns = ();
    while ($#rdns > -1) {
	push @rev_rdns, pop @rdns;
    }
    return join(",",@rev_rdns);
}

#
# SavePemFileByBase64
#
sub SavePemFileByBase64 {
    local($file, $label, $base64) = @_;
    my $col = 0;
    my $len = length($base64);

    open(FO, "> $file") || die "can't write to '$file'";
    print FO "-----BEGIN $label-----\n";

    do {
	print FO substr($base64, $col, 64), "\n";
	$col += 64;
    } until ($col >= $len);

    print FO "-----END $label-----\n";
    close FO;
}

#
# LdifConvertOneLine
#
#	read LDIF contents as a file stream, convert multi line
#	attribute into one line, then write to output stream.
#
sub LdifConvertOneLine {
    local($fin, $fout) = @_;
    my $prev_line = "";

    while (<$fin>) {
	s/\r?\n$//;
	if (/^ (.*)$/) {$prev_line .= $1; next;}
	print $fout $prev_line, "\n";
	$prev_line = $_;
    }
    print $fout $prev_line, "\n";
}

sub printLdifWrapString {
    local($FO, $s) = @_;
    $s = &_printCol($FO, $s, 77);
    print $FO $s, "\n" if $s ne "";
}

sub printLdifWrapBase64File {
    local($FO, $attrTag, $derFile) = @_;
    open(FOPENSSL, "openssl base64 -e -in $derFile |") || die;
    my $buf = $attrTag;
    my $firsttime = 1;
    while (<FOPENSSL>) {
	chop;
	$buf = &_printCol($FO, $buf . $_, 77);
    }
    print $FO $buf, "\n" if $buf ne "";
    close FOPENSSL;
}

sub _printCol {
    local($FO, $s, $ncol) = @_;
    while (($s =~ s/^(.{77})/ /)) {
	print $FO $1, "\n"; 
    }
    return $s;
}

1;
