$gencfgX509ExtProc = ();

$gencfgX509ExtProc->{keyUsage} = sub {
    local($h, $FOUT) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
   
    print $FOUT "KeyUsage=$critical:$info->{value}\n";
};

$gencfgX509ExtProc->{certificatePolicies} = sub {
    local($h, $FOUT) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    print $FOUT "[certificatePolicies]\n";
    print $FOUT "policyIdentifier=$critical:$info->{value}\n";
    print $FOUT "[certificatePolicies end]\n";
};

$gencfgX509ExtProc->{policyMappings} = sub {
    local($h, $FOUT) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    print $FOUT "PolicyMappings=$critical:$info->{value}\n";
};

$gencfgX509ExtProc->{basicConstraints} = sub {
    local($h, $FOUT) = @_;

    print $FOUT "[BasicConstraints]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[BasicConstraints end]\n";
};

$gencfgX509ExtProc->{authorityKeyIdentifier} = sub {
    local($h) = @_;

    print $FOUT "[AuthorityKeyIdentifier]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[AuthorityKeyIdentifier end]\n";
};

$gencfgX509ExtProc->{subjectKeyIdentifier} = sub {
    local($h) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    print $FOUT "SubjectKeyIdentifier=$critical:$info->{value}\n";
};

$gencfgX509ExtProc->{cRLDistributionPoint} = sub {
    local($h) = @_;

    print $FOUT "[CRLDistributionPoints]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[CRLDistributionPoints end]\n";
};

$gencfgX509ExtProc->{oCSPNoCheck} = sub {
    local($h) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    print $FOUT "ocsp-nocheck=$critical:TRUE\n";
};

$gencfgX509ExtProc->{privateExtension} = sub {
    local($h) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    $info->{value} =~ s/^\s+//;
    $info->{value} =~ s/\s+$//;
    for my $keyvalue (split(/\s+/, $info->{value})) {
	if ($keyvalue =~ /^[0-9.]+=[cn]:[0-9A-Fa-f]+$/) {
	    print $FOUT $keyvalue, "\n";
	} else {
	    print $FOUT "# SKIP - malformed privateExtension $keyvalue\n";
	}
    }
};


$gencfgX509ExtProc->{cRLNumber} = sub {
    local($h) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";
    
    print $FOUT "CRLNumber=$critical:$info->{value}\n";
};

$gencfgX509ExtProc->{issuingDistributionPoint} = sub {
    local($h) = @_;

    print $FOUT "[issuingDistributionPoint]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[issuingDistributionPoint end]\n";
};

# DATABASE ENTRY
#  EauthorityInformationAccess
#  caIssuers=http://pkiiwg.chttl.com.tw/repository/CertIssuedToThisCA.p7b
# CONFIGURATION
#  [authorityInfoAccess]
#  caIssuers=n:http://pkiiwg.chttl.com.tw/repository/CertIssuedToThisCA.p7b
#  [authorityInfoAccess end]

$gencfgX509ExtProc->{authorityInformationAccess} = sub {
    local($h, $FOUT) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";

    my ($type, $value) = ($info->{value} =~ /^([^=]+)=(.+)$/);
    
    print $FOUT "[authorityInfoAccess]\n";
    print $FOUT "$type=$critical:$value\n";
    print $FOUT "[authorityInfoAccess end]\n";
};

# DATABASE ENTRY
#  EextendedKeyUsage
#  1.3.6.1.5.5.7.3.9
# CONFIGURATION
#  extKeyUsage=n:1.3.6.1.5.5.7.3.9

$gencfgX509ExtProc->{extendedKeyUsage} = sub {
    local($h, $FOUT) = @_;

    my ($seqno) = keys(%{$h});
    my $info = $h->{$seqno};
    my $critical = ($info->{critical} == 1) ? "c" : "n";

    my ($type, $value) = ($info->{value} =~ /^([^=]+)=(.+)$/);
    
    print $FOUT "extKeyUsage=$critical:$info->{value}\n";
};

# DATABASE ENTRY
#  EsubjectAltName
#  directoryName=UTF8:CN=TestCA, OU=BB, O=PPTG, C=JP
# CONFIGURATION
#  [SubjectAltName]
#  directoryName=n:UTF8:CN=TestCA, OU=BB, O=PPTG, C=JP
#  [SubjectAltName end]

$gencfgX509ExtProc->{subjectAltName} = sub {
    local($h) = @_;

    print $FOUT "[SubjectAltName]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[SubjectAltName end]\n";
};

# DATABASE ENTRY
#  EissuerAltName
#  directoryName=UTF8:CN=TestCA, OU=BB, O=PPTG, C=JP
# CONFIGURATION
#  [IssuerAltName]
#  directoryName=n:UTF8:CN=TestCA, OU=BB, O=PPTG, C=JP
#  [IssuerAltName end]

$gencfgX509ExtProc->{issuerAltName} = sub {
    local($h) = @_;

    print $FOUT "[IssuerAltName]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[IssuerAltName end]\n";
};

# DATABASE ENTRY
#  EpolicyConstraints
#  requireExplicitPolicy=0
#  EpolicyConstraints
#  inhibitPolicyMapping=0
# CONFIGURATION
#  [PolicyConstraints]
#  requireExplicitPolicy=c:1
#  [PolicyConstraints end]

$gencfgX509ExtProc->{policyConstraints} = sub {
    local($h) = @_;

    print $FOUT "[PolicyConstraints]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[PolicyConstraints end]\n";
};

# DATABASE ENTRY
#  EnameConstraints
#  permittedSubtrees=UTF8:OU=Int.CC.RP.37.01_YY, O=PPTG, C=JP
#  EnameConstraints
#  excludedSubtrees=UTF8:OU=AnotherUnit, OU=Int.CC.RP.37.01_YY, O=PPTG, C=JP
# CONFIGURATION
#  [NameConstraints]
#  permittedSubtrees=c:Printable:OU=Int.CC.RP.35.01_PKI-J, O=PPTG, C=JP
#  [NameConstraints end]

$gencfgX509ExtProc->{nameConstraints} = sub {
    local($h) = @_;

    print $FOUT "[NameConstraints]\n";

    for $seqno (sort {$a <=> $b} keys(%{$h})) {
	my $info = $h->{$seqno};
	my $critical = ($info->{critical} == 1) ? "c" : "n";

	if ($info->{value} =~ /^([^=]+)=(.+)$/) {
	    print $FOUT "$1=$critical:$2\n";
	    next;
	}
    }

    print $FOUT "[NameConstraints end]\n";
};

1;

