/* crlconf.c */
/*
 * Copyright (C) 2002 IPA (?)
 *
 * All rights reserved.
 *
 */
#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_pem.h"
#include "ok_err.h"
#include "crlgen.h"

int CRLG_get_extReasonCode(char *buf, Revoked *rv);

/*--------------------------------------------
  get information from conf file
--------------------------------------------*/
CRL *CRLG_open_conf(char *file){
	char *buf=NULL;
	CRL *ret=NULL;
	int err=-1,len;
  
	if((buf=(char*)get_file2buf(file,&len))==NULL){
	  printf("cannot open CRL config file : %s\n",(file)?(file):("<null>"));
	  goto done;
	}
	
	if((ret=CRL_new())==NULL) goto done;
	if(CRLG_read_common(buf,ret)) goto done;
	if(CRLG_read_ext(buf,ret)) goto done;
	err=0;
done:
	if(buf) AIFREE(buf);
	if(err){CRL_free(ret); ret=NULL;}
	return ret;
}

/*--------------------------------------------
  read common settings
--------------------------------------------*/
int CRLG_read_common(char *org, CRL *crl){
	char *buf=NULL,*hd,*ed;
	int ok=-1,len;
  
	/* find "common" section */
	if((hd=strstr(org,"[CertificateList Fields]"))==NULL){
		printf("cannot find [CertificateList Fields] tag.\n");
		goto done;
	}
	if((ed=strstr(hd,"[CertificateList Fields end]"))==NULL){
		printf("cannot find [CertificateList Fields end] tag.\n");
		goto done;
	}
	*ed=0; hd+=sizeof("[CertificateList Fields]")-1;
	len=strlen(hd);
	/* allocate & copy buffer */
	if((buf=(char*)AIMALLOC(len+2))==NULL){
		printf("cannot allocate memory!\n");
		goto done;
	}
	memcpy(buf,hd,len+2);
	*ed='[';

	if(CRLG_com_syntax(buf,crl)) goto done;
  
	ok=0;
done:
	if(buf) AIFREE(buf);
	return ok;
}

int CRLG_com_syntax(char *buf, CRL *crl){
        char *ep,*cp,*dp; /* dp:my add */
	int line=0,ok=-1;

	crl->version=1;
	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */

		cp=buf;
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}
		
		while(*cp){
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case '\t':
			case ' ': /* ignore these chars */
				cp++; continue;

			case 'v': /* version */
			        /* version is fixed to 2(1) */
				break;

			case 'i': /* issuer */
				if((crl->issuer=CRLG_get_dn_base(cp,"issuer=",&crl->issuer_dn))==NULL){
					printf("CRLG_com_syntax: bad issuer dn (%d)\n",line);
					goto done;
				}
				break;

			case 't': /* thisUpdate */
				if(CRLG_get_time(cp,"thisUpdate=",&crl->lastUpdate)){
					printf("CRLG_com_syntax: bad thisUpdate (%d)\n",line);
					goto done;
				}
				break;
	
			case 'n': /* nextUpdate */
				if(CRLG_get_time(cp,"nextUpdate=",&crl->nextUpdate)){
					printf("CRLG_com_syntax: bad nextUpdate (%d)\n",line);
					goto done;
				}
				break;
	
			case '[': /* revokedCertificate */
				if((cp=strstr(cp,"[revokedCertificates]"))==NULL){
					printf("CRLG_com_syntax: bad [revokedCertificates] tag (%d)\n",line);
					goto done;
				}
				cp += sizeof("[revokedCertificates]");

				if((ep=strstr(cp,"[revokedCertificates end]"))==NULL){
					printf("CRLG_com_syntax: bad [revokedCertificates end] tag (%d)\n",line);
					goto done;
				}
				*ep = 0;
				ep += sizeof("[revokedCertificates end]")-1;
				if(CRLG_rvk_syntax(cp, crl)) goto done;
				break;	

			default:
				printf("CRLG_com_syntax: syntax error (%d)\n>>%s\n",line,buf);
				goto done;
			}
			break;
		}
		buf=ep; line++;
		if(dp){
		  *dp='#';
		}
	}
	ok = 0;

done:
	return ok;
}

/*
 * CRLG_rvk_syntax(buf, crl)
 *
 *	generate CRL object from 'buf' and store to 'crl'.
 *	'buf' may have following syntax.
 *
 *	dec:1=UTC:1
 *	dec:2=UTC:2	reasonCode:20
 *
 *	You can specify 'reasonCode' as option.
 */
int CRLG_rvk_syntax(char *buf, CRL *crl){
	Revoked *hd=NULL,*rv=NULL;
	char *ep,*cp,*sp,*dp;
	char *extensionArgument1;
	int line=0,ok=-1;
	int snlen;
	int numBytesSN = 0; /* num bytes of long userCertificate SN */

	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */

		cp=buf;
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}
		
		while(*cp){
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case ' ':
			case '\t': /* ignore these chars */
				cp++; continue;

			default:
				/* find '=' then decode line */
				if((sp=strchr(cp,'='))==NULL){
					printf("CRLG_rev_syntax: cannot find '=' separator (%d)\n",line);
					goto done;
				}
				*sp=0; sp++; /* sp = at the beginning of time */

				/* find a '\t' for extension */
				extensionArgument1 = strchr(sp, '\t');
				if (extensionArgument1 != NULL) {
				  *extensionArgument1 = '\0';
				  extensionArgument1++;
				}

				/* create revoked list */
				if((rv=Revoked_new())==NULL) goto done;

				/*
				 * at this time cp is 'hex:aa...' or 'dec:1...'
				 */
				if((snlen=strlen(cp))<=11){
				  if((rv->serialNumber=CRLG_line2int_base(cp,""))<0){
				    printf("CRLG_rev_syntax: bad serial number (%d)\n",line);
				    goto done;
				  }
				}else if(snlen<=53){
				  /* printf("snlen:%d\n", snlen); */
				  numBytesSN = (int) (snlen - 4) / 2.0;
				  if((rv->long_sn=(unsigned char *)calloc(32,sizeof(char)))==NULL)
				    goto done;
				  /* snlen overwriting X-< */
				  if((snlen=CRLG_line2bin_base(cp,"",&rv->long_sn[2],numBytesSN))<0){
				    printf("CRLG_rev_syntax: bad serial number (%d)\n",line);
				    goto done;
				  }
				  rv->long_sn[0]=ASN1_INTEGER;
				  rv->long_sn[1]=snlen;
				  /* printf("last snlen:%d\n", numBytesSN); */
				}else{
				    printf("CRLG_rev_syntax: bad serial number (%d)\n",line); 
				}

				if(CRLG_get_time(sp,"",&rv->revocationDate)){
					printf("CRLG_com_syntax: bad revocationDate (%d)\n",line);
					goto done;
				}

				if (extensionArgument1 != NULL) {
				  if (CRLG_get_extReasonCode(extensionArgument1,rv)) {
				    printf("CRLG_rev_syntax: bad extension (%d)\n", line);
				    goto done;
				  }
				}

				/* add to CRL */
				if(hd){
					hd->next=rv; hd=rv;
				}else{
					crl->next=hd=rv;
				}
				break;
			}
			break;
		}
		buf=ep; line++;
	}

	ok = 0;
done:
	if(ok) Revoked_free(rv);
	return ok;
}

/*--------------------------------------------
  configure file analyzing tools
--------------------------------------------*/
int CRLG_line2int(char *buf, char *tag,int type,int *critical){
	char *hd;
	int ret=-1;
	int tmp=UNSET;
  
	if((hd=strstr(buf,tag))==NULL) goto done;
	hd+=strlen(tag);
	
	if(type==EXTENSION){
	  if((hd=(char *)get_critical(hd,&tmp))==NULL)
	    goto done;
	  if(*critical!=CRITICAL||*critical!=NON_CRITICAL){
	    *critical=tmp;
	  }
	  hd+=2;
	}

	while(*hd){
		switch(*hd){
		case ' ':
		case '\t':
			hd++; continue;
		case 'h': /* hex string */
			if(hd=strstr(hd,"hex:")){
				ret = h_atoi(hd+4);
			}
			break;
		default:
			if((*hd>='0')&&(*hd<='9')){
				ret = atoi(hd);
			}else if(hd=strstr(hd,"dec:")){
				ret = atoi(hd+4);
			}
			break;
		}
		break;
	}
done:
	return ret;
}

int h_atoi(char *asc){
	int ret=0;

	while(*asc){
		if((*asc>='0')&&(*asc<='9')){
			ret = (ret<<4) | (*asc - '0');
		}else if((*asc>='a')&&(*asc<='f')){
			ret = (ret<<4) | (*asc - 'a' + 0x0a);
		}else if((*asc>='A')&&(*asc<='F')){
			ret = (ret<<4) | (*asc - 'A' + 0x0a);
		}else{
			break;
		}
		asc++;
	}
	return ret;
}

int CRLG_line2bin(char *buf, char *tag, unsigned char *ret, int ret_max,int type, int *critical){
	char *hd,*t;
	int i,j,len=-1;
	int blen;
	unsigned char *der;
	char *hdd;
	int cr;
  
	if((hd=strstr(buf,tag))==NULL) goto done;
	hd+=strlen(tag);

	while(*hd){
	        switch(*hd){
		case ' ':
		case '\t':
			hd++; continue;
		case 'n': /*set critical*/
		case 'c':
		        if(strstr(hd,"c:")||strstr(hd,"n:")){
			  if(type==EXTENSION){
			    if((hd=get_critical(hd,&cr))==NULL) goto done;
			    if(*critical!=CRITICAL&&*critical!=NON_CRITICAL){
			      *critical=cr;
			    }
			    hd+=2; continue;
			  }else{
			    goto done;
			  }
			}else{ /* for keyID(start c) */
			  len=h_atobin(hd,ret,ret_max);
			}
		case 'd': /* dec string */
		        hdd=hd;
			if(hd=strstr(hd,"dec:")){
			  if((der=(unsigned char *)str2binary(hd+4,&len))==NULL)
			    goto done;

			  if(len<=ret_max){
			    memcpy(ret,der,len);
			  }else{
			    len=-1;
			    goto done;
			  }
			}else{ /* for keyID(start d) */
			  len=h_atobin(hdd,ret,ret_max);
			}
			break;
		case 'h': /* hex string */
		default:
			if(t=strstr(hd,"hex:")){
				len = h_atobin(t+4,ret,ret_max);
			}else{
				len = h_atobin(hd,ret,ret_max);
			}
			break;
		}
		if(type==EXTENSION&&*critical==UNSET){
		  printf("err in set bin critical\n");
		  len=-1;
		  goto done;
		}


		break;
	}
done:
	return len;
}

int h_atobin(char *asc,unsigned char *bin,int max){
	int i=0,j=0,num=0;

	while((*asc)&&(j<max)){
		if((*asc>='0')&&(*asc<='9')){
			num = (num<<4) | (*asc - '0');
		}else if((*asc>='a')&&(*asc<='f')){
			num = (num<<4) | (*asc - 'a' + 0x0a);
		}else if((*asc>='A')&&(*asc<='F')){
			num = (num<<4) | (*asc - 'A' + 0x0a);
		}else{
			break;
		}
		if(i&0x1) bin[j] = num & 0xff;
		asc++; i++; j=i>>1; /* j=i/2 */
	}
	return j;
}

char *CRLG_get_dn(char *buf, char *tag, CertDN *dn,int type,int *critical){
	char *hd,*sp,*ed,*ret=NULL,tmp[256];
	int num=0,i,dq,lp,dform=ASN1_PRINTABLE_STRING;
	char *eq,*co;

	if((hd=strstr(buf,tag))==NULL) goto done;
	hd+=strlen(tag);

	/* get DER Format */
	while(*hd){
		switch(*hd){
		case ' ':
		case '\t':
			hd++; continue;
		case 'P': /* Printable */
			if((hd=strstr(hd,"Printable:"))==NULL) goto done;
			hd += sizeof("Printable:")-1;
			break;
		case 'U': /* UTF8 */
			if((hd=strstr(hd,"UTF8:"))==NULL) goto done;
			dform=ASN1_UTF8STRING;
			hd += sizeof("UTF8:")-1;
			break;
		case 'c':/* set critical */
		case 'n':
		        if(type==EXTENSION){
			  if((hd=get_critical(hd,critical))==NULL) goto done;
			  hd+=2;
			  continue;
			}else{
			  goto done;
			}
		}
		break;
	}

	if(type==EXTENSION&&*critical==UNSET){
	  printf("err in set dn cricital\n");
	  goto done;
	}


	/* get DN string */
	while(*hd &&(num<RDN_MAX)){
		/* find DN attribute */
		while(*hd==' ') hd++; /* skip space char */

		if((sp=strchr(hd,'='))==NULL) goto done;
		*sp=0; sp++; ed=sp;

		dq = 0; /* clear double quote flag */
		lp = 1; /* set loop flag */
		i = 0;
		memset(tmp,0,256);

		/* get attribute context */
		while(*ed && lp && (i<250)){
			switch(*ed){
			case '"':
				if(dq){
					dq=0; *ed=0; /* set end pointer */
				}else{
					dq=1; sp=ed; /* reset start pointer */
				}
				break;
			case '\\':
				ed++; /* escape one charactor */
				tmp[i]=*ed; i++;
				break;
			case ',':
				if(!dq){
				  eq=strchr_nesc(ed+1,'=');
				  co=strchr_nesc(ed+1,',');
				  if(eq==NULL||(co&&eq&&(eq-co>0))){
				    tmp[i]=*ed; i++;
				  }else{ /* clear loop flag */
				    lp=0; *ed=0;
				  }
				}
				else{ tmp[i]=*ed; i++; }
				break;
			case '\r':
				break; /* Windows text file has this... */
			default:
				tmp[i]=*ed; i++; 
				break;
			}
			ed++;
		}

		/* stack RDNs */
		if((dn->rdn[num].tagoid=i=attchar2int(hd))<0) goto done;
		if((STRDUP(dn->rdn[num].tag,tmp))==NULL) goto done;
		dn->rdn[num].derform = 
			(i==OBJ_DIR_C)?(ASN1_PRINTABLE_STRING):(
			(i==OBJ_DIR_EMAIL)?(ASN1_IA5STRING):(dform));

		hd = ed;
		num++;
	}

	dn->num = num;

	/* swap DN */
	if(dn->rdn[0].tagoid > dn->rdn[num-1].tagoid)
		if(swap_dn(dn))
			goto done;

	ret = Cert_subject_str(dn);

done:
	return ret;
}

int attchar2int(char *att){
	int ret=-1;

	switch(*att){
	case 'C':
	case 'c':
		if((att[1]=='N')||(att[1]=='n'))
			ret = OBJ_DIR_CN;
		else if(att[1]==0)
			ret = OBJ_DIR_C;
		break;
	case 'L':
	case 'l':
		ret = OBJ_DIR_L;
		break;
	case 'O':
	case 'o':
		if((att[1]=='U')||(att[1]=='u'))
			ret = OBJ_DIR_OU;
		else if(att[1]==0)
			ret = OBJ_DIR_O;
		break;
	case 'S':
	case 's':
		if((att[1]=='T')||(att[1]=='t'))
			ret = OBJ_DIR_ST;
		break;
	case 'E':
	case 'e':
		if((att[1]=='M')||(att[1]=='m'))
			ret = OBJ_DIR_EMAIL;
		break;
	}
	return ret;
}

int swap_dn(CertDN *dn){
	char *tc;
	int i,j,tk,sk,max = dn->num >> 1;

	for(i=0;i<max;i++){
		j = dn->num-i-1;
		tk = dn->rdn[i].tagoid;
		dn->rdn[i].tagoid = dn->rdn[j].tagoid;
		dn->rdn[j].tagoid = tk;

		sk = dn->rdn[i].derform;
		dn->rdn[i].derform = dn->rdn[j].derform;
		dn->rdn[j].derform = sk;

		tc = dn->rdn[i].tag;
		dn->rdn[i].tag = dn->rdn[j].tag;
		dn->rdn[j].tag = tc;
	}
	return 0;
}

int CRLG_get_time(char *buf, char *tag, struct tm *time){
	unsigned char der[32];
	char *hd;
	int ret=-1;
  
	if((buf=strstr(buf,tag))==NULL) goto done;
	buf+=strlen(tag);
	memset(der,0,32);

	/* get ASN1 */
	if(hd=strstr(buf,"UTC:")){
		der[0]=ASN1_UTCTIME; der[1]=12;
		memcpy(&der[2],hd+4,12);
	}else if(hd=strstr(buf,"Generalized:")){
		der[0]=ASN1_GENERALIZEDTIME; der[1]=14;
		memcpy(&der[2],hd+12,14);
	}else{
		goto done;
	}
	ret = CRLG_UTC2stm(der,time);
done:
	return ret;
}


int CRLG_UTC2stm(unsigned char *utc, struct tm *ctm){
	unsigned char buf[16];
	int i;
	char tmp[16]="";

	memset(ctm,0,sizeof(struct tm));
	memset(buf,0,16);
	if(*utc==ASN1_GENERALIZEDTIME){
		buf[0]=utc[2];buf[1]=utc[3];
		buf[2]=utc[4];buf[3]=utc[5]; i=6;
		ctm->tm_year = atoi(buf) - 1900;
	}else if(*utc==ASN1_UTCTIME){
		buf[0]=utc[2];buf[1]=utc[3]; i=4;
		ctm->tm_year = atoi(buf);
		if(ctm->tm_year<50) ctm->tm_year+=100;	/* 2000 year problem */
	}else{
		OK_set_error(ERR_ST_BADFORMAT,ERR_LC_X509,ERR_PT_X509TIME,NULL);
		return -1;
	}

	buf[2]=0;
	memcpy(buf,&utc[i],2);
	ctm->tm_mon = (atoi(buf)-1) % 12;

	memcpy(buf,&utc[i+2],2);
	ctm->tm_mday = atoi(buf) % 32;

	memcpy(buf,&utc[i+4],2);
	ctm->tm_hour = atoi(buf) % 24;

	memcpy(buf,&utc[i+6],2);
	ctm->tm_min = atoi(buf) % 60;
  
	if((utc[i+8]=='Z')||(utc[i+9]=='+')||(utc[i+10]=='-')){
		ctm->tm_sec = 0;
	}else{
		memcpy(buf,&utc[i+8],2);
		ctm->tm_sec = atoi(buf) % 60;
	}

	if(utc[0]==ASN1_UTCTIME){
	  ctm->tm_wday=0;
	}else if(utc[0]==ASN1_GENERALIZEDTIME){
	  ctm->tm_wday=1;
	}

	return 0;
}

char *strchr_nesc(char *in,char target){
  char *now,*ret=NULL;

  for(now=in;now;now=strchr(now+1,target)){
    if(now>in){
      if((*(now-1))=='\\'){
      }else{
	ret=now;
	break;
      }
    }
  }
  return ret;
}


/* print tool */
void write_hex(unsigned char *in,int size,FILE *out){
  int i;
  if(in){
    for(i=0;i < size;i++){
      fprintf(out,"%02X ",in[i]);
    }
    printf("\n");
  }
}

int write_binary_file(char * filename,char *ret,int size){
  FILE *fp;
  if((fp=fopen(filename,"wb"))==NULL){
    fprintf(stderr,"can't open %s\n",filename);
    return(-1);
  }else{
    fwrite(ret,sizeof(char),size,fp);
    fclose(fp);
    return(0);
  }
}

void print_dn_all(CertDN *dn){
  int i;
  
  printf("========= dn value =======\n");
  printf("num = %d\n",dn->num);
  for(i=0;i<dn->num;i++){
    switch(dn->rdn[i].derform){
    case ASN1_UTF8STRING:
      printf("UTF8_STRING      :");
      break;
    case ASN1_PRINTABLE_STRING:
      printf("PRINTABLE_STRING :");
      break;
    default:
      printf("type=%d:other string\n",dn->rdn[i].derform);
    }
    switch(dn->rdn[i].tagoid){
    case OBJ_DIR_C:  printf("C  "); break;
    case OBJ_DIR_ST: printf("ST "); break;
    case OBJ_DIR_L:  printf("L  "); break;
    case OBJ_DIR_O:  printf("O  "); break;
    case OBJ_DIR_OU: printf("OU "); break;
    case OBJ_DIR_CN: printf("CN "); break;
    case OBJ_DIR_EMAIL: printf("EMAIL "); break;
    }
    printf(":%s\n",dn->rdn[i].tag);
  }
  printf("========= dn value =======\n");
}

/*
 * CRLG_get_extReasonCode
 *
 *	append 'reasonCode' CRL entry extension to 'rv'.
 *	The argument 'buf' may have format like 'reasonCode:NUM'.
 */
int CRLG_get_extReasonCode(char *buf, Revoked *rv) {
  int err = 0;
  int code = -1;
  char *valueCp;
  CertExt *extReasonCode;
  CertExt *ep;

  if (! strstr(buf, "reasonCode:")) {
    goto done;
  }

  valueCp = buf + 11;
  code = atoi(valueCp);

  /* CertExt *Extnew_reason_code(int code) */
  if ((extReasonCode = Extnew_reason_code(code)) == NULL) {
    err = 1;
    goto done;
  }

  if (rv->entExt == NULL) {
    rv->entExt = extReasonCode;
  } else {
    ep = rv->entExt;
    while (ep->next != NULL) ep = ep->next;
    ep->next = extReasonCode;
  }

 done:
  if (err == 1) printf("CRL entExt reasonCodeObj generating error\n");
  return err;
}
