/* crlder.c */
/*
 * Copyright (C) 2002 IPA (?)
 *
 * All rights reserved.
 *
 */
#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_tool.h"
#include "ok_uconv.h"
#include "ok_err.h"

#include "crlgen.h"

int asn1_time_tag;

/*-----------------------------------------
  Get CRL DER from CRL
-----------------------------------------*/
unsigned char *CRLG_CRL_toDER(CRL *crl,Key *prv,unsigned char *buf,int *ret_len){
	unsigned char *cp,*ret;
	int	i,j;

	if(buf==NULL){
		if((i=CRL_estimate_der_size(crl))<=0)
			return NULL;

		if((ret=(unsigned char*)MALLOC(i))==NULL){
			OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509CRL,ERR_PT_CRLASN1,NULL);
			return NULL;
		}
	}else{
		ret=buf;
	}
	if(CRL_set_sigalgo(crl,prv)) goto error;

	if(CRLG_CRL_DER_data(crl,ret,&i)) goto error;

	if(x509_set_signature(ret,prv,&crl->signature,&crl->siglen)) goto error;
	cp = ret+i;

	if(x509_DER_algoid(crl->signature_algo,NULL,cp,&j)) goto error;
	cp+=j; i+=j;

	ASN1_set_bitstring(0,crl->siglen,crl->signature,cp,&j);
	i+=j;

	ASN1_set_sequence(i,ret,ret_len);
	return ret;
error:
	if(ret!=buf) FREE(ret);
	return NULL;
}

int CRLG_CRL_DER_data(CRL *crl,unsigned char *ret,int *ret_len){
	unsigned char *cp;
	int	i=0,j,err=-1;

	cp=ret;
	if(crl->version){ /* version 2 or more */
		ASN1_set_integer(crl->version,cp,&j);
		cp+=j; i+=j;
	}
	if(x509_DER_algoid(crl->signature_algo,NULL,cp,&j)) goto done;
	cp+=j; i+=j;

	#ifdef __PRINTABLE_STRING_NOCHECK__
	if(Cert_DER_subject_printable_noCheck(&(crl->issuer_dn),cp,&j)) goto done;
	#else
	if(Cert_DER_subject(&(crl->issuer_dn),cp,&j)) goto done;
	#endif
	cp+=j; i+=j;

	if(CRLG_Cert_DER_time(&crl->lastUpdate,cp,&j)) goto done;
	cp+=j; i+=j;

	if(CRLG_Cert_DER_time(&crl->nextUpdate,cp,&j)) goto done;
	cp+=j; i+=j;

	if(CRLG_CRL_DER_revoked(crl,cp,&j)) goto done;
	cp+=j; i+=j;

	if(crl->version){ /* version 2 or more */
		if(x509_DER_exts(crl->ext,cp,&j)) goto done;
		if(j) ASN1_set_explicit(j,0,cp,&j);
		i+=j;
	}

	ASN1_set_sequence(i,ret,ret_len);
	err=0;
done:
	return err;
}



/*-----------------------------------------
  Get CRL data DER from CRL
-----------------------------------------*/
int CRLG_CRL_DER_revoked(CRL *crl,unsigned char *ret,int *ret_len){
	unsigned char *sq,*cp;
	Revoked *rv;
	int	i,j,k;
	int	entExtsLen;

	*ret_len=i=0;
	if(!crl->next) return 0; /* not error */

	sq=ret;
	for(rv=crl->next;rv!=NULL;rv=rv->next){
	        if(rv->long_sn){
		  j=ASN1_tlen(rv->long_sn);
		  ASN1_set_binary(ASN1_INTEGER,j,&rv->long_sn[2],sq,&j);
		}else{
		  ASN1_set_integer(rv->serialNumber,sq,&j);
		}
		cp=sq+j;

		if(CRLG_Cert_DER_time(&rv->revocationDate,cp,&k)) return -1;
		j+=k;
		cp = cp + k;

		if (rv->entExt != NULL) {
		  if (x509_DER_exts(rv->entExt,cp,&entExtsLen)) return -1;
		  j += entExtsLen;
		}

		ASN1_set_sequence(j,sq,&j);
		i+=j; sq+=j;
	}
	ASN1_set_sequence(i,ret,ret_len);
	return 0;
}


/*-------------------------------------------------------------------
  Get UTC or General time DER for CRLG
  -------------------------------------------------------------------*/
int CRLG_Cert_DER_time(struct tm *time,unsigned char *ret,int *ret_len){
        if(time->tm_wday==0){ /* This means UTC */
	        asn1_time_tag=ASN1_UTCTIME;
	}else if(time->tm_wday==1){
	        asn1_time_tag=ASN1_GENERALIZEDTIME;
	}

        if(stm2UTC(time,ret,(unsigned char)asn1_time_tag)==NULL)
                return -1;

        *ret_len=ASN1_tlen(ret)+2;
        return 0;
}



/*-------------------------------------------------------------------
  MAX_DEC_SIZE =  "1461501637330902918203684832716283019655932542975"
  -------------------------------------------------------------------*/
unsigned char *str2binary(char *input,int *len){
  char *buf;
  unsigned char *ret;
  int i=0,j;
  char table[20][9];
  int tlen;
  char *in;

  in=input;
  memset(table,0x00,160);

  ret=(unsigned char *)calloc(20+1,sizeof(char));

  do{
    tlen=i/8;
    if((buf=str_mod(in,&(table[tlen][i%8])))==NULL)goto done;
    in=buf;
    i++;
    if(i>160)goto done;
  }while(*buf);

  for(j=0;j<tlen+1;j++){
    for(i=0;i<strlen(table[j]);i++){
      if(table[j][strlen(table[j])-i-1]=='1'){
	ret[tlen-j]=(ret[tlen-j]<<1)|0x01;
      }else if(table[j][strlen(table[j])-i-1]=='0'){
	ret[tlen-j]=ret[tlen-j]<<1;
      }
    }
  }
  *len=tlen +1;
  return ret;

 done:
  free(ret);
  return(NULL);
}

char *str_mod(char *in,char *rem){
  char *ret;
  int num,syou,amari=0;
  int i,j=0,flag=0;

  ret=(char *)calloc(strlen(in)+1,sizeof(char));
  for(i=0;i<strlen(in);i++){
    if((num=char2i(in[i]))<0)goto done;
    num+=amari*10;
    amari=num % 2;
    syou = num /2;
    if((syou!=0)||flag){
      if((ret[j]=i2char(syou))<0) goto done;
      j++;
      flag=1;
    }
  }
  ret[strlen(ret)]='\0';
  *rem= i2char(amari);
  return ret;
 done:
  return NULL;
}

int char2i(int in){
  switch(in){
  case '1': return 1; break;
  case '2': return 2; break;
  case '3': return 3; break;
  case '4': return 4; break;
  case '5': return 5; break;
  case '6': return 6; break;
  case '7': return 7; break;
  case '8': return 8; break;
  case '9': return 9; break;
  case '0': return 0; break;
  default:
    return(-1);
  }
}

char i2char(int in){
  switch(in){
  case 1: return '1'; break;
  case 2: return '2'; break;
  case 3: return '3'; break;
  case 4: return '4'; break;
  case 5: return '5'; break;
  case 6: return '6'; break;
  case 7: return '7'; break;
  case 8: return '8'; break;
  case 9: return '9'; break;
  case 0: return '0'; break;
  default:
    return(-1);
  }
}


CertExt *Extnew_crl_issdistpt_printable_noCheck(ExtGenNames *distp,unsigned char *rflg,int bflg){
	CE_IssDistPt *ret;
	unsigned char *cp;
	int i,j,k=16,l;
	
	if((ret=(CE_IssDistPt*)CertExt_new(OBJ_X509v3_IssDistPoint))==NULL) goto error;

	/* estimate DER size */
	if(distp){
		if((i=ExtGN_estimate_der_size(distp))<0) goto error;
		k+=i;
	}
	k += ((bflg&EXT_IDP_UCert)?(4):(0)) + ((bflg&EXT_IDP_CACert)?(4):(0))
		+((bflg&EXT_IDP_indCRL)?(4):(0));

	if((ret->der=(unsigned char*)MALLOC(k))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509EXT,ERR_PT_EXTCRL+2,NULL);
		goto error;
	}

	i=0; cp=ret->der;
	if(distp){
		/* set data */
		ret->distp.fullName = distp;
		ret->distp.FullorRDN= 1;
		/* get DER */
		if(ExtGN_toDER_printable_noCheck(distp,cp,&j)==NULL) goto error;	
		*cp = 0xa0; /* implicit */
		ASN1_set_explicit(j,0,cp,&j);
		cp+=j; i+=j;
	}
	if(bflg&EXT_IDP_UCert){
		ret->onlyContainsUserCerts = 1; /* TRUE */
		ASN1_set_boolean(1,cp,&j);
		*cp = 0x81; /* implicit */
		cp+=j; i+=j;
	}
	if(bflg&EXT_IDP_CACert){
		ret->onlyContainsCACerts = 1; /* TRUE */
		ASN1_set_boolean(1,cp,&j);
		*cp = 0x82; /* implicit */
		cp+=j; i+=j;		
	}
	if(rflg){
		/* set data */
		memcpy(ret->rflag,rflg,2);
		/* get DER */
		asn1_check_derbit(2,rflg,&k,&l);
		ASN1_set_bitstring(k,l,rflg,cp,&j);
		*cp = 0x83; /* implicit */
		cp+=j; i+=j;
	}
	if(bflg&EXT_IDP_indCRL){
		ret->indirectCRL = 1; /* TRUE */
		ASN1_set_boolean(1,cp,&j);
		*cp = 0x84; /* implicit */
		i+=j;		
	}
	ASN1_set_sequence(i,ret->der,&ret->dlen);

	return (CertExt*)ret;
error:
	CertExt_free((CertExt*)ret);
	return NULL;
}


CertExt *Extnew_altname_printable_noCheck(int id, ExtGenNames *top){
	CE_SbjAltName *ret;

	if((ret=(CE_SbjAltName*)CertExt_new(id))==NULL) 
		goto error;

	/* set information */
	ret->egn = top;

	/* get DER */
	if((ret->der=ExtGN_toDER_printable_noCheck(top,NULL,&ret->dlen))==NULL)
		goto error;

	return (CertExt*)ret;
error:
	CertExt_free((CertExt*)ret);
	return NULL;
}

unsigned char *ExtGN_toDER_printable_noCheck(ExtGenNames *top,unsigned char *buf,int *ret_len){
	ExtGenNames	*now;
	unsigned char *cp,*ret;
	int i,j=0;

	if(buf==NULL){
		if((i=ExtGN_estimate_der_size(top))<=0)
			return NULL;

		if((ret=(unsigned char*)MALLOC(i))==NULL){
			OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509EXT,ERR_PT_EXTGN+9,NULL);
			return NULL;
		}
	}else{
		ret=buf;
	}

	*ret_len=0; cp=ret;
	for(now=top;now!=NULL;now=now->next){
		if(ExtGN_DER_gname_printable_noCheck(now,cp,&i)) goto error;
		cp+=i; j +=i;
	}
	ASN1_set_sequence(j,ret,ret_len);
	return ret;
error:
	if(ret!=buf) FREE(ret);
	return NULL;
}

int ExtGN_DER_gname_printable_noCheck(ExtGenNames *now,unsigned char *ret,int *ret_len){
	int i;
	switch(now->type){
	case 1:	/* rfc822Name (IA5String) */
	case 2:	/* dNSName (IA5String) */
	case 6:	/* uniformResourceIdentifier (IA5String) */
		if(ASN1_set_ia5(now->name,ret,ret_len)) goto error;
		ASN1_set_implicit(now->type,ret);
		break;
	case 4:	/* directoryName */
	        #ifdef __PRINTABLE_STRING_NOCHECK__
		if(Cert_DER_subject_printable_noCheck((CertDN*)now->name,ret,&i)) goto error;
		#else
		if(Cert_DER_subject((CertDN*)now->name,ret,&i)) goto error;
		#endif
		ASN1_set_explicit(i,4,ret,ret_len);
		break;
	case 7:	/* iPAddress (OCTET STRING) */
		ASN1_set_octetstring(now->name_len,now->name,ret,ret_len);
		*ret = 0x87; /* implicit */
		break;
	case 8:	/* registeredID (OBJECT IDENTIFIER) */
		if((str2objid(now->name,ret,32))<0) goto error;
		*ret = 0x88; /* implicit */
		break;
	case 0: /* otherName */
		if(ExtGN_DER_othname((OtherName*)now->name,ret,ret_len)) goto error;
		break;
	default:
		OK_set_error(ERR_ST_UNSUPPORTED_PARAM,ERR_LC_X509EXT,ERR_PT_EXTGN+8,NULL);
		goto error;
	}
	return 0;
error:
	return -1;
}

int Cert_DER_subject_printable_noCheck(CertDN *dn,unsigned char *ret,int *ret_len){
	unsigned char *tmp,*cp,uc[256];
	int		i,j,k,l,t;

	tmp=ret;
	memset(uc,0,256);
	for(l=k=0;k<dn->num;k++){
		cp =tmp;

		if(ASN1_int_2object(dn->rdn[k].tagoid,cp,&i)) goto error;
		cp +=i;

		switch(dn->rdn[k].derform){
		case ASN1_UTF8STRING:
			if((t=UC_conv(UC_LOCAL_JCODE,UC_CODE_UTF8,dn->rdn[k].tag,strlen(dn->rdn[k].tag),uc,254))<0) goto error;
			if(ASN1_set_utf8(uc,cp,&j)) goto error;
			break;
		case ASN1_IA5STRING:
			if(ASN1_set_ia5(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		case ASN1_T61STRING:
			if(ASN1_set_t61(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		case ASN1_PRINTABLE_STRING:
		default:
			if(ASN1_set_printable_noCheck(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		}

		i+=j;
		ASN1_set_sequence(i,tmp,&i);
		ASN1_set_set(i,tmp,&i);
		tmp+=i; l+=i;
	}

	ASN1_set_sequence(l,ret,ret_len);
	return 0;
error:
	return -1;
}

int ASN1_set_printable_noCheck(char *str,unsigned char *ret,int *ret_len){
  asn1_set_str(ASN1_PRINTABLE_STRING,str,ret,ret_len);

  return 0;
}
