/* crlext.c */
/*
 * Copyright (C) 2002 IPA (?)
 *
 * All rights reserved.
 *
 */

#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_pem.h"
#include "ok_err.h"
#include "aiconfig.h"

#include "crlgen.h"


/*--------------------------------------------
  read extension settings
--------------------------------------------*/
int CRLG_read_ext(char *org, CRL *crl){
	char *buf=NULL,*hd,*ed;
	int ok=-1,len;

	/* find "extension" section */
	if((hd=strstr(org,"[CRL Extensions]"))==NULL){
		printf("cannot find [CertificateList Fields] tag.\n");
		goto done;
	}
	if((ed=strstr(hd,"[CRL Extensions end]"))==NULL){
		printf("cannot find [CertificateList Fields end] tag.\n");
		goto done;
	}
	*ed=0; hd+=sizeof("[CRL Extensions]")-1;
	len=strlen(hd);

	/* allocate & copy buffer */
	if((buf=(char*)AIMALLOC(len+2))==NULL){
		printf("cannot allocate memory!\n");
		goto done;
	}
	memcpy(buf,hd,len+2);
	*ed='[';

	if(CRLG_ext_syntax(buf,crl)) goto done;
  
	ok=0;
done:
	if(buf) AIFREE(buf);
	return ok;
}

char *cnf_gets(char *buf){
	char *ret=NULL;

	if(ret=strchr(buf,'\n')){
		if('\r'==*(ret-1)){ *(ret-1)=0; }
		*ret=0; ret++;
	}
	return ret;
}

int CRLG_ext_syntax(char *buf, CRL *crl){
	CertExt *hd=NULL,*ce;
	char *ep,*cp,*t,*dp;
	int line=0,i,ok=-1;
	unsigned char long_sn[32]="";
	int snlen=-1,critical,status;
	
	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */

		ce=NULL;
		critical=UNSET;
		cp=buf;
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}

		while(*cp){
		        status=0;
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case '\t':
			case ' ': /* ignore these chars */
				cp++; continue;

			case 'C': /* CRLNumber */
			  if((snlen=CRLG_line2bin_ext(cp,"CRLNumber=",&long_sn[2],20,&critical))>=0){
				        long_sn[0]=ASN1_INTEGER;
				        long_sn[1]=snlen;
				        if((ce=Extnew_crl_number_long_sn(long_sn))==NULL) goto done;
					if(critical!=UNSET)ce->critical=critical;
				}else{
					printf("CRLG_ext_syntax: bad CRL number (%d)\n",line);
					goto done;
				}
				break;
	
			case '[': /* CRL extension Sections */
				if((t=strchr(cp,']'))==NULL){
					printf("CRLG_ext_syntax: bad section [] (%d)\n",line);
					goto done;
				}
				*t=0; cp++; t+=2;

				if(!strcmp(cp,"AuthorityKeyIdentifier")){
					i=1; cp=t; t="[AuthorityKeyIdentifier end]";
				}else if(!strcmp(cp,"IssuerAltName")){
					i=2; cp=t; t="[IssuerAltName end]";
				}else if(!strcmp(cp,"issuingDistributionPoint")){
					i=3; cp=t; t="[issuingDistributionPoint end]";
				}else{
					printf("CRLG_ext_syntax: unknown section name (%d)\n>>%s\n",line,cp);
					goto done;
				}

				if((ep=strstr(cp,t))==NULL){
					printf("CRLG_com_syntax: bad %s tag (%d)\n",t,line);
					goto done;
				}
				*ep = 0;
				ep += strlen(t);

				switch(i){
				case 1: ce=CRLG_exAKI_syntax(cp, crl,&status); break;
				case 2: ce=CRLG_exIAN_syntax(cp, crl,&status); break;
				case 3: ce=CRLG_exIDP_syntax(cp, crl,&status); break;
				}

				if(status){
				  switch(i){
				  case 1: fprintf(stderr,"err in CRLG_exAKI_syntax\n"); break;
				  case 2: fprintf(stderr,"err in CRLG_exIAN_syntax\n"); break;
				  case 3: fprintf(stderr,"err in CRLG_exIDP_syntax\n"); break;
				  }
				  goto done;
				}
				break;
			case '0':
			case '1':
			case '2':
			        if((ce=CRLG_exPrv_syntax(cp,crl))==NULL)goto done;
			  break;
			default:
				printf("CRLG_ext_syntax: syntax error (%d)\n>>%s\n",line,buf);
				goto done;
			}
			break;
		}
		buf=ep; line++;

		if(ce){
			if(hd){
				hd->next=ce; hd=ce;
			}else{
				crl->ext=hd=ce;
			}
		}
	}
	ok = 0;
done:
	return ok;
}

/*--------------------------------------------
  Authority Key Identifier syntax
--------------------------------------------*/
CertExt *CRLG_exAKI_syntax(char *buf, CRL *crl,int *status){
	ExtGenNames *gn=NULL;
	CertExt *ret=NULL;
	CertDN tdn,*dn=NULL;
	char *cp,*ep,*t,*tmp=NULL,*dp;
	unsigned char kid[32],serial[32];
	int klen=0,slen=0;
	int critical=UNSET;
	int err=0;

	cert_dn_init(&tdn);

	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */

		gn=NULL;
		cp=buf;
		
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}

		while(*cp){
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case '\t':
			case ' ': /* ignore these chars */
				cp++; continue;

			case 'k':
				if(t=strstr(cp,"keyIdentifier=")){
				        err=-1;
					if((klen=CRLG_line2bin_ext(cp,"keyIdentifier=",kid,20,&critical))<0){
						printf("CRLG_exIAN_syntax: bad key Identifier \n");
						goto done;
					}
				}
				break;

			case 'a': /* authorityCertIssuer or authorityCertSerialNumber */	
			        if(t=strstr(cp,"authorityCertSerialNumber=")){
				        err=-1;
					if((slen=CRLG_line2bin_ext(cp,"authorityCertSerialNumber=",serial,20,&critical))<0){
						printf("CRLG_exIAN_syntax: bad authorityCertSerialNumber \n");
						goto done;
					}
				}else if(t=strstr(cp,"authorityCertIssuer=")){
				        err=-1;
					if(strstr(cp,"issuer")){
				                if((cp=get_critical(cp+strlen("authorityCertIssuer="),&critical))==NULL) goto done;
						dn = &crl->issuer_dn;
					}else{
						if((tmp=CRLG_get_dn_ext(cp,"authorityCertIssuer=",&tdn,&critical))==NULL) goto done;
						AIFREE(tmp);
						dn = &tdn;
					}
				}
				break;

			default:
			        err=-1;
				printf("CRLG_exIAN_syntax: unknown tag error \n");
				goto done;
			}
			break;
		}
		buf=ep;
	}

	if(err){
	  if((ret = Extnew_authkey_id2((klen)?(kid):(NULL),klen,dn,(slen)?(serial):(NULL),slen))==NULL){
	    fprintf(stderr,"err Extnew_auhtkey_id2()\n");
	    goto done;
	  }
	  if(critical!=UNSET){
	    ret->critical=critical;
	  }
	}
	err=0;

done:
	*status=err;
	cert_dn_free(&tdn);
	if(err){CertExt_free(ret);ret=NULL;}
	return ret;
}

/*--------------------------------------------
  Issuer Alt Name
--------------------------------------------*/
CertExt *CRLG_exIAN_syntax(char *buf, CRL *crl,int *status){
	ExtGenNames *hd=NULL,*tp=NULL,*gn;
	CertExt *ret=NULL;
	CertDN dn;
	unsigned char ip[16];
	char *cp,*ep,*t,*tmp=NULL,*dp;
	int critical=UNSET,err=0;

	cert_dn_init(&dn);

	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */
	  
		gn=NULL;
		cp=buf;
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}

		while(*cp){
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case '\t':
			case ' ': /* ignore these chars */
				cp++; continue;

			case 'r': /* rfc822Name */
				if(cp=strstr(cp,"rfc822Name=")){
				        err=-1;
				        if((get_critical(cp+strlen("rfc822Name="),&critical))==NULL) goto done;
					cp+=strlen("rfc822Name=")+2;
					gn = ExtGN_set_email(cp);
				}
				break;
			case 'i': /* IP Adderss */
				if(cp=strstr(cp,"iPAddress=")){
				        err=-1;
				        if((get_critical(cp+strlen("iPAddress="),&critical))==NULL) goto done;
					cp+=strlen("iPAddress=")+2;
					str2ip(cp,ip);
					gn = ExtGN_set_ip(ip,4);
				}
				break;
			case 'd': /* dNSName or directoryName */
				if(t=strstr(cp,"dNSName=")){
				        err=-1;
				        if((get_critical(cp+strlen("dNSName="),&critical))==NULL) goto done;
					gn = ExtGN_set_dns(t+strlen("dNSName=")+2);
				}else if(t=strstr(cp,"directoryName=")){
				        err=-1;
					if(strstr(cp,"issuer")){
					        if((get_critical(cp+strlen("directoryName="),&critical))==NULL) goto done;
						gn = ExtGN_set_dn(&crl->issuer_dn);
					}else{
						if((tmp=CRLG_get_dn_ext(cp,"directoryName=",&dn,&critical))==NULL) goto done;
						gn = ExtGN_set_dn(&dn);
						AIFREE(tmp);
						cert_dn_free(&dn);
					}
				}
				break;
			default:
			        err=-1;
				printf("CRLG_exIAN_syntax: unknown tag error \n");
				goto done;
			}
			break;
		}
		buf=ep;

		if(gn){
			if(hd){
				hd->next=gn; hd=gn;
			}else{
				tp=hd=gn;
			}
		}
	}
	if(err){
	  if((ret = Extnew_altname_printable_noCheck(OBJ_X509v3_IssAltName,tp))==NULL)
	    goto done;
	  if(critical!=UNSET){
	    ret->critical=critical;
	  }
	}
	err=0;

 done:
	*status=err;
	cert_dn_free(&dn);
	if(err){CertExt_free(ret);ret=NULL;}
	return ret;
}

CertExt *CRLG_exPrv_syntax(char *in,CRL *crl){
  CertExt *ret=NULL;
  char *hp;
  int err=-1;

  if((ret=calloc(1,sizeof(CertExt)))==NULL)
    goto done;
  ret->extnID=0;
  ret->critical=UNSET;
  if((ret->objid=(unsigned char *)calloc(64,sizeof(char)))==NULL)
    goto done;

  if((hp=strchr(in,'='))==NULL)
    goto done;
  hp+=1;
  if(get_critical(hp,&(ret->critical))==NULL)
     goto done;
  *(hp-1)='\0';
  hp+=2;

  if(str2objid(in,ret->objid,32)<0)
    goto done;
  ret->dlen=strlen(hp)/2;
  if((ret->der=(unsigned char *)calloc(ret->dlen+1,sizeof(char)))==NULL)
    goto done;
  CRLG_line2bin_base(hp,"",ret->der,ret->dlen);

  err=0;
  
 done:
  if(err&&ret){CertExt_free(ret);ret=NULL;}
  return ret;
}  


int str2ip(char *str, unsigned char *bin){
	char *n,*cp=str;
	int i;

	memset(bin,0,4);
	for(i=0;(n=strchr(cp,'.'))&&(i<3); cp=n+1,i++)
		bin[i] = atoi(cp);
	
	bin[i] = atoi(cp); /* last byte */

	return 0;
}

/*--------------------------------------------
  Issuing Distribution Point
--------------------------------------------*/
CertExt *CRLG_exIDP_syntax(char *buf, CRL *crl,int *status){
	ExtGenNames *hd=NULL,*tp=NULL,*gn;
	CertExt *ret=NULL;
	CertDN dn;
	char *cp,*ep,*t,*tmp=NULL;
	unsigned char bflg;

	char *dp;
	int critical=UNSET,err=0;

	bflg = 0;

	cert_dn_init(&dn);

	while(ep=cnf_gets(buf)){ /* get 1 line buffer (ep is next line) */
		gn=NULL;
		cp=buf;
		if((dp=strchr(cp,'#'))!=NULL){
		  *dp='\0';
		}

		while(*cp){
			switch(*cp){
			case '#': /* comment line */
			case '\0': /* \r & \n are replaced by \0 -- end line */
				break;

			case '\t':
			case ' ': /* ignore these chars */
				cp++; continue;

			case 'U': /* URI */
				if(cp=strstr(cp,"URI=")){
				  if((cp=get_critical(cp+strlen("URI="),&critical))==NULL)goto done;
				  gn=ExtGN_set_url_list(cp+2);
				}
				break;
			case 'd': /* directoryName */
				if(t=strstr(cp,"directoryName=")){
				        err=-1;
					if(strstr(cp,"issuer")){
					        if((get_critical(cp+strlen("directoryName="),&critical))==NULL) goto done;
						gn = ExtGN_set_dn(&crl->issuer_dn);
					}else{
						if((tmp=CRLG_get_dn_ext(cp,"directoryName=",&dn,&critical))==NULL) goto done;
						gn = ExtGN_set_dn(&dn);
						AIFREE(tmp);
						cert_dn_free(&dn);
					}
				}
				break;
			case 'o': /* onlyContainsUserCerts or onlyContainsCACerts */
				if(t=strstr(cp,"onlyContainsUserCerts=")){
				        err=-1;
				        if(get_critical(cp+strlen("onlyContainsUserCerts="),&critical)==NULL) goto done;
					if(strstr(t,"TRUE"))
						bflg |= EXT_IDP_UCert;
				}else if(t=strstr(cp,"onlyContainsCACerts=")){
				        err=-1;
				        if(get_critical(cp+strlen("onlyContainsCACerts="),&critical)==NULL) goto done;
					        if(strstr(t,"TRUE"))
						bflg |= EXT_IDP_CACert;
				}
				break;
			default:
			        err=-1;
				printf("CRLG_exIDP_syntax: unknown tag error \n");
				goto done;
			}
			break;
		}
		buf=ep;

		if(gn){
			if(hd){
				hd->next=gn;
				while(gn->next)gn=gn->next;
				hd=gn;
			}else{
				tp=hd=gn;
			}
		}
	}

	if((tp==NULL)&&(bflg==0)){
	  goto done;
	}
	if(err){
	  if((ret = (CertExt *)Extnew_crl_issdistpt_printable_noCheck(tp, NULL, bflg))==NULL){
	    goto done;
	  }
	  if(critical!=UNSET){
	    ret->critical=critical;
	  }
	}
	
	err=0;

done:
	*status=err;
	cert_dn_free(&dn);
	if(err&&ret){CertExt_free(ret); ret=NULL;}
	return ret;
}

/*--------------------------------------------
  Authority Key Identifier
--------------------------------------------*/
CertExt *Extnew_authkey_id2(unsigned char *kid, int klen, CertDN *dn, unsigned char *serial, int slen){
	CE_AuthKID *ret=NULL;
	unsigned char *cp,*pub=NULL,*buf=NULL;
	int i,j=0,err=-1;

	/* set Authority Key ID */
	if((ret=(CE_AuthKID*)CertExt_new(OBJ_X509v3_AuthKeyIdt))==NULL) goto done;

	/* check subject size */
	if(dn){
		for(i=j=0;i<RDN_MAX;i++){
			if(dn->rdn[i].tag)
				j+=strlen(dn->rdn[i].tag)+20;
		}
	}
	if((buf=(unsigned char*)MALLOC(j+128))==NULL) goto done;

	i=0; cp=buf;
	/* set Identifier... */
	if(kid){
		if((ret->keyid=(unsigned char*)MALLOC(klen))==NULL) goto done;
		memcpy(ret->keyid,kid,klen);
		ret->klen = klen;

		/* get DER */
		ASN1_set_octetstring(ret->klen,ret->keyid,cp,&i);
		ASN1_set_implicit(0,cp); /* implicit OCTET STRING */
		cp+=i;
	}

	/* set Directory... */
	if(dn){
		if((ret->authorityCertIssuer=ExtGN_set_dn(dn))==NULL) goto done;

		/* get DER */
		#ifdef __PRINTABLE_STRING_NOCHECK__
		if(Cert_DER_subject_printable_noCheck(dn,cp,&j)) goto done;
		#else
		if(Cert_DER_subject(dn,cp,&j)) goto done;
		#endif

		ASN1_set_explicit(j,4,cp,&j);
		ASN1_set_explicit(j,1,cp,&j); /* implicit GeneralNames */
		cp+=j; i+=j;


	}

	/* set Serial Number... */
	if(serial){
		/* set long integer */
		ret->slen = slen;

		if((ret->long_sn=(unsigned char*)MALLOC(slen+2))==NULL) goto done;
		memcpy(&ret->long_sn[2],serial,slen);
		ret->long_sn[0] = ASN1_INTEGER;
		ret->long_sn[1] = slen;

		memcpy(cp,ret->long_sn,slen+2);
		ASN1_set_implicit(2,cp); /* implicit INTEGER */
		i+=slen+2;
	}

	if(i==0)goto done;
	ASN1_set_sequence(i,buf,&i);

	ret->dlen = i;
	if((ret->der=(unsigned char*)MALLOC(i))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509EXT,ERR_PT_EXTCERT,NULL);
		goto done;
	}
	memcpy(ret->der,buf,i);
	err=0;

done:
	if(pub) FREE(pub);
	if(buf) FREE(buf);
	if(err&&ret){CertExt_free((CertExt*)ret);ret=NULL;}
	return (CertExt*)ret;
}

/*-----------------------------------
  set long serialNumber for CRLNumber 
  -----------------------------------*/
CertExt *Extnew_crl_number_long_sn(unsigned char *long_sn){
  CertExt *ret;
  int len;

  if((ret=(CertExt *)CertExt_new(OBJ_X509v3_CRLNumber))==NULL)
    goto done;

  len=ASN1_tlen(long_sn)+2;
  if((ret->der=(unsigned char *)calloc(len,sizeof(char)))==NULL)
    goto done;
  memcpy(ret->der,long_sn,len);

  ret->dlen=len;

  return ret;

 done:
  CertExt_free(ret);
  return NULL;
}
  
/*----------------------------------------
  set URI  list Separator=tab(0x09)
  -----------------------------------------*/
ExtGenNames *ExtGN_set_url_list(char *in){
  ExtGenNames *top,*now;
  char *hp,*ep;
  int flag=0;
  
  top=now=NULL;

  if(in==NULL)goto error;
  if((now=(ExtGenNames *)calloc(1,sizeof(ExtGenNames)))==NULL)
    goto error;

  top=now;

  for(hp=in;flag!=1;now=now->next,hp=ep+1){
    if((ep=strchr(hp,0x09))==NULL){
      flag=1;
    }else{
      *ep='\0';
    }

    if((now->next=ExtGN_set_url(hp))==NULL)
      goto error;
  }
  
  return top->next;

 error:
  if(top)ExtGN_free(top);
  return NULL;
}


char *get_critical(char *in,int *critical){
  char *ret;
  int flag=0;

  
  for(ret=in;(*ret==' ')||(*ret=='\t');ret++){
    /* step space and tab */
  }
  
  
  if(!strncmp(ret,"n:",2)){
    if(*critical!=CRITICAL&&*critical!=NON_CRITICAL)
      *critical=NON_CRITICAL;
  }else if(!strncmp(ret,"c:",2)){
    if(*critical!=CRITICAL&&*critical!=NON_CRITICAL)
      *critical=CRITICAL;
  }else{
    *critical=UNSET;
    printf("err in set critical\n");
    return NULL;
  }

  return ret;
}
