/* crlgen.c */
/*
 * Copyright (C) 2002 IPA (?)
 *
 * All rights reserved.
 *
 */
#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_tool.h"
#include "ok_err.h"

#include "crlgen.h"

/* global values */
char *valfile =NULL;
char *keyfile =NULL;
char *outfile =NULL;
int sighash=OBJ_HASH_SHA1;

char program_name[] = "crlgen";  
char program_version[] = "1.9";

/*-----------------------------------------
  main
-----------------------------------------*/
int main(int argc,char **argv){
	FILE *fp=NULL;
	CRL *crl=NULL;
	Key *cakey=NULL;
	int dlen,err=-1;
  
	OK_set_sbjdn_conv_flag(0);

	options(argc,argv);

	/* clear error buffer */
	OK_clear_error();

	/* get CA private key */
	if((cakey=CRLG_get_prvkey(keyfile))==NULL){
		printf("cannot open CA private key : %s\n",(keyfile)?(keyfile):("<null>"));
		goto done;
	}
	OK_clear_error();

	/* get CRL information */
	if((crl=CRLG_open_conf(valfile))==NULL) goto done;
	
	/* get CRL DER */
	OK_set_crl_sig_algo(get_sigalgo(cakey,sighash));
	if((crl->der=CRLG_CRL_toDER(crl,cakey,NULL,&dlen))==NULL) goto done;

	/* output data */
	if(outfile){
		if((fp=fopen(outfile,"wb"))==NULL){
			printf("cannot open a output file : %s\n",outfile);
			goto done;
		}
		if((signed)fwrite(crl->der,sizeof(char),dlen,fp)<dlen){
			printf("cannot write a CRL to the file : %s\n",outfile);
			goto done;
		}
	}else{
		if((signed)fwrite(crl->der,sizeof(char),dlen,stdout)<dlen){
			printf("cannot write a CRL to stdout : %s\n",outfile);
			goto done;
		}
	}

#ifdef _DEBUG
{
	Revoked *rv;
	printf("crl->ver:%d\n",crl->version);
	printf("crl->issuer:%s\n",crl->issuer);
	printf("crl->thisUpdate:%s",asctime(&crl->lastUpdate));
	printf("crl->nextUpdate:%s",asctime(&crl->nextUpdate));
	printf("crl->revoked:\n");
	for(rv=crl->next; rv ; rv=rv->next){
		printf("  serial=%d, revokedDate=%s",rv->serialNumber,asctime(&rv->revocationDate));
	}
	printf("crl->ext:\n");
	if(crl->ext) print_v3_extensions(crl->ext,2);
}


#endif /* _DEBUG */
/*  	PEM_write_crl(crl,"crl.pem");  *//* out put crl pem form for check */

	err = 0;
done:
	if(err) printf("%s\n",OK_get_errstr());
	if(fp) fclose(fp);
	CRL_free(crl);
	Key_free(cakey);
	return err;
}
/*-----------------------------------------
  options
-----------------------------------------*/
void options(int argc,char **argv){
	int   i;
	if(argc<2) goto error;
    
	for(i=1;i<argc;i++){
		if(!strcmp(argv[i],"-K")){
			if((++i)>=argc) goto error;
			keyfile = argv[i];

		}else if(!strcmp(argv[i],"-v")){
			if((++i)>=argc) goto error;
			valfile = argv[i];
      
		}else if(!strcmp(argv[i],"-a")){
			if((++i)>=argc) goto error;
			if(!strcmp(argv[i],"sha1")){
				sighash=OBJ_HASH_SHA1;
			}else if(!strcmp(argv[i],"md5")){
				sighash=OBJ_HASH_MD5;
			}else{
				goto error;
			}
		}else if(!strcmp(argv[i],"-o")){
			if((++i)>=argc) goto error;
			outfile = argv[i];

		}else if(!strcmp(argv[i],"-h")){
			usage();
			exit(0);
		}else{
			goto error;
		}
	}
  
	if((keyfile==NULL)||(valfile==NULL)) goto error;
    
	return;
error:
	usage();
	exit(-1);
}

/*-----------------------------------------
  usage
-----------------------------------------*/
void usage() {
     printf("%s version %s:\n", program_name, program_version);
     printf("usage:\n");
     printf(" %s -K <CA private key file> -v <crl value file>\n",
	    program_name);
     printf("	[-a sha1|md5] [-o output file] [-h]\n");
     printf("  -K\tCA private key file.\n");
     printf("  -v\tCRL value file.\n");
     printf("  -a\tCRL signature algorithm. (default 'sha1')\n");
     printf("  -o\toutput CRL file. (if not specified, CRL data will be output to stdout)\n");
     printf("  -h\tshow command usage.\n");
     printf("\n");
     printf("NOTE: Max number of RDN is limited to %d.\n", RDN_MAX);
}
/*-----------------------------------------
  tool
-----------------------------------------*/
int get_sigalgo(Key *key, int hash){
	int ret = -1;

	switch(key->key_type){
	case KEY_RSA_PUB:
	case KEY_RSA_PRV:
		switch(hash){
		case OBJ_HASH_SHA1: ret=OBJ_SIG_SHA1RSA; break;
		case OBJ_HASH_MD2: ret=OBJ_SIG_MD2RSA; break;
		case OBJ_HASH_MD5: ret=OBJ_SIG_MD5RSA; break;
		}
		break;
	case KEY_DSA_PUB:
	case KEY_DSA_PRV:
		ret=OBJ_SIG_SHA1DSA;
		break;
	case KEY_ECDSA_PUB:
	case KEY_ECDSA_PRV:
		ret=OBJ_SIG_SHA1ECDSA;
		break;
	}
	return ret;
}

