#include<stdio.h>
#include<string.h>
#include<ctype.h>
#include"ok_x509.h"
#include"ok_asn1.h"
#include"cert_opt.h"
#include"ext.h"

char program_name[] = "certgen";  
char program_version[] = "1.9";

unsigned char cert_section=SECTION_NULL;

int set_options(int argc,char **argv);
void help_print(void);

CertOptions opt;

int main(int argc,char **argv){
  char *tmp;
  char line[MAX_LEN]="";
  char *hp,*hpp,*lp;
  int len,llen,dlen,i;
  int err=-1;
  Cert *ct;
  Key *caprvkey;
  unsigned char *der;
  int size,ret_len,state;
  FILE *fp;

  OK_set_sbjdn_conv_flag(0);

  cert_opt_init(&opt);

  if((state=set_options(argc,argv))==1){
    return(0);
  }else if(state<0){
    fprintf(stderr,"err in set_options\n");
    goto done;
  } 

  /* read conf file */
  if((tmp=(char *)get_file2buf(opt.conf,&len))==NULL){
    fprintf(stderr,"\"%s\" is not enough conf file\n",opt.conf);
    goto done;
  }
  while(hp=strrchr(tmp,'\n')){
    lp=strchr(tmp,'\0');
    hpp=hp+1;
    if((hpp!=lp)&&(*hpp!='#')){/* start '#' or \n only  line check */
      llen=strlen(hpp)+2;
      strncpy(line,hpp,llen);
      if((hpp=strchr(line,'#'))!=NULL){
	*hpp='\0';
      }
      if(line_comment_check(line)){
	if(cert_line_opt_check(line,&opt)){
	  goto done;
	}
      }else{
	if(set_cert_section(line,&cert_section)){
	  goto done;
	}
      }
      memset(line,'\0',256);
    }
    *hp='\0';
  }

  /* set cert info */
  if((ct=(Cert *)set_Cert(&opt,&dlen))==NULL){
    fprintf(stderr,"err in set Cert\n");
    goto done;
  }

  if(*opt.cert_out){
    ASN1_write_der(ct->der,opt.cert_out);
  }else{
    dlen=ASN1_length((ct->der+1),&i);
    dlen+=i+1;

    if((signed)fwrite(ct->der,sizeof(char),dlen,stdout)<dlen){
      printf("cannot write a Cert to stdout\n");
      goto done;
    }
  }

  /* tmp for check signature */
  if(*opt.pemout){
    PEM_write_cert(ct,opt.pemout);
  }

  /* output in text form */
  if(opt.text==1)Cert_print(ct);

  return(0);

 done:
  fprintf(stderr,"err in certgen.c\n");
  fprintf(stderr,"%s\n",OK_get_errstr());
  return err;
}

int set_options(int argc,char **argv){
  int err=-1;
  if(argc < 2){
    help_print();
    goto done;
  }
  while(*++argv){
    if(!strcmp("-K",*argv)){
      if(*++argv==NULL)goto done;
      strncpy(opt.caprvkey,*argv,MAX_LEN-1);
    }else if(!strcmp("-p",*argv)){
      if(*++argv==NULL)goto done;
      strncpy(opt.targetkey,*argv,MAX_LEN-1);
    }else if(!strcmp("-v",*argv)){
      if(*++argv==NULL)goto done;
      strncpy(opt.conf,*argv,MAX_LEN-1);
    }else if(!strcmp("-a",*argv)){
      if(*++argv==NULL)goto done;
      if(!strcmp("md5",*argv)){
	opt.hash_algo=OBJ_HASH_MD5;
      }
    }else if(!strcmp("-o",*argv)){
      if(*++argv==NULL)goto done;
      strncpy(opt.cert_out,*argv,MAX_LEN-1);
    }else if(!strcmp("-h",*argv)){
      help_print();
      return 1;
      /* deleate after */
    }else if(!strcmp("-pemout",*argv)){
      if(*++argv==NULL)goto done;
      strncpy(opt.pemout,*argv,MAX_LEN-1);
      /* deleate after end */
    }else if(!strcmp("-t",*argv)){
      opt.text=1;
    }else{
      help_print();
      return err;
    }
  }

  if(*opt.caprvkey&&*opt.targetkey){
    err=0;
  }
 done:
  
  return err;
}
  
void help_print(void){
  printf("%s version %s:\n", program_name, program_version);
  printf("usage:\n%s -K <CA private key file>\n", program_name);
  printf("\t-p <target public key file> -v <certificate value file>\n");
  printf("\t[-a sha1|md5] [-h]\n");
  printf("------------------------ option list --------------------------\n");
  printf("\"-K file\" :set CAprvkey\n");
  printf("\"-p file\" :set targetpub.key\n");
  printf("\"-v file\" :set certvalue.cf\n");
  printf("\"-a kind\" :set hash algo(kind: md5 or sha1)\n");
  printf("\"-o file\" :set output file name\n");
  printf("\"-h \"     :options list\n");
  printf("---------------------------------------------------------------\n");
  printf("NOTE: Max number of RDN is limited to %d.\n", RDN_MAX);
}
