#include<stdio.h>
#include<string.h>
#include <ctype.h>
#include"ok_pem.h"
#include"ok_x509.h"
#include"ocsp_opt.h"
#include"ocsp_req.h"
#include"ocsp_res.h"
#include"ext.h"

int set_options(int argc,char **argv);
void help_print(void);

OCSPOptions opt;

int main(int argc,char **argv){
  char *tmp;
  char line[256]="";
  char *hp,*hpp,*lp;
  int len,llen,state;
  FILE *fp=stdout;
  unsigned char *der;
  int ret_len=0;

  OCSPresp *oresp;

  OK_set_sbjdn_conv_flag(0);

  ocsp_opt_init(&opt);

  if ((state=set_options(argc,argv)) == 1) {
    return(0);
  } else if (state < 0) {
    fprintf(stderr,"err in set_options\n");
    goto done;
  }
  if ((tmp = get_file2buf(opt.conf,&len)) == NULL) {
    fprintf(stderr,"%s is not enough conf file\n",opt.conf);
    goto done;
  }

  while (hp = strrchr(tmp,'\n')) {
    lp=strchr(tmp,'\0');
    hpp=hp+1;
    if ((hpp!=lp) && (*hpp!='#')) {/* only '\n' or start '#' line check */
      llen=strlen(hpp)+2;
      strncpy(line,hpp,llen);
    
      if ((hpp = strchr(line,'#')) != NULL) {
	*hpp='\0';
      }

      if (line_comment_check(line)) {
	if (ocsp_line_opt_check(line,&opt)) {
	  goto done;
	}
      }
      memset(line,'\0',256);
    }
    *hp='\0';
  }

  if ((oresp = (OCSPresp *)set_OCSPresp(&opt)) == NULL)
    goto done;
  /* *** */
  if ((der = (unsigned char *)calloc(2048,sizeof(char))) == NULL)
    goto done;

  if(ASN1_set_OCSPresp(oresp,der,&ret_len))
    goto done;
  if(*(opt.resout)){
    if((fp=fopen(opt.resout,"wb"))==NULL)
      goto done;
  }else{
    fp=stdout;
  }
  if((fwrite(der,sizeof(char),ret_len,fp))<ret_len){
    goto done;
  }
  if(fp)fclose(fp);

  return(0);
 done:
  fprintf(stderr,"err in ocspres.c\n");
  fprintf(stderr,"%s\n",OK_get_errstr());
  return(1);
}

/*
 * set_options()
 *
 *	process command line arguments.
 */
int set_options(int argc, char **argv) {
  int err=-1;

  opt.nonce[0] = '\0';

  if (argc < 2) {
    help_print();
    goto done;
  }
  while (*++argv) {
    if (! strcmp("-K",*argv)) {
      if (*++argv == NULL) goto done;
      strncpy(opt.rkey,*argv,MAX_LEN-1);
    } else if (! strcmp("-c",*argv)) {
      if (*++argv == NULL) goto done;
      strncpy(opt.rcert,*argv,MAX_LEN-1);
    } else if(! strcmp("-v",*argv)) {
      if (*++argv == NULL)goto done;
      strncpy(opt.conf,*argv,MAX_LEN-1);
    } else if(! strcmp("-n",*argv)) {
      if (*++argv == NULL) goto done;
      strncpy(opt.nonce,*argv,MAX_LEN-1);
    } else if(! strcmp("-s",*argv)) {
      if (*++argv == NULL) goto done;
      strncpy(opt.serial,*argv,MAX_LEN-1);
    } else if (! strcmp("-o",*argv)) {
      if (*++argv==NULL) goto done;
      strncpy(opt.resout,*argv,MAX_LEN-1);
    } else if (! strcmp("-h",*argv)) {
      help_print();
      return 1;
    } else {
      help_print();
      goto done;
    }
  }
  if(*opt.rkey){
    err=0;
  }

 done:
  if(!opt.conf[0]){
    sprintf(opt.conf,"./ocspresvalue.cf",NULL);
  }
  return err;
}

void help_print(void){
  printf("usage:\n./ocspresgen -K <OCSP private key file> -c <OCSP responder certificate file> -v <OCSP response value file> -n <nonce value> [-s <serialNumber>] [-o output file] [-h]\n");
  printf("------------------------ option list --------------------------\n");
  printf("\"-K file\"  :set responder prvkey\n");
  printf("\"-c file\"  :set responder cert\n");
  printf("\"-v file\"  :set ocsprespgen.cf\n");
  printf("\"-n value\" :set nonce value(hex string)\n");
  printf("\"-s value\" :set serialNumber value(hex string) OPTIONAL\n");
  printf("\"-o file\"  :set output file name\n");
  printf("\"-h \"      :options list\n");
  printf("---------------------------------------------------------------\n");
}
