//#include<stdio.h>
#include<string.h>
#include"ok_x509.h"
#include"ok_x509ext.h"
#include"ok_asn1.h"
#include"cert_opt.h"
#include"ocsp_req.h"

#define RSA 0
#define DSA 1

int asn1_time_tag;
int asn1_before_tag=ASN1_UTCTIME;
int asn1_after_tag=ASN1_UTCTIME;


Cert *set_Cert(CertOptions *opt,int *ret_len){
  Cert *ct;
  int len;
  struct tm stm;
  Key *caprvkey;

  if((ct=(Cert *)Cert_new())==NULL)
    goto done;

  ct->version=2;
  /* set serial */
  if((ct->long_sn=(unsigned char *)set_cert_serial(opt->snum,opt->snum_type))==NULL){
    fprintf(stderr,"err in set serialnumber\n");
    goto done;
  }
  /* set sig algo */
  if((caprvkey=(Key *)set_prvkey(opt->caprvkey))==NULL){
    fprintf(stderr,"err in read caprvkey \"%s\"\n",opt->caprvkey);
    goto done;
  }
  if((default_cert_sig_algo=set_sig_algo(opt->hash_algo,caprvkey->key_type))==UNSET)
    goto done;
  /* set issuer */
  if(Cert_dncopy(&(opt->issuer),&(ct->issuer_dn))){
    fprintf(stderr,"err in copy issuer\n");
    goto done;
  }
  if((ct->issuer=(char *)Cert_subject_str(&(ct->issuer_dn)))==NULL){
    fprintf(stderr,"err in set issuer\n");
    goto done;
  }
  /* set validity */
  asn1_before_tag=opt->before_type;
  if(str2tm(opt->notBefore,&(ct->time.notBefore))){
    fprintf(stderr,"err in set notBefore\n");
    goto done;
  }
  asn1_after_tag=opt->after_type;
  if(str2tm(opt->notAfter,&(ct->time.notAfter))){
    fprintf(stderr,"err in set notAfter\n");
    goto done;
  }
  /* set subject */
  if(Cert_dncopy(&(opt->subject),&(ct->subject_dn))){
    fprintf(stderr,"err in copy subject");
    goto done;
  }
  if((ct->subject=(char *)Cert_subject_str(&(ct->subject_dn)))==NULL){
    fprintf(stderr,"err in set subject\n");
    goto done;
  }
  /* set target pubkey */
  if((ct->pubkey=(Key *)set_pubkey(opt->targetkey))==NULL){
    fprintf(stderr,"err in set pubkey \"%s\"\n",opt->targetkey);
    goto done;
  }
  ct->pubkey_algo=ct->pubkey->key_type;
  /* set CertExt */
  if((ct->ext=(CertExt *)option2CertExt(opt))==NULL){
    fprintf(stderr,"err in option2CertExt()\n");
    goto done;
  }

  /* set signeture */
  if((ct->der=Cert_toDER_opt(ct,caprvkey,NULL,ret_len))==NULL){
    fprintf(stderr,"err in set cert der\n");
    goto done;
  }

  return ct;

 done:
  fprintf(stderr,"err in set_Cert\n");
  if(ct)Cert_free(ct);
  return NULL;
  
}
/*-----------------------------------------
  set prv & pub key
 ------------------------------------------*/
Key *set_prvkey(char *file){
  unsigned char *der=NULL;
  Key *ret=NULL;
  
  if(!file) goto error;

  if(ret = (Key*)PEM_read_rsaprv(file)){
    /* read rsa private key */
  }else if(ret = (Key*)PEM_read_dsaprv(file)){
    /* read dsa private key */
  }else if(ret = (Key*)PEM_read_ecdsaprv(file)){
    /* read ecdsa private key */
  }else if(ret = (Key*)P8_read_file(file)){
    /* read pkcs8 file */
  }else if(ret = (Key*)P8enc_read_file(file)){
    /* read pkcs8 (encrypted) file */
  }else if(der = ASN1_read_der(file)){
    if(ret = (Key*)ASN1_read_rsaprv(der)){
      /* read rsa private key */
    }else if(ret = (Key*)ASN1_read_dsaprv(der)){
      /* read dsa private key */
    }else if(ret = (Key*)ASN1_read_ecdsaprv(der)){
      /* read ecdsa private key */
    }
    AIFREE(der); der=NULL;
  }
  
  return ret;
 error:
  return NULL;
}

Key *set_pubkey(char *pubkey){
  Key *ret=NULL;
  unsigned char *der=NULL;  

  if(!pubkey)
    goto done;
/*    if((der=(unsigned char *)pem_read2der("-----BEGIN PUBLIC KEY-----\n","-----END PUBLIC KEY-----\n",pubkey))==NULL){ */
  if((der=(unsigned char *)pem_read2der("-----BEGIN PUBLIC KEY-----","-----END PUBLIC KEY-----",pubkey))==NULL){
    /* read public key */
    goto done;
  }

  if((ret=(Key *)ASN1_get_pubkey(der))==NULL){
    fprintf(stderr,"err der to key");
    goto done;
  }
  free(der);

 done:
  return ret;
}

/*---------------------------
  set signature  algo
  cert:default_cert_sig_algo
  crl:default_crl_sig_algo
  ---------------------------*/
int set_sig_algo(int hash_algo,int prvkey_type){
  int ret=UNSET;
  switch(hash_algo){
  case OBJ_HASH_SHA1:
    switch(prvkey_type){
    case KEY_RSA_PRV:
      ret=OBJ_SIG_SHA1RSA;
      break;
    case KEY_DSA_PRV:
      ret=OBJ_SIG_SHA1DSA;
      break;
    }
    break;
  case OBJ_HASH_MD5:
    switch(prvkey_type){
    case KEY_RSA_PRV:
      ret=OBJ_SIG_MD5RSA;
	break;
    case KEY_DSA_PRV:
      /*        ret=OBJ_SIG_MD5DSA; */ /* OBJ_SIG_MD5DSA isn't define */
      break;
    }
    break;
  }

  if(ret==UNSET)fprintf(stderr,"err in set sig algo\n");
  return ret;
}

unsigned char *set_cert_serial(char *in,int snum_type){
  unsigned char *ret=NULL,*buf;
  int len;

  switch(snum_type){
  case DEC:
    if((buf=str2binary(in,&len))==NULL)
      goto done;
    break;
  case HEX:
    if((buf=str2hex(in,&len))==NULL)
      goto done;
    break;
  default:
    goto done;
  }
  if((ret=(unsigned char *)calloc(len+4,sizeof(char)))==NULL)
    goto done;
  ret[0]=0x02;
  ret[1]=(char)len;
  memcpy(ret+2,buf,len);

 done:
  if(ret==NULL)fprintf(stderr,"err in set cert serial\n");
  return ret;
}

/*--------------------------
  set certificate extensions 
  --------------------------*/
CertExt *option2CertExt(CertOptions *opt){
  CertExt *top=NULL,*ext=NULL;
  ExtGenNames *extgname=NULL;
  Opt_PrvExt *pext;
  char *accessMethod;
  int err=-1;

  if((ext=(CertExt *)CertExt_new(OBJ_DUMMY))==NULL)
    goto done;
  top=ext;

  /* authorityKeyIdentifier */
  if((opt->auth_kid!=NULL)&&(opt->auth_kid->critical!=UNSET)){
    if((ext->next=(CertExt *)Extnew_authkey_id_opt(opt->auth_kid))==NULL)
      goto done;
    ext->next->critical=opt->auth_kid->critical;
    ext=ext->next;
  }

  /* SubjectKeyIdentifier */
  if(opt->skid!=NULL){
    if((ext->next=(CertExt *)Extnew_sbjkey_id_opt(opt->skid))==NULL)
      goto done;
    ext->next->critical=opt->skid->critical;
    ext=ext->next;
  }

  /* KeyUsage */
  if(opt->keyUsage!=NULL){
    if((ext->next=(CertExt *)Extnew_keyusage(opt->keyUsage->flag))==NULL)
      goto done;
    ext->next->critical=opt->keyUsage->critical;
    ext=ext->next;
  }

  /* policyIdentifier */
  if(opt->ct_pol!=NULL){
    if((ext->next=(CertExt *)Extnew_cert_policyList(OBJ_X509v3_CERT_Pol,opt->ct_pol->plist))==NULL)
      goto done;

    ext->next->critical=opt->ct_pol->critical;
    ext=ext->next;
  }

  /* PolicyMapping */
  if(opt->pmap!=NULL){
    if((ext->next=(CertExt *)Extnew_policyMap_opt(opt->pmap))==NULL)
      goto done;
    ext=ext->next;
  }

  /* SubjectAltName */
  if(opt->sub_alt_name!=NULL){
    if((ext->next=(CertExt *)Extnew_altname_printable_noCheck(opt->sub_alt_name->type,opt->sub_alt_name->gname))==NULL)
      goto done;
    ext->next->critical=opt->sub_alt_name->critical;
    ext=ext->next;
  }
  /* IssuerAltName */
  if(opt->iss_alt_name!=NULL){
    if((ext->next=(CertExt *)Extnew_altname_printable_noCheck(opt->iss_alt_name->type,opt->iss_alt_name->gname))==NULL)
      goto done;
    ext->next->critical=opt->iss_alt_name->critical;
    ext=ext->next;
  }

  /* BasicConstraint */
  if(opt->bconst!=NULL){
    if((ext->next=(CertExt *)Extnew_basic_cons(opt->bconst->ca,opt->bconst->path_len))==NULL)
      goto done;
    ext->next->critical=opt->bconst->critical;
    ext=ext->next;
  }
  /* PolicyConstraints */
  if(opt->pconst!=NULL){
    if((ext->next=(CertExt *)Extnew_policy_cons(opt->pconst->req,opt->pconst->inhibit))==NULL)
      goto done;
    ext->next->critical=opt->pconst->critical;
    ext=ext->next;
  }

  /* NameConstraints */
  if(opt->nconst!=NULL){
    if((ext->next=(CertExt *)Extnew_name_cons_opt(opt->nconst->ptree,opt->nconst->etree))==NULL){
      fprintf(stderr,"err set name const\n");
      goto done;
    }
    ext->next->critical=opt->nconst->critical;
    ext=ext->next;
  }

  /* extKeyUsage */
  if(opt->ext_kusage!=NULL){
    if((ext->next=(CertExt *)Extnew_cert_policyList(OBJ_X509v3_ExtKeyUsage,opt->ext_kusage->plist))==NULL)
      goto done;
    ext->next->critical=opt->ext_kusage->critical;
    ext=ext->next;
  }
  
  /* CRLDistributionPoints */
  if(opt->crl_dp!=NULL){
    if((ext->next=(CertExt *)Extnew_crl_distpoint_printable_noCheck(opt->crl_dp->crldp,NULL,NULL))==NULL)
      goto done;
    ext->next->critical=opt->crl_dp->critical;
    ext=ext->next;
  }

  /* authorityInfoAccess */
  if(opt->aia!=NULL){
    if((extgname=(ExtGenNames *)ExtGN_set_str(opt->aia->uri,6))==NULL)
      goto done;
    if (opt->aia->accessMethod == AIA_ACCESS_CAISSUERS) {
      accessMethod = "1.3.6.1.5.5.7.48.2";
    } else {
      accessMethod = "1.3.6.1.5.5.7.48.1";
    }
    if((ext->next=(CertExt *)Extnew_pkix_aia(accessMethod,extgname))==NULL)
      goto done;   
    ext->next->critical=opt->aia->critical;
    ext=ext->next;
  }

  /* ocsp-nocheck */
  if(opt->ncheck!=NULL){
    if((ext->next=(CertExt *)Extnew_ocsp_nocheck())==NULL)
      goto done;
    ext->next->critical=opt->ncheck->critical;
    ext=ext->next;
  }

  /* private ext */
  if(opt->pext){
    for(pext=opt->pext;pext;pext=pext->next,ext=ext->next){
      if((ext->next=(CertExt *)Extnew_PrvExt(pext))==NULL)
	goto done;
    }
  }
  err=0;

 done:
  if (err != 0) 
    return NULL;

  return top;
}

CertExt *Extnew_authkey_id_opt(Opt_AuthKeyID *auth_kid){
  CE_AuthKID *ret=NULL;
  unsigned char *buf=NULL,*cp;
  int i,j,err=-1;

  if(auth_kid==NULL)
    goto done;
  if((ret=(CE_AuthKID *)CertExt_new(OBJ_X509v3_AuthKeyIdt))==NULL)
    goto done;
  
  /* chech subject size */
  for(i=j=0;i<RDN_MAX;i++){
    if(auth_kid->certIssuer.rdn[i].tag)
      j+=strlen(auth_kid->certIssuer.rdn[i].tag)+20;
  }
  if((buf=(unsigned char *)calloc(j+128,sizeof(char)))==NULL)
    goto done;

  i=0;
  cp=buf;

  /* set keyIdentifier */
  if(*auth_kid->keyID){
    if((ret->keyid=(unsigned char *)str2hex(auth_kid->keyID,&(ret->klen)))==NULL)
      goto done;
    if(ret->klen!=20)
      goto done;
  }
  ASN1_set_octetstring(ret->klen,ret->keyid,cp,&i);
  ASN1_set_implicit(0,cp);
  cp+=i;

  /* set authorityCertIsser */
  if(auth_kid->certIssuer_type!=UNSET){
    if((ret->authorityCertIssuer=ExtGN_set_dn(&auth_kid->certIssuer))==NULL)
      goto done;
    /* get DER */
    #ifdef __PRINTABLE_STRING_NOCHECK__
    if(Cert_DER_subject_printable_noCheck(&(auth_kid->certIssuer),cp,&j))goto done;
    #else
    if(Cert_DER_subject(&(auth_kid->certIssuer),cp,&j))goto done;
    #endif
    ASN1_set_explicit(j,4,cp,&j);
    ASN1_set_explicit(j,1,cp,&j); /* implicit GeneralNames */
    cp+=j;i+=j;
  }

  /* set authorityCertSerialNumber */
  if(auth_kid->snum_type!=UNSET){
    if((ret->long_sn=(unsigned char *)set_cert_serial(auth_kid->snum,auth_kid->snum_type))==NULL)
    goto done;

    ret->slen=j=ASN1_tlen(ret->long_sn)+2;
    memcpy(cp,ret->long_sn,j);

    ASN1_set_implicit(2,cp); /* implicit INTEGER */
    i+=j;
  }
  ASN1_set_sequence(i,buf,&i);

  ret->dlen=i;
  if((ret->der=(unsigned char *)calloc(i,sizeof(char)))==NULL)
    goto done;
  memcpy(ret->der,buf,i);
  err=0;

 done:
  if(buf)FREE(buf);
  if(err&&ret){CertExt_free((CertExt *)ret);ret=NULL;}
  if(err)fprintf(stderr,"err in Extnew_authkey_id_opt()\n");
  return (CertExt *)ret;
}


CertExt *Extnew_sbjkey_id_opt(Opt_SubKeyID *skid){
  CE_SbjKID *ret=NULL;
  int len,err=-1;

  if((ret=(CE_SbjKID *)CertExt_new(OBJ_X509v3_SbjKeyIdt))==NULL)
    goto done;
  if((ret->der=(unsigned char *)calloc(32,sizeof(char)))==NULL)
    goto done;
  /* set data */
  ret->klen=20;
  if((ret->keyid=(unsigned char *)str2hex(skid->sub_keyID,&len))==NULL)
    goto done;
  ASN1_set_octetstring(20,ret->keyid,ret->der,&ret->dlen);
  err=0;

 done:
  if(err&&ret){CertExt_free((CertExt *)ret);ret=NULL;}
  return (CertExt *)ret;
}

/* PoliciMapping */
CertExt *Extnew_policyMap_opt(Opt_PolicyMap *pmap){
  CertExt *ret;
  unsigned char *top;
  int err=-1;

  if(pmap==NULL)
    goto done;
  if((ret=(CertExt *)calloc(1,sizeof(CertExt)))==NULL)
    goto done;
  ret->extnID=OBJ_X509v3_CertPolMap;
  ret->critical=pmap->critical;
  if((ret->der=(unsigned char *)calloc(1024,sizeof(char)))==NULL)
    goto done;
  top=ret->der;
  
  if(PairPolicy_toDER(pmap->oids,ret->der,&ret->dlen))
    goto done;

  err=0;

 done:
  if(err){
    if(ret->der!=NULL)free(ret->der);
    if(ret!=NULL)free(ret);
    ret=NULL;
  }
  return ret;
}

CertExt *Extnew_cert_policyList(int ext_type,ExtCertPol *top){
  CertExt *ret;
  int size=0,err=-1;
  
  if((ret=(CertExt *)CertExt_new(ext_type))==NULL)
    goto done;
  
  /*estimate der size */
  if((size=PolicyList_estimate_der_size(top))<0)
    goto done;
  
  if((ret->der=(unsigned char *)calloc(size,sizeof(char)))==NULL)
    goto done;
  if(PolicyList_toDER(top,ext_type,ret->der,&ret->dlen))
    goto done;
  
  err=0;

 done:
  if(err){
    if(ret->der!=NULL)free(ret->der);
    if(ret!=NULL)free(ret);
    ret=NULL;
  }
  return ret;
}



int PolicyList_estimate_der_size(ExtCertPol *top){
  ExtCertPol *cp;
  int ret=8;

  if(top==NULL){
    return -1;
  }

  for(cp=top;cp!=NULL;cp=cp->next){
    ret+=(strlen(cp->policyID)/2) +2+2;
  }
  return ret;
}


int PolicyList_toDER(ExtCertPol *in,int ext_type,unsigned char *ret,int *ret_len){
  int i,len,tlen=0,err=-1;
  ExtCertPol *plist;
  unsigned char *top,*hp;

  top=ret;
  
  for(plist=in;plist!=NULL;plist=plist->next){
    if(*plist->policyID){
      hp=ret;
      if((i=str2objid(plist->policyID,ret,32))<0)
	goto done;
      if(ext_type==OBJ_X509v3_CERT_Pol){
	ASN1_set_sequence(i,hp,&i);
      }
      ret+=i;
      tlen+=i;
    }
  }
  ASN1_set_sequence(tlen,top,ret_len);

  err=0;

 done:
  if(err)fprintf(stderr,"err in policyList toDER\n");
  return err;
}

int PairPolicy_toDER(PairOID *in,unsigned char *ret,int *ret_len){
  int i,j,len,tlen=0,err=-1;
  PairOID *poid,*next;
  unsigned char *top,*hp;

  top=ret;
  
  for(poid=in;poid!=NULL;poid=next){
    next=poid->next;
    if(*poid->issPolicy&&*poid->subPolicy){
      
      hp=ret;
      if((i=str2objid(poid->issPolicy,ret,32))<0)
	goto done;
      ret+=i;
      if((j=str2objid(poid->subPolicy,ret,32))<0)
	goto done;

      ASN1_set_sequence(i+j,hp,&len);
      tlen+=len;
      ret+=len-i;
    }
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;

 done:
  if(err)fprintf(stderr,"err in PairPolicy_toDER\n");
  return err;
}


CertExt *Extnew_name_cons_opt(ExtSubTrees *permit,ExtSubTrees *exclude){
  CE_NameCons *ret;
  unsigned char *cp;
  int i,tlen=0;

  if((ret=(CE_NameCons *)CertExt_new(OBJ_X509v3_NameConst))==NULL)
    goto done;
  /* estimate size & malloc DER */
  if(permit){
    if((i=ExtSubT_estimate_der_size(permit))<0)
      goto done;
    tlen+=i;
  }
  if(exclude){
    if((i=ExtSubT_estimate_der_size(exclude))<0)
      goto done;
    tlen+=i;
  }

  if((ret->der=(unsigned char *)calloc(tlen+4,sizeof(char)))==NULL)
    goto done;

  /* set data */
  ret->permittedSubtrees=permit;
  ret->excludedSubtrees=exclude;

  /* get DER */
  cp=ret->der;tlen=0;
  if(permit){
    if(ExtSubT_toDER_opt(permit,cp,&i)==NULL) goto done;
    *cp=0xa0; /* 0x30 to IMPLICIT[0] */
    cp+=i; tlen+=i;
  }

  if(exclude){
    if(ExtSubT_toDER_opt(exclude,cp,&i)==NULL) goto done;
    *cp=0xa1; /* 0x30 to IMPLICIT[1] */
    cp+=i; tlen+=i;
  }
  ASN1_set_sequence(tlen,ret->der,&ret->dlen);

  return (CertExt *)ret;

 done:
  fprintf(stderr,"err in Extnew_name_cons_opt()\n");
  if(ret)CertExt_free((CertExt *)ret);
  return NULL;
}

unsigned char *ExtSubT_toDER_opt(ExtSubTrees *top,unsigned char *buf,int *ret_len){
  ExtSubTrees *now;
  unsigned char *cp,*t,*ret;
  int i,j,k;

  if(buf==NULL){
    if((i=ExtSubT_estimate_der_size(top))<=0)
      return NULL;
    if((ret=(unsigned char *)calloc(i,sizeof(char)))==NULL)
      return NULL;
  }else{
    ret=buf;
  }

  *ret_len=0;
  cp=t=ret;j=k=0;
  for(now=top;now;now=now->next){
    if(ExtGN_DER_gname_printable_noCheck(now->base,cp,&j)){
      goto done;
    }
    cp+=j;

    if(now->minimum>=0){
      ASN1_set_integer(now->minimum,cp,&i);*cp=0x80; /* implicit [0]*/
      cp+=i;j+=i;
    }    

    if(now->maximum>=0){
      ASN1_set_integer(now->maximum,cp,&i);*cp=0x81; /* implicit [1] */
      cp+=i;j+=i;
    }
    ASN1_set_sequence(j,t,&j);
    t+=j; k+=j; cp=t;
  }
  ASN1_set_sequence(k,ret,ret_len);
  return ret;

 done:
  fprintf(stderr,"err in ExtSubT_toDER_opt()\n");
  if(ret!=buf)free(ret);
  return NULL;
}


/*-----------------------------------------
  Get certificate DER from Cert (for different time type)
-----------------------------------------*/
unsigned char *Cert_toDER_opt(Cert *ct,Key *prv,unsigned char *buf,int *ret_len){
	unsigned char *cp,*ret;
	int	i,j;

	if(buf==NULL){
		if((i=Cert_estimate_der_size(ct))<=0)
			return NULL;

		if((ret=(unsigned char*)MALLOC(i))==NULL){
			OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509CERT,ERR_PT_CERTASN1,NULL);
			return NULL;
		}
	}else{
		ret=buf;
	}
	if(Cert_set_sigalgo(ct,prv)) goto error;

	if(Cert_DER_data_opt(ct,ret,&i)) goto error;

	if(x509_set_signature(ret,prv,&ct->signature,&ct->siglen)) goto error;
	cp = ret+i;

	if(x509_DER_algoid(ct->signature_algo,NULL,cp,&j)) goto error;
	cp+=j; i+=j;

	ASN1_set_bitstring(0,ct->siglen,ct->signature,cp,&j);
	i+=j;

	ASN1_set_sequence(i,ret,ret_len);
	return ret;
error:
	fprintf(stderr,"err in Cert_toDER_opt()\n");
	if(ret!=buf) FREE(ret);
	return NULL;
}

/*-----------------------------------------
  Get certificate data DER from cert(for different time type)
-----------------------------------------*/
int Cert_DER_data_opt(Cert *ct,unsigned char *ret,int *ret_len){
	unsigned char *cp,*sq;
	int i,j,k,err=-1;

	ASN1_set_integer(ct->version,ret,&i);
	ASN1_set_explicit(i,0,ret,&i);
	cp = ret+i;

	if(ct->long_sn){
		/* set long integer */
		j = ASN1_tlen(ct->long_sn);
		ASN1_set_binary(ASN1_INTEGER,j,&ct->long_sn[2],cp,&j);
	}else{
		ASN1_set_integer(ct->serialNumber,cp,&j);
	}
	cp+=j; i+=j;

	if(x509_DER_algoid(ct->signature_algo,NULL,cp,&j)) goto done;
	cp+=j; i+=j;
	#ifdef __PRINTABLE_STRING_NOCHECK__
	if(Cert_DER_subject_printable_noCheck(&(ct->issuer_dn),cp,&j)) goto done;
	#else
	if(Cert_DER_subject(&(ct->issuer_dn),cp,&j)) goto done;
	#endif
	sq=cp+j; i+=j;
  
	asn1_time_tag=asn1_before_tag;
	if(Cert_DER_time(&ct->time.notBefore,sq,&j)) goto done;
	cp=sq+j;
	asn1_time_tag=asn1_after_tag;
	if(Cert_DER_time(&ct->time.notAfter,cp,&k)) goto done;
	j+=k;
	ASN1_set_sequence(j,sq,&j);
	cp=sq+j; i+=j;

	#ifdef __PRINTABLE_STRING_NOCHECK__
	if(Cert_DER_subject_printable_noCheck(&(ct->subject_dn),cp,&j)) goto done;
	#else
	if(Cert_DER_subject(&(ct->subject_dn),cp,&j)) goto done;
	#endif
	cp+=j; i+=j;

	if(x509_DER_pubkey(ct->pubkey,cp,&j)) goto done;
	cp+=j; i+=j;

	if(ct->version){ /* version should be 3 or more */
		if(Cert_DER_certext(ct,cp,&j)) goto done;
		i+=j;
	}
	ASN1_set_sequence(i,ret,ret_len);
	err=0;
 done:
	if(err)fprintf(stderr,"err in set Cert_DER_data_opt()\n");
	return err;
}

CertExt *Extnew_PrvExt(Opt_PrvExt *pext){
  CertExt *ret;
  int err=-1;

  if((ret=(CertExt *)calloc(1,sizeof(CertExt)))==NULL)
    goto done;

  ret->extnID=0;
  if((ret->objid=(unsigned char *)calloc(64,sizeof(char)))==NULL)
    goto done;
  
  if((str2objid(pext->oid,ret->objid,32))<0)
    goto done;

  if(pext->critical==CRITICAL){
    ret->critical=CRITICAL;
  }

  if((ret->der=str2hex(pext->value,&(ret->dlen)))==NULL)
    goto done;

  err=0;

 done:
  if(err&&ret){free(ret);ret=NULL;}
  return ret;
}
