#ifndef __CERT_OPT_H__
#define __CERT_OPT_H__

#include"ok_x509.h"
#include"opt.h"
/* Authority Key Identifier */
typedef struct Authority_Key_Identifier{
  int critical;
  char keyID[64];
  CertDN certIssuer;
  int certIssuer_type;
  char snum[64];
  int snum_type;
}Opt_AuthKeyID;

/* KeyUsage */
typedef struct certificate_ext_KeyUsage{
  int critical;
  unsigned char flag;
}Opt_KeyUsage;

/* subject key identifier */
typedef struct certificate_ext_subjectKeyIdentifier{
  int critical;
  char sub_keyID[64];
}Opt_SubKeyID;

/* Certificate Policy */
typedef struct certificate_ext_CertificatePolicy{
  int critical;
  ExtCertPol *plist;
}Opt_CertPol;

/* Policy Mapping */
typedef struct pair_certificate_policy PairOID;
struct pair_certificate_policy{
  char issPolicy[MAX_LEN];
  char subPolicy[MAX_LEN];
  PairOID *next;
};
typedef struct certificate_ext_poilcyMappings{
  int critical;
  PairOID *oids;
}Opt_PolicyMap;

/* SubjectAltName,IssuerAltName */
typedef struct certificate_ext_GeneralName{
  int type; /* SubjectAltName or IssuerAltName */
  int critical;
  ExtGenNames *gname;
}Opt_GNames;

/* basic constraints */
typedef struct certificate_ext_Basic_Constraints{
  int critical;
  int ca;  /*TRUE or FALSE */
  int path_len;
}Opt_BConst;

/* name constraints */
typedef struct certificate_ext_NameConstraints{
  int critical;
  ExtSubTrees *ptree;
  ExtSubTrees *etree;
}Opt_NConst;
  
/* policy constraints */
typedef struct certificate_ext_plicyConstraints{
  int critical;
  int req;
  int inhibit;
}Opt_PConst;

/* extKeyUsage */
typedef struct certificate_ext_extKeyUsage{
  int critical;
  ExtCertPol *plist;
}Opt_ExtKUsage;

/* CRL Distribution points */
typedef struct certificate_ext_CRLDistributionPoints{
  int critical;
  ExtGenNames *crldp;
}Opt_CRLDP;

/* authorityInfoAccess */
typedef struct certificate_ext_authorityInfoAccess{
  int critical;
  int accessMethod;
  char *uri;
}Opt_AIA;

/* ocsp nocheck */
typedef struct certificate_ext_ocsp_nocheck{
  int critical;
}Opt_OCSP_nocheck;

/* private extension */
typedef struct certificate_ext_private_extensions Opt_PrvExt;
struct certificate_ext_private_extensions{
  int critical;
  char *oid;
  char *value;
  Opt_PrvExt *next;
};


/*----------------------------
  File conf data
  --------------------------*/
typedef struct Certificate_options{
  /* Basic Certificate Fields */
  int snum_type;
  char snum[64];
  int issuer_type;
  CertDN issuer;
  int before_type;
  int after_type;
  char notBefore[16];
  char notAfter[16];
  int subject_type;
  CertDN subject;
  /* Certificate Extensions  */
  Opt_AuthKeyID *auth_kid;
  Opt_KeyUsage *keyUsage;
  Opt_SubKeyID *skid;
  Opt_CertPol *ct_pol;
  Opt_PolicyMap *pmap;
  Opt_GNames *sub_alt_name;
  Opt_GNames *iss_alt_name;
  Opt_BConst *bconst;
  Opt_NConst *nconst;
  Opt_PConst *pconst;
  Opt_ExtKUsage *ext_kusage;
  Opt_CRLDP *crl_dp;
  Opt_AIA *aia;
  Opt_OCSP_nocheck *ncheck;
  Opt_PrvExt *pext;

  /*------------------------
    options
    -----------------------*/
  char caprvkey[MAX_LEN];
  char pemout[MAX_LEN]; /* tmp for check signature */
  char targetkey[MAX_LEN];
  int hash_algo;
  char conf[MAX_LEN];
  char cert_out[MAX_LEN];
  int text;
}CertOptions;

/* cert_opt.c */
void cert_opt_init(CertOptions *opt);
void cert_opt_print(CertOptions *opt);
unsigned char set_key_usage(char *in);

Opt_ExtKUsage *set_Opt_Ext_KeyUsage(char *in);
Opt_CertPol *set_Opt_Cert_Pol(char *in);
void print_PolicyList(ExtCertPol *top);
void free_PolicyList(ExtCertPol *top);
ExtCertPol *set_PolicyList(char *in);

Opt_PolicyMap *set_Opt_PolicyMap(char *in);
PairOID *set_PairOID_List(char *in);
PairOID *set_PairOID(char *in);
void free_Opt_PolicyMap(Opt_PolicyMap *pmap);
void free_PairOID_List(PairOID *top);
void print_PairOID_List(PairOID *top);
ExtSubTrees *ExtSubTrees_new(void);
ExtSubTrees *set_ExtSubTrees(char *in);
ExtGenNames *set_ExtGenNames(char *in,ExtGenNames *gname,int gn_type);
ExtGenNames *set_CRL_DP(char *in,ExtGenNames *gname,int dp_type);
Opt_PrvExt *set_PrvExt(Opt_PrvExt *pext,int critical,char *oid,char *value);
void print_PrvExt(Opt_PrvExt *top);

int cert_line_opt_check(char *line,CertOptions *opt);
/* section */
#define SECTION_NULL                      0x00
#define SECTION_CERT_BASIC_CERT           0x40
#define SECTION_CERT_EXT                  0x80
#define SECTION_CERT_EXT_AUTH_KEY_ID      0x81
#define SECTION_CERT_EXT_CERT_POLICY      0x82
#define SECTION_CERT_EXT_SUB_ALT_NAME     0x83
#define SECTION_CERT_EXT_ISSUER_ALT_NAME  0x84
#define SECTION_CERT_EXT_BASIC_CONST      0x85
#define SECTION_CERT_EXT_NAME_CONST       0x86
#define SECTION_CERT_EXT_POLICY_CONST     0x87
#define SECTION_CERT_EXT_CRL_DP           0x88
#define SECTION_CERT_EXT_AUTH_INFO_ACCESS 0x89

#define AIA_ACCESS_OCSP			  0x01
#define AIA_ACCESS_CAISSUERS		  0x02

extern unsigned char cert_section;
int set_cert_section(char *in,unsigned char *now);


/* cert_set.c */
Cert *set_Cert(CertOptions *opt,int *ret_len);
Key *set_prvkey(char *prvkey);
Key *set_pubkey(char *pubkey);
int set_sig_algo(int hash_algo,int prvkey_type);
unsigned char *set_cert_serial(char *in,int snum_type);
CertExt *option2CertExt(CertOptions *opt);
CertExt *Extnew_authkey_id_opt(Opt_AuthKeyID *aut_kid);
CertExt *Extnew_sbjkey_id_opt(Opt_SubKeyID *skid);
CertExt *Extnew_policyMap_opt(Opt_PolicyMap *pmap);
CertExt *Extnew_cert_policyList(int ext_type,ExtCertPol *top);
int PolicyList_estimate_der_size(ExtCertPol *top);
int PolicyList_toDER(ExtCertPol *in,int ext_type,unsigned char *ret,int *ret_len);
int PairPolicy_toDER(PairOID *in,unsigned char *ret,int *ret_len);
CertExt *Extnew_name_cons_opt(ExtSubTrees *permit,ExtSubTrees *exclude);
unsigned char *ExtSubT_toDER_opt(ExtSubTrees *top,unsigned char *buf,int *ret_len);
/* for different validity time type */
unsigned char *Cert_toDER_opt(Cert *ct,Key *prv,unsigned char *buf,int *ret_len);
int Cert_DER_data_opt(Cert *ct,unsigned char *ret,int *ret_len);
CertExt *Extnew_PrvExt(Opt_PrvExt *pext);

#endif /* __CERT_OPT_H__ */
