#ifndef __OCSP_RES_H__
#define __OCSP_RES_H__

#include "ocsp_req.h" 
#include<stdio.h>
#include"ok_x509.h"

/* ResponseStatus */
#define SUCCESSFUL 0
#define MALFORMEDREQUEST 1
#define INTERNALERROR 2
#define TRYLATER 3
#define SIGREQUIRED 4
#define UNAUTHORIZED 5

/*Cert status */
#define GOOD 0
#define REVOKED 1
#define UNKNOWN 2

/* OID for ocsp */
#define OBJ_OCSP_BASIC        9101
#define OBJ_OCSP_NONCE        9102
#define OBJ_OCSP_CRL          9103
#define OBJ_OCSP_RESPONSE     5104
#define OBJ_OCSP_NOCHECK      9105
#define OBJ_OCSP_ARCH_CUTOFF  9106
#define OBJ_OCSP_SERV_LOCATOR 9107
/*OID for GPKI */
#define OBJ_GPKI_SUBSCRIBERCERT		    8201
#define OBJ_GPKI_INTERMEDIATECERTS	    8202
#define OBJ_GPKI_TRUSTANCHORCERT	    8203
#define OBJ_GPKI_REQUIREDPOLICY		    8204
#define OBJ_GPKI_REQUIRED_EXPLICIT_POLICY   8205
#define OBJ_GPKI_INHIBIT_POLICY_MAPPING	    8206
#define OBJ_GPKI_RESPONSEFORMAT		    8207
#define OBJ_GPKI_CERTPATHSTATUS		    8208
#define OBJ_GPKI_CERTPATH		    8209
#define OBJ_GPKI_REVOCATIONLIST		    8210
#define OBJ_GPKI_OCSPRESPONSE		    8211
#define OBJ_GPKI_MAPPEDPOLICY		    8212
/* OID for */
#define OBJ_ID_REGCERT_MG_GENMINFOREQ           9108 
#define OBJ_ID_REGCERT_MG_GENPINFORES           9109
#define OBJ_ID_REGCERT_MG_CONFIRMATIONTIME      9110
#define OBJ_ID_REGCERT_MG_OCSPSTATUSCODE        9111
#define OBJ_ID_REGCERT_MG_EFFECTIVETIMELIMIT    9112
#define OBJ_ID_REGCERT_MG_SUSPENSIONSECRETCODE  9113
/* certPathStatus For CVS */
#define CERT_PATH_GOOD                    0
#define CERT_PATH_CONSTRUCTIONERR       101
#define CERT_PATH_INACCURATE_CERT       202
#define CERT_PATH_REVOKED_CERT          203
#define CERT_PATH_POLICY_DISAGREEMENT   204
#define CERT_PATH_RESTRICTION_VIOLATION 205
#define CERT_PATH_UNKNOWN_CERT          206
#define CERT_PATH_SERVER_REFUSE         901
#define CERT_PATH_REQ_TIMEOUT           902

typedef struct revokedInfo{
  struct tm revTime;
  int rreason;
}RevInfo;

typedef struct CertStatus{
  int state;
  RevInfo *rev;
}CertState;

typedef struct singleResponse{
  CertID *cid;
  CertState *cstate;
  struct tm thisUpdate;
  struct tm nextUpdate;
  CertExt *singleExt;
}SingleResp;

typedef struct Responses Resps;
struct Responses{
  SingleResp *sresp;
  Resps *next;
};

typedef struct ResponderID{
  int type;              /* byname=0,bykey=1 */
  int ctype;
  CertDN subject_dn;
  char *subject;
  unsigned char *key_hash;
}RespID;

typedef struct ResponseData{
  int version; /* default v1(0) */
  RespID *rid;
  struct tm producedAt;
  Resps *resps;
  CertExt *respExt;
}TBSresp;

typedef struct BasicOCSPresponse{
  TBSresp *tresp;
  int sig_algo;
  unsigned char *signature;
  int siglen;
  char *rkey;
  Cert*certs;
  int ct_len;
  unsigned char *ct_der;
}BaseResp;

typedef struct ResponseBytes{
  int respType;
  BaseResp *bresp;
}RespBytes;

typedef struct OCSPResponse{
  int respState;
  RespBytes *rbytes;
}OCSPresp;

/* res_set.c */
OCSPresp *set_OCSPresp(OCSPOptions *opt);
RespBytes *set_RespBytes(OCSPOptions *opt);
BaseResp *set_BaseResp(OCSPOptions *opt);
TBSresp *set_TBSresp(OCSPOptions *opt);
RespID *set_RespID(OCSPOptions *opt);
Resps *set_Resps(OCSPOptions *opt);
SingleResp *set_SingleResp(OCSPOptions *opt);
CertState *set_CertState(OCSPOptions *opt);
RevInfo *set_RevInfo(OCSPOptions *opt);
/* ocsp_obj.c */
int ASN1_int_2object_ocsp(int obj,unsigned char *ret,int *ret_len);
int ASN1_object_2int_ocsp(unsigned char *in);
/* res_der.c */
int ASN1_set_OCSPresp(OCSPresp *oresp,unsigned char *ret,int *ret_len);
int ASN1_set_TBSresp(TBSresp *tresp,unsigned char *ret,int *ret_len);
int ASN1_set_RespID(RespID *rid,unsigned char *ret,int *ret_len);
int ASN1_set_Resps(Resps *resps,unsigned char *ret,int *ret_len);
int ASN1_set_SingleResp(SingleResp *sresp,unsigned char *ret,int *ret_len);
int ASN1_set_CertState(CertState *cstate,unsigned char *ret,int *ret_len);
int ASN1_set_RevInfo(RevInfo *rinfo, unsigned char *ret,int *ret_len);
int ASN1_set_time(struct tm *time,int type,unsigned char *ret,int *ret_len);
void ASN1_mset_implicit(int len,char num,unsigned char *der,int *ret_len);

#endif /* __OCSP_RES_H__ */
