#include<stdio.h>
#include<string.h>
#include"ok_asn1.h"
#include"cert_opt.h"
#include"opt.h"

void cert_opt_init(CertOptions *opt){
  opt->snum_type=UNSET;
  opt->issuer_type=UNSET;
  opt->after_type=UNSET;
  opt->before_type=UNSET;
  opt->subject_type=UNSET;
  opt->hash_algo=OBJ_HASH_SHA1;
  opt->text=UNSET;
  sprintf(opt->conf,"./certvalue.cf",NULL);
}

void cert_opt_print(CertOptions *opt){
  printf("### set options list ###\n");
  printf("snum_type = %d\n",opt->snum_type);
  if(*opt->snum)printf("snum = %s\n",opt->snum);
  printf("issuer_type = %d\n",opt->issuer_type);
  cert_dn_print(&(opt->issuer));
  if(*opt->notBefore)printf("notBefore = %s\n",opt->notBefore);
  if(*opt->notAfter)printf("notAfter = %s\n",opt->notAfter);
  printf("subject_type = %d\n",opt->subject_type);
  cert_dn_print(&(opt->subject));
}

int cert_line_opt_check(char *line,CertOptions *opt){
  int err=-1,len,gn_type=-1;
  char *hp,*lp;
  int critical;
  
  if((hp=strchr(line,'#'))!=NULL){
    *hp='\0';
  }

  switch(cert_section){
  case SECTION_CERT_BASIC_CERT:
    if(strstr(line,"issuer=")!=NULL){
      if(set_dn(line,&(opt->issuer),&(opt->issuer_type)))
	goto done;
    }else if((hp=strstr(line,"subject="))!=NULL){
      hp+=strlen("subject=");
      if(set_dn(hp,&(opt->subject),&(opt->subject_type)))
	goto done;
    }else if(strstr(line,"validity=")!=NULL){
      if(strstr(line,"UTC")){
	opt->before_type=ASN1_UTCTIME;
	opt->after_type=ASN1_UTCTIME;	
      }else if(strstr(line,"Generalized")){
	opt->before_type=ASN1_GENERALIZEDTIME;
	opt->after_type=ASN1_GENERALIZEDTIME;
      }
      if(hp=(char *)strchr(line,':')){
	if(lp=(char *)strchr(hp,',')){
	  strncpy(opt->notAfter,lp+1,16);
	  *lp='\0';
	  strncpy(opt->notBefore,hp+1,16);
	}
      }
    }else if(strstr(line,"notBefore=")!=NULL){
      if(strstr(line,"UTC")){
	opt->before_type=ASN1_UTCTIME;
      }else if(strstr(line,"Generalized")){
	opt->before_type=ASN1_GENERALIZEDTIME;
      }
      if(hp=(char *)strchr(line,':')){
	strncpy(opt->notBefore,hp+1,16);
      }
    }else if(strstr(line,"notAfter=")!=NULL){
      if(strstr(line,"UTC")){
	opt->after_type=ASN1_UTCTIME;
      }else if(strstr(line,"Generalized")){
	opt->after_type=ASN1_GENERALIZEDTIME;
      }
      if(hp=(char *)strchr(line,':')){
	strncpy(opt->notAfter,hp+1,16);
      }
    }else if(strstr(line,"serialNumber=")!=NULL){
      if(set_serial(line,opt->snum,&(opt->snum_type)))
	goto done;
    }else if(strstr(line,"version=")!=NULL){
      /* cert version is fixed to  3(2) */
    }else{
      goto done;
    }
    break;

  case SECTION_CERT_EXT:
    if((hp=strstr(line,"SubjectKeyIdentifier="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      if((opt->skid=(Opt_SubKeyID *)calloc(1,sizeof(Opt_SubKeyID)))==NULL)
	goto done;
      if((opt->skid->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      strncpy(opt->skid->sub_keyID,hp+2,63);
    }else if((hp=strstr(line,"extKeyUsage="))!=NULL){
      if((opt->ext_kusage=(Opt_ExtKUsage *)set_Opt_Ext_KeyUsage(line))==NULL)
	goto done;
    }else if((hp=strstr(line,"KeyUsage="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      if((opt->keyUsage=(Opt_KeyUsage *)calloc(1,sizeof(Opt_KeyUsage)))==NULL)
	goto done;
      if((opt->keyUsage->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      if((opt->keyUsage->flag=(unsigned char)set_key_usage(hp))==0x00)
	goto done;
    }else if((hp=strstr(line,"PolicyMappings="))!=NULL){
      if((opt->pmap=(Opt_PolicyMap *)set_Opt_PolicyMap(hp))==NULL)
	goto done;
    }else if((hp=strstr(line,"ocsp-nocheck="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      if((opt->ncheck=(Opt_OCSP_nocheck *)calloc(1,sizeof(Opt_OCSP_nocheck)))==NULL)
	goto done;
      hp+=1;
      if((opt->ncheck->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      if(strstr(hp,"TRUE")==NULL){
	free(opt->ncheck);
	opt->ncheck=NULL;
      }
    }else{
      if((hp=strchr(line,'.'))==NULL)
	goto done;
      switch(*(--hp)){
      case '0':
      case '1':
      case '2':
	/* Private Extensions */
	critical=UNSET;
	if((hp=strchr(line,'='))==NULL)
	  goto done;
	hp+=1;
	if((critical=set_critical(*hp))==UNSET)
	  goto done;
	lp=hp+2;
	*(hp-1)='\0';
	if((opt->pext=set_PrvExt(opt->pext,critical,line,lp))==NULL)
	  goto done;
	break;
      default:
	goto done;
      }
    }
    break;

  case SECTION_CERT_EXT_AUTH_KEY_ID:
    if(opt->auth_kid==NULL){
      if((opt->auth_kid=(Opt_AuthKeyID*)calloc(1,sizeof(Opt_AuthKeyID)))==NULL)
	goto done;
      opt->auth_kid->critical=UNSET;
      opt->auth_kid->snum_type=UNSET;
      opt->auth_kid->certIssuer_type=UNSET;
    }

    if((hp=strstr(line,"authorityCertIssuer="))!=NULL){
      if((hp=strchr(line,'='))==NULL)
	goto done;
      if((opt->auth_kid->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      if(set_dn(hp+2,&(opt->auth_kid->certIssuer),&(opt->auth_kid->certIssuer_type)))
	goto done;
    }else if((hp=strstr(line,"keyIdentifier="))!=NULL){
      if((hp=strchr(line,'='))==NULL)
	goto done;
      if((opt->auth_kid->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      strncpy(opt->auth_kid->keyID,hp+2,63);
    }else if((hp=strstr(line,"authorityCertIssuer="))!=NULL){
      if((hp=strchr(line,'='))==NULL)
	goto done;
      if((opt->auth_kid->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      if(set_dn(hp+2,&(opt->auth_kid->certIssuer),&(opt->auth_kid->certIssuer_type)))
	goto done;
    }else if((hp=strstr(line,"authorityCertSerialNumber="))!=NULL){
      if((hp=strchr(line,'='))==NULL)
	goto done;
      if((opt->auth_kid->critical=set_critical(*(++hp)))==UNSET)
	goto done;
      if(set_serial(hp+2,opt->auth_kid->snum,&(opt->auth_kid->snum_type)))
	goto done;
    }else{
      goto done;
    }

    break;

  case SECTION_CERT_EXT_CERT_POLICY:
    if((opt->ct_pol=(Opt_CertPol *)set_Opt_Cert_Pol(line))==NULL)
      goto done;
    break;

  case SECTION_CERT_EXT_SUB_ALT_NAME:
    if(opt->sub_alt_name==NULL){
      if((opt->sub_alt_name=(Opt_GNames *)calloc(1,sizeof(Opt_GNames)))==NULL)
	goto done;
      opt->sub_alt_name->type=OBJ_X509v3_SbjAltName;
      opt->sub_alt_name->critical=UNSET;
      opt->sub_alt_name->gname=NULL;
    }

    if((hp=strstr(line,"directoryName="))!=NULL){
      gn_type=4; /* explicit [4]*/
      hp+=strlen("directoryName=");
      if((opt->sub_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else if((hp=strstr(line,"rfc822Name="))!=NULL){
      gn_type=1; /* explicit [1] */
      hp+=strlen("rfc822Name=");
      if((opt->sub_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else if((hp=strstr(line,"dNSName="))!=NULL){
      gn_type=2; /* explicit [2] */
      hp+=strlen("dNSName=");
      if((opt->sub_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;

    }else if((hp=strstr(line,"iPAddress="))!=NULL){
      gn_type=7; /* explicit [7]*/
      hp+=strlen("iPAddress=");
      if((opt->sub_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else{
      goto done;
    }
    hp+=2;
    
    switch(gn_type){
    case 1:
    case 2:
    case 4:
    case 7:
      if((opt->sub_alt_name->gname=(ExtGenNames *)set_ExtGenNames(hp,opt->sub_alt_name->gname,gn_type))==NULL)
	goto done;
      break;
    default:
      goto done;
    }
    break;

  case SECTION_CERT_EXT_ISSUER_ALT_NAME:
    if(opt->iss_alt_name==NULL){
      if((opt->iss_alt_name=(Opt_GNames *)calloc(1,sizeof(Opt_GNames)))==NULL)
	goto done;
      opt->iss_alt_name->type=OBJ_X509v3_IssAltName;
      opt->iss_alt_name->critical=UNSET;
      opt->iss_alt_name->gname=NULL;
    }

    if((hp=strstr(line,"directoryName="))!=NULL){
      gn_type=4; /* explicit [4]*/
      hp+=strlen("directoryName=");
      if((opt->iss_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else if((hp=strstr(line,"rfc822Name="))!=NULL){
      gn_type=1; /* explicit [1] */
      hp+=strlen("rfc822Name=");
      if((opt->iss_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else if((hp=strstr(line,"dNSName="))!=NULL){
      gn_type=2; /* explicit [2] */
      hp+=strlen("dNSName=");
      if((opt->iss_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else if((hp=strstr(line,"iPAddress="))!=NULL){
      gn_type=7; /* explicit [7]*/
      hp+=strlen("iPAddress=");
      if((opt->iss_alt_name->critical=set_critical(*hp))==UNSET)
	goto done;
    }else{
      goto done;
    }
    hp+=2;

    switch(gn_type){
    case 1:
    case 2:
    case 4:
    case 7:
      if((opt->iss_alt_name->gname=(ExtGenNames *)set_ExtGenNames(hp,opt->iss_alt_name->gname,gn_type))==NULL)
	goto done;
      break;
    default:
      goto done;
    }
    break;

  case SECTION_CERT_EXT_BASIC_CONST:
    if(opt->bconst==NULL){
      if((opt->bconst=(Opt_BConst *)calloc(1,sizeof(Opt_BConst)))==NULL)
	goto done;
      opt->bconst->critical=UNSET;
      opt->bconst->ca=FALSE;
      opt->bconst->path_len=UNSET;
    }

    if((hp=strstr(line,"cA="))!=NULL){
      hp+=strlen("cA=");
      if((opt->bconst->critical=set_critical(*hp))==UNSET)
	goto done;
      if(strstr(hp,"TRUE")){
	opt->bconst->ca=TRUE;
      }else if(strstr(hp,"FALSE")){
	opt->bconst->ca=FALSE;
      }else{
	goto done;
      }
    }else if((hp=strstr(line,"pathLenConstraint="))!=NULL){
      hp+=strlen("pathLenConstraint=");
      if((opt->bconst->critical=set_critical(*hp))==UNSET)
	goto done;
      opt->bconst->path_len=atoi(hp+2);
    }else{
      goto done;
    }
    break;

  case SECTION_CERT_EXT_NAME_CONST:
    if(opt->nconst==NULL){
      if((opt->nconst=(Opt_NConst *)calloc(1,sizeof(Opt_NConst)))==NULL)
	goto done;
      opt->nconst->critical=UNSET;
      opt->nconst->ptree=NULL;
      opt->nconst->etree=NULL;
    }
    if((hp=strstr(line,"permittedSubtrees="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->nconst->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      if((opt->nconst->ptree=(ExtSubTrees *)set_ExtSubTrees(hp))==NULL)
	goto done;
    }else if((hp=strstr(line,"excludedSubtrees="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->nconst->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      if((opt->nconst->etree=(ExtSubTrees *)set_ExtSubTrees(hp))==NULL)
	goto done;
    }else{
      goto done;
    }
    break;

  case SECTION_CERT_EXT_POLICY_CONST:
    if(opt->pconst==NULL){
      if((opt->pconst=(Opt_PConst *)calloc(1,sizeof(Opt_PConst)))==NULL)
	goto done;
      opt->pconst->critical=UNSET;
      opt->pconst->req=UNSET;
      opt->pconst->inhibit=UNSET;
    }
    if((hp=strstr(line,"requireExplicitPolicy="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->pconst->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      opt->pconst->req=atoi(hp);
    }else if((hp=strstr(line,"inhibitPolicyMapping="))!=NULL){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->pconst->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      opt->pconst->inhibit=atoi(hp);
    }else{
      goto done;
    }
    break;

  case SECTION_CERT_EXT_CRL_DP:
    if(opt->crl_dp==NULL){
      if((opt->crl_dp=(Opt_CRLDP *)calloc(1,sizeof(Opt_CRLDP)))==NULL)
	goto done;
      opt->crl_dp->critical=UNSET;
      opt->crl_dp->crldp=NULL;
    }
    if((hp=strstr(line,"directoryName="))!=NULL){
      gn_type=4; /* GeneralName type is directoryName */
    }else if((hp=strstr(line,"URI="))!=NULL){
      gn_type=6; /* GeneralName type is URI */
    }else{
      goto done;
    }
    
    if(gn_type==4||gn_type==6){
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->crl_dp->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      if((opt->crl_dp->crldp=(ExtGenNames *)set_CRL_DP(hp,opt->crl_dp->crldp,gn_type))==NULL)
	goto done;

    }
    break;

  case SECTION_CERT_EXT_AUTH_INFO_ACCESS:
    if((hp=strstr(line,"ocsp="))!=NULL){
      if(opt->aia==NULL){
	if((opt->aia=(Opt_AIA *)calloc(1,sizeof(Opt_AIA)))==NULL)
	  goto done;
	opt->aia->critical=UNSET;
	opt->aia->accessMethod=UNSET;
	opt->aia->uri=NULL;
      }
      opt->aia->accessMethod = AIA_ACCESS_OCSP;
      if((hp=strchr(hp,'='))==NULL)
 	goto done;
      hp+=1;
      if((opt->aia->critical=set_critical(*hp))==UNSET)
 	goto done;
      hp+=2;
      len=strlen(hp);
      if((opt->aia->uri=(char *)calloc(len+1,sizeof(char)))==NULL)
 	goto done;
      strncpy(opt->aia->uri,hp,len);
    } else if((hp=strstr(line,"caIssuers="))!=NULL){
      if(opt->aia==NULL){
 	if((opt->aia=(Opt_AIA *)calloc(1,sizeof(Opt_AIA)))==NULL)
 	  goto done;
 	opt->aia->critical=UNSET;
 	opt->aia->accessMethod=UNSET;
 	opt->aia->uri=NULL;
      }
      opt->aia->accessMethod = AIA_ACCESS_CAISSUERS;
      if((hp=strchr(hp,'='))==NULL)
	goto done;
      hp+=1;
      if((opt->aia->critical=set_critical(*hp))==UNSET)
	goto done;
      hp+=2;
      len=strlen(hp);
      if((opt->aia->uri=(char *)calloc(len+1,sizeof(char)))==NULL)
	goto done;
      strncpy(opt->aia->uri,hp,len);
    }else{
      goto done;
    }
    break;

  default:
    goto done;
  }
  err=0;

 done:
  if(err)fprintf(stderr,"err in check cert line=[\"%s\"]\n",line);
  return err;
}

unsigned char set_key_usage(char *in){
  unsigned char ret=0x00;

  if(strstr(in,"digitalSignature")!=NULL){
    ret|=0x80;
  }
  if(strstr(in,"nonRepudiation")!=NULL){
    ret|=0x40;
  }
  if(strstr(in,"keyEncipherment")!=NULL){
    ret|=0x20;
  }
  if(strstr(in,"dataEncipherment")!=NULL){
    ret|=0x10;
  }
  if(strstr(in,"keyAgreement")!=NULL){
    ret|=0x08;
  }
  if(strstr(in,"keyCertSign")!=NULL){
    ret|=0x04;
  }
  if(strstr(in,"cRLSign")!=NULL){
    ret|=0x02;
  }

  return ret;
}

Opt_ExtKUsage *set_Opt_Ext_KeyUsage(char *in){
  Opt_ExtKUsage *ret;
  char *hp;
  int err=-1;

  if((ret=(Opt_ExtKUsage *)calloc(1,sizeof(Opt_ExtKUsage)))==NULL)
    goto done;
  if((hp=strchr(in,'='))==NULL)
    goto done;
  hp+=1;
  if((ret->critical=set_critical(*hp))==UNSET)
    goto done;
  hp+=2;
  if((ret->plist=(ExtCertPol *)set_PolicyList(hp))==NULL)
    goto done;

  err=0;
 done:
  if(err){
    if(ret->plist)free_PolicyList(ret->plist);
    if(ret)free(ret);
    ret=NULL;
  }
  return ret;
}

Opt_CertPol *set_Opt_Cert_Pol(char *in){
  Opt_CertPol *ret;
  char *hp;
  int err=-1;

  if((ret=(Opt_CertPol *)calloc(1,sizeof(Opt_CertPol)))==NULL)
    goto done;

  if((hp=strchr(in,'='))==NULL)
    goto done;

  hp+=1;
  if((ret->critical=set_critical(*hp))==UNSET)
    goto done;
  hp+=2;
  if((ret->plist=(ExtCertPol *)set_PolicyList(hp))==NULL)
    goto done;

  err=0;

 done:
  if(err){
    if(ret->plist)free_PolicyList(ret->plist);
    if(ret)free(ret);
    ret=NULL;
  }
  return ret;
}

ExtCertPol *set_PolicyList(char *in){
  int flag,len,err=-1;
  ExtCertPol *ret,*plist;
  char *hp,*hpp;
  char *anyPolicy="2.5.29.32.0";

  if((plist=(ExtCertPol *)calloc(1,sizeof(ExtCertPol)))==NULL)
    goto done;
  plist->next=NULL;
  plist->info=NULL;
  ret=plist;

  for(flag=0;flag!=1;plist=plist->next){
    if((hp=strrchr(in,0x09))==NULL){
      hpp=in;
      flag=1;
    }else{
      hpp=hp+1;
    }
    if((plist->next=(ExtCertPol *)calloc(1,sizeof(ExtCertPol)))==NULL)
      goto done;
    plist->next->info=NULL;
    if(!strcmp(hpp,"anyPolicy")){
      len=strlen(anyPolicy);
      if((plist->next->policyID=(char *)calloc(len+1,sizeof(char)))==NULL)
	goto done;
      strncpy(plist->next->policyID,anyPolicy,len);
    }else{
      len=strlen(hpp);
      if((plist->next->policyID=(char *)calloc(len+1,sizeof(char)))==NULL)
	goto done;
      strncpy(plist->next->policyID,hpp,len);
    }
    if(flag==0)*hp='\0';
  }
  ret=ret->next;
  
  err=0;
 done:
  if(err){free_PolicyList(ret);ret=NULL;}
  return ret;
}

void print_PolicyList(ExtCertPol *top){
  ExtCertPol *plist;
  
  for(plist=top;plist!=NULL;plist=plist->next){
    if(plist->policyID){
      printf("%s\n",plist->policyID);
    }
  }
}

void free_PolicyList(ExtCertPol *top){
  ExtCertPol *plist,*next;
  for(plist=top;plist!=NULL;plist=next){
    next=plist->next;
    free(plist);
  }
}


Opt_PolicyMap *set_Opt_PolicyMap(char *in){
  Opt_PolicyMap *ret;
  char *hp;
  int err=-1;

  if((ret=(Opt_PolicyMap *)calloc(1,sizeof(Opt_PolicyMap)))==NULL)
    goto done;
  ret->critical=UNSET;
  ret->oids=NULL;

  if((hp=strchr(in,'='))==NULL)
    goto done;
  hp+=1;
  if((ret->critical=set_critical(*hp))==UNSET)
    goto done;
  hp+=2;
  if((ret->oids=set_PairOID_List(hp))==NULL)
    goto done;

  err=0;
 done:
  if(err&&ret){free_Opt_PolicyMap(ret);ret=NULL;}
  return ret;
}

PairOID *set_PairOID_List(char *in){
  int flag,err=-1;
  PairOID *ret,*poid;
  char *hp,*hpp;

  if((poid=(PairOID *)calloc(1,sizeof(PairOID)))==NULL)
    goto done;
  poid->next=NULL;
  ret=poid;
  
  for(flag=0;flag!=1;poid=poid->next){
    if((hp=strrchr(in,0x09))==NULL){
      hpp=in;
      flag=1;
    }else{
      hpp=hp+1;
    }

    if((poid->next=(PairOID *)set_PairOID(hpp))==NULL)
      goto done;
    if(flag==0)*hp='\0';
  }
  err=0;

  if(ret->next)ret=ret->next;

 done:
  if(err&&ret){free_PairOID_List(ret);ret=NULL;}
  return ret;
}


PairOID *set_PairOID(char *in){
  PairOID *pid=NULL;
  char *hp,*hpp;
  int err=-1;

  if((pid=(PairOID *)calloc(1,sizeof(PairOID)))==NULL)
    goto done;
  if((hp=strchr(in,'='))==NULL)
    goto done;
  
  hpp=hp+1;
  
  strncpy(pid->subPolicy,hpp,MAX_LEN-1);
  *hp='\0';
  strncpy(pid->issPolicy,in,MAX_LEN-1);
  err=0;
  
 done:
  if(err&&pid){free(pid);pid=NULL;}
  return pid;
}

void free_Opt_PolicyMap(Opt_PolicyMap *pmap){
  if(pmap->oids!=NULL)free_PairOID_List(pmap->oids);
  if(pmap)free(pmap);
}

void free_PairOID_List(PairOID *top){
  PairOID *poid,*next;

  for(poid=top;poid!=NULL;poid=next){
    next=poid->next;
    free(poid);
  }
}

void print_PairOID_List(PairOID *top){
  PairOID *poid,*next;
  for(poid=top;poid!=NULL;poid=next){
    next=poid->next;
    if(*poid->issPolicy&&*poid->subPolicy){
      printf("issuerPolicy  %s\n",poid->issPolicy);
      printf("subjectPolicy %s\n",poid->subPolicy);    
    }
  }
}

ExtSubTrees *ExtSubTrees_new(void){
  ExtSubTrees *ret;
  
  if((ret=(ExtSubTrees *)calloc(1,sizeof(ExtSubTrees)))==NULL)
    return NULL;
  ret->base=NULL;
  ret->next=NULL;
  ret->minimum=UNSET;
  ret->maximum=UNSET;
  
  return ret;
}

ExtSubTrees *set_ExtSubTrees(char *in){
  int flag,ctype,err=-1;
  ExtSubTrees *ret,*list;
  char *hp,*hpp;
  CertDN tmp_dn;

  if((list=(ExtSubTrees *)ExtSubTrees_new())==NULL)
    goto done;
  ret=list;

  for(flag=0;flag!=1;list=list->next){
    if((hp=strchr(in,0x09))==NULL){
      hpp=in;
      flag=1;
    }else{
      hpp=hp+1;
    }
    if((list->next=(ExtSubTrees *)ExtSubTrees_new())==NULL)
      goto done;
    cert_dn_init(&tmp_dn);
    if(set_dn(hpp,&tmp_dn,&ctype))
      goto done;
    if((list->next->base=ExtGN_set_dn(&tmp_dn))==NULL)
      goto done;
    if(flag==0)*hp='\0';
  }
  err=0;
  if(ret->next)ret=ret->next;
 done:
  if(err){if(ret)ExtSubT_free_all(ret);ret=NULL;}
  return ret;
}

ExtGenNames *set_ExtGenNames(char *in,ExtGenNames *gname,int gn_type){
  ExtGenNames *ret;
  CertDN tmp_dn;
  int dn_type,err=-1;
  unsigned char ip[16]="";
  
  
  if((ret=(ExtGenNames *)calloc(1,sizeof(ExtGenNames)))==NULL)
    goto done;
  ret->next=NULL;

  switch(gn_type){
  case 1: /* rfc822Name */
  case 2: /* dNSName */
    if((ret=(ExtGenNames *)ExtGN_set_str(in,gn_type))==NULL)
      goto done;
    break;
  case 7: /* iPAddress */
    str2ip(in,ip);
    if((ret=(ExtGenNames *)ExtGN_set_ip(ip,4))==NULL)
      goto done;
    break;
  case 4: /* directoryName */
    cert_dn_init(&tmp_dn);
    if(set_dn(in,&tmp_dn,&dn_type))
      goto done;
    if((ret=(ExtGenNames *)ExtGN_set_dn(&tmp_dn))==NULL)
      goto done;
    break;
  default:
    goto done;
  }

  if(gname!=NULL){
    if((ret->next=(ExtGenNames *)ExtGN_dup_all(gname))==NULL)
      goto done;
  }

  err=0;

 done:
  if(err&&ret){ExtGN_free(ret);ret=NULL;}
  return ret;
}


ExtGenNames *set_CRL_DP(char *in,ExtGenNames *gname,int gn_type){
  int flag,err=-1,dn_type;
  ExtGenNames *ret,*list;
  char *hp,*hpp;
  CertDN dn;
  
  if((list=(ExtGenNames *)calloc(1,sizeof(ExtGenNames)))==NULL)
    goto done;
  list->next=NULL;
  ret=list;
  
  for(flag=0;flag!=1;list=list->next){
    if((hp=strrchr(in,0x09))==NULL){
      hpp=in;
      flag=1;
    }else{
      hpp=hp+1;
    }
    switch(gn_type){
    case 4: /* directoryName */
      cert_dn_init(&dn);
      if(set_dn(hpp,&dn,&dn_type))
	goto done;
      if((list->next=(ExtGenNames *)ExtGN_set_dn(&dn))==NULL)
	goto done; 
      break;
    case 6: /* URI */
      if((list->next=(ExtGenNames *)ExtGN_set_str(hpp,6))==NULL)   
	goto done;
      break;
    default:
      goto done;
    }
    if(flag==0)*hp='\0';
  }
  if(gname!=NULL){
    if((list->next=(ExtGenNames *)ExtGN_dup_all(gname))==NULL)
      goto done;
  }
  if(ret->next){
    ret=ret->next;
  }
    err=0;
  done:
    if(err){if(ret)ExtGN_free(ret);ret=NULL;}
    return ret;
  }


Opt_PrvExt *set_PrvExt(Opt_PrvExt *top,int critical,char *oid,char *value){
  Opt_PrvExt *pext,*pre=NULL,*hd=NULL;
  int err=-1;

  if((pext=(Opt_PrvExt *)calloc(1,sizeof(Opt_PrvExt)))==NULL)
    goto done;
  if((pext->oid=(char *)calloc(strlen(oid)+1,sizeof(char)))==NULL)
    goto done;
  strncpy(pext->oid,oid,strlen(oid));
  if((pext->value=(char *)calloc(strlen(value)+1,sizeof(char)))==NULL)
    goto done;
  strncpy(pext->value,value,strlen(value));

  if((pext->critical=critical)==UNSET)
    goto done;

  if(top==NULL){
    top=pext;
  }else{
    for(hd=top;hd;hd=hd->next){
      pre=hd;
    }
    pre->next=pext;
  }

  err=0;
 done:
  if(err){printf("err in set Private ext \n");free(pext);pext=NULL;}
  return top;
}

void print_PrvExt(Opt_PrvExt *top){
  Opt_PrvExt *pext;

  for(pext=top;pext!=NULL;pext=pext->next){
    printf("oid[%s]\tvalue[%s] ",pext->oid,pext->value);
    printf("%s\n",(pext->critical==CRITICAL)?"Critical":"Non Critical");
  }
} 

/*----------------------
set cert section
-----------------------*/
int set_cert_section(char *in,unsigned char *now){
  unsigned char dif,next=0x00;
  int err=UNSET;
  
  if(strstr(in,"[Basic Certificate Fields")){
    next=SECTION_CERT_BASIC_CERT;
  }else if(strstr(in,"[Certificate Extensions")){
    next=SECTION_CERT_EXT;
  }else if(strstr(in,"[AuthorityKeyIdentifier")){
    next=SECTION_CERT_EXT_AUTH_KEY_ID;
  }else if(strstr(in,"[certificatePolicies")){
    next=SECTION_CERT_EXT_CERT_POLICY;
  }else if(strstr(in,"[SubjectAltName")){
    next=SECTION_CERT_EXT_SUB_ALT_NAME;
  }else if(strstr(in,"[IssuerAltName")){
    next=SECTION_CERT_EXT_ISSUER_ALT_NAME;
  }else if(strstr(in,"[BasicConstraints")){
    next=SECTION_CERT_EXT_BASIC_CONST;
  }else if(strstr(in,"[NameConstraints")){
    next=SECTION_CERT_EXT_NAME_CONST;
  }else if(strstr(in,"[PolicyConstraints")){
    next=SECTION_CERT_EXT_POLICY_CONST;
  }else if(strstr(in,"[CRLDistributionPoints")){
    next=SECTION_CERT_EXT_CRL_DP;
  }else if(strstr(in,"[authorityInfoAccess")){
    next=SECTION_CERT_EXT_AUTH_INFO_ACCESS;
  }else{
    goto done;
  }

  dif=next^(*now);
 
  switch(next){
  case SECTION_NULL:
  case SECTION_CERT_BASIC_CERT: 
  case SECTION_CERT_EXT:
    *now=(dif!=SECTION_NULL)?next:SECTION_NULL;
    break;
  case SECTION_CERT_EXT_AUTH_KEY_ID:
  case SECTION_CERT_EXT_CERT_POLICY :
  case SECTION_CERT_EXT_SUB_ALT_NAME:
  case SECTION_CERT_EXT_ISSUER_ALT_NAME:
  case SECTION_CERT_EXT_BASIC_CONST:
  case SECTION_CERT_EXT_NAME_CONST :
  case SECTION_CERT_EXT_POLICY_CONST:
  case SECTION_CERT_EXT_CRL_DP:
  case SECTION_CERT_EXT_AUTH_INFO_ACCESS:
    *now=(dif!=SECTION_NULL)?next:SECTION_CERT_EXT;
    break;
  default:
    goto done;
  }

  err=0;

 done:
  if(err==UNSET)fprintf(stderr,"err in set cert seciton:\"%s\"\n",in);
  return err;
}


