#include<stdio.h>
#include<string.h>
#include"ok_asn1.h"
#include "ocsp_opt.h"
#include "ext.h"

/*---------------------------------
  Initialize OCSP Options
  ---------------------------------*/
void ocsp_opt_init(OCSPOptions *opt){
  opt->rState=UNSET;
  opt->rid_type=UNSET;
  opt->rid_ctype=UNSET;
  opt->inhash_type=UNSET;
  opt->snum_type=UNSET;
  opt->cState=UNSET;
  opt->rreason=UNSET;
  opt->nonce_type=UNSET;
  opt->oscode_type=UNSET;
  opt->cpstate_type=UNSET;
}


/*--------------------------------
  print setup OCSP options list
  ---------------------------------*/
void ocsp_opt_print(OCSPOptions *opt){
  printf("### set options list ###\n");

  if(opt->rState!=UNSET){
    printf("rState = %d\n",opt->rState);
  }
  /**responderID**/
  switch(opt->rid_type){
  case BYNAME:
    printf("RID_TYPE= BYNAME\n");
    printf("ctype = %d\n",opt->rid_ctype);
    if(opt->rid.num!=0){
      cert_dn_print(&(opt->rid));
    }
    break;
  case BYKEY:
    printf("RID_TYEP= BYKEY\n");
    printf("key_hash= %s\n",opt->rid_value);
    break;
  default:
    printf("not set rid\n");
  }
  if(*(opt->pAt)){
    printf("pAt = %s\n",opt->pAt);
  }
  /** issuerNameHash **/
  switch(opt->inhash_type){
  case ASN1_PRINTABLE_STRING:
    printf("inhash type = PRINTABLE\n");
    break;
  case ASN1_UTF8STRING:
    printf("inhash type = UTF8\n");
    break;
  default:
    printf("inhash type = UNSET\n");
  }
  if(opt->inhash.num!=0){
    cert_dn_print(&(opt->inhash));
  }

  if(*(opt->ikhash)){
    printf("ikhash = %s\n",opt->ikhash);
  }
  if(opt->snum_type!=UNSET){
    printf("snum_type = %d\n",opt->snum_type);
  }
  if(*(opt->snum_value)){
    printf("snum_value = %s\n",opt->snum_value);
  }
  if(opt->cState!=UNSET){
    printf("cState = %d\n",opt->cState);
  }
  if(*(opt->rTime)){
    printf("rTime = %s\n",opt->rTime);
  }
  if(opt->rreason!=UNSET){
    printf("rreason = %d\n",opt->rreason);
  }
  if(*(opt->tup)){
    printf("tup = %s\n",opt->tup);
  }
}

/*------------------------------------------
  check ocsp options
  ----------------------------------------*/
int ocsp_line_opt_check(char *line,OCSPOptions *opt){
  char opt_value[256]="";
  char *hp,*hpp,*lp;
  int dn_oid,flag=1,st;

  if((hp=strchr(line,'#'))!=NULL){
    *hp='0';
  }

  if((hp=strstr(line,"responderID="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    if(strstr(hp,"byKey:")){
      opt->rid_type=BYKEY;
      if((hp=strchr(hp,':'))==NULL)
	goto done;
      strncpy(opt->rid_value,hp+1,strlen(hp+1));
    }else if(hp=strstr(hp,"byName:")){
      opt->rid_type=BYNAME;
      hpp=hp+7;

      if((hp=(char *)strrchr(hp,':'))==NULL)
	goto done;
      cert_dn_init(&(opt->rid));
      if(set_dn(hpp,&(opt->rid),&(opt->rid_ctype))){
	goto done;
      }
    }
  }else if((hp=strstr(line,"issuerNameHash="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    if(strstr(hp,"Printable:")){
      opt->inhash_type=ASN1_PRINTABLE_STRING;
    }else if(strstr(hp,"UTF8:")){
      opt->inhash_type=ASN1_PRINTABLE_STRING;
    }else{
      goto done;
    }
    cert_dn_init(&(opt->inhash));
    while(flag){
      if((hpp=strrchr(hp,','))==NULL){
	hpp=strchr(hp,'=')+1;
	flag=0;
      }
      if((lp=DN_kind_check(hpp,&dn_oid))!=0){
	cert_dn_set(&(opt->inhash),opt->inhash_type,dn_oid,lp);
      }
      *hpp='\0';
    }
  } else if((hp=strstr(line,"responseStatus="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    opt->rState=atoi(++hp);

  }else if((hp=strstr(line,"producedAt="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    strncpy(opt->pAt,++hp,16);
  }else if((hp=strstr(line,"issuerKeyHash="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    strncpy(opt->ikhash,++hp,64);
  } else if ((hp=strstr(line,"serialNumber=")) != NULL) {
    if (set_serial(hp,opt->snum_value,&(opt->snum_type))) {
      goto done;
    } else {
      if (opt->serial[0] != '\0' && 
	  strcmp(opt->snum_value, "%%AUTO%%") == 0) {
	strcpy(opt->snum_value, opt->serial);
	opt->snum_type = HEX;
      }
    }
  } else if((hp=strstr(line,"CertStatus=")) != NULL) {
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    st=atoi(++hp);
    if((st<0)||(st>2))goto done;
    opt->cState = st;
  }else if((hp = strstr(line,"revocationTime=")) != NULL){
    if((hp=strchr(hp,'='))==NULL){
      goto done;
    }else{
      strncpy(opt->rTime,++hp,16);
    }
  }else if((hp=strstr(line,"revocationReason="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    opt->rreason=atoi(++hp);
  }else if((hp=strstr(line,"thisUpdate="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    strncpy(opt->tup,++hp,16);
  }else if((hp=strstr(line,"certPathStatus="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    if((opt->cpstate_type=set_critical(hp[1]))==UNSET)
      goto done;
    opt->cpstate_value=atoi(hp+3);
  }else if((hp=strstr(line,"ocspStatusCode="))!=NULL){
    if((hp=strchr(hp,'='))==NULL)
      goto done;
    if((opt->oscode_type=set_critical(hp[1]))==UNSET)
      goto done;
    opt->oscode_value=atoi(hp+3);
  }else if((hp=strstr(line,"version="))!=NULL){
    /* version is fixed to 1 */
  }else{
    goto done;
  }
  return(0);

 done:
  printf("option file format error. line = [\"%s\"]\n",line);
  return(1);
}
