#include"ok_x509.h"
#include"ok_asn1.h"
#include"opt.h"
#include"ext.h"

/*---------------------------------
check comment or option
---------------------------------*/
int line_comment_check(char *line){
  int i;
  
  for(i=0;i<strlen(line);i++){
    if((line[i]=='#')||(line[i]=='[')){
      return(0);
    }
    if(isspace(line[i])==0){
      return(1);
    }
  }
}

/*---------------------------------
  check DN kind
  -------------------------------*/
char *DN_kind_check(char *line,int *dn_kind){
  char *ret=NULL;
  
  if(strstr(line,"c=")||strstr(line,"C=")){
    *dn_kind=OBJ_DIR_C;
  }else if(strstr(line,"st=")||strstr(line,"ST=")){
    *dn_kind=OBJ_DIR_ST;
  }else if(strstr(line,"email=")||strstr(line,"EMAIL=")){
    *dn_kind=OBJ_DIR_EMAIL;
  }else if(strstr(line,"l=")||strstr(line,"L=")){
    *dn_kind=OBJ_DIR_L;
  }else if(strstr(line,"o=")||strstr(line,"O=")){
    *dn_kind=OBJ_DIR_O;
  }else if(strstr(line,"ou=")||strstr(line,"OU=")){
    *dn_kind=OBJ_DIR_OU;
  }else if(strstr(line,"cn=")||strstr(line,"CN=")){
    *dn_kind=OBJ_DIR_CN;
  }else if(strstr(line,"uid=")||strstr(line,"UID=")){
    *dn_kind=OBJ_DIR_UID;
  }

  if((ret=(char *)strchr(line,'='))==NULL){
    *dn_kind=0;
  }else{
    ret += 1;
  }

  return ret;
}

int set_serial(char *in,char *ret,int *num_type){
  char *hp;
  int err=-1;

  if((hp=(char *)strstr(in,"dec:"))!=NULL){
    *num_type=DEC;
  }else if((hp=(char *)strstr(in,"hex:"))!=NULL){
    *num_type=HEX;
  }else{
    goto done;
  }
  hp+=4;
  strncpy(ret,hp,strlen(hp));

  err=0;

 done:
  if(err)fprintf(stderr,"err in set_serial\n");
  return err;
}

/*----------------------
 need cert_dn_init()
 ----------------------*/
int set_dn(char *in,CertDN *ret,int *ret_ctype){
  char *hp,*hpp,*lp;
  int err=-1;
  int flag=1,dn_oid,fg=1,i;

  if(strstr(in,"Printable:")){
    *ret_ctype=ASN1_PRINTABLE_STRING;
  }else if(strstr(in,"UTF8")){
    *ret_ctype=ASN1_UTF8STRING;
  }else{
    goto done;
  }
  if((hp=(char *)strchr(in,':'))==NULL)
    goto done;
  while(flag){
    if((hpp=(char *)strrchr(hp,','))!=NULL){
      for(i=1;!strchr(hpp,'=');i++){
	if((hpp=strnrchr(hp,',',i))==NULL){
	  if(strchr(hp,'=')==strrchr(hp,'=')){
	    hpp=hp;
	    flag=0;
	  }else{
	    goto done;
	  }
	}
      }
    }else{
      hpp=hp;
      flag=0;
    }

    if((lp=DN_kind_check(hpp,&dn_oid))!=0){
      cert_dn_set(ret,*ret_ctype,dn_oid,lp);
    }
    *hpp='\0';
  }

  err=0;
 done:
  if(err)fprintf(stderr,"err in set_dn\n");
  return err;
}

void cert_dn_set(CertDN *dn,int dn_ctype,int dn_oid,char *dn_value){
  int num,vlen,len,tlen=0,i;
  char *hp,*lp;

  num=dn->num;
  if(dn_ctype==ASN1_UTF8STRING&&dn_oid==OBJ_DIR_C){
    dn->rdn[num].derform=ASN1_PRINTABLE_STRING;
  }else if(dn_oid==OBJ_DIR_EMAIL){
    dn->rdn[num].derform=ASN1_IA5STRING;
  }else{
    dn->rdn[num].derform=dn_ctype;
  }
  dn->rdn[num].tagoid=dn_oid;
  vlen=strlen(dn_value);
  dn->rdn[num].tag=(char *)calloc(sizeof(char),vlen+1);
  
  for(hp=dn_value,i=0;tlen+i<vlen;hp=lp+1,i++){
    if((lp=(char *)strstr(hp,"\\,"))==NULL){
      lp=hp+strlen(hp);
    }
    len=lp-hp;
    strncpy(dn->rdn[num].tag+tlen,hp,len);
    tlen+=len;
  }
  dn->num=num+1;
}

void cert_dn_print(CertDN *dn){
  int i;
  
  for(i=0;i<dn->num;i++){
    printf("ctype = %d ",dn->rdn[i].derform);
    printf("oid = %d ",dn->rdn[i].tagoid);
    printf("value = %s\n",dn->rdn[i].tag);
  }
}

int set_critical(char in){
  switch(in){
  case 'c':return CRITICAL;
  case 'n':return NON_CRITICAL;
  default:
    return UNSET;
  }
}

unsigned char *str2hex(char *input,int *len){
  unsigned char *ret;
  unsigned char tmp,tmp2;
  int ilen,i,j;
  
  ilen=strlen(input);
  *len=ilen/2;
  ret=(unsigned char *)calloc(*len+1,sizeof(char));
  
  for(i=0,j=0;i<ilen;i+=2){
    tmp=char2hex(input[i]);
    tmp2=char2hex(input[i+1]);
    tmp=(tmp<<4)|tmp2;
    ret[i/2]=tmp;
  }
  return(ret);
}

unsigned char char2hex(char input){
  unsigned char ret;
  switch(input){
  case '0':
    ret=0x00;break;
  case '1':
    ret=0x01;break;
  case '2':
    ret=0x02;break;
  case '3':
    ret=0x03;break;
  case '4':
    ret=0x04;break;
  case '5':
    ret=0x05;break;
  case '6':
    ret=0x06;break;
  case '7':
    ret=0x07;break;
  case '8':
    ret=0x08;break;
  case '9':
    ret=0x09;break;
  case 'a':
  case 'A':
    ret=0x0a;break;
  case 'b':
  case 'B':
    ret=0x0b;break;
  case 'c':
  case 'C':
    ret=0x0c;break;
  case 'd':
  case 'D':
    ret=0x0d;break;
  case 'e':
  case 'E':
    ret=0x0e;break;
  case 'f':
  case 'F':
    ret=0x0f;break;
  }
  return ret;
}

/*-------------------------------------------------------------------
  MAX_DEC_SIZE =  "730750818665451459101842416358141509827966271487"
  -------------------------------------------------------------------*/
unsigned char *str2binary(char *input,int *len){
  char *buf;
  unsigned char *ret;
  int i=0,j;
  char table[20][9];
  int tlen;
  char *in;

  in=input;
  memset(table,0x00,160);

  if((ret=(unsigned char *)calloc(20+2,sizeof(char)))==NULL) goto done;
  ret[0]=0x00;
  ret++;
  
  do{
    tlen=i/8;
    if((buf=str_mod(in,&(table[tlen][i%8])))==NULL)goto done;
    in=buf;
    i++;
    if(i>160)goto done;
  }while(*buf);

  for(j=0;j<tlen+1;j++){
    for(i=0;i<strlen(table[j]);i++){
      if(table[j][strlen(table[j])-i-1]=='1'){
	ret[tlen-j]=(ret[tlen-j]<<1)|0x01;
      }else if(table[j][strlen(table[j])-i-1]=='0'){
	ret[tlen-j]=ret[tlen-j]<<1;
      }
    }
  }
  *len=tlen +1;

  if((*ret&0x80)==0x80){
    ret--;
    *len+=1;
  }

  return ret;

 done:
  free(ret);
  return(NULL);
}

char *str_mod(char *in,char *rem){
  char *ret;
  int num,syou,amari=0;
  int i,j=0,flag=0;

  ret=(char *)calloc(strlen(in)+1,sizeof(char));
  for(i=0;i<strlen(in);i++){
    if((num=char2i(in[i]))<0)goto done;
    num+=amari*10;
    amari=num % 2;
    syou = num /2;
    if((syou!=0)||flag){
      if((ret[j]=i2char(syou))<0) goto done;
      j++;
      flag=1;
    }
  }
  ret[strlen(ret)]='\0';
  *rem= i2char(amari);
  return ret;
 done:
  return NULL;
}

int char2i(int in){
  switch(in){
  case '1':
    return 1;break;
  case '2':
    return 2;break;
  case '3':
    return 3;break;
  case '4':
    return 4;break;
  case '5':
    return 5;break;
  case '6':
    return 6;break;
  case '7':
    return 7;break;
  case '8':
    return 8;break;
  case '9':
    return 9;break;
  case '0':
    return 0;break;
  default:
    return(-1);
  }
}

char i2char(int in){
  switch(in){
    case 1:
      return '1';break;
   case 2:
      return '2';break;
   case 3:
      return '3';break;
   case 4:
      return '4';break;
   case 5:
      return '5';break;
   case 6:
      return '6';break;
   case 7:
      return '7';break;
   case 8:
      return '8';break;
   case 9:
      return '9';break;
   case 0:
      return '0';break;
  default:
    return(-1);
  }
}

int str2tm(char *input,struct tm *stm){
  char buf[20];
  int ylen,base,ilen;



  switch(ilen=strlen(input)){
  case 12:
    ylen=2;
    base=0;
    stm->tm_wday=0;/* This means UTC */
    break;
  case 14:
    ylen=4;
    base=-1900;
    stm->tm_wday=1; /* This means Generalized */
    break;
  default:
    return(1);
  }

  strncpy(buf,input,ylen);
  buf[ylen]='\0';
  stm->tm_year=atoi(buf)+base;
  if((ilen==12)&&(stm->tm_year<50))stm->tm_year+=100;
  
  strncpy(buf,input+ylen,2);
  buf[2]='\0';
  stm->tm_mon=atoi(buf)-1;
  strncpy(buf,input+ylen+2,2);
  buf[2]='\0';
  stm->tm_mday=atoi(buf);
  strncpy(buf,input+ylen+2+2,2);
  buf[2]='\0';
  stm->tm_hour=atoi(buf);
  strncpy(buf,input+ylen+2+2+2,2);
  buf[2]='\0';
  stm->tm_min=atoi(buf);
  strncpy(buf,input+ylen+2+2+2+2,2);
  stm->tm_sec=atoi(buf);

  return(0);
}

int str2ip(char *str, unsigned char *bin){
	char *n,*cp=str;
	int i;

	memset(bin,0,4);
	for(i=0;(n=(char *)strchr(cp,'.'))&&(i<3); cp=(char *)n+1,i++)
		bin[i] = atoi(cp);
	
	bin[i] = atoi(cp); /* last byte */

	return 0;
}

char *strnrchr(char *in,int target,int n){
  int i,ilen,rlen;
  char *buf,*ret;
 
  ilen=strlen(in);
  if((buf=(char *)calloc(ilen+1,sizeof(char)))==NULL) return NULL;
  strncpy(buf,in,ilen);

  for(i=0;i<n;i++){
    if((ret=(char  *)strrchr(buf,target))==NULL) return NULL;
    *ret='.';
  }
  rlen=strlen(ret);

  return(in+(ilen-rlen));
}


/*--------------------------------------------------
Printable String$B$r%A%'%C%/$7$J$$%P!<%8%g%s(B
 ---------------------------------------------------*/
CertExt *Extnew_altname_printable_noCheck(int id, ExtGenNames *top){
	CE_SbjAltName *ret;

	if((ret=(CE_SbjAltName*)CertExt_new(id))==NULL) 
		goto error;

	/* set information */
	ret->egn = top;

	/* get DER */
	if((ret->der=ExtGN_toDER_printable_noCheck(top,NULL,&ret->dlen))==NULL)
		goto error;

	return (CertExt*)ret;
error:
	CertExt_free((CertExt*)ret);
	return NULL;
}

CertExt *Extnew_crl_distpoint_printable_noCheck(ExtGenNames *distp,unsigned char *flg,ExtGenNames *issuer){
	CE_CRLDistPt *ret;
	unsigned char *cp;
	int i,j,k=16,l;

	/* estimate size & malloc DER */
	if(distp){
		if((i=ExtGN_estimate_der_size(distp))<0) goto error;
		k+=i;
	}
	if(issuer){
		if((i=ExtGN_estimate_der_size(issuer))<0) goto error;
		k+=i;
	}
	k+=(flg)?(16):(0);

	if((ret=(CE_CRLDistPt*)CertExt_new(OBJ_X509v3_CRL_Point))==NULL) goto error;

	if((ret->der=(unsigned char*)MALLOC(k))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509EXT,ERR_PT_EXTCERT+9,NULL);
		goto error;
	}

	i=0; cp=ret->der;
	if(distp){
		/* set data */
		ret->distp[0].distp.fullName = distp;
		ret->distp[0].distp.FullorRDN= 1;
		/* get DER */
		if(ExtGN_toDER_printable_noCheck(distp,cp,&j)==NULL) goto error;	
		*cp = 0xa0; /* implicit */
		ASN1_set_explicit(j,0,cp,&j);
		cp+=j; i+=j;
	}
	if(flg){
		/* set data */
		memcpy(ret->distp[0].flag,flg,2);
		/* get DER */
		asn1_check_derbit(2,flg,&k,&l);
		ASN1_set_bitstring(k,l,flg,cp,&j);
		*cp = 0x81; /* implicit */
		cp+=j; i+=j;
	}
	if(issuer){
		/* set data */
		ret->distp[0].cRLIssuer = issuer;
		/* get DER */
		if(ExtGN_toDER_printable_noCheck(issuer,cp,&j)==NULL) goto error;
		*cp = 0xa2; /* implicit */
		i+=j;
	}

	ASN1_set_sequence(i,ret->der,&i);
	ASN1_set_sequence(i,ret->der,&ret->dlen);
	ret->pnum = 1;
	return (CertExt*)ret;
error:
	CertExt_free((CertExt*)ret);
	return NULL;
}

unsigned char *ExtGN_toDER_printable_noCheck(ExtGenNames *top,unsigned char *buf,int *ret_len){
	ExtGenNames	*now;
	unsigned char *cp,*ret;
	int i,j=0;

	if(buf==NULL){
		if((i=ExtGN_estimate_der_size(top))<=0)
			return NULL;

		if((ret=(unsigned char*)MALLOC(i))==NULL){
			OK_set_error(ERR_ST_MEMALLOC,ERR_LC_X509EXT,ERR_PT_EXTGN+9,NULL);
			return NULL;
		}
	}else{
		ret=buf;
	}

	*ret_len=0; cp=ret;
	for(now=top;now!=NULL;now=now->next){
		if(ExtGN_DER_gname_printable_noCheck(now,cp,&i)) goto error;
		cp+=i; j +=i;
	}
	ASN1_set_sequence(j,ret,ret_len);
	return ret;
error:
	if(ret!=buf) FREE(ret);
	return NULL;
}

int ExtGN_DER_gname_printable_noCheck(ExtGenNames *now,unsigned char *ret,int *ret_len){
	int i;
	switch(now->type){
	case 1:	/* rfc822Name (IA5String) */
	case 2:	/* dNSName (IA5String) */
	case 6:	/* uniformResourceIdentifier (IA5String) */
		if(ASN1_set_ia5(now->name,ret,ret_len)) goto error;
		ASN1_set_implicit(now->type,ret);
		break;
	case 4:	/* directoryName */
	        #ifdef __PRINTABLE_STRING_NOCHECK__
		if(Cert_DER_subject_printable_noCheck((CertDN*)now->name,ret,&i)) goto error;
		#else
		if(Cert_DER_subject((CertDN*)now->name,ret,&i)) goto error;
		#endif
		ASN1_set_explicit(i,4,ret,ret_len);
		break;
	case 7:	/* iPAddress (OCTET STRING) */
		ASN1_set_octetstring(now->name_len,now->name,ret,ret_len);
		*ret = 0x87; /* implicit */
		break;
	case 8:	/* registeredID (OBJECT IDENTIFIER) */
		if((str2objid(now->name,ret,32))<0) goto error;
		*ret = 0x88; /* implicit */
		break;
	case 0: /* otherName */
		if(ExtGN_DER_othname((OtherName*)now->name,ret,ret_len)) goto error;
		break;
	default:
		OK_set_error(ERR_ST_UNSUPPORTED_PARAM,ERR_LC_X509EXT,ERR_PT_EXTGN+8,NULL);
		goto error;
	}
	return 0;
error:
	return -1;
}

int Cert_DER_subject_printable_noCheck(CertDN *dn,unsigned char *ret,int *ret_len){
	unsigned char *tmp,*cp,uc[256];
	int		i,j,k,l,t;

	tmp=ret;
	memset(uc,0,256);
	for(l=k=0;k<dn->num;k++){
		cp =tmp;

		if(ASN1_int_2object(dn->rdn[k].tagoid,cp,&i)) goto error;
		cp +=i;

		switch(dn->rdn[k].derform){
		case ASN1_UTF8STRING:
			/* if((t=UC_conv(UC_LOCAL_JCODE,UC_CODE_UTF8,dn->rdn[k].tag,strlen(dn->rdn[k].tag),uc,254))<0) goto error; */
			strcpy(uc,dn->rdn[k].tag);
			if(ASN1_set_utf8(uc,cp,&j)) goto error;
			break;
		case ASN1_IA5STRING:
			if(ASN1_set_ia5(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		case ASN1_T61STRING:
			if(ASN1_set_t61(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		case ASN1_PRINTABLE_STRING:
		default:
			if(ASN1_set_printable_noCheck(dn->rdn[k].tag,cp,&j)) goto error;
			break;
		}

		i+=j;
		ASN1_set_sequence(i,tmp,&i);
		ASN1_set_set(i,tmp,&i);
		tmp+=i; l+=i;
	}

	ASN1_set_sequence(l,ret,ret_len);
	return 0;
error:
	return -1;
}

int ASN1_set_printable_noCheck(char *str,unsigned char *ret,int *ret_len){
  asn1_set_str(ASN1_PRINTABLE_STRING,str,ret,ret_len);

  return 0;
}
