#include"ok_asn1.h"
#include"ocsp_opt.h"
#include"ocsp_res.h"

/*-----------------
object for ocsp 
------------------*/
/*static unsigned char PKIX_OCSP[]={0x2b,0x06,0x01,0x05,0x05,0x07,0x30,0x01};*/
typedef unsigned char Uchar[];
static Uchar PKIX_OCSP = {0x2b,0x06,0x01,0x05,0x05,0x07,0x30,0x01};
static Uchar GPKI_OID = {0x2a,0x83,0x08,0x8c,0x9a,0x4a,0x0a};
static Uchar ID_REGCERT_MG = {0x2a,0x83,0x08,0x86,0x8f,0x4c,0x01,0x02};

/*----------------------------------------
  get OBJECT IDENTIFIER form int value
  return -1..not object id.
  return 0...no err
  --------------------------------------*/
int ASN1_int_2object_ocsp(int obj,unsigned char *ret,int *ret_len){
  int len = obj/1000;

  *ret = 0x06; ret++;
  *ret = len; ret++;
  *ret_len = 2+len;
  switch(len){
  case 8:
    if(obj>=8201&&obj<=8212){
      memcpy(ret,GPKI_OID,7);ret+=7;
      switch(obj){
      case OBJ_GPKI_SUBSCRIBERCERT:           *ret=0x01;break;
      case OBJ_GPKI_INTERMEDIATECERTS:        *ret=0x02;break;
      case OBJ_GPKI_TRUSTANCHORCERT:          *ret=0x03;break;
      case OBJ_GPKI_REQUIREDPOLICY:           *ret=0x04;break;
      case OBJ_GPKI_REQUIRED_EXPLICIT_POLICY: *ret=0x05;break;
      case OBJ_GPKI_INHIBIT_POLICY_MAPPING:   *ret=0x06;break;
      case OBJ_GPKI_RESPONSEFORMAT:           *ret=0x07;break;
      case OBJ_GPKI_CERTPATHSTATUS:           *ret=0x08;break;
      case OBJ_GPKI_CERTPATH:                 *ret=0x09;break;
      case OBJ_GPKI_REVOCATIONLIST:           *ret=0x0a;break;
      case OBJ_GPKI_OCSPRESPONSE:             *ret=0x0b;break;
      case OBJ_GPKI_MAPPEDPOLICY:             *ret=0x0c;break;
      default:goto err;
      }
    }else goto err;
    break;
  case 9:
    if(obj>=9101 && obj<=9107){
      memcpy(ret,PKIX_OCSP,8); ret+=8;
      switch(obj){
      case OBJ_OCSP_BASIC:        *ret =0x01; break;
      case OBJ_OCSP_NONCE:        *ret =0x02; break;
      case OBJ_OCSP_CRL:          *ret =0x03; break;
      case OBJ_OCSP_RESPONSE:     *ret =0x04; break;
      case OBJ_OCSP_NOCHECK:      *ret =0x05; break;
      case OBJ_OCSP_ARCH_CUTOFF:  *ret =0x06; break;
      case OBJ_OCSP_SERV_LOCATOR: *ret =0x07; break;
      default: goto err;
      }
    }else if(obj>=9108,obj<=9113){
      memcpy(ret,ID_REGCERT_MG,8);ret+=8;
      switch(obj){
      case OBJ_ID_REGCERT_MG_GENMINFOREQ:          *ret=0x15; break; 
      case OBJ_ID_REGCERT_MG_GENPINFORES:          *ret=0x22; break;
      case OBJ_ID_REGCERT_MG_CONFIRMATIONTIME:     *ret=0x66; break;
      case OBJ_ID_REGCERT_MG_OCSPSTATUSCODE:       *ret=0x67; break;
      case OBJ_ID_REGCERT_MG_EFFECTIVETIMELIMIT:   *ret=0x68; break;
      case OBJ_ID_REGCERT_MG_SUSPENSIONSECRETCODE: *ret=0x69; break;
      default: goto err;
      }
    }else goto err;
    break;
  dafault: goto err;
  }
  return(0);
 err:
  fprintf(stderr,"err in object\n");
  return (-1);
}

/*----------------------------------------
  get int value form OBJECT IDENTIFIER
  return -1..not object id.
  return 0...unknown object.
  return OID.else.
  --------------------------------------*/
int ASN1_object_2int_ocsp(unsigned char *in){
  int i,len;
  if(*in!=ASN1_OBJECT_IDENTIFIER){
    printf("not oid input data\n");
    return(-1);
  }
  len=ASN1_length((++in),&i);
  in+=i;
  switch(len){
  case 8:
    if(!memcmp(in,ID_REGCERT_MG,7)){
      in+=8;
      switch(*in){
      case 0x15: return OBJ_ID_REGCERT_MG_GENMINFOREQ; 
      case 0x16: return OBJ_ID_REGCERT_MG_GENPINFORES;
      case 0x66: return OBJ_ID_REGCERT_MG_CONFIRMATIONTIME;
      case 0x67: return OBJ_ID_REGCERT_MG_OCSPSTATUSCODE;
      case 0x68: return OBJ_ID_REGCERT_MG_EFFECTIVETIMELIMIT;
      case 0x69: return OBJ_ID_REGCERT_MG_SUSPENSIONSECRETCODE;
      default:
	return 0;
      }
    }
  case 9:
    if(!memcmp(in,PKIX_OCSP,8)){  /* This is OCSP object */
      in+=8;
      switch(*in){
      case 0x01: return OBJ_OCSP_BASIC;
      case 0x02: return OBJ_OCSP_NONCE;
      case 0x03: return OBJ_OCSP_CRL;
      case 0x04: return OBJ_OCSP_RESPONSE;
      case 0x05: return OBJ_OCSP_NOCHECK;
      case 0x06: return OBJ_OCSP_ARCH_CUTOFF;
      case 0x07: return OBJ_OCSP_SERV_LOCATOR;
      default:
	return 0;
      }
    }else if(!memcmp(in,GPKI_OID,8)){
      in+=8;
      switch(*in){
      case 0x01: return OBJ_GPKI_SUBSCRIBERCERT;
      case 0x02: return OBJ_GPKI_INTERMEDIATECERTS;
      case 0x03: return OBJ_GPKI_TRUSTANCHORCERT;
      case 0x04: return OBJ_GPKI_REQUIREDPOLICY;
      case 0x05: return OBJ_GPKI_REQUIRED_EXPLICIT_POLICY;
      case 0x06: return OBJ_GPKI_INHIBIT_POLICY_MAPPING;
      case 0x07: return OBJ_GPKI_RESPONSEFORMAT;
      case 0x08: return OBJ_GPKI_CERTPATHSTATUS;
      case 0x09: return OBJ_GPKI_CERTPATH;
      case 0x0a: return OBJ_GPKI_REVOCATIONLIST;
      case 0x0b: return OBJ_GPKI_OCSPRESPONSE;
      case 0x0c: return OBJ_GPKI_MAPPEDPOLICY;
      default: return 0;
      }
    }
  default:
    return 0;
  }
}

void OCSP_obj_print(int oid){
  switch(oid){
  case OBJ_OCSP_BASIC:printf("id-pkix-ocsp-basic\n"); break;
  case OBJ_OCSP_NONCE:printf("id-pkix-ocsp-nonce\n"); break;
  case OBJ_OCSP_CRL:printf("id-pkix-ocsp-crl\n"); break;
  case OBJ_OCSP_RESPONSE:printf("id-pkix-ocsp-response\n"); break;
  case OBJ_OCSP_NOCHECK:printf("id-pkix-ocsp-nocheck\n"); break;
  case OBJ_OCSP_ARCH_CUTOFF:printf("id-pkix-ocsp-archive-cutoff\n"); break;
  case OBJ_OCSP_SERV_LOCATOR:printf("id-pkix-ocsp-service-locator\n"); break;
  default:
    printf("unknown object.\n");
  }
}
