#include"ok_x509.h"
#include"ok_asn1.h"
#include"ocsp_opt.h"
#include"ocsp_req.h"
#include"ocsp_res.h"

int ASN1_set_OCSPresp(OCSPresp *oresp,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;
  
  if(oresp==NULL)
    goto done;
  ASN1_set_enumerated(oresp->respState,ret,&tlen);
  ret+=tlen;
  switch(oresp->respState){
  case SUCCESSFUL:
    if(ASN1_set_RespBytes(oresp->rbytes,ret,&len))
      goto done;
    ASN1_set_explicit(len,0,ret,&len);
    ret+=len,tlen+=len;
    break;
  case MALFORMEDREQUEST:
  case INTERNALERROR:
  case TRYLATER:
  case SIGREQUIRED:
  case UNAUTHORIZED:
    break;
  default:
    goto done;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;

 done:
  if(err)fprintf(stderr,"err in ASN1_set oresp\n");
  return err;
}

int ASN1_set_RespBytes(RespBytes *rbytes,unsigned char *ret,int *ret_len){
  unsigned char *top=ret,*in=NULL;
  int len,tlen=0,err=-1;
  
  if(rbytes==NULL)
    goto done;
  if(ASN1_int_2object_ocsp(rbytes->respType,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(ASN1_set_BaseResp(rbytes->bresp,ret,&len))
    goto done;
  if((in=(unsigned char *)calloc(len+1,sizeof(char)))==NULL)
    goto done;
  memcpy(in,ret,len);
  ASN1_set_octetstring(len,in,ret,&len);
  tlen+=len;
  ASN1_set_sequence(tlen,top,ret_len);

  err=0;

 done:
  if(err)fprintf(stderr,"err in ASN1_set rbytes\n");
  return err;
}
  
int ASN1_set_BaseResp(BaseResp *bresp,unsigned char *ret,int *ret_len){
  unsigned char *der=NULL,*top=ret;
  int len,tlen=0,err=-1;
  Prvkey_RSA *prv;

  if(bresp==NULL)
    goto done;
  if(ASN1_set_TBSresp(bresp->tresp,ret,&len))
    goto done;
  if((der=(unsigned char *)calloc(len,sizeof(char)))==NULL)
    goto done;
  memcpy(der,ret,len);
  ret+=len;tlen+=len;

  if(x509_DER_algoid(bresp->sig_algo,NULL,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if((prv=(Prvkey_RSA *)PEM_read_p8(bresp->rkey))==NULL){
    if((prv=(Prvkey_RSA *)PEM_read_rsaprv(bresp->rkey))==NULL){
      fprintf(stderr,"err in read PEM prvkey");
      goto done;
    }
  }
  set_digalgo_from_sigalgo(OBJ_SIG_SHA1RSA);
  if(x509_set_signature(der,(Key *)prv,&(bresp->signature),&(bresp->siglen)))
    goto done;
  ASN1_set_bitstring(0,bresp->siglen,bresp->signature,ret,&len);
  ret+=len;tlen+=len;

  if(bresp->ct_der){
    memcpy(ret,bresp->ct_der,bresp->ct_len);
    ASN1_set_sequence(bresp->ct_len,ret,&len);
    ASN1_set_explicit(len,0,ret,&len);
    ret+=len;tlen+=len;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;

 done:
  if(der)free(der);
  if(err)fprintf(stderr,"err in ASN1_set bresp\n");
  return err;
}

int ASN1_set_TBSresp(TBSresp *tresp,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;

  if(tresp==NULL)
    goto done;
  if(tresp->version!=0){
    ASN1_set_integer(tresp->version,ret,&len);
    ASN1_set_explicit(len,0,ret,&len);
    ret+=len;tlen+=len;
  }
  if(ASN1_set_RespID(tresp->rid,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(ASN1_set_time(&(tresp->producedAt),ASN1_GENERALIZEDTIME,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(ASN1_set_Resps(tresp->resps,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(tresp->respExt!=NULL){
    /* nonce set */
    if(x509_DER_exts_ocsp(tresp->respExt,ret,&len))
      goto done;
    ASN1_set_explicit(len,1,ret,&len);
    ret+=len;tlen+=len;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;
  
 done:
  if(err)fprintf(stderr,"err in ASN1_set TBSresp \n");
  return err;
}

int ASN1_set_RespID(RespID *rid,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;

  if(rid==NULL)
    goto done;
  switch(rid->type){
  case BYNAME:
    #ifdef __PRINTABLE_STRING_NOCHECK__
    if(Cert_DER_subject_printable_noCheck(&(rid->subject_dn),ret,&len)) goto done;
    #else
    if(Cert_DER_subject(&(rid->subject_dn),ret,&len)) goto done;
    #endif
    ASN1_set_explicit(len,BYNAME,ret,ret_len);
    break;
  case BYKEY:
    if(rid->key_hash==NULL)
      goto done;
    ASN1_set_octetstring(20,rid->key_hash,ret,&len);
    ASN1_set_explicit(len,BYKEY,ret,ret_len);
    break;
  default:
    goto done;
  }
  err=0;

 done:
  if(err)fprintf(stderr,"err in ASN1_set rid\n");
  return err;
}

int ASN1_set_Resps(Resps *resps,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;
  
  if(resps==NULL)goto done;
  while(resps){
    if(ASN1_set_SingleResp(resps->sresp,ret,&len))
      goto done;
    ret+=len;tlen+=len;
    resps=resps->next;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;
 done:
  if(err)fprintf(stderr,"err in ASN1_set resps\n");
  return err;
}

int ASN1_set_SingleResp(SingleResp *sresp,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;
  
  if(ASN1_set_CertID(sresp->cid,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(ASN1_set_CertState(sresp->cstate,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(ASN1_set_time(&(sresp->thisUpdate),ASN1_GENERALIZEDTIME,ret,&len))
    goto done;
  ret+=len;tlen+=len;
  if(sresp->nextUpdate.tm_year){
    if(ASN1_set_time(&(sresp->nextUpdate),ASN1_GENERALIZEDTIME,ret,&len))
      goto done;
      ASN1_set_explicit(len,0,ret,&len);
      ret+=len;tlen+=len;
  }
  if(sresp->singleExt){
    if(x509_DER_exts_ocsp(sresp->singleExt,ret,&len))
      goto done;
    ASN1_set_explicit(len,1,ret,&len);
    ret+=len;tlen+=len;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;
 done:
  if(err)fprintf(stderr,"err in ASN1_set_SingleResp\n");
  return err;
}

int ASN1_set_CertState(CertState *cstate,unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;
  
  if(cstate==NULL)goto done;
  switch(cstate->state){
  case GOOD: /*GOOD = 1*/
    ASN1_set_null(ret);
    ASN1_mset_implicit(2,GOOD,ret,ret_len);
    break;
  case REVOKED: /*REVOKED = 1*/
    if(ASN1_set_RevInfo(cstate->rev,ret,&len)){
      goto done;
    }else{
      ASN1_mset_implicit(len,REVOKED,ret,ret_len);
    }
    break;
  case UNKNOWN:
    ASN1_set_null(ret);
    ASN1_mset_implicit(2,UNKNOWN,ret,ret_len);
    break;
  default:
    goto done;
  }
  err = 0;

 done:
  if(err)printf("err in cstate set\n");
  return err;
}

int ASN1_set_RevInfo(RevInfo *rinfo, unsigned char *ret,int *ret_len){
  unsigned char *top=ret;
  int len,tlen=0,err=-1;

  if(rinfo==NULL)goto done;
  if(ASN1_set_time(&(rinfo->revTime),ASN1_GENERALIZEDTIME,ret,&len))goto done;

  ret+=len;tlen+=len;
  if(rinfo->rreason!=UNSET){
    ASN1_set_enumerated(rinfo->rreason,ret,&len);
    ASN1_set_explicit(len,0,ret,&len);
    ret+=len;tlen+=len;
  }
  ASN1_set_sequence(tlen,top,ret_len);
  err=0;

 done:
  return err;
}

int ASN1_set_time(struct tm *time,int type,unsigned char *ret,int *ret_len){
	if(stm2UTC(time,ret,(unsigned char)type)==NULL)
		return -1;
	*ret_len=ASN1_tlen(ret)+2;
	return 0;
}

void ASN1_mset_implicit(int len,char num,unsigned char *der,int *ret_len){
  if(*der&0x20){
    *der=0xa0|num;
  }else{
    *der=0x80|num;
  }
  *ret_len=len;
}
