#include<string.h>
#include"ok_asn1.h"
#include"ok_tool.h"
#include"ocsp_opt.h"
#include"ocsp_res.h"
#include"ocsp_req.h"
#include"ext.h"

OCSPresp *set_OCSPresp(OCSPOptions *opt){
  OCSPresp *oresp;
  
  if((oresp=(OCSPresp *)calloc(1,sizeof(OCSPresp)))==NULL)
    goto done;
  switch(opt->rState){
  case SUCCESSFUL: 
    oresp->respState=SUCCESSFUL;
    break;
  case MALFORMEDREQUEST:
    oresp->respState=MALFORMEDREQUEST;
    break;
  default:
    goto done;
  }
  if((oresp->rbytes=(RespBytes *)set_RespBytes(opt))==NULL)
    goto done;

  return oresp;
  
 done:
  if(oresp)free(oresp);
  fprintf(stderr,"err in oresp\n");
  return NULL;
}

RespBytes *set_RespBytes(OCSPOptions *opt){
  RespBytes *rbytes;

  if((rbytes=(RespBytes *)calloc(1,sizeof(RespBytes)))==NULL)
    goto done;
  rbytes->respType=OBJ_OCSP_BASIC;
  if((rbytes->bresp=(BaseResp *)set_BaseResp(opt))==NULL)
    goto done;

  return rbytes;

 done:
  fprintf(stderr,"err in rbyte\n");
  if(rbytes)free(rbytes);
  return NULL;
}

BaseResp *set_BaseResp(OCSPOptions *opt){
  BaseResp *bresp;
  unsigned char *der;
  FILE *fp;
  int len;

  if((bresp=(BaseResp *)calloc(1,sizeof(BaseResp)))==NULL)
    goto done;
  if((bresp->tresp=(TBSresp *)set_TBSresp(opt))==NULL)
    goto done;
  bresp->sig_algo=OBJ_SIG_SHA1RSA;
  len=strlen(opt->rkey);
  if((bresp->rkey=(char *)calloc(len+1,sizeof(char)))==NULL)
    goto done;
  strncpy(bresp->rkey,opt->rkey,len);
  /*------------
    set signature
    ------------*/
  if(*(opt->rcert)){
    if((fp=fopen(opt->rcert,"rb"))==NULL)
      goto done;
    bresp->ct_len=ok_get_flen(fp);
    if((bresp->ct_der=(unsigned char *)ASN1_read_der(opt->rcert))==NULL)
      goto done;
    if((bresp->certs=(Cert *)ASN1_read_cert(bresp->ct_der))==NULL)
      goto done;
  }

  return bresp;

 done:
  fprintf(stderr,"err in bresp\n");
  return NULL;
}

TBSresp *set_TBSresp(OCSPOptions *opt){
  TBSresp *tresp;
  int len;
  unsigned char *buf;

  if((tresp=(TBSresp *)calloc(1,sizeof(TBSresp)))==NULL)
    goto done;

  tresp->version=0; /* this means version 1 */
  if((tresp->rid=(RespID *)set_RespID(opt))==NULL)
    goto done;
  if(str2tm(opt->pAt,&(tresp->producedAt)))
    goto done;
  if((tresp->resps=set_Resps(opt))==NULL)
    goto done;
  /*--------------------
    set nonce value
    -------------------*/
  if(*(opt->nonce)){
    if((tresp->respExt=(CertExt *)calloc(1,sizeof(CertExt)))==NULL)
      goto done;
    tresp->respExt->extnID=OBJ_OCSP_NONCE;
    tresp->respExt->critical=TRUE;

    if((tresp->respExt->der=(unsigned char *)str2hex(opt->nonce,&(tresp->respExt->dlen)))==NULL)
      goto done;

    tresp->respExt->next=NULL;
  }
  return tresp;
  
 done:
  if(tresp)free(tresp);
  fprintf(stderr,"err in tresp\n");
  return NULL;
}

RespID *set_RespID(OCSPOptions *opt){
  RespID *rid;
  int len;

  if((rid=(RespID *)calloc(1,sizeof(RespID)))==NULL)
    goto done;
  rid->type=UNSET;
  switch(opt->rid_type){
  case BYNAME:
    rid->type=BYNAME;
    if((rid->ctype=opt->rid_ctype)==UNSET)
      goto done;
    if(Cert_dncopy(&(opt->rid),&(rid->subject_dn)))
      goto done;
    if((rid->subject=(char *)Cert_subject_str(&(opt->rid)))==NULL)
      goto done;
    break;
  case BYKEY:
    rid->type=BYKEY;
    rid->key_hash=(unsigned char *)str2hex(opt->rid_value,&len);
    break;
  default:
    goto done;
  }
  return rid;
  
 done:
  fprintf(stderr,"err in set rid\n");
  if(rid)free(rid);
  return NULL;
}

Resps *set_Resps(OCSPOptions *opt){
  Resps *resps;

  if((resps=(Resps *)calloc(1,sizeof(Resps)))==NULL)
    goto done;
  if((resps->sresp=(SingleResp *)set_SingleResp(opt))==NULL)
    goto done;
  resps->next = NULL;

  return resps;
 done:
  fprintf(stderr,"err in resps");
  if(resps)free(resps);
  return NULL;
}

SingleResp *set_SingleResp(OCSPOptions *opt){
  SingleResp *sresp;

  if((sresp=(SingleResp *)calloc(1,sizeof(SingleResp)))==NULL)
    goto done;

  if((sresp->cid=(CertID *)set_CertID(opt))==NULL)
    goto done;
  if((sresp->cstate=set_CertState(opt))==NULL)
    goto done;
  if(*(opt->tup)){
    if(str2tm(opt->tup,&(sresp->thisUpdate))) goto done;
  }
  if(*(opt->nup)){
    if(str2tm(opt->nup,&(sresp->thisUpdate))) goto done;
  }
  /*--------------------------
    set single Extentions
    ---------------------------*/
  if((opt->cpstate_type!=UNSET)&&(opt->cpstate_value!=UNSET)){
    if(add_CertExt_int(&(sresp->singleExt),OBJ_GPKI_CERTPATHSTATUS,opt->cpstate_type,opt->cpstate_value)){
      goto done;
    }
  }
  if((opt->oscode_type!=UNSET)&&(opt->oscode_value!=UNSET)){
    if(add_CertExt_int(&(sresp->singleExt),OBJ_ID_REGCERT_MG_OCSPSTATUSCODE,opt->oscode_type,opt->oscode_value))
      goto done;
  }

  return sresp;

 done:
  fprintf(stderr,"err in set sresp\n");
  if(sresp)free(sresp);
  return NULL;
}

CertState *set_CertState(OCSPOptions *opt){
  CertState *cstate;

  if((cstate=(CertState *)calloc(1,sizeof(CertState)))==NULL)
    goto done;
  if(opt->cState!=UNSET)cstate->state=opt->cState;
  if(opt->cState==REVOKED){
    if((cstate->rev=(RevInfo *)set_RevInfo(opt))==NULL)
      goto done;
  }

  return cstate;

 done:
  fprintf(stderr,"err in set cstate\n");
  if(cstate)free(cstate);
  return NULL;

}

RevInfo *set_RevInfo(OCSPOptions *opt){
  RevInfo *rinfo;

  if((rinfo=(RevInfo *)calloc(1,sizeof(RevInfo)))==NULL)
    goto done;
  if(opt->rTime){
    if(str2tm(opt->rTime,&(rinfo->revTime)))
      goto done;
  }
  if(opt->rreason!=UNSET){
    rinfo->rreason=opt->rreason;
  }else{
    rinfo->rreason=UNSET;
  }
  return rinfo;

 done:
  fprintf(stderr,"err in set rinfo\n");
  if(rinfo)free(rinfo);
  return NULL;
}
