/* $Id: crt2ccp.c,v 1.4 2004/07/09 09:48:37 urushima Exp $
 *******************************************************************
 *
 * Challenge PKI Test Suite 2.0
 *
 *							Jul 7, 2004
 *
 *			Copyright (c) 2002-2004 
 *			Challenge PKI Project http://www.jnsa.org/mpki/
 *			  IPA-ISEC http://www.ipa.go.jp/security/
 *			  NPO-JNSA http://www.jnsa.org/
 *			  SECOM Co.,Ltd. http://www.secom.co.jp/
 *			  All Rights Reserved.
 *
 *******************************************************************
 */

/*
 * $Id: crt2ccp.c,v 1.4 2004/07/09 09:48:37 urushima Exp $
 *
 * crt2ccp
 * 
 * 	generate a cross certificate pair in DER format from 
 * 	issuedToThisCA and issuedByThisCA certificate. At least
 *	one of there certificate is certificate is necessary.
 *	
 *	% crt2ccp [-issuedto <cert>] [-issuedby <cert>] -out <ccp>
 *	
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ok_asn1.h"
#include "ok_tool.h"
#include "ok_x509.h"

char *toCertFile = NULL; /* issuedToThisCA certificate file */
char *byCertFile = NULL; /* issuedByThisCA certificate file */
char *ccpFile = NULL;    /* output cross certificate pair file */

int parseOption(int argc, char **argv);
void errorExit(int code, char *message);
void usage();
void printUsage();

int main(int argc, char **argv) {
  Cert *issuedToCert = NULL;
  Cert *issuedByCert = NULL;
  CertPair *certPair = NULL;

  int junk;

  OK_set_sbjdn_conv_flag(0);

  parseOption(argc, argv);

  if (toCertFile != NULL) issuedToCert = Cert_read_file(toCertFile);
  if (byCertFile != NULL) issuedByCert = Cert_read_file(byCertFile);

  certPair = CertPair_new();
  if (issuedToCert != NULL) certPair->issuedToThisCA = issuedToCert;
  if (issuedByCert != NULL) certPair->issuedByThisCA = issuedByCert;

  if ((certPair->der = CertPair_toDER(certPair, NULL, &junk)) == NULL) {
    printf("crt2ccp: error in [CertPair_toDER]\n");
    exit(-1);
  }

  ASN1_write_der(certPair->der, ccpFile);
  
  exit(0);
}

int parseOption(int argc, char **argv) {
  if (argc < 3) usage();

  while (*++argv) {
    if (!strcmp(*argv, "-h")) {
      usage();
    } else if (!strcmp(*argv, "-issuedto")) {
      if (*++argv == NULL) goto error;
      toCertFile = *argv;
    } else if (!strcmp(*argv, "-issuedby")) {
      if (*++argv == NULL) goto error;
      byCertFile = *argv;
    } else if (!strcmp(*argv, "-out")) {
      if (*++argv == NULL) goto error;
      ccpFile = *argv;
    }
  }

  if (toCertFile == NULL && byCertFile == NULL) goto error;
  if (ccpFile == NULL) goto error;

  return 1;
 error:
  printUsage();
  errorExit(1, "illeagal option");
  return -1;
}

void printUsage() {
  printf("usage: ");
  printf("crt2ccp [-issuedto <cert>] [-issuedby <cert>] [-out <ccp>]\n");
}

void usage() {
  printUsage();
  exit(0);
}

void errorExit(int code, char *message) {
  printf("ERROR: %s\n", message);
  exit(code);
}

