/* $Id: p7b2crt.c,v 1.3 2004/07/09 09:48:37 urushima Exp $
 *******************************************************************
 *
 * Challenge PKI Test Suite 2.0
 *
 *							Jul 7, 2004
 *
 *			Copyright (c) 2002-2004 
 *			Challenge PKI Project http://www.jnsa.org/mpki/
 *			  IPA-ISEC http://www.ipa.go.jp/security/
 *			  NPO-JNSA http://www.jnsa.org/
 *			  SECOM Co.,Ltd. http://www.secom.co.jp/
 *			  All Rights Reserved.
 *
 *******************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ok_asn1.h"
#include "ok_pkcs.h"

void parseOption(int argc, char **argv);
void usage();

unsigned char *p7bFileName = NULL;
unsigned char *outputFileHead = NULL;

int main(int argc, char **argv) {
  PKCS7		*p7s;
  P12_Baggage	*bg;
  int		cert_idx = 1;
  int		crl_idx = 1;
  unsigned char	*der;
  Cert		*cert;
  CRL		*crl;
  P12_CertBag	*certBag;
  P12_CRLBag	*crlBag;
  char		buf[256];

  parseOption(argc, argv);

  if ((p7s = P7s_read_file(p7bFileName)) == NULL) {
    printf("can't read '%s' PKCS#7 file.\n", p7bFileName);
    return -1;
  }

  for (bg = p7s->bag; bg != NULL; bg = bg->next) {
    switch (bg->type) {
    case OBJ_P12v1Bag_CERT:
      sprintf(buf, "%s.%d.cer", outputFileHead, cert_idx++);
      certBag = (P12_CertBag*)bg;
      cert = certBag->cert;
      der = cert->der;
      ASN1_write_der(der, buf);
      break;
    case OBJ_P12v1Bag_CRL:
      sprintf(buf, "%s.%d.crl", outputFileHead, crl_idx++);
      crlBag = (P12_CRLBag*)bg;
      crl = crlBag->crl;
      der = crl->der;
      ASN1_write_der(der, buf);
      break;
    }
  }

  /* P7_print(p7s); */
  P7_free(p7s);

  return 0;
}

void parseOption(int argc, char **argv) {
  int i = 0;
  for (i = 1; i < argc; i++) {
    if (!strcmp("-f", argv[i])) {
      if (++i < argc)
	p7bFileName = argv[i];
      else 
	goto error;
    } else if (!strcmp("-c", argv[i])) {
      if (++i < argc)
	outputFileHead = argv[i];
      else 
	goto error;
    } else if (!strcmp("-h", argv[i])) {
      goto error;
    } else {
      goto error;
    }
  }

  if (p7bFileName != NULL && outputFileHead != NULL) return;

 error:
  usage();
  exit(-1);
}

void usage() {
  printf("usage:\n");
  printf("  p7b2crt -f P7BFILENAME -c OUTPUTFILEHEAD\n");
  printf("  p7b2crt -h\n");
}
