/*
$Id: CPKI2003_P7.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
*/
#include "CPKI2003.h"
#include <string.h>
#include "SignerInfo.h"


static int CPKI2003_P7_DER_signed_cert(PKCS7 *p7,unsigned char *cn0,int *ret_len, Cert** cert_list){
	int index, len, cert_len;
	Cert **list;
	Cert *cert;
	
	index = 0;
	list = cert_list;
	if(*list == NULL){
		*ret_len = 0;
		return 0;
	}
	while(*list != NULL){
		cert = *list;
		cert_len = ASN1_length(cert->der + 1, &len);
		cert_len += len + 1;
		memcpy(cn0 + index, cert->der, cert_len);
		index += cert_len;
		list ++;
	}
	ASN1_set_explicit(index,0,cn0,ret_len);
	return 0;
}



static int CPKI2003_P7_DER_sigCont(Config *conf, P7_Signed *sig,unsigned char *cp,int *ret_len){
	int	index,len;	
	index = 0;

	if(contains_Config(conf, CONFIG_eContentType)){
		setObjectID(getObjectID_Config(conf, CONFIG_eContentType), cp, &len);
	}
	else{
		setObjectID(OID_TIMESTAMPTOKEN, cp, &len);
	}
	index = len;
	ASN1_set_octetstring(sig->cnt_size, sig->content, cp+index, &len);
	setDepth(len, cp+index, &len);
	*(cp+index) = 0xa0;	
	ASN1_set_sequence(len+index, cp, &len);
	*ret_len = len;
	return 0;
}


unsigned char *CPKI2003_P7_signed_toDER(Config *conf, PKCS7 *p7
				,unsigned char *buf,int *ret_len, Cert** cert_list){
	P7_Signed *p7sig;
	unsigned char *cp,*ret;
	int len,i,j,k,sigf=0;

	int index, index_0;
	
	if((i=P7s_estimate_der_size(p7))<=0)
		return NULL;
	
	if(buf==NULL){
		if((ret=(unsigned char*)MALLOC(i))==NULL){
			OK_set_error(ERR_ST_MEMALLOC,ERR_LC_PKCS7,ERR_PT_P7SASN1,NULL);
			return NULL;
		}
	}else{
		ret=buf;
	}
	
	index = 0;
	cp = ret;
	// ContentInfo.conetntType
	if(contains_Config(conf, CONFIG_contentType)){
		setObjectID(getObjectID_Config(conf, CONFIG_contentType), cp, &len);
	}
	else{
		ASN1_int_2object(OBJ_P7_SIGNED,cp,&len);
	}
	index += len;
	index_0 = index;
	
	if((p7sig=(P7_Signed*)(p7->cont)))
		if(p7sig->digest_algo) sigf=1;
	
	/* version number */
	(p7sig)?(k=p7sig->version):(k=1);
	ASN1_set_integer(k,cp+index,&len);
	index += len;
	
	/* digestAlgorithms */
	(sigf)?(k=p7sig->digest_algo):(k=0);
	if(k){
		if(P7_DER_algoId(k,cp+index,&len)) goto error;
		ASN1_set_set(len,cp+index,&len);
		index+=len;
	}else{
		ASN1_set_null(cp+index); *(cp+index)=0x31; /* set [31 00] */
		index+= 2;
	}
	
	/* contentInfo */
	if(sigf){
		if(CPKI2003_P7_DER_sigCont(conf, p7sig,cp+index,&j)) goto error;
		index+=j;
	}else{
		ASN1_int_2object(OBJ_P7_DATA,cp+index,&j);
		ASN1_set_sequence(j,cp+index,&j);
		index += j;
	}
	
	/* check chain first */
	/* if(P12_check_chain((PKCS12*)p7,0)) goto error; */
	
	/* [0] ExtendedCertificatesAndCertificates */

	if(CPKI2003_P7_DER_signed_cert(p7,cp+index,&j, cert_list)) goto error;
	index += j;

	/* [1] CertificateRevocationLists */
	if(P7_DER_signed_crl(p7,cp+index,&j)) goto error;
	index += j;
	
	/* SignerInfos */
	if(sigf && p7sig->signer != NULL &&
	   (contains_Config(conf, CONFIG_signerInfoCount) == false ||
	   getInt_Config(conf, CONFIG_signerInfoCount) > 0)){

		if(getBoolean_Config(conf, CONFIG_signatureTampered) == true){
		  p7sig->signer->signature[1] += 1;
		}
		
		if(P7_DER_signerInfo(p7sig->signer,cp+index,&j)) goto error;
		index +=j;
	}else{
		ASN1_set_null(cp+index); *(cp+index)=0x31; /* set [31 00] */
		index += 2;
	}

	ASN1_set_sequence(index - index_0, ret + index_0, &len);
	ASN1_set_explicit(len, 0, ret+index_0, &len);
	ASN1_set_sequence(len+index_0, ret, &len);
	*ret_len = len;
	return ret;
error:
		if(ret!=buf) FREE(ret);
	return NULL;
}

static AuthAtt *CPKI2003_P7s_attr_cntType(/*int type*/){
	AuthAtt *ret=NULL;
	unsigned char buf[64],*cp;
	int	i,j;
	
	ASN1_int_2object(OBJ_P9_CONTENT_TYPE,buf,&i);
	cp=buf+i;
	
	
	setObjectID(OID_TIMESTAMPTOKEN, cp, &j);
//	if(ASN1_int_2object(type,cp,&j)) goto error;
	ASN1_set_set(j,cp,&j);
	
	ASN1_set_sequence(i+j,buf,&i);
	
	if((ret=P7_authatt_new())==NULL) goto error;
	if((ret->der=(unsigned char*)MALLOC(i))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_PKCS7,ERR_PT_P7SATTR+1,NULL);
		goto error;
	}
	ret->der_size=i;
	memcpy(ret->der,buf,i);
	return ret;
error:
		P7_authatt_free(ret);
	return NULL;
}

static int CPKI2003_P7s_get_authatt(SignerInfo *sig,unsigned char *data,int len
							 ,Cert *cert, Config *conf){
	AuthAtt *att;
	
	if((att=CPKI2003_P7s_attr_cntType(/*OBJ_P7_DATA*/))==NULL) goto error;
	sig->auth=att;
	
	if((att->next=P7s_attr_signtime())==NULL) goto error;
	att=att->next;

	/* in this case just set one cryptograph */
	/* it should be listed .... */ 
	/*
	if((att->next=P7s_attr_smimecap(default_p7env_cry_algo,default_p7env_passwd_len))==NULL)
		goto error;
	att=att->next;
	 */



	if((att->next=P7s_attr_digest(sig,data,len))==NULL) goto error;

	att=att->next;
	att->next = createSigningCertificateAttr_SignerInfo(cert, conf);

	return 0;
error:
		P7_authatt_free(sig->auth);
	sig->auth=NULL;
	return -1;
}

static int setSignerInfo(SignerInfo *si, unsigned char *data, int len, Config *conf
						 , Cert *cert, Key *key, int digest_algo){
	
	/* set version */
	si->version = 1;
	
	/* set issuerAndSerialNumber */
	si->serialNum = cert->serialNumber;
	if(Cert_dncopy(&(cert->issuer_dn),&(si->iss_dn))) goto error;
	
	/* set digestAlgorithm */
	si->digest_algo = digest_algo;
	
	/* set authenticatedAttribute (optional) */
	if(CPKI2003_P7s_get_authatt(si,data,len, cert, conf)) goto error;
	
	/* set encryptedDigest */
	if(P7s_get_signature(si,key,data,len)) goto error;
	
	/* set unauthenticatedAttribute (optional) */
	/* si->unauth = NULL */
	return 0;
error:
		return -1;

}

static int CPKI2003_P7s_get_signerInfo(PKCS7 *p7
	   , unsigned char *data, int len, Config *conf, Cert *signer_cert, Key *signer_key){
	SignerInfo	*si;
	Cert *cert;
	Key *key;
	int i, count;

	cert = signer_cert;
	key = signer_key;

//	if(get_usercert_and_key(p7,&cert,&key)) goto error;
	
	if((si=((P7_Signed*)p7->cont)->signer)==NULL){
		OK_set_error(ERR_ST_NULLPOINTER,ERR_LC_PKCS7,ERR_PT_P7SIGN+1,NULL);
		goto error;
	}


	setSignerInfo(si, data, len, conf, cert, key, ((P7_Signed*)p7->cont)->digest_algo);

	count = getInt_Config(conf, CONFIG_signerInfoCount);
	for(i = 1; i < count; i++){
	  si->next = P7_signer_new();
	  si = si->next;
	  setSignerInfo(si, data, len, conf, cert, key, ((P7_Signed*)p7->cont)->digest_algo);
	}
	return 0;
error:
		return -1;
}


PKCS7 *CPKI2003_P7s_get_signed(Config *conf, PKCS12 *p12
			       , Cert *signer_cert, Key *signer_key
			       , unsigned char *data, int len, int digest_algo){
	PKCS7	*p7;
	P7_Signed	*p7sig;
	
	if((p7=P7_new(OBJ_P7_SIGNED))==NULL) goto error;
	p7sig = (P7_Signed*)p7->cont;
	
	/* set version 1 */
	if(contains_Config(conf, CONFIG_SignedData_version)){
		p7sig->version = getInt_Config(conf, CONFIG_SignedData_version);
	}
	else{
		p7sig->version = 3;
	}
	p7sig->digest_algo = digest_algo;
	
	/* set context of PKCS#7 Signed Data*/
	p7sig->cnt_size = len;
	if((p7sig->content=(unsigned char*)MALLOC(len))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_PKCS7,ERR_PT_P7SIGN,NULL);
		goto error;
	}
	memcpy(p7sig->content,data,len);
	
	/* copy user certs and private key into p7 */
	if(P12_copy_p12bags((PKCS12*)p7,p12)) goto error;
	
	if(CPKI2003_P7s_get_signerInfo(p7, data,len, conf, signer_cert, signer_key)) goto error;
	
	return p7;
error:
		P7_free(p7);
	return NULL;
}
