/*
$Id: Config.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include "util.h"
#include "Config.h"
#include "TimeStamp.h"
#include "StringToken.h"

static OctetString *toOctetString_(char *value){
	char *c;
    int i, j, length, size;
	OctetString *os;
	unsigned char *buf;
	int odd = true;
	
	os = (OctetString *)MALLOC(sizeof(OctetString));
	c = value;
	length = strlen(c);
    size = length;
    if((size % 2) == 1){
        size++;
        odd = false;
    }
    size /= 2;
    os->length = size;
    buf = (unsigned char *)MALLOC(size);
    i = j = 0;
    if(odd == false){
        buf[0] = hex2int(*c) & 0x0F;
        i = j = 1;
    }
    while(*(c + i) != '\0'){
        buf[j] = ((hex2int(*(c+i)) & 0x0F) << 4) | (hex2int(*(c+i+1)) & 0x0F);
        j++;
        i+=2;
    }
    os->data = buf;
    return os;
}

static char* getDN__(char *src, char *tag){
	char *c, *ret;
	ret = NULL;
	c = strstr(src, tag);
	if(c != NULL){
		ret = strdup(c+strlen(tag));
		if((c = strchr(ret, ',')) != NULL){
			*c = '\0';
		}
	}
	return ret;
}

static DirectoryName *toDirectoryName_(char *src){
	DirectoryName *dn = (DirectoryName *)MALLOC(sizeof(DirectoryName));
	dn->c = getDN__(src, "c=");
	dn->st = getDN__(src, "st=");
	dn->l = getDN__(src, "l=");
	dn->o = getDN__(src, "o=");
	dn->ou = getDN__(src, "ou=");
	dn->cn = getDN__(src, "cn=");
	dn->email = getDN__(src, "email=");
	return dn;
}

static int parse_Config_(Config** conf, char* line){
  char* sep;
  *conf = null;
  if((sep = strchr(line, '=')) == null){
    return true;
  }
  *conf = (Config*)MALLOC(sizeof(Config));
  (*conf)->buffer = MALLOC(strlen(line)+1);
  strcpy((*conf)->buffer, line);
  (*conf)->name = (*conf)->buffer;
  sep = strchr((*conf)->buffer, '=');
  *sep = '\0';
  (*conf)->value = sep + 1;
  return true;
}

static Integer *decStringToInteger_(char *str){
	// intň͈͂̂ݑΉ
	Integer *result;
	unsigned char *buf;
	int i;
	int length;
	int val;
	int tmp;
	val = atoi(str);
	
	length = 0;
	tmp = val;
	while(tmp > 0){
		tmp /= 0x100;
		length ++;
	}
	if(val == 0){
		length = 1;
	}
	buf = (unsigned char *)MALLOC(length);
	result = (Integer *)MALLOC(sizeof(Integer));
	result->length = length;
	result->data = buf;
	
	for(i = length-1; i >= 0; i --){
		buf[i] = val % 0x100;
		val /= 0x100;
	}
	return result;
}

static Integer *hexStringToInteger_(char *str){
	char *c;
    int i, j, length, size;
	Integer *result;
	unsigned char *buf;
	int odd = true;
	
	c = str;
	while((*c == '0')){
		c++;
	}
	result = (Integer *)MALLOC(sizeof(Integer));
	length = strlen(c);
    size = length;
    if((size % 2) == 1){
        size++;
        odd = false;
    }
    size /= 2;
    result->length = size;
    buf = (unsigned char *)MALLOC(size);
    i = j = 0;
    if(odd == false){
        buf[0] = hex2int(*c) & 0x0F;
        i = j = 1;
    }
    while(*(c + i) != '\0'){
        buf[j] = ((hex2int(*(c+i)) & 0x0F) << 4) | (hex2int(*(c+i+1)) & 0x0F);
        j++;
        i+=2;
    }
    result->data = buf;
    return result;
}

Config* new_Config(char* fname){
  Config* conf = null;
  Config* tmp;
  char *line;
  FILE *file;
  if((file = fopen(fname, "r")) == null){
    char msg[256];
    sprintf(msg, "config file open error[%s]", fname);
    die(msg);
  }
  while(true){
    if(!readLine(file, &line)){
      die("config line too long");
    }
    if(line == null){
      break;
    }
    if(parse_Config_(&tmp, line) == false){
      die("config parse error");
    }
    if(tmp != null){
      tmp->next = conf;
      conf = tmp;
    }
  }
  fclose(file);
  return conf;
}

char* get_Config(Config *conf, char* name){
  Config *c;
  for(c = conf; c != null; c = c->next){
    if(strcmp(c->name, name) == 0){
      return(c->value);
    }
  }
  return null;
}

StringTokenList getList_Config(Config *conf, char *tag){
	Config *c;
	StringTokenList list;
	int count;
	count = 0;
	for(c = conf; c != null; c = c->next){
		if(strcmp(c->name, tag) == 0){
			count ++;
		}
	}
	list = (StringTokenList)MALLOC(sizeof(char *) * (count + 1));
	count = 0;
	for(c = conf; c != null; c = c->next){
		if(strcmp(c->name, tag) != 0){
			continue;
		}
		*(list+count++) = c->value;
	}
	*(list+count) = NULL;
	return list;
}	

int contains_Config(Config *conf, char *tag){
	char *c = get_Config(conf, tag);
	if(c == NULL){
		return false;
	}
	if(strlen(c) > 0){
		return true;
	}
	return false;

}

ObjectID *getObjectID_Config(Config *conf, char *tag){
	char *c = get_Config(conf, tag);
	if(strcasecmp("SHA-1", c) == 0 || strcasecmp("SHA1", c) == 0){
		c = "1.3.14.3.2.26";
	}else if(strcasecmp("MD5", c) == 0){
		c = "1.2.840.113549.2.5";
	}else if(strcasecmp("RIPEMD-160", c) == 0){
		c = "1.3.36.3.2.1";
	}else if(strcasecmp("MD2", c) == 0){
		c = "1.2.840.113549.2.2";
	}
	return((ObjectID*)c);
}

#define DIRECTORY_NAME_TAG "directoryname:"
#define PRINTABLE_TAG "printable:"
#define UTF8_TAG "utf8:"
#define URI_TAG "uri::"

int isDirectoryName_Config(Config *conf, char *tag){
	char *c;
	c = get_Config(conf, tag);
	return(strncasecmp(c, DIRECTORY_NAME_TAG, strlen(DIRECTORY_NAME_TAG)) == 0);
}
DirectoryName *getDirectoryName_Config(Config *conf, char *tag){
	DirectoryName *dn;
	char *type, *value;
	char *c0 = get_Config(conf, tag);

	type = strchr(c0, ':')+1;
	value = strchr(type, ':')+1;

	dn = toDirectoryName_(value);
	if(strncasecmp(type, "utf8:", 5) == 0){
		dn->type = utf8;
	}else if(strncasecmp(type, "printable:", 10) == 0){
		dn->type = printable;
	}else{
		die(type);
	}
	return dn;
}

int isURI_Config(Config *conf, char *tag){
	char *c = get_Config(conf, tag);
	return(strncasecmp(c, URI_TAG, strlen(URI_TAG)) == 0);
}
char *getURI_Config(Config *conf, char *tag){
	char *c = get_Config(conf, tag);
	return(c+strlen(URI_TAG));
}

Accuracy *getAccuracy_Config(Config *conf, char *tag){
	Accuracy *accuracy;
	char *p1, *p2;
	char *c = get_Config(conf, tag);
	if(c == NULL){
		return null;
	}

	accuracy = (Accuracy *)MALLOC(sizeof(Accuracy));
	accuracy->seconds = -1;
	accuracy->millis = -1;
	accuracy->micros = -1;
	
	p1 = strchr(c, '.');
	p2 = strrchr(c, '.');
	
	if(p1 != c){
		*p1 = '\0';
		accuracy->seconds = atoi(c);
	}
	if((p2 - p1) > 1){
		*p2 = '\0';
		accuracy->millis = atoi(p1+1);
	}
	if(*(p2+1) != '\0'){
		accuracy->micros = atoi(p2+1);
	}
	return accuracy;
}

GeneralizedTime *getGeneralizedTime_Config(Config *conf, char *tag){
	struct tm *now;
	char *c, *result;
	c = get_Config(conf, tag);
	if(*c == '+' || *c == '-'){
		int diff = atoi(c+1);
		time_t t;
		time(&t);
		if(*c == '+'){
			t+=diff;
		}else{
			t-=diff;
		}
		now = gmtime(&t);
		result = (char *)MALLOC(sizeof(char) * 16);
		sprintf(result, "%04d%02d%02d%02d%02d%02dZ"
				, now->tm_year + 1900
				, now->tm_mon + 1
				, now->tm_mday
				, now->tm_hour
				, now->tm_min
				, now->tm_sec);
	}
	else{
		result = c;
	}
	return((GeneralizedTime*)result);
}

Integer *getInteger_Config(Config *conf, char *tag){
	Integer *result;
	char *c;
	c = get_Config(conf, tag);
	if(strncmp(c, "dec:", 4) == 0){
		result = decStringToInteger_(c + 4);
	}
	else if(strncmp(c, "hex:", 4) == 0){
		result = hexStringToInteger_(c + 4);
	}
	else {
		result = decStringToInteger_(c);
	}
	return result;
}

unsigned char *getUTF8String_Config(Config *conf, char *tag){
	return getUnsignedChar_Config(conf, tag);
}

BitString *getBitString_Config(Config *conf, char *tag){
	StringTokenList tokens, token;
	BitString *res;
	int i;
	tokens = split_StringToken(get_Config(conf, tag), ',');

	res = (BitString*)MALLOC(sizeof(BitString));
	res->count = length_StringToken(tokens);
	res->bits = (int*)MALLOC(sizeof(int) * res->count);
	res->maxbit = -1;
	for(token = tokens, i = 0; *token != NULL; token ++, i ++){
		res->bits[i] = atoi(*token);
		if(res->maxbit < res->bits[i]){
			res->maxbit = res->bits[i];
		}
	}
	return res;
}

#define BADFORMAT_DIE {char msg[256];sprintf(msg, "bad extension format(%s)", str);die(msg);}

static Extension *toExtension(char *str){
	Extension *ext;
	char *c, *c2;
	ext = (Extension *)MALLOC(sizeof(Extension));
	c = str;
	if(*c == 'c'){
		ext->critical = true;
	}else if(*c == 'n'){
		ext->critical = false;
	}
	else{
		BADFORMAT_DIE;
	}
	// skip ':'
	c+=2;
	if((c2 = strchr(c, ':')) == NULL){
		BADFORMAT_DIE;
	}
	*c2 = '\0';
	c2 ++;
	ext->extnID = (ObjectID*)c;
	ext->extnValue = toOctetString_(c2);
	return ext;
}
Extension **getExtensions_Config(Config *conf, char *tag){
	int i;
	StringTokenList list;
	StringTokenList l;
	Extension **exts;
	list = getList_Config(conf, tag);
	i = length_StringToken(list);
	exts = (Extension **)MALLOC(sizeof(Extension) * (i + 1));
	for(l = list, i = 0; *l != NULL; l ++, i ++){
		exts[i] = toExtension(*l);
	}
	exts[i] = NULL;
	return exts;
}


int getInt_Config(Config *conf, char *tag){
    char *c;
    c = get_Config(conf, tag);
    if(c == null){
        return(-1);
    }
    return atoi(c);
}

unsigned char* getUnsignedChar_Config(Config *conf, char *tag){
    char *c;
    c = get_Config(conf, tag);
    return((unsigned char *)(c));
}

OctetString *getOctetString_Config(Config *conf, char *tag){
    char *c;
    c = get_Config(conf, tag);
    if(c == null){
		return NULL;
    }
	return toOctetString_(c);
}

int getBoolean_Config(Config *conf, char *tag){
    char *c;
    c = get_Config(conf, tag);
    if(c == null){
		return -1;
    }
    if(strcasecmp(c, "true") == 0){
        return true;
    }else{
        return false;
    }
}

unsigned char* getUnsignedChar(Config *conf, char *tag){
    char *c;
    c = get_Config(conf, tag);
    return((unsigned char *)(c));
}

OctetString *getOctetString(Config *conf, char *tag){
    char *c;
    OctetString *os;
    os = (OctetString *)MALLOC(sizeof(OctetString));
    c = get_Config(conf, tag);
    if(c == null){
        os->data = null;
        return os;
    }
    assert(true);
    return os;
}



void put_Config(Config *conf, char* name, char* value){
  // not implemented
}

void free_Config(Config *conf){
  Config* tmp;
  while(conf != null){
    if(conf->buffer != null){
      free(conf->buffer);
    }
    tmp = conf->next;
    free(conf);
    conf = tmp;
  }
}
