/*
 $Id: TimeStampInfo.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
 */
#include "TimeStampInfo.h"
#include "TimeStampRequest.h"
#include "TimeStamp.h"
#include "string.h"
static void setExtensions(Extension **exts, unsigned char *der_buffer, int *ret_len){
	Extension *ext;
	int index, len, index_0;
	index = 0;
	for(ext = *exts; ext != NULL; ext = *(++exts)){
		index_0 = index;
		setObjectID(ext->extnID, der_buffer+index_0, &len);
		index_0 += len;
		ASN1_set_boolean(ext->critical, der_buffer+index_0, &len);
		index_0 += len;
		setOctetString(ext->extnValue, der_buffer+index_0, &len);
		index_0 += len;
		ASN1_set_sequence(index_0-index, der_buffer+index, &len);
		index += len;
	}
	ASN1_set_sequence(index, der_buffer, &len);
	setDepth(len, der_buffer, &len);
	*der_buffer = 0x80;
	*ret_len = len;
}
static void addAccuracy_(Config *conf, unsigned char *der_buffer, int *ret_len){
	int index, len;
	unsigned char tmp[2];
	Accuracy *accuracy = getAccuracy_Config(conf, CONFIG_accuracy);
	index = 0;
	if(accuracy->seconds >= 0){
		ASN1_set_integer(accuracy->seconds, der_buffer, &len);
		index += len;
	}
	if(accuracy->millis >= 0){ // TAG = b10000000
		tmp[0] = accuracy->millis / 0x100;
		tmp[1] = accuracy->millis % 0x100;
		
		if(tmp[0] == 0){
			ASN1_set_binary(0x80, 1, tmp+1, der_buffer+index, &len);
		}
		else{
			ASN1_set_binary(0x80, 2, tmp, der_buffer+index, &len);
		}
		index += len;
	}
	if(accuracy->micros >= 0){ // TAG = b10000001
		tmp[0] = accuracy->micros / 0x100;
		tmp[1] = accuracy->micros % 0x100;
		if(tmp[0] == 0){
			ASN1_set_binary(0x81, 1, tmp+1, der_buffer+index, &len);
		}
		else{
			ASN1_set_binary(0x81, 2, tmp, der_buffer+index, &len);
		}
		index += len;
	}
	ASN1_set_sequence(index, der_buffer, &len);
	*ret_len = len;
}

static void addMessageImprint_(Config *conf, unsigned char *der_buffer, int *ret_len){
	int len = 0;
	int index = 0;
	if(contains_Config(conf, CONFIG_messageImprint_hashAlgorithm) == false ||
	   contains_Config(conf, CONFIG_messageImprint_hashedMessage) == false){
		die("need messageImprint.hashAlgorithm & messageImprint.hashedMessage in conf");
	}
	{// AlgorithmIdentifier
		/*
		 from RFC2459
		 AlgorithmIdentifier  ::=  SEQUENCE {
			 algorithm               OBJECT IDENTIFIER,
			 parameters              ANY DEFINED BY algorithm OPTIONAL }
		 */
		
		setObjectID(getObjectID_Config(conf, CONFIG_messageImprint_hashAlgorithm), der_buffer, &len);
		
		//if parameter exists, append to buffer here.
		/*
		ASN1_set_null(der_buffer+len);
		len += 2;
		*/

		ASN1_set_sequence(len, der_buffer, &len);
		index += len;
	}
	setOctetString(getOctetString_Config(conf, CONFIG_messageImprint_hashedMessage), der_buffer+index, &len);
	index += len;
	
	ASN1_set_sequence(index, der_buffer, &len);
	*ret_len = len;
}

TimeStampInfo *createTimeStampInfo(Config *conf, TimeStampRequest *request){
	TimeStampInfo *result;
	char *der_buffer;
	int len, index;
	
	
	if(contains_Config(conf, CONFIG_TSTInfo_binary)){
		OctetString *os = getOctetString_Config(conf, CONFIG_TSTInfo_binary);
		return((TimeStampInfo*)os);
	}
	
	der_buffer = MALLOC(TimeStampInfo_ENOGH_SIZE);

	index = 0;
	//version
	setInteger(getInteger_Config(conf, CONFIG_version), der_buffer, &len);
	index += len;
	
	//policy
	setObjectID(getObjectID_Config(conf, CONFIG_policy), der_buffer+index, &len);
	index += len;

	// message imprint
	if(contains_Config(conf, CONFIG_messageImprint_AUTO) &&
	   getBoolean_Config(conf, CONFIG_messageImprint_AUTO) == true){
		MessageImprint *mi;
		if(request == NULL){
			die("Though messageImprint.AUTO are set TRUE, TSQ is not specified.");
		}
		mi = getMessageImprint_TimeStampRequest(request);
		memcpy(der_buffer+index, mi->data, mi->length);
		len = mi->length;
	}
	else{
		addMessageImprint_(conf, der_buffer + index, &len);
	}
	index += len;
	
	// serial number
	setInteger(getInteger_Config(conf, CONFIG_serialNumber), der_buffer+index, &len);
	index += len;

	// genTime
	setGeneralizedTime(getGeneralizedTime_Config(conf, CONFIG_genTime), der_buffer+index, &len);
	index += len;

	// accuracy
	if(contains_Config(conf, CONFIG_accuracy)){
		addAccuracy_(conf, der_buffer+index, &len);
		index += len;
	}
	
	//ordering
	if(contains_Config(conf, CONFIG_ordering)){
		ASN1_set_boolean(getBoolean_Config(conf, CONFIG_ordering), der_buffer+index, &len);
		index+=len;
	}

	// nonce
	if(contains_Config(conf, CONFIG_nonce_AUTO) &&
	   getBoolean_Config(conf, CONFIG_nonce_AUTO) == true &&
	   request != NULL){
		Nonce *nonce = getNonce_TimeStampRequest(request);
		if(nonce != NULL){
		  memcpy(der_buffer+index, nonce->data, nonce->length);
		  index += nonce->length;
		}
	}
	else{
		if(contains_Config(conf, CONFIG_nonce)){
			setInteger(getInteger_Config(conf, CONFIG_nonce), der_buffer+index, &len);
			index += len;
		}
	}
	
	// tsa
	if(contains_Config(conf, CONFIG_tsa)){
		if(isURI_Config(conf, CONFIG_tsa)){
			setURI(getURI_Config(conf, CONFIG_tsa), der_buffer+index, &len);
			index += len;
		}
		else if(isDirectoryName_Config(conf, CONFIG_tsa)){
			setDirectoryName(getDirectoryName_Config(conf, CONFIG_tsa), der_buffer+index, &len);
			index += len;
		}
		else{
			die(get_Config(conf, CONFIG_tsa));
		}
	}

	//extensions
	if(contains_Config(conf, CONFIG_extensions)){
		setExtensions(getExtensions_Config(conf, CONFIG_extensions), der_buffer+index, &len);
		index += len;
	}
	
	ASN1_set_sequence(index, der_buffer, &len);
	result = MALLOC(sizeof(TimeStampInfo));
    result->data = der_buffer;
    result->length = len;
    return result;
}



