/*
 $Id: r_asn1.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
 */
#include "memory.h"
#include "ok_asn1.h"
#include "util.h"
#include "r_asn1.h"

void setDepth(int len,unsigned char *der,int *ret_len){
	unsigned char   buf[32],*cp;
	int i,j;
	
	ASN1_set_length(len,&buf[1],&i);
	cp = &(der[len+i]);
	
	for(j=len-1;j>=0;j--){
		*cp = der[j]; cp--;
	}
	memcpy(der,buf,i+1);
	*ret_len = len+i+1;
}

static void setDN__(int type, int oid, char *value, unsigned char *der, int *ret_len){
	int index, len;
	index = 0;
	ASN1_int_2object(oid, der, &len);
	index += len;
	if(type == 0){ // utf8
		ASN1_set_utf8(value, der+index, &len);
		index += len;
	}
	else{ // printable
		ASN1_set_printable(value, der+index, &len);
		index += len;
	}
	ASN1_set_sequence(index, der, &len);
	ASN1_set_set(len, der, &len);
	*ret_len = len;
}

void setBitString(BitString *bs, unsigned char *der, int *ret_len){
	int bytes, mod, bit, pos_bit, pos_byte, index, len, i;
	unsigned char *buf;
	bytes = (bs->maxbit / 8) + 1;
	mod = 7 - (bs->maxbit % 8);
	buf = (unsigned char *)MALLOC(bytes);
	memset(buf, 0x00, bytes);
	
	for(i = 0; i < bs->count; i ++){
		bit = bs->bits[i];
		pos_byte = bit / 8;
		pos_bit = bit % 8;
		buf[pos_byte] |= (0x80 >> pos_bit);
	}
	index = 0;
	*der = ASN1_BITSTRING;
	index ++;
	
	ASN1_set_length(bytes + 1, (der + index), &len);
	index += len;
	
	*(der + index++) = (unsigned char)mod;
	memcpy((der+index), buf, bytes);
	*ret_len = index + bytes;
}

void setURI(char *uri, unsigned char *der, int *ret_len){
	int len;
	ASN1_set_ia5(uri, der, &len);
	setDepth(len, der, &len);
	*der = 0xA6;
	setDepth(len, der, &len);
	*der = 0xA0;
	*ret_len = len;
}


void setDirectoryName(DirectoryName *dn, unsigned char *der, int *ret_len){
	int index, len;
	index = 0;
	if(dn->c != NULL){
		setDN__(dn->type, OBJ_DIR_C, dn->c, der, &len);
		index += len;
	}

	if(dn->st != NULL){		
		setDN__(dn->type, OBJ_DIR_ST, dn->st, der+index, &len);
		index += len;
	}
	if(dn->l != NULL){		
		setDN__(dn->type, OBJ_DIR_L, dn->l, der+index, &len);
		index += len;
	}
	if(dn->o != NULL){		
		setDN__(dn->type, OBJ_DIR_O, dn->o, der+index, &len);
		index += len;
	}
	if(dn->ou != NULL){		
		setDN__(dn->type, OBJ_DIR_OU, dn->ou, der+index, &len);
		index += len;
	}
	if(dn->cn != NULL){		
		setDN__(dn->type, OBJ_DIR_CN, dn->cn, der+index, &len);
		index += len;
	}
	if(dn->email != NULL){		
		setDN__(dn->type, OBJ_DIR_EMAIL, dn->email, der+index, &len);
		index += len;
	}

	ASN1_set_sequence(index, der, &len);
	setDepth(len, der, &len);
	*der = 0xA4;
	setDepth(len, der, &len);
	*der = 0xA0;
	*ret_len = len;
}

void setInteger(Integer *data, unsigned char *der, int *ret_len){
	int len, index;
	index = 0;
	*der = ASN1_INTEGER;
	index ++;
	ASN1_set_length(data->length, der+index, &len);
	index += len;
	memcpy(der+index, data->data, data->length);
	index += data->length;
	*ret_len = index;
}

void setObjectID(ObjectID *data, unsigned char *der, int *ret_len){
	unsigned char tmp[64];
	int len, i;
	len = str2objid((char *)data, tmp, sizeof(tmp));
	for(i = 0; i < len; i ++){
		der[i] = tmp[i];
	}
	*ret_len = len;
}

void setGeneralizedTime(GeneralizedTime *gentime, unsigned char *der, int *ret_len){
	int len;
	asn1_set_str(ASN1_GENERALIZEDTIME, (char *)gentime, der, &len);
	*ret_len = len;
}
