/*
$Id: tsrgen.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "util.h"
#include "Config.h"
#include "TimeStampToken.h"
#include "TimeStampResponse.h"

static void usage(const char *command){
	char *c;
	if((c = strrchr(command, '/')) != NULL){
		c++;
	}
	else{
		c = (char *)command;
	}
	fprintf(stderr, "usage:\n");
	fprintf(stderr, "   %s --conf <file> [--tstfile <file>] --out <file>\n", c);
}

static BootOptions *parseOptions(int argc, const char *argv[]){
	char *confFile = NULL;
	char *tstFile = NULL;
	char *outFile = NULL;
	int isError = false;
	int i;
	BootOptions *opt;
	
	enum {other, conf, tstfile, out} phase;
	phase = other;
	
	for(i = 1; i < argc && isError == false; i++){
		if(strcmp(argv[i], "--conf") == 0){
			phase = conf;
		}
		else if(strcmp(argv[i], "--tstfile") == 0){
			phase = tstfile;
		}
		else if(strcmp(argv[i], "--out") == 0){
			phase = out;
		}
		else{
			switch(phase){
				case conf:
					confFile = strdup(argv[i]);phase = other;
					break;
				case tstfile:
					tstFile = strdup(argv[i]);phase = other;
					break;
				case out:
					outFile = strdup(argv[i]);phase = other;
					break;
				default:
					isError = true;
					break;
			}
		}
	}
	if(confFile == NULL ||
	   outFile == NULL ||
	   isError == true){
		usage(argv[0]);
		return NULL;
	}
	opt = (BootOptions *)MALLOC(sizeof(BootOptions));
	opt->confFile = confFile;
	opt->tstFile = tstFile;
	opt->outFile = outFile;
	return opt;
}

int main (int argc, const char *argv[], const char *envp[]) {
    FILE *out;
    Config *conf;
    BootOptions *opt;
    TimeStampToken *token;
	TimeStampResponse *response;
#ifdef DEBUG
	char cmd[256];
#endif
    if((opt = parseOptions(argc, argv)) == NULL){
		return(1);
    }
    conf = new_Config(opt->confFile);
    token = NULL;
    if(opt->tstFile != NULL){
	token = readFromFile_TimeStampToken(opt->tstFile);
    }
    response = createTimeStampResponse(conf, token);
	
	out = fopen(opt->outFile, "w");
    fwrite(response->data, response->length, 1, out);
    //    fwrite(info->data, info->length, 1, out);
    fclose(out);

#ifdef DEBUG
	 sprintf(cmd, "/Users/itoh/unix/bin/dumpasn1 %s", opt->outFile);
	 system(cmd);
#endif
    return 0;
}
