/*
$Id: tstgen.c,v 1.1 2004/04/21 12:52:30 urushima Exp $
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "util.h"
#include "Config.h"
#include "TimeStampInfo.h"
#include "TimeStampToken.h"
#include "TimeStampRequest.h"

static void usage(const char *command){
	char *c;
	if((c = strrchr(command, '/')) != NULL){
		c++;
	}
	else{
		c = (char *)command;
	}
	fprintf(stderr, "usage:\n");
	fprintf(stderr, "   %s --conf <file> --tsakey <file> --tsacert <file> --out <file>\n", c);
	fprintf(stderr, "      [--certs certfile certfile ...] [--query queryfile]\n");
}

static BootOptions *parseOptions(int argc, const char *argv[]){
	char *confFile = NULL;
	char *certFiles = NULL;
	char *keyFiles = NULL;
	char *tsaCertFile = NULL;
	char *tsaKeyFile = NULL;
	char *queryFile = NULL;
	char *outFile = NULL;
	int isError = false;
	int i;
	BootOptions *opt;

	enum {other, conf, tsacert, certs, keys, tsakey, query, out} phase;
	phase = other;

	for(i = 1; i < argc && isError == false; i++){
		if(strcmp(argv[i], "--conf") == 0){
			phase = conf;
		}
		else if(strcmp(argv[i], "--tsacert") == 0){
			phase = tsacert;
		}
		else if(strcmp(argv[i], "--out") == 0){
			phase = out;
		}
		else if(strcmp(argv[i], "--certs") == 0){
			phase = certs;
		}
		else if(strcmp(argv[i], "--keys") == 0){
			phase = keys;
		}
		else if(strcmp(argv[i], "--tsakey") == 0){
			phase = tsakey;
		}
		else if(strcmp(argv[i], "--query") == 0){
			phase = query;
		}
		else{
			switch(phase){
				case conf:
					confFile = strdup(argv[i]);phase = other;
					break;
				case certs:
					if(certFiles == NULL){
						certFiles = MALLOC(1024);
						certFiles[0] = '\0';
					}
					strcat(certFiles, " ");
					strcat(certFiles, argv[i]);
					break;
				case keys:
					if(keyFiles == NULL){
						keyFiles = MALLOC(1024);
						keyFiles[0] = '\0';
					}
					strcat(keyFiles, " ");
					strcat(keyFiles, argv[i]);
					break;
				case tsacert:
					tsaCertFile = strdup(argv[i]);phase = other;
					break;
				case tsakey:
					tsaKeyFile = strdup(argv[i]);phase = other;
					break;
				case query:
					queryFile = strdup(argv[i]);phase = other;
					break;
				case out:
					outFile = strdup(argv[i]);phase = other;
					break;
				default:
					isError = true;
					break;
			}
		}
	}
	if(confFile == NULL ||
	   tsaKeyFile == NULL ||
	   tsaCertFile == NULL ||
	   outFile == NULL ||
	   isError == true){
		usage(argv[0]);
		return NULL;
	}
	opt = (BootOptions *)MALLOC(sizeof(BootOptions));
	opt->confFile = confFile;
	opt->tsaKeyFile = tsaKeyFile;
	opt->tsaCertFile = tsaCertFile;
	opt->outFile = outFile;
	opt->queryFile = queryFile;
	if(certFiles != NULL){
		opt->certFiles = split_StringToken(certFiles, ' ');
	}
	if(keyFiles != NULL){
		opt->keyFiles = split_StringToken(keyFiles, ' ');
	}
	return opt;
}
int xmain(){
	FILE *out;
	Config *conf = new_Config("test");
	unsigned char *buf = malloc(10000);
	int len, index, i;
	index = 0;
	for(i = 0; i< 104; i ++){
		setOctetString(getOctetString_Config(conf, "test"), buf+index, &len);
		index += len;
	}
	printf("LEN=%d\n", index);
	ASN1_set_sequence(index, buf, &len);
    out = fopen("der.dat", "w");
    fwrite(buf, len, 1, out);
    fclose(out);
	
	system("/Users/itoh/unix/bin/dumpasn1 der.dat");
	return 0;
}
int main (int argc, const char *argv[], const char *envp[]) {
    FILE *out;
    Config *conf;
    BootOptions *opt;
	TimeStampRequest *request;
    TimeStampInfo *info;
    TimeStampToken *token;

#ifdef DEBUG
    char cmd[256];
#endif
	
    if((opt = parseOptions(argc, argv)) == NULL){
      return(1);
    }
    conf = new_Config(opt->confFile);
	
	if(contains_Config(conf, CONFIG_messageImprint_AUTO) &&
	   getBoolean_Config(conf, CONFIG_messageImprint_AUTO) == true &&
	   opt->queryFile == NULL){
		die("set queryfile");
	}
	
	request = NULL;
	if(opt->queryFile != NULL){
		request = readFromFile_TimeStampRequest(opt->queryFile);
	}
    info = createTimeStampInfo(conf, request);
    token = createTimeStampToken(conf, info, opt->tsaCertFile, opt->tsaKeyFile
								 , opt->certFiles
								 , opt->keyFiles
								 , request);
    out = fopen(opt->outFile, "w");
    fwrite(token->data, token->length, 1, out);
    //    fwrite(info->data, info->length, 1, out);
    fclose(out);

#ifdef DEBUG
    sprintf(cmd, "/Users/itoh/unix/bin/dumpasn1 %s", opt->outFile);
    system(cmd);
#endif
    return 0;
}
